//ProjectContents.cpp

#include "Application.h"
#include "Frame.h"
#include "ProjectContents.h"
#include "Panel.h"
#include "TreeControl.h"
#include "DefaultTreeItem.h"
#include "TreeModel.h"
#include "VCFFile.h"
#include "StringUtils.h"

using namespace VCF;

ProjectContents::ProjectContents()
{

	Application* app = Application::getRunningInstance();
	if ( NULL != app ){
		Frame* frame = this;
		app->loadFrame( &frame );
	}

	vcfInit();
}

ProjectContents::~ProjectContents()
{

}

void ProjectContents::vcfInit()
{
	//VCF::ComponentListener for ProjectContents, event: onComponentCreated, handler: ProjectContents_onComponentCreated
	VCF::ComponentHandler* projectContentsComponentHandler = new VCF::ComponentHandler( this );
	projectContentsComponentHandler->m_componentCreated = (OnComponentEvent)ProjectContents::ProjectContents_onComponentCreated;
	this->addComponentListener( projectContentsComponentHandler );
	this->addEventHandler( "projectContentsComponentHandler", projectContentsComponentHandler );

	this->bindVariable( (VCF::Component**)&m_projectTree, "m_projectTree" );
	this->bindVariable( (VCF::Component**)&m_statBar, "m_statBar" );

}//vcfInit finished


void ProjectContents::ProjectContents_onComponentCreated( VCF::ComponentEvent* event )
{

}

void ProjectContents::setCurrentProject( VCFProject* project )
{
	TreeModel* tm = m_projectTree->getTreeModel();
	if ( NULL != tm ){
		tm->empty();
		DefaultTreeItem* forms = new DefaultTreeItem( "Forms", tm);
		
		DefaultTreeItem* headers = new DefaultTreeItem( "Headers", tm);
		
		DefaultTreeItem* impl = new DefaultTreeItem( "Implementation", tm);		
		

		Enumerator<VCFFile*>* files = project->getFiles();
		while ( files->hasMoreElements() ){
			VCFFile* file = files->nextElement();
			forms->addChild( new DefaultTreeItem( StringUtils::getFileName( file->getVFFFileName() ), tm ) );

			headers->addChild( new DefaultTreeItem( StringUtils::getFileName( file->getHFileName() ), tm ) );

			impl->addChild( new DefaultTreeItem( StringUtils::getFileName( file->getCppFileName() ), tm ) );
		}

		tm->addNodeItem( forms );
		tm->addNodeItem( headers );
		tm->addNodeItem( impl );
	}
}