/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
// ObjectInspector.h

#ifndef _OBJECTINSPECTOR_H__
#define _OBJECTINSPECTOR_H__


#include "Window.h"
#include <vector>

namespace VCF {

class TabbedPages;
class Panel;
class ComboBoxControl;
class TableControl;
class ItemEvent;
class ItemEditorEvent;
class ItemEditorEventAdapter;
class TableItemEditor;

class EventKey {	
public:
	EventKey(){
		EventMethod = "";
		ComponentName = "";
		ListenerClassName = "";
	}
	
	String EventMethod;
	String ComponentName;
	String ListenerClassName;
};

class EventKeyLess{
public:
	/**
	*returns true if a != b !!!!! 
	*/
	bool operator() ( EventKey* a, EventKey* b )const{
		bool result = false;
		if ( (NULL == a) || (NULL == b) ){
			result = true;
		}
		result = !( (a->ComponentName == b->ComponentName) && (a->EventMethod == b->EventMethod) && (a->ListenerClassName == b->ListenerClassName));
		return result;
	}
};

class ObjectInspector : public Window  
{
public:
	ObjectInspector();

	virtual ~ObjectInspector();

	void selectComponent( Component* c );

	void updateProperties();

	void onPropertyItemDblClick( MouseEvent* e );
	
	void onEventItemDblClick( MouseEvent* e );
	
	void onItemChanged( ItemEditorEvent* e ); 

	void addComponent( Frame* currentFrame, Component* component );

	void clearComponents(){
		m_componentList.clear();
	}

	Enumerator<Component*>* getComponentList( Frame* frame );

	void onObjectListSelectionChanged( ItemEvent* event );

	void setCurrentFrame( Frame* currentFrame ){
		m_currentFrame = currentFrame;
	}
private:
	String getUniqueName( Component* component );
	void updateObjectList( Component* component );

	void updateComponentEntries( Component* component );

	TabbedPages* m_tabs;
	TableControl* m_propertyTable;
	TableControl* m_eventTable;
	Panel* m_propPanel;
	Panel* m_eventPanel;
	ComboBoxControl* m_objectsList;
	std::vector<Component*> m_componentList;
	EnumeratorContainer<std::vector<Component*>,Component*> m_compListContainer;
	Component* m_currentComponent;
	ItemEditorEventAdapter* m_itemEventAdapter;
	TableItemEditor* m_prevItemEditor;
	Frame* m_currentFrame;
	void updateEventCode( EventProperty* ep );
	std::map<EventProperty*,String> m_eventMethodMap;
	void readVCFInitEventEntries();
	/**
	*mapped by Listener name to implementing method name
	*/
//	std::map<EventKey*,String,EventKeyLess> m_currentEventEntries;
	std::map<String,String> m_currentEventEntries;
	void clearEventEntries();
	bool hasControl( const String& name );
};

}

#endif //_OBJECTINSPECTOR_H__
