/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 


//VCFBuilderMainWin.h

#ifndef _VCFBUILDERMAINWIN_H__
#define _VCFBUILDERMAINWIN_H__

#include "Window.h"

#define  MAINWIN_COMPONENT_MOUSE_LISTENER	"MainWinSelectedCompMouseListener"

class ProjectContents;

namespace VCF {

class MenuItemEvent;
class MenuItem;
class DragEvent;
class DropEvent;
class ObjectInspector;
class VCFProject;


class VCFBuilderMainWin : public Window  
{
public:
	VCFBuilderMainWin();

	virtual ~VCFBuilderMainWin();

	virtual void paint( GraphicsContext* gc );

	void initObjectInspector();

	void initProjectContents();

	VCFProject* getCurrentProject(){
		return m_currentProject;
	}

	void setCurrentProject( VCFProject* project );
protected:
	

	MenuItem* m_file;
	MenuItem* m_fileNewForm;
	MenuItem* m_fileNewDialog;
	MenuItem* m_fileNewProject;
	MenuItem* m_fileOpenProject;
	MenuItem* m_fileSaveProject;
	MenuItem* m_fileOpenForm;
	MenuItem* m_fileSaveForm;
	MenuItem* m_fileCloseForm;
	MenuItem* m_filePrintForm;
	MenuItem* m_fileQuit;
	MenuItem* m_edit;
	MenuItem* m_editCut;
	MenuItem* m_editCopy;
	MenuItem* m_editPaste;
	MenuItem* m_editSelectAll;
	MenuItem* m_editDeselectAll;
	MenuItem* m_editDelete;

	MenuItem* m_view;
	MenuItem* m_viewObjectInspector;
	MenuItem* m_viewProjectWindow;

	MenuItem* m_insertComponent;
	MenuItem* m_insertPanel;
	MenuItem* m_insertButton;
	
	MenuItem* m_components;
	MenuItem* m_addVPL;

	MenuItem* m_help;
	MenuItem* m_helpAbout;


	ObjectInspector* m_objectInspector;

	ProjectContents* m_projectContents;

	void fileNewFormClick( MenuItemEvent* e );

	void fileNewDialogClick( MenuItemEvent* e );

	void fileNewProjectClick( MenuItemEvent* e );

	void fileOpenProjectClick( MenuItemEvent* e );

	void fileSaveProjectClick( MenuItemEvent* e );

	void fileOpenFormClick( MenuItemEvent* e );

	void fileSaveFormClick( MenuItemEvent* e );

	void fileCloseFormClick( MenuItemEvent* e );

	void filePrintFormClick( MenuItemEvent* e );

	void fileQuitClick( MenuItemEvent* e );

	void editCutClick( MenuItemEvent* e );

	void editCopyClick( MenuItemEvent* e );

	void editPasteClick( MenuItemEvent* e );

	void editSelectAllClick( MenuItemEvent* e );

	void editDeselectAllClick( MenuItemEvent* e );

	void editDeleteClick( MenuItemEvent* e );

	void insertPanelClick( MenuItemEvent* e );
	
	void insertButtonClick( MenuItemEvent* e );

	void helpAboutClick( MenuItemEvent* e );

	void mainWinDragEntered( DragEvent* event );

	void mainWinDropped( DropEvent* event );

	void selectComponent( Component* c );

	void selectedCompMouseDown( MouseEvent* e );
	
	void selectedCompMouseMove( MouseEvent* e );

	void selectedCompMouseUp( MouseEvent* e );

	void insertNewComponentClick( MenuItemEvent* e );

	void addVPL( MenuItemEvent* e );

	void viewObjectInspectorClick( MenuItemEvent* e );

	void viewProjectContentsClick( MenuItemEvent* e );

	Component* m_selectedComponent;

	Frame* m_currentWin;

	Point m_tDragOffset;

	VCFProject* m_currentProject;
	String generateCPP( Frame* frame, const bool& newForm=false, const String& fileDir="" );

	void loadComponents();
	
};

};

#endif // _VCFBUILDERMAINWIN_H__

