/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
// ObjectInspector.cpp

#pragma warning (disable : 4786)

#include "Application.h"
#include "ObjectInspector.h"
#include "TabbedPages.h"
#include "TabPage.h"
#include "Panel.h"
#include "TableControl.h"
#include "PropertyTableModel.h"
#include "ComboBoxControl.h"
#include "DefaultListItem.h"
#include "ListModel.h"
#include "TableCellItem.h"
#include "PropertyCellItem.h"
#include "ItemEvent.h"
#include "ItemEditorListener.h"
#include "TableItemEditor.h"
#include "EventCellItem.h"
#include "EventTableModel.h"
#include "FileStream.h"
#include "FileUtils.h"
#include "VCFBuilderMainWin.h"
#include "VCFProject.h"

using namespace VCF;


ObjectInspector::ObjectInspector()
{
	m_currentComponent = NULL;

	
	m_tabs = new TabbedPages();

	m_objectsList = new ComboBoxControl();
	m_objectsList->setHeight( 25 );	

	ComboBoxAdapter* cba = new ComboBoxAdapter( this );
	cba->m_selectionChange = (OnComboBoxEvent)ObjectInspector::onObjectListSelectionChanged;
	m_objectsList->addComboBoxListener( cba );
	this->addEventHandler( "ComboBoxListener", cba );

	this->add( m_objectsList, ALIGN_TOP );
	this->add( m_tabs, ALIGN_CLIENT );	
	
	TabPage* propPage = m_tabs->addNewPage( "Properties" );
	m_propPanel = (Panel*)propPage->getPageComponent();

	m_propertyTable = new TableControl( new PropertyTableModel() );	
	TableModel* model = m_propertyTable->getTableModel();

	MouseHandler* mh = new MouseHandler( this );
	this->addEventHandler( "PropertyTableMouseHandler", mh );
	mh->m_mouseDoubleClicked = (OnMouseEvent)onPropertyItemDblClick;
	m_propertyTable->addMouseListener( mh );

	m_propPanel->add( m_propertyTable, ALIGN_CLIENT );
	TabPage* eventPage = m_tabs->addNewPage( "Events" );	
	m_eventPanel = (Panel*)eventPage->getPageComponent();	
	this->setBounds( &Rect(20, 100, 150, 600) );

	m_eventTable = new TableControl( new EventTableModel() );
	
	mh = new MouseHandler( this );
	this->addEventHandler( "EventTableMouseHandler", mh );
	mh->m_mouseDoubleClicked = (OnMouseEvent)onEventItemDblClick;
	m_eventTable->addMouseListener( mh );

	m_eventPanel->add( m_eventTable, ALIGN_CLIENT );

	m_compListContainer.initContainer( m_componentList );

	m_itemEventAdapter = new ItemEditorEventAdapter( this );
	m_itemEventAdapter->m_change = (OnItemEditorEvent)onItemChanged;
	this->addEventHandler( "ItemEventHandler", m_itemEventAdapter );

	m_prevItemEditor = NULL;
}

ObjectInspector::~ObjectInspector()
{

}


void ObjectInspector::selectComponent( Component* c )
{
	m_currentComponent = c;
	Frame* frame = dynamic_cast<Frame*>( c );
	if ( NULL != frame ){
		this->setCurrentFrame( frame );
	}
	m_propertyTable->setSelectedItem( NULL );
	m_eventMethodMap.clear();
	TableModel* tm = m_propertyTable->getTableModel();
	if ( NULL != tm ){
		tm->empty();
		if ( NULL != c ) {
			tm->addColumn();
			tm->addColumn();			
			
			TableCellItem* item = tm->getItem( 0, 0 );
			item->setCaption( "Name" );
			
			item = tm->getItem( 0, 1 );
			item->setCaption( "Value" );

			Class* clazz = c->getClass();
			if ( NULL != clazz ){
				Enumerator<Property*>* props = clazz->getProperties();
				if ( NULL != props ){
					int row = 1;
					while ( props->hasMoreElements() ){
						Property* p = props->nextElement();
						tm->addRow();
						item = tm->getItem( row, 0 );
						item->setCaption( p->getName() );

						item = tm->getItem( row, 1 );						
						PropertyCellItem* pci = (PropertyCellItem*)item;
						pci->setProperty( p );
						row++;
					}
				}
			}
			m_propertyTable->setFixedColumn( true, 0 );
			m_propertyTable->setFixedRow( true, 0 );
			m_propertyTable->setColumnWidth( 1, 200 );
		}
	}
	m_propertyTable->repaint();
	
	m_eventTable->setSelectedItem( NULL );
	tm = m_eventTable->getTableModel();
	if ( NULL != tm ){
		tm->empty();
		if ( NULL != c ) {
			tm->addColumn();
			tm->addColumn();			
			
			TableCellItem* item = tm->getItem( 0, 0 );
			item->setCaption( "Event" );
			
			item = tm->getItem( 0, 1 );
			item->setCaption( "Handler" );

			this->readVCFInitEventEntries();

			Class* clazz = c->getClass();
			if ( NULL != clazz ){
				Enumerator<EventPropertySet*>* eventSets = clazz->getEventSets();
				if ( NULL != eventSets ){					
					int row = 1;
					while ( true == eventSets->hasMoreElements() ){
						EventPropertySet* eps = eventSets->nextElement();
						if ( NULL != eps ){
							Enumerator<EventProperty*>* events = eps->getEvents();
							if ( NULL != events ){
								while ( true == events->hasMoreElements() ){
									EventProperty* event = events->nextElement();
									if ( NULL != event ){
										tm->addRow();
										
										item = tm->getItem( row, 0 );
										EventCellItem* eci = dynamic_cast<EventCellItem*>(item);
										if ( NULL != eci ){
											eci->setEvent( event );
										}
										item->setCaption( eps->getListenerClassName() + "::" + event->getEventMethod() );
										
										item = tm->getItem( row, 1 );
										eci = dynamic_cast<EventCellItem*>(item);
										/*
										EventKey evKey;
										evKey.ComponentName = m_currentComponent->getName();
										evKey.EventMethod = event->getEventMethod();
										evKey.ListenerClassName = eps->getListenerClassName();
										*/
										String evKey = m_currentComponent->getName();
										evKey += event->getEventMethod();
										evKey += eps->getListenerClassName();
										StringUtils::trace( evKey + "\n" );
										//std::map<EventKey*,String,EventKeyLess>::iterator found = 
										std::map<String,String>::iterator found = 
											this->m_currentEventEntries.find( evKey );
										if ( found != m_currentEventEntries.end() ){
											//EventKey* first = found->first;
											StringUtils::trace( found->first + "\n" );
											item->setCaption( found->second );
										}
										if ( NULL != eci ){
											eci->setEvent( event );
										}
										row++;
									}
								}
							}
						}
					}
				}
			}
			m_eventTable->setFixedColumn( true, 0 );
			m_eventTable->setFixedRow( true, 0 );
			m_eventTable->setColumnWidth( 0, 300 );
			m_eventTable->setColumnWidth( 1, 250 );
		}
	}
	m_eventTable->repaint();

	updateObjectList( c );
}

void ObjectInspector::updateProperties()
{
	m_propertyTable->repaint();
	m_eventTable->repaint();
}

void ObjectInspector::onPropertyItemDblClick( MouseEvent* e )
{
	TableCellItem* selectedItem = this->m_propertyTable->getSelectedItem();
	if ( NULL != selectedItem ){
		StringUtils::trace( selectedItem->getCaption() + "\n" );
		PropertyCellItem* pci = dynamic_cast<PropertyCellItem*>(selectedItem);
		if ( NULL != pci ){
			
		}
	}
}

void ObjectInspector::addComponent( Frame* currentFrame, Component* component )
{
	m_componentList.push_back( component );
	ListModel* lm = m_objectsList->getListModel();
	String componentName = getUniqueName( component );
	component->setName( componentName );
	ListItem* item = new DefaultListItem( lm, componentName );
	item->setData( (void*)component );
	lm->addItem( item );
	this->setCurrentFrame( currentFrame );
	updateComponentEntries( component );
}

String ObjectInspector::getUniqueName( Component* component )
{
	String result = "";	
	if ( NULL != component ){
		String name;
		name = component->getClassName();
		String tmp = name;
		std::string::size_type idx = tmp.find( "::" );
		if ( idx == tmp.npos ) {
			name = tmp;  // not part of a namespace
		} else {
			name = tmp.substr( idx + 2 );  // strip namespace off from string
		}
		
		std::vector<Component*>::iterator it = m_componentList.begin();
		int nameCount = 1;
		while ( it != m_componentList.end() ){
			Component* c = *it;
			tmp = c->getName();
			idx = tmp.find( name );
			if ( idx != tmp.npos ){
				nameCount ++;
			}			
			it++;
		}
		
		name += StringUtils::toString( nameCount );
		result = name;
	}
	else {

	}
	return result;
}

void ObjectInspector::updateObjectList( Component* component )
{
	String name = component->getName();
	ListModel* lm = m_objectsList->getListModel();
	Enumerator<ListItem*>* items = lm->getItems();
	if ( NULL != items ){
		while ( true == items->hasMoreElements() ){
			ListItem* item = items->nextElement();			
			if ( NULL != item ){
				Component* itemComp = (Component*)item->getData();
				if ( itemComp == component ){
					item->setCaption( name );
					m_objectsList->setSelectedItemIndex( item->getIndex() );
					break;
				}
			}
		}
	}
}

Enumerator<Component*>* ObjectInspector::getComponentList( Frame* frame )
{
	return m_compListContainer.getEnumerator();
}

void ObjectInspector::onObjectListSelectionChanged( ItemEvent* event )
{
	ListItem* item = (ListItem*)event->getSource();
	Component* c = (Component*)item->getData();
	if ( c != m_currentComponent ){
		this->selectComponent( c );
	}
}

void ObjectInspector::onEventItemDblClick( MouseEvent* e )
{
	TableCellItem* selectedItem = this->m_eventTable->getSelectedItem();
	if ( NULL != m_prevItemEditor ){
		//m_prevItemEditor->removeItemEditorListener( m_itemEventAdapter );
	}
	if ( NULL != selectedItem ){
		EventCellItem* eci = dynamic_cast<EventCellItem*>(selectedItem);
		TableItemEditor* tie = selectedItem->getItemEditor();		
		tie->addItemEditorListener( m_itemEventAdapter );
		m_prevItemEditor = tie;
		if ( NULL != eci ){
			String itemCaption = selectedItem->getCaption();
			if ( itemCaption == "" ){
				EventProperty* event = eci->getEvent();
				String methName = m_currentComponent->getName() + "_" + event->getEventMethod();
				selectedItem->setCaption( methName );
				m_eventMethodMap[event] = methName;				
				//tie->setItemToEdit( selectedItem );
				updateEventCode( event );
			}			
		}
	}
	else {
		m_prevItemEditor = NULL;
	}
}

void ObjectInspector::onItemChanged( ItemEditorEvent* e )
{
	StringUtils::trace( "ObjectInspector::onItemChanged()\n" );
	StringUtils::trace( "caption: " + e->getItemBeingEdited()->getCaption() + "\n" );
	
}

void ObjectInspector::readVCFInitEventEntries()
{
	clearEventEntries();
	Application* app = Application::getRunningInstance();		
	String appFileName = app->getFileName();
	VCFBuilderMainWin* mainWin = (VCFBuilderMainWin*)app->getMainWindow();
	
	String startDir = mainWin->getCurrentProject()->getProjectDir(); 
	String cppFileName = startDir + "\\" + m_currentFrame->getName() + ".cpp";

	if ( true == FileUtils::fileExists( cppFileName ) ){
		String classImpl;
		FileStream readFs( cppFileName );
		readFs >> classImpl;

		String className = m_currentFrame->getName();
		String initMethod = "void " + className + "::vcfInit()";
		int pos = classImpl.find( initMethod );
		if ( pos != String::npos ){
			/**start searching for the following pattern:
			*	find the window class name + "::"
			*	what follows should be a valid member function
			*so now we have the member function, what we also 
			*need is the Listener class, the listener method that is being
			*linked to by the handler.
			*The form of Listener method vs Handler method pointer variable
			*is the following:
			*	
			*/
			String vcfInitEndMethod = "}//vcfInit finished";
			int endPos = classImpl.find( vcfInitEndMethod );
			int startPos = pos + initMethod.size();
			String searchStr = "//";

			if ( endPos != String::npos ){
				int pos1 = classImpl.find( searchStr, startPos );
				int nextEventMethodPos = pos1;
				while ( (nextEventMethodPos < endPos) && (nextEventMethodPos != String::npos) ){					
					pos1 += 2;
					int pos2 = classImpl.find( " ", pos1 );
					String listenerClass = classImpl.substr( pos1, pos2 - pos1 );
					pos2 = pos2 + listenerClass.size();
					int pos3 = classImpl.find( "for ", pos1 );
					pos3 += 4;
					int pos4 = classImpl.find( ',', pos3 );
					String componentName = classImpl.substr( pos3, pos4 - pos3 );
					int pos5 = classImpl.find( "event: ", pos1 );
					pos5 +=7;
					
					int pos6 = classImpl.find( ',', pos5 );
					String eventMethod = classImpl.substr( pos5,  pos6 - pos5 );

					int pos7 = classImpl.find( "handler: ", pos1 );
					pos7 += 9;
					int pos8 = classImpl.find( '\n', pos1 );
					
					String handlerMethod = classImpl.substr( pos7,  pos8 - pos7 );

					pos1 = classImpl.find( searchStr, pos7 );
					if ( pos1 != String::npos ){
						nextEventMethodPos = pos1;
					}
					else {
						nextEventMethodPos = String::npos;
					}
					
					/*
					EventKey* evKey = new EventKey;
					evKey->ComponentName = this->m_currentComponent->getName();
					evKey->EventMethod = eventMethod;
					evKey->ListenerClassName = listenerClass;
					*/
					String evKey = componentName;
					evKey += eventMethod;
					evKey += listenerClass;
					m_currentEventEntries[evKey] = handlerMethod;
				}
			}
		}
	}
}

/**
*rules for adding the events
*1.	look for the existing .cpp file where the name is the 
*	name of the current window frame
*2.	the class name is the name of the current frame
*3.	the init method is in the form "void classname::vcfInit()"
*4. once this located, store the posiiton - this is the insert position
*	of all the listener code.
*5.	the formula for creating the listener code is as follows:
		comment:
			//"Listener class name" for current component name
		creation of Listener class on the heap
			"Listener Handler class name" * "handler variable" = new "Listener Handler class name" ( this );

*/
void ObjectInspector::updateEventCode( EventProperty* ep )
{
	if ( NULL != ep ){
		String methName = m_currentComponent->getName() + ep->getEventMethod();
		Application* app = Application::getRunningInstance();		
		VCFBuilderMainWin* mainWin = (VCFBuilderMainWin*)app->getMainWindow();
		String appFileName = app->getFileName();
		String startDir = mainWin->getCurrentProject()->getProjectDir(); //StringUtils::getFileDir( appFileName );
		String cppFileName = startDir + "\\" + m_currentFrame->getName() + ".cpp";
		String hFileName = startDir + "\\" + m_currentFrame->getName() + ".h";
		String className = m_currentFrame->getName();
		String classDecl;
		String classImpl;
		String listenerImpl;
		if ( true == FileUtils::fileExists( cppFileName ) ){
			//look for the init method
			FileStream readFs( cppFileName );

			readFs >> classImpl;
			String initMethod = "void " + className + "::vcfInit()";
			int pos = classImpl.find( initMethod );
			if ( pos != String::npos ){
				int insertVCFInitCode = classImpl.find_first_of( '{', pos + initMethod.size() );
				insertVCFInitCode +=2;
				std::map<EventProperty*,String> eventsToImpl;
				std::map<EventProperty*,String>::iterator it = m_eventMethodMap.begin();
				while ( it != m_eventMethodMap.end() ){
					int methodPos = classImpl.find( it->second );
					if ( methodPos == String::npos ){
						eventsToImpl[it->first] = it->second;
					}
					it++;
				}

				it = eventsToImpl.begin();
				String eventMethodsImpl;
				while ( it != eventsToImpl.end() ){						
					EventProperty* event = it->first;
					String listenerClassName = event->getListenerClassName();
					String adapterClassName = event->getEventSet()->getAdapterClassName();
					String adapterVarName = adapterClassName;
					
					String eventMethodName = event->getEventMethod();
					/**
					*method names are ALWAYS of the form
					*	"on" Event name
					*and the Handler class's method pointer
					*member variable is ALWAYS of the form
					*	"m_" event, with the first leter of the event
					*	being lowercase
					*failure to observe this will FUCK UP the 
					*Builder, as there is no other way to do
					*this just yet
					*/
					VCFChar firstChar = '\0';
					int onPos = eventMethodName.find( "on" );
					if ( onPos != String::npos ){
						firstChar = eventMethodName[onPos + 2];
						eventMethodName[onPos + 2] = tolower( firstChar );
						eventMethodName.erase( onPos, onPos + 2 );
					}					
					
					eventMethodName = "m_" + eventMethodName;
					/**
					*this ius to get rid of namespaces - this 
					*only gets rid of single namespaces - not nested - needs to be improved
					*/

					int nmspPos = adapterVarName.find( "::" );
					if ( nmspPos != String::npos ){
						adapterVarName.erase( 0, nmspPos + 2 );
					}
					adapterVarName = this->m_currentComponent->getName() + adapterVarName;
					
					nmspPos = listenerClassName.find( "::" );
					if ( nmspPos != String::npos ){
						listenerClassName.erase( 0, nmspPos + 2 );
					}


					firstChar = adapterVarName[0];
					adapterVarName[0] = tolower( firstChar );

					eventMethodsImpl += "\n\nvoid " + className + "::" + 
							it->second + "( " + event->getEventClassName() + "* event )\n{\n\n}\n";
					listenerImpl += "\t//" + event->getListenerClassName() + " for " + m_currentComponent->getName() + 
						            ", event: " + event->getEventMethod() + ", handler: " + it->second + "\n" ;
					listenerImpl += "\t" + adapterClassName + "* " + adapterVarName + " = new " + 
						 adapterClassName + "( this );\n";

					listenerImpl += "\t" + adapterVarName + "->" + eventMethodName + " = " +
						className + "::" + it->second + ";\n";

					listenerImpl += "\t" + m_currentComponent->getName() + "->add" + listenerClassName + "( " +
						adapterVarName + " );\n";
					listenerImpl += "\tthis->addEventHandler( \"" + adapterVarName + "\", " + adapterVarName + " );\n\n";
					
					it++;
				}

				classImpl.insert( insertVCFInitCode, listenerImpl );

				StringUtils::trace( classImpl );
				String vcfInitEndMethod = "}//vcfInit finished";
				int insertpos = classImpl.find( vcfInitEndMethod );
				insertpos += vcfInitEndMethod.size() + 1;

				classImpl.insert( insertpos, eventMethodsImpl );
				StringUtils::trace( classImpl );


			}
			else {

			}
			readFs.close();
			readFs.open( cppFileName, FS_WRITE );
			StringUtils::trace( "File \"" + cppFileName + "\" is " + 
				StringUtils::toString(readFs.getSize()) + " bytes\n" );
			readFs << classImpl;
		}
		else {

		}

		if ( true == FileUtils::fileExists( hFileName ) ){
			FileStream readFs( hFileName );
			readFs >> classDecl;			
			
			String vcfInitDecl = "void vcfInit();";
			String firstProtectedDecl = "protected:";

			int vcfInitPos = classDecl.find( vcfInitDecl );

			int firstProtectedPos = classDecl.find( firstProtectedDecl );
			
			if ( vcfInitPos != String::npos  ){
				vcfInitPos += vcfInitDecl.size();
				std::map<EventProperty*,String> eventsToImpl;
				std::map<EventProperty*,String>::iterator it = m_eventMethodMap.begin();
				while ( it != m_eventMethodMap.end() ){
					int methodPos = classDecl.find( it->second, vcfInitPos );
					if ( (methodPos == String::npos) && (methodPos < firstProtectedPos) ){
						eventsToImpl[it->first] = it->second;
					}
					it++;
				}
				
				it = eventsToImpl.begin();
				String eventMethodDecl = "";
				while ( it != eventsToImpl.end() ){
					EventProperty* event = it->first;
					eventMethodDecl  += "\n\tvoid " + it->second + "( " + event->getEventClassName() + "* event );\n";					
					it++;
				}
				classDecl.insert( vcfInitPos + 1, eventMethodDecl );
				readFs.close();
				readFs.open( hFileName, FS_WRITE );				
				readFs << classDecl;
			}			
		}
		else{
			
		}
	}
}

void ObjectInspector::clearEventEntries()
{
	m_currentEventEntries.clear();
}

bool ObjectInspector::hasControl( const String& name ){
	bool result = false;
	std::vector<Component*>::iterator it = m_componentList.begin();
	while ( it != m_componentList.end() ){
		Component* comp = *it;
		if ( name == comp->getName() ){
			result = true;
			break;
		}
		it++;
	}
	
	return result;
}

void ObjectInspector::updateComponentEntries( Component* component )
{
	Application* app = Application::getRunningInstance();		
	VCFBuilderMainWin* mainWin = (VCFBuilderMainWin*)app->getMainWindow();
	String appFileName = app->getFileName();
	String startDir = mainWin->getCurrentProject()->getProjectDir(); //StringUtils::getFileDir( appFileName );
	String cppFileName = startDir + "\\" + m_currentFrame->getName() + ".cpp";
	String hFileName = startDir + "\\" + m_currentFrame->getName() + ".h";
	String classImpl;
	String className = m_currentFrame->getName();
	if ( true == FileUtils::fileExists( cppFileName ) ){
		FileStream readFs( cppFileName );
		
		readFs >> classImpl;
		readFs.close();

		String initMethod = "void " + className + "::vcfInit()";
		int pos = classImpl.find( initMethod );
		if ( pos != String::npos ){
			int insertVCFInitCode = classImpl.find_first_of( '{', pos + initMethod.size() );
			insertVCFInitCode +=2;
			String vcfInitEndMethod = "}//vcfInit finished";
			int endPos = classImpl.find( vcfInitEndMethod );

			std::vector<String> bindList;
			std::vector<Component*>::iterator it = this->m_componentList.begin();
			while ( it != m_componentList.end() ){
				if ( (*it)->getName() != className ) {
					bindList.push_back( (*it)->getName() );
				}
				it++;
			}

			String findBind = "bindVariable(";
			int bindPos = classImpl.find( findBind, insertVCFInitCode );
			while ( (bindPos != String::npos) && (bindPos < endPos) ){
				int methEnd = classImpl.find( ");", insertVCFInitCode );
				if ( methEnd != String::npos ){
					String convert = "(VCF::Component**)&";
					int varStart = classImpl.find( convert, insertVCFInitCode );
					varStart += convert.size();
					int varEnd = classImpl.find( ',', insertVCFInitCode );
					String varName = classImpl.substr( varStart, varEnd - varStart );
					std::vector<String>::iterator found = std::find( bindList.begin(), bindList.end(), varName );
					if ( found != bindList.end() ){
						bindList.erase( found );
					}
					bindPos = classImpl.find( findBind, insertVCFInitCode );
					insertVCFInitCode = methEnd + 3;
				}
				else {
					bindPos = endPos;
				}
				
			}

			std::vector<String>::iterator bindIt = bindList.begin();
			String tmpImpl;	
			while ( bindIt != bindList.end() ){				
				tmpImpl += "\tthis->bindVariable( (VCF::Component**)&" + *bindIt + ", \"" + *bindIt + "\" );\n";				
				bindIt++;
			}
			if ( false == tmpImpl.empty() ){
				classImpl.insert( insertVCFInitCode, tmpImpl );
				FileStream saveFs( cppFileName );
				saveFs << classImpl;
			}

		}
	}
}