/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "PropertyCellItemEditor.h"
#include "PropertyCellItem.h"
#include "PropertyEditorManager.h"
#include "PropertyEditor.h"

using namespace VCF;

PropertyCellItemEditor::PropertyCellItemEditor( TableCellItem* itemToEdit ):
	BasicTableItemEditor( itemToEdit )
{
	m_currentPropertyCellItem = NULL;
	m_currentPropertyEditor = NULL;
}

PropertyCellItemEditor::~PropertyCellItemEditor()
{

}

void PropertyCellItemEditor::setItemToEdit( TableCellItem* itemToEdit )
{
	BasicTableItemEditor::setItemToEdit( itemToEdit );
	m_currentPropertyCellItem = dynamic_cast<PropertyCellItem*>( itemToEdit );
	m_currentPropertyEditor = NULL;
}

void PropertyCellItemEditor::updateItem()
{
	if ( (NULL != m_currentPropertyEditor) && (NULL != m_currentPropertyCellItem) ){
		Property* property = m_currentPropertyCellItem->getProperty();
		if ( NULL != property ){
			if ( m_currentPropertyEditor->hasCustomEditor() ){
				//assume the value is already in the PropertyEditor
			}
			else {
				BasicTableItemEditor::updateItem();
				m_currentPropertyEditor->setValueAsText( m_currentPropertyCellItem->getCaption() );
			}

			property->set( m_currentPropertyEditor->getValue() );

		}
	}
}

Control* PropertyCellItemEditor::getEditingControl()
{
	Control* result = NULL;
	if ( NULL != m_currentPropertyCellItem ){
		Property* property = m_currentPropertyCellItem->getProperty();
		if ( NULL != property ){
			String propertyClassName = Class::getClassNameForProperty( property );
			PropertyEditor* pe = PropertyEditorManager::findEditor( propertyClassName );	
			m_currentPropertyEditor = pe;
			if ( NULL != pe ){
				pe->setValue( property->get() );
				if ( true == pe->hasCustomEditor() ){
					result = pe->getCustomEditor();
				}
				else {
					result = BasicTableItemEditor::getEditingControl();
				}
			}
			else if ( PROP_OBJECT == property->getType() ){ 
				/**
				*if we don't have a property editor, and the property is a
				*component then create a combobox that lists all the known 
				*components of correct class name
				*/
								
				//Object* o = property
			}
		}
	}
	return result;
}