/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "VCFBuilderApp.h"
#include "Registry.h"
#include "Dialog.h"
#include "ClassInfo.h"
#include "NewProjectDlg.h"
#include "ProjectContents.h"
#include "Rect.h"
#include "RenderPaths.h"
#include "GraphicsContext.h"
#include "MicroTiles.h"

using namespace VCF;

class GTest : Window {
public:
	GTest(){
		this->setCaption( "Graphics Test" );
	};

	virtual ~GTest(){};

	virtual void paint( GraphicsContext* ctx ){
		Window::paint( ctx );
		Color* c = ctx->getColor();
		VectorPath vp( 200.0, 300.5, 200.3 );

		
		VectorPath vp2( 250.0, 350.5, 200.3 );
		VectorPathDash vpd( &vp2 );
		vpd.addDash( 5.0 );
		vpd.addDash( 15.0 );

		int32 i = vp.getPointCount();
		ctx->setColor( Color::getColor("black" ) );
		for (int32 j=0;j<i;j++){
			VectorPathPoint* pt = vp[j];
			switch ( pt->m_code ) {
				case PCT_MOVETO :{
					ctx->moveTo( pt->m_x, pt->m_y );
				}
				break;

				case PCT_LINETO :{
					ctx->lineTo( pt->m_x, pt->m_y );
				}
				break;

				case PCT_MOVETO_OPEN :{
					ctx->strokePath();
					ctx->moveTo( pt->m_x, pt->m_y );
				}
				break;
			}
		}

		ctx->strokePath();
		//vpd.setOffset( 10 );
		VectorPath* vp3 = vpd.doDash();

		i = vp3->getPointCount();
		
		ctx->setColor( Color::getColor("yellow" ) );

		for (int32 k=0;k<i;k++){
			VectorPathPoint* pt = (*vp3)[k];
			switch ( pt->m_code ) {
				case PCT_MOVETO :{
					ctx->moveTo( pt->m_x, pt->m_y );
				}
				break;

				case PCT_LINETO :{
					ctx->lineTo( pt->m_x, pt->m_y );
				}
				break;

				case PCT_MOVETO_OPEN :{
					ctx->strokePath();
					ctx->moveTo( pt->m_x, pt->m_y );
				}
				break;
				
			}
		}
		
		ctx->strokePath();

		

		Rect r(0, 0, 250, 250 );
		MicroTileArray u(vp3);
		
		uint32 tc = u.getTileCount();
		for ( int l=0;l<tc;l++){
			MicroTileBBox* bbx = u[l];
			if ( NULL != bbx ){
				ctx->rectangle( &(Rect)*bbx );
				ctx->strokePath();
			}
		}

		ctx->setColor( c );

		delete vp3;
		vp3 = NULL;
	}
};


VCFBuilderApp::VCFBuilderApp()
{

}

VCFBuilderApp::~VCFBuilderApp()
{

}

bool VCFBuilderApp::initRunningApplication()
{
	
	REGISTER_CLASSINFO(NewProjectDlg)
	REGISTER_CLASSINFO(ProjectContents)

	bool result = Application::initRunningApplication();
	

	Registry reg;
	reg.setRoot( RKT_CURRENT_USER );
	if ( true == reg.openKey( "Software\\VCF\\Builder\\VPLS", true ) ){		
		
		Enumerator<RegistryValueInfo*>* regVals = reg.getValues();
		while ( regVals->hasMoreElements() ){
			RegistryValueInfo* valInfo = regVals->nextElement();
			String vplName = valInfo->m_data;
			try {
				this->loadVPL( vplName );
				m_vplList.push_back( vplName );
			}
			catch ( BasicException &e ){
				Dialog::showMessage( e.getMessage() );
			}
		}	
		
	}
	
	VCFBuilderMainWin* mainWnd = new VCFBuilderMainWin();
	this->setMainWindow( mainWnd );
	mainWnd->initObjectInspector();
	mainWnd->initProjectContents();
	this->getMainWindow()->setCaption( "VCF Builder - [No Form]" );

	//GTest* g = new GTest();

	return result;
}

void VCFBuilderApp::terminateRunningApplication()
{
	

	Registry reg;
	reg.setRoot( RKT_CURRENT_USER );

	if ( true == reg.openKey( "Software\\VCF\\Builder\\VPLS", true ) ){
		std::vector<String>::iterator it = m_vplList.begin();
		int i = 1;
		while ( it != m_vplList.end() ){
			String name = *it;
			reg.setValue( name, "VPL" + StringUtils::toString(i) );
			it++;
			i++;
		}
	}
	else {
		Dialog::showMessage( "Error trying to save VCFBuilder information in Registry." );
	}
	
	Application::terminateRunningApplication();
}

void VCFBuilderApp::addVPLToList( const String& vplName  )
{
	m_vplList.push_back( vplName );
}