/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
//VCFBuilderMainWin.cpp

#include "VCF.h"
#include "VCFBuilderMainWin.h"
#include "DefaultMenuItem.h"
#include "CommonFileOpen.h"
#include "CommonFileSave.h"
#include "NewFormDlg.h"
#include "Application.h"
#include "DropTarget.h"
#include "DropEvent.h"

#include "ProjectContents.h"

#include "Panel.h"
#include "Label.h"
#include "ObjectInspector.h"
#include "GraphicsContext.h"
#include "Rect.h"
#include "Basic3DBorder.h"
#include "CommandButton.h"
#include "FileStream.h"
#include "ComponentOutputStream.h"
#include "ComponentInputStream.h"
#include "FileUtils.h"
#include "Enumerator.h"

#include "Library.h"
#include "BasicException.h"
#include "ResourceBundle.h"
#include "PopupMenu.h"
#include "File.h"
#include "VCFBuilderApp.h"

#include "NewProjectDlg.h"
#include "VCFProject.h"
#include "VCFAbout.h"



using namespace VCF;



class TmpPanel : public Panel{
public:
	TmpPanel() : Panel(){
		Label* l = new Label();
		l->setBounds( &Rect(12,12,100,35) );
		l->setCaption("Foo Label");
		add( l );
	}

	virtual ~TmpPanel(){};

	virtual void mouseDown( MouseEvent* e ){
		Panel::mouseDown( e );
		if ( e->hasLeftButton() ){
			m_tDragOffset = *e->getPoint();
			this->keepMouseEvents();
		}
		else if ( e->hasRightButton() ){
			StringUtils::trace( "TmpPanel::mouseDown() \n\te->hasRightButton()\n" );
			GraphicsContext* ctx = this->getContext();
			ctx->setColor( &Color(0.0f,0.0f,0.0f) );
			
			oldPt = *e->getPoint();
			ctx->moveTo( oldPt.m_x, oldPt.m_y );
		}
	}

	virtual void mouseMove( MouseEvent* e ){
		Panel::mouseMove(e);
		if ( e->hasLeftButton() ){
			this->setLeft(getLeft() + e->getPoint()->m_x - m_tDragOffset.m_x);
			this->setTop(getTop() + e->getPoint()->m_y - m_tDragOffset.m_y);			
		}
		else if ( e->hasRightButton() ){
			StringUtils::trace( "TmpPanel::mouseMove() \n\te->hasRightButton()\n" );
			GraphicsContext* ctx = this->getContext();
			ctx->moveTo( oldPt.m_x, oldPt.m_y );
			ctx->lineTo( e->getPoint()->m_x, e->getPoint()->m_y );
			ctx->strokePath();
			oldPt = *e->getPoint();
		}
	}

	virtual void mouseUp( MouseEvent* e ){
		Panel::mouseUp(e);
		GraphicsContext* ctx = this->getContext();
		ctx->strokePath();
		this->releaseMouseEvents();
		
	}


	virtual void paint( GraphicsContext* g ){
		Panel::paint( g );
	}

	Point m_tDragOffset;
	Point oldPt;

};

class TmpLabel : public Label{
public:
	TmpLabel() : Label(){
		this->setCaption( "Foo Fighters" );
	}

	virtual ~TmpLabel(){};

	virtual void mouseDown( MouseEvent* e ){
		Label::mouseDown( e );
		m_tDragOffset = *e->getPoint();

		this->keepMouseEvents();
	}

	virtual void mouseMove( MouseEvent* e ){
		Label::mouseMove(e);
		if ( e->hasLeftButton() ){
			Rect* oldBounds = this->getBounds();
			this->setLeft(getLeft() + e->getPoint()->m_x - m_tDragOffset.m_x);
			this->setTop(getTop() + e->getPoint()->m_y - m_tDragOffset.m_y);
			this->getParent()->repaint( oldBounds );
			this->repaint();
		}

		
	}

	virtual void mouseUp( MouseEvent* e ){
		Label::mouseUp(e);

		this->releaseMouseEvents();
	}

	virtual void paint( GraphicsContext* g ){
		Label::paint( g );
		g->circle( 100, 100, 50 );
		g->setColor( this->getColor() );
		g->fillPath();
	}

	Point m_tDragOffset;
};


DropTarget* dt;


VCFBuilderMainWin::VCFBuilderMainWin()
{
	m_selectedComponent = NULL;
	m_currentWin = NULL;

	m_currentProject = NULL;

	this->setBounds( &Rect(0,0,1024,50 ) );

	this->setMenuBar( new MenuBar() );
	
	MenuBar* mb = this->getMenuBar();

	MenuItem* item = mb->getRootMenuItem();
	this->m_file = new DefaultMenuItem( "&File", NULL, mb ); 	
	
	m_fileNewProject = new DefaultMenuItem( "New &Project", m_file, mb ); 
	m_fileOpenProject = new DefaultMenuItem( "Open Project", m_file, mb ); 
	m_fileSaveProject = new DefaultMenuItem( "Save Project", m_file, mb ); 
	
	this->m_fileNewForm = new DefaultMenuItem( "New &Form", m_file, mb ); 
	this->m_fileNewDialog = new DefaultMenuItem( "New &Dialog", m_file, mb ); 	


	this->m_fileOpenForm = new DefaultMenuItem( "&Open Form...", m_file, mb ); 
	this->m_fileSaveForm = new DefaultMenuItem( "&Save Form...", m_file, mb ); 
	this->m_fileCloseForm = new DefaultMenuItem( "&Close", m_file, mb ); 
	this->m_filePrintForm = new DefaultMenuItem( "&Print...", m_file, mb ); 
	this->m_fileQuit = new DefaultMenuItem( "&Quit", m_file, mb ); 

	MenuItemHandler* menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileNewFormClick );
	m_fileNewForm->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileNewForm", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileNewDialogClick );
	m_fileNewDialog->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileNewDialog", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileNewProjectClick );
	m_fileNewProject->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileNewProject", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileOpenProjectClick );
	m_fileOpenProject->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileOpenProject", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileSaveProjectClick );
	m_fileSaveProject->addMenuItemListener( menuHandler );	
	
	this->addEventHandler( "FileSaveProject", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileQuitClick );
	m_fileQuit->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileQuit", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileOpenFormClick );
	m_fileOpenForm->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileOpenForm", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileSaveFormClick );
	m_fileSaveForm->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileSaveForm", menuHandler );

	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)fileCloseFormClick );
	m_fileCloseForm->addMenuItemListener( menuHandler );
	this->addEventHandler( "FileCloseForm", menuHandler );

	this->m_edit = new DefaultMenuItem( "&Edit", NULL, mb ); 

	this->m_editCut = new DefaultMenuItem( "C&ut", m_edit, mb ); 
	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)editCutClick );
	m_editCut->addMenuItemListener( menuHandler );
	this->addEventHandler( "EditCutClick", menuHandler );

	this->m_editCopy = new DefaultMenuItem( "&Copy", m_edit, mb ); 
	this->m_editPaste = new DefaultMenuItem( "&Paste", m_edit, mb ); 
	this->m_editSelectAll = new DefaultMenuItem( "&Select All", m_edit, mb ); 
	this->m_editDeselectAll = new DefaultMenuItem( "Deselect &All", m_edit, mb ); 
	this->m_editDelete = new DefaultMenuItem( "&Delete", m_edit, mb ); 

	this->m_view = new DefaultMenuItem( "&View", NULL, mb ); 
	this->m_viewObjectInspector = new DefaultMenuItem( "Object Inspector", m_view, mb ); 
	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)viewObjectInspectorClick );
	m_viewObjectInspector->addMenuItemListener( menuHandler );
	this->addEventHandler( "ViewObjectInspectorClick", menuHandler );


	this->m_viewProjectWindow = new DefaultMenuItem( "&Project Window", m_view, mb ); 
	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)viewProjectContentsClick );
	m_viewProjectWindow->addMenuItemListener( menuHandler );
	this->addEventHandler( "ViewProjectWindowClick", menuHandler );


	m_components = new DefaultMenuItem( "&Components",NULL, mb );
	
	m_addVPL = new DefaultMenuItem( "&Add Visual Package Library", m_components, mb );
	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)addVPL );
	m_addVPL->addMenuItemListener( menuHandler );
	this->addEventHandler( "ComponentsAddVPL", menuHandler ); 

	this->m_help = new DefaultMenuItem( "&Help", NULL, mb ); 
	this->m_helpAbout = new DefaultMenuItem( "&About", m_help, mb ); 
	menuHandler = new MenuItemHandler( this, (OnMenuItemEvent)helpAboutClick );
	m_helpAbout->addMenuItemListener( menuHandler );
	this->addEventHandler( "HelpAboutClick", menuHandler );
	

	m_insertComponent = new DefaultMenuItem( "&Insert Component", NULL, mb ); 
	
	PopupMenu* popup = new PopupMenu( this );
	
	DefaultMenuItem* root = new DefaultMenuItem( "Foo1", NULL, popup );

	root->addChild( new DefaultMenuItem( "Foo1a", NULL, popup ) );
	root->addChild( new DefaultMenuItem( "Foo1b", NULL, popup ) );
	MenuItem* sep = new DefaultMenuItem();
	sep->setSeparator( true );
	root->addChild( sep );
	root->addChild( new DefaultMenuItem( "Foo1c", NULL, popup ) );
	
	
	root->getChildAt(1)->addMenuItemListener( (MenuItemListener*)this->getEventHandler( "EditCutClick" ) );
	popup->setRootMenuItem( root );
	
	this->setPopupMenu( popup );


	item->addChild( m_file );

	item->addChild( m_edit ); 

	item->addChild( m_view ); 

	loadComponents();	

	item->addChild( m_insertComponent ); 
	
	item->addChild( m_components );  
		
	item->addChild( m_help ); 

	dt = new DropTarget( this );
	DropTargetHandler* dtHandler = new DropTargetHandler(this);
	dtHandler->m_dropTargetEntered = (OnDragTargetEvent1)mainWinDragEntered;
	dtHandler->m_dropTargetDropped = (OnDragTargetEvent2)mainWinDropped;
	
	this->addEventHandler( "DropTargetHandler", dtHandler );
	dt->addDropTargetListener( dtHandler );		
	m_objectInspector = NULL;

	this->m_projectContents = NULL;

	
	this->setName( "MainWindow" );
}

void VCFBuilderMainWin::initObjectInspector()
{
	m_objectInspector = new ObjectInspector();	
}

void VCFBuilderMainWin::initProjectContents()
{
	m_projectContents = new ProjectContents();
}

VCFBuilderMainWin::~VCFBuilderMainWin()
{
	delete m_objectInspector;
	delete dt;
}

void VCFBuilderMainWin::fileNewFormClick( MenuItemEvent* e )
{
	Window* newForm = new Window();
	m_currentWin = newForm;
	m_currentWin->setComponentState( CS_DESIGNING );
	m_objectInspector->clearComponents();
	m_objectInspector->addComponent( m_currentWin, newForm );
	this->generateCPP( m_currentWin, true );
	this->selectComponent( newForm );

	this->m_currentProject->addToProject( m_currentWin );
}

void VCFBuilderMainWin::fileOpenFormClick( MenuItemEvent* e )
{
	CommonFileOpen open( this );
	if ( true == open.execute() ){
		String fileName = open.getFileName();
		FileStream fs( fileName );
		ComponentInputStream cis( &fs );
		Control* c = NULL;
		cis.readControl( &c );
		if ( NULL != c ){
			m_currentWin = dynamic_cast<Frame*>(c);
			if ( NULL != m_currentWin ){				
				m_currentWin->setVisible( true );
				m_currentWin->setComponentState( CS_DESIGNING );
				m_objectInspector->clearComponents();
				m_objectInspector->addComponent( m_currentWin, m_currentWin );
				this->generateCPP( m_currentWin, true );
				this->selectComponent( m_currentWin );
			}
		}
	}
}

void VCFBuilderMainWin::fileSaveFormClick( MenuItemEvent* e )
{
	CommonFileSave save( this );
	save.setFileName( m_currentWin->getName() + ".vff" );
	if ( true == save.execute() ){
		String fileName = save.getFileName();
		FileStream fs( fileName );
		ComponentOutputStream cos( &fs );
		cos.writeControl( m_currentWin );
		this->generateCPP( m_currentWin, false, StringUtils::getFileDir( fileName ) );
	}
}



void VCFBuilderMainWin::fileCloseFormClick( MenuItemEvent* e )
{
	this->m_currentWin->setDoubleBuffered( !m_currentWin->isDoubleBuffered() );
		
}

void VCFBuilderMainWin::filePrintFormClick( MenuItemEvent* e )
{

}

void VCFBuilderMainWin::fileQuitClick( MenuItemEvent* e )
{	
	this->close();
}

void VCFBuilderMainWin::editCutClick( MenuItemEvent* e )
{
	Dialog::showMessage( "You clicked VCFBuilderMainWin::editCutClick()" );
}

void VCFBuilderMainWin::editCopyClick( MenuItemEvent* e )
{
	
}

void VCFBuilderMainWin::editPasteClick( MenuItemEvent* e )
{

}

void VCFBuilderMainWin::editSelectAllClick( MenuItemEvent* e )
{

}

void VCFBuilderMainWin::editDeselectAllClick( MenuItemEvent* e )
{

}

void VCFBuilderMainWin::editDeleteClick( MenuItemEvent* e )
{

}

void VCFBuilderMainWin::paint( GraphicsContext* gc )
{
	Window::paint( gc );
	//Application::getRunningInstance()->getResourceBundle()->getResource( "IDS_FOO" );
	//Image* img = Application::getRunningInstance()->getResourceBundle()->getImage( "IDB_BITMAP1" );
	//gc->drawImage( 20, 50, img );
	//delete img;
}

void VCFBuilderMainWin::mainWinDragEntered( DragEvent* event )
{	
	StringUtils::trace( "VCFBuilderMainWin::mainWinDragEntered()\n" );
}

void VCFBuilderMainWin::mainWinDropped( DropEvent* event )
{
	ClipboardDataObject* obj = event->getDataObject();
	//if ( true == obj->isTypeSupported( STRING_DATA_TYPE ) ){
	StringUtils::trace( "ClipboardDataObject has data\n" );		
	//}
}

void VCFBuilderMainWin::selectComponent( Component* c )
{
	
	if ( NULL != m_selectedComponent ){
		Listener* l = this->getEventHandler( MAINWIN_COMPONENT_MOUSE_LISTENER );
		if ( NULL != l ){
			Control* control = dynamic_cast<Control*>(c);
			if ( NULL != control ){
				control->removeMouseListener( (MouseListener*)l );
			}
		}	
	}
	m_selectedComponent = c;
	if ( NULL != m_selectedComponent ){
		Listener* l = this->getEventHandler( MAINWIN_COMPONENT_MOUSE_LISTENER );
		MouseHandler* mh = NULL;
		if ( NULL == l ){
			mh = new MouseHandler( this );	
			mh->m_mouseDown = (OnMouseEvent)selectedCompMouseDown;
			mh->m_mouseMove = (OnMouseEvent)selectedCompMouseMove;
			mh->m_mouseUp = (OnMouseEvent)selectedCompMouseUp;		
			this->addEventHandler( MAINWIN_COMPONENT_MOUSE_LISTENER, mh );
		}
		else {
			mh = dynamic_cast<MouseHandler*>(l);
		}
		
		if ( NULL != mh ){
			Control* control = dynamic_cast<Control*>(c);
			if ( NULL != control ){
				control->addMouseListener( mh );
			}
		}		
	}
	this->m_objectInspector->selectComponent( m_selectedComponent );
}

bool dragCorner = false;

void VCFBuilderMainWin::selectedCompMouseDown( MouseEvent* e )
{
	dragCorner = false;
	Component* c = dynamic_cast<Component*>( e->getSource() );

	this->selectComponent( c );
	
	Control* control = dynamic_cast<Control*>(e->getSource());
	Frame* f = dynamic_cast<Frame*>(e->getSource());
	if ( (e->hasLeftButton()) && (NULL != control) && (NULL == f) ){
		m_tDragOffset = *e->getPoint();
		Rect* bounds= control->getBounds();
		Point pt( e->getPoint()->m_x + bounds->m_left, e->getPoint()->m_y + bounds->m_top );
		Rect r(bounds->m_right-5, bounds->m_bottom-5, bounds->m_right, bounds->m_bottom );
		dragCorner = r.containsPt( &pt );
		control->keepMouseEvents();	
		e->setConsumed( true );
	}	
}


	
void VCFBuilderMainWin::selectedCompMouseMove( MouseEvent* e )
{	
	Control* control = dynamic_cast<Control*>(e->getSource());
	Frame* f = dynamic_cast<Frame*>(e->getSource());
	if ( (e->hasLeftButton()) && (NULL != control) && (NULL == f) ){
		if ( true == dragCorner ){						
			double w = control->getLeft() + (e->getPoint()->m_x - m_tDragOffset.m_x);
			//w += control->getParent()->getLeft();			
			double h = control->getTop() + (e->getPoint()->m_y - m_tDragOffset.m_y);
			control->setWidth( w );
			control->setHeight( h );
		}
		else {
			control->setLeft(control->getLeft() + e->getPoint()->m_x - m_tDragOffset.m_x);
			control->setTop(control->getTop() + e->getPoint()->m_y - m_tDragOffset.m_y);			
		}

		e->setConsumed( true );
		
		m_objectInspector->updateProperties();
	}
}

void VCFBuilderMainWin::selectedCompMouseUp( MouseEvent* e )
{	
	Control* control = dynamic_cast<Control*>(e->getSource());
	Frame* f = dynamic_cast<Frame*>(e->getSource());
	if ( (NULL ==f) && (NULL != control) ){
		control->releaseMouseEvents();
		e->setConsumed( true );
	}
}

void VCFBuilderMainWin::insertPanelClick( MenuItemEvent* e )
{
	if ( NULL != this->m_selectedComponent ){
		Panel* newPanel = new Panel();
		newPanel->setBounds( &Rect( 20, 20, 200, 200 ) );
		Container* cont = dynamic_cast<Container*>(m_selectedComponent);
		if ( NULL != cont ){
			cont->add( newPanel );
		}
		else if ( NULL != m_currentWin ){
			m_currentWin->add( newPanel );
		}

		m_objectInspector->addComponent( m_currentWin, newPanel );

		generateCPP( m_currentWin );
		this->selectComponent( newPanel );
	}
}

void VCFBuilderMainWin::insertButtonClick( MenuItemEvent* e )
{
	if ( NULL != this->m_selectedComponent ){
		CommandButton* newButton = new CommandButton();
		newButton->setBounds( &Rect( 20, 20, 200, 200 ) );
		Container* cont = dynamic_cast<Container*>(m_selectedComponent);
		if ( NULL != cont ){
			cont->add( newButton );
		}
		else if ( NULL != m_currentWin ){
			m_currentWin->add( newButton );
		}

		m_objectInspector->addComponent( m_currentWin, newButton );
		generateCPP( m_currentWin );
		this->selectComponent( newButton );
	}
}

String VCFBuilderMainWin::generateCPP( Frame* frame, const bool& newForm, const String& fileDir ){
	String result;
	/**
	*steps to generate the code
	*1. look for file with (frame->getName).cpp and (frame->getName).h
	*2. if not found then create
	*3. Load and store data as a string otherwise create an empty string
	*4. get a list of all components added for the frame
	*5. create a class as follows:
	*	class (frame->getName()) : public (frame->getClass()->getSuperClass()->getName) {
	*	public:
	*		(frame->getName()) ();
	*		virtual ~(frame->getName()) ();
	*		void init();
	*	private:
	*	};
	*6. for each comp in list, add a member variable to the class 
	*	under the private scope section as follows:
	*		(comp->getClassName) * m_(comp->getName());


  note need a way to bind the variable names created here to 
  the actual components created when the form is loaded
  this could be done in the init() function ????
	*/
	Application* app = Application::getRunningInstance();
	if ( NULL != this->m_currentProject ){
		String cppFileName = "";
		String hFileName = "";
		if ( true == fileDir.empty() ){		
			
			String startDir = m_currentProject->getProjectDir();

			cppFileName = startDir + "\\" + frame->getName() + ".cpp";
			hFileName = startDir + "\\" + frame->getName() + ".h";
		}
		else {
			cppFileName = fileDir + "\\" + frame->getName() + ".cpp";
			hFileName = fileDir + "\\" + frame->getName() + ".h";
		}

		String className = frame->getName();
		String classDecl;
		String classImpl;
		if ( true == newForm ){
			try{
				File f;
				try {
					f.setName( cppFileName );				
					f.remove();
				}
				catch (...){
					throw BasicException("Error deleting \"" + cppFileName + "\""  );
				}
				
				try{
					f.setName( hFileName );
					f.remove();
				}
				catch (...){
					throw BasicException("Error deleting \"" + hFileName + "\""  );
				}
			}
			catch ( BasicException &e ) {
				Dialog::showMessage( e.getMessage() );
			}
			
		}
		if ( true == FileUtils::fileExists( cppFileName ) ){
			
		}
		else {			
			classImpl += "//" + className + ".cpp\n\n";
			classImpl += "#include \"Application.h\"\n";
			classImpl += "#include \"Frame.h\"\n";
			classImpl += "#include \"" + className + ".h\"\n\n\n";			
			classImpl += className + "::" + className + "()\n{\n";
			
			classImpl += "\n\tApplication* app = Application::getRunningInstance();\n";
			classImpl += "\tif ( NULL != app ){\n";
			classImpl += "\t\tFrame* frame = this;\n";
			classImpl += "\t\tapp->loadFrame( &frame );\n";
			classImpl += "\t}\n";
			classImpl += "\n\tvcfInit();\n}\n\n";
			classImpl += className + "::~" + className + "()\n{\n\n}\n\n";
			classImpl += "void " + className + "::vcfInit()\n{\n\n}//vcfInit finished\n\n";

			FileStream CPPFile( cppFileName );
			CPPFile << classImpl;
		}

		if ( true == FileUtils::fileExists( hFileName ) ){
			//search for existing info
			FileStream readFs( hFileName );

			readFs >> classDecl;
			
			int pos = classDecl.find_first_of( frame->getName() );
			if ( pos != String::npos ){ //found the class decl
				Enumerator<Component*>* comps = m_objectInspector->getComponentList( frame );
				if ( NULL != comps ){
					while ( true == comps->hasMoreElements() ){
						Component* c = comps->nextElement();
						if ( c != frame ){
							String compName = "m_" + c->getName();
							pos = classDecl.find( compName );
							if ( pos == String::npos ){ //add a new variable
								String s = "protected:";
								int protectedPos = classDecl.find( s );
								if ( String::npos != protectedPos ){
									String memberVar = "\n\t" + c->getClassName() + "* " + compName + ";\n";
									classDecl.insert( protectedPos + s.size() + 1, memberVar );
								}
							}
						}
					}
				}
			}
			else{

			}			
		}
		else {
			classDecl += "//" + className + ".h\n\n";			
			classDecl += "#ifndef _" + StringUtils::upperCase( className ) + "_H__\n";
			classDecl += "#define _" + StringUtils::upperCase( className ) + "_H__\n\n\n";
			classDecl += "using namespace VCF;\n\n\n";
			String classID = StringUtils::upperCase( className ) + "_CLASSID";
			String uuid = StringUtils::upperCase( StringUtils::newUUID() );
			classDecl += "#define " + classID + "\t\t\"" + uuid + "\"\n\n";
			classDecl += "class " + className + " : public " + frame->getClassName() + "{\n";
			classDecl += "public:\n";
			classDecl += "\tBEGIN_CLASSINFO(" + className + ", \"" + className + "\", \"" + frame->getClassName() + "\", " + classID + " )\n";
			classDecl += "\tEND_CLASSINFO(" + className + ")\n\n";
			classDecl += "\t" + className + "();\n\n";
			classDecl += "\tvirtual ~" + className + "();\n\n";
			classDecl += "\t/**\n\t*vcfInit() is used by VCF Builder to initialize variables\n";
			classDecl += "\t*and linkup listener objects.\n\t*/\n";
			classDecl += "\tvoid vcfInit();\n\n";
			classDecl += "protected:\n";
			
			Enumerator<Component*>* comps = m_objectInspector->getComponentList( frame );
			if ( NULL != comps ){
				while ( true == comps->hasMoreElements() ){
					Component* c = comps->nextElement();
					if ( NULL != c ){
						if ( c != frame ){
							classDecl += "\t" + c->getClassName() + "* " + c->getName() + ";\n";
						}
					}
				}
			}
			classDecl += "};\n\n\n";
			classDecl += "#endif // _" + StringUtils::upperCase( frame->getName() ) + "_H__\n";			
		}

		FileStream HFile( hFileName );
		HFile << classDecl;		

		StringUtils::trace( "cpp: \n" + classImpl );
		StringUtils::trace( "h: \n" + classDecl );
	}
	return result;
}

void VCFBuilderMainWin::loadComponents()
{
	m_insertComponent->clearChildren();
	Enumerator<String>* cats = Component::getRegisteredComponentCategories();
	if ( NULL != cats ){
		MenuItemHandler* menuHandler = 
							new MenuItemHandler( this,
							                     (OnMenuItemEvent)VCFBuilderMainWin::insertNewComponentClick );
		this->addEventHandler( "InsertNewComponentHandler", menuHandler );
		while ( true == cats->hasMoreElements() ){
			String cat = cats->nextElement();
			Enumerator<Class*>* componentClasses = Component::getComponentClassesForCategory( cat );
			if ( NULL != componentClasses ){
				while ( true == componentClasses->hasMoreElements() ){
					Class* compClass = componentClasses->nextElement();
					if ( NULL != compClass ){
						String compClassName = compClass->getClassName();
						StringUtils::trace( "\"" + compClassName + "\" in category \"" + cat + "\"\n" );
						
						

						MenuItem* insertNewComp = new DefaultMenuItem( "Insert " + compClassName, m_insertComponent );
						insertNewComp->setData( (void*)compClass );
						insertNewComp->addMenuItemListener( menuHandler );
						
					}
				}
			}
		}
	}
}

void VCFBuilderMainWin::insertNewComponentClick( MenuItemEvent* e )
{
	MenuItem* item = (MenuItem*)e->getSource();
	if ( NULL != item ){
		Class* compClass = (Class*)item->getData();
		if ( (NULL != this->m_selectedComponent) && (compClass != NULL) ){
			Component* newComponent = NULL;
			compClass->createInstance( (Object**)&newComponent );
			if ( NULL != newComponent ){
				newComponent->setComponentState( CS_DESIGNING );
				newComponent->setTop( 20 );
				newComponent->setLeft( 20 );
				Control* control = dynamic_cast<Control*>( newComponent );
				if ( NULL != control ){
					control->setBounds( &Rect( 20, 20, 200, 200 ) );
					Container* cont = dynamic_cast<Container*>(m_selectedComponent);
					if ( NULL != cont ){
						cont->add( control );
					}
					else if ( NULL != m_currentWin ){
						m_currentWin->add( control );
					}
				}
				else {
					
					Container* cont = dynamic_cast<Container*>(m_selectedComponent);
					if ( NULL != cont ){
						m_selectedComponent->addComponent( newComponent );
					}
					else if ( NULL != m_currentWin ){
						m_currentWin->addComponent( newComponent );
					}
				}
				
				m_objectInspector->addComponent( m_currentWin, newComponent );
				generateCPP( m_currentWin );
				this->selectComponent( newComponent );
			}
		}
	}
}

void VCFBuilderMainWin::addVPL( MenuItemEvent* e )
{
	CommonFileOpen open( this );
	open.addFilter( "Visual Package Libraries (*.vpl)", "*.vpl" );
	if ( true == open.execute() ){
		
		String vplName = open.getFileName();	
		
		try{
			VCFBuilderApp* app =  (VCFBuilderApp*)Application::getRunningInstance();
			app->loadVPL( vplName );
			app->addVPLToList( vplName );
			this->loadComponents();
		}
		catch ( BasicException& ex ){
			StringUtils::trace( ex.getMessage() );
		}		
	}
}

void VCFBuilderMainWin::fileNewDialogClick( MenuItemEvent* e )
{
	Dialog* newForm = new Dialog();
	
	m_currentWin = newForm;	
	m_objectInspector->clearComponents();
	m_objectInspector->addComponent( m_currentWin, newForm );
	this->generateCPP( m_currentWin, true );	
	this->selectComponent( newForm );

	this->m_currentProject->addToProject( m_currentWin );

	newForm->show();
	
}

void VCFBuilderMainWin::fileNewProjectClick( MenuItemEvent* e )
{
	if ( NULL != m_currentProject ){
		delete m_currentProject;
		m_currentProject = NULL;
	}
	
	NewProjectDlg* newProj = new NewProjectDlg();//(NewProjectDlg*)Application::getRunningInstance()->createFrame( "NewProjectDlg" );
	newProj->setParent( this );

	if ( MRT_OK == newProj->showModal() ){	
		
		m_currentProject = new VCFProject();

		m_currentProject->setProjectDir( newProj->getProjDir() );

		m_currentProject->setName( newProj->getProjName() );		
	}

	delete newProj;
	newProj = NULL;

}

void VCFBuilderMainWin::fileOpenProjectClick( MenuItemEvent* e )
{	
		CommonFileOpen openDlg;
		openDlg.addFilter( "Visual Component Builder Project (*.vbp)", "*.vbp" );
		if ( true == openDlg.execute() ){
			if ( NULL != m_currentProject ){
				delete m_currentProject;
				m_currentProject = NULL;
			}
			String s = openDlg.getFileName();
			FileStream fs( s );
			
			VCFProject* openProj = new VCFProject();
			fs >> openProj;

			setCurrentProject( openProj );
		}
}

void VCFBuilderMainWin::fileSaveProjectClick( MenuItemEvent* e )
{
	if ( NULL != m_currentProject ){
		CommonFileSave saveDlg;
		saveDlg.setFileName( m_currentProject->getName() + ".vbp" );
		saveDlg.addFilter( "Visual Component Builder Project (*.vbp)", "*.vbp" );
		if ( true == saveDlg.execute() ){
			String s = saveDlg.getFileName();
			String ext = StringUtils::getFileExt( s );
			if ( true == ext.empty() ){
				s += ".vbp";
			}
			FileStream fs( s );
			fs << m_currentProject;
		}
	}
}

void VCFBuilderMainWin::setCurrentProject( VCFProject* project )
{
	if ( NULL != m_currentProject ){
		delete m_currentProject;
		m_currentProject = NULL;
	}

	m_currentProject = project;

	if ( NULL != m_currentProject ){
		this->setCaption( "Project: [ " + m_currentProject->getName() + " ]" );
		Application::getRunningInstance()->setCurrentDirectory( m_currentProject->getProjectDir() );
	}

	this->m_projectContents->setCurrentProject( m_currentProject );
}

void VCFBuilderMainWin::helpAboutClick( MenuItemEvent* e )
{
	VCFAbout* about = new VCFAbout();
	about->showModal();
	delete about;
	about = NULL;
}

void VCFBuilderMainWin::viewObjectInspectorClick( MenuItemEvent* e )
{
	if ( NULL != this->m_objectInspector ){
		MenuItem* item = (MenuItem*)e->getSource();
		m_objectInspector->setVisible( !m_objectInspector->getVisible() );
		item->setChecked( m_objectInspector->getVisible() );
	}
}

void VCFBuilderMainWin::viewProjectContentsClick( MenuItemEvent* e )
{
	if ( NULL != this->m_objectInspector ){
		MenuItem* item = (MenuItem*)e->getSource();
		this->m_projectContents->setVisible( !m_projectContents->getVisible() );
		item->setChecked( m_projectContents->getVisible() );
	}
}