/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "VCFProject.h"
#include "VCFFile.h"
#include "Frame.h"

using namespace VCF;

VCFProject::VCFProject()
{
	m_fileContainer.initContainer( m_fileListingMap );
}

VCFProject::~VCFProject()
{

}

void VCFProject::addToProject( Frame* frameWindow )
{
	String name = frameWindow->getName();
	m_fileListingMap[name] = new VCFFile( this, frameWindow );
}

void VCFProject::saveToStream( OutputStream * stream )
{
	std::map<String,VCFFile*>::iterator it = m_fileListingMap.begin();
	
	OutputStream& outStrRef = *stream;

	outStrRef << this->m_name;

	outStrRef << this->m_projDir;
	
	int count = m_fileListingMap.size();

	outStrRef << count;

	while ( it != m_fileListingMap.end() ){
		VCFFile* vcfFile = it->second;
		vcfFile->updateFileNames();
		outStrRef << vcfFile->getFrame()->getName();
		outStrRef << vcfFile->getHFileName();
		outStrRef << vcfFile->getCppFileName();
		outStrRef << vcfFile->getVFFFileName();
		it++;
	}
}
	
void VCFProject::loadFromStream( InputStream * stream )
{
	InputStream& inStrRef = *stream;

	inStrRef >> this->m_name;

	inStrRef >> this->m_projDir;

	int count = 0;

	inStrRef >> count;
	
	for ( int i=0;i<count;i++){
		String s1;
		String s2;
		String s3;
		String s4;
	
		inStrRef >> s1;	
		inStrRef >> s2;
		inStrRef >> s3;
		inStrRef >> s4;
		VCFFile* vcfFile = new VCFFile( this, NULL );
		vcfFile->setHFileName( s2 );
		vcfFile->setCppFileName( s3 );
		vcfFile->setVFFFileName( s4 );
		m_fileListingMap[s1] = vcfFile;
	}

}

Enumerator<VCFFile*>* VCFProject::getFiles()
{
	return m_fileContainer.getEnumerator();
}