/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 


//ApplicationPeer.h


#ifndef _APPLICATION_Peer_H__
#define _APPLICATION_Peer_H__

#include "String.h"
#include <vector>


namespace VCF
{

class Application;
class ResourceBundle;
/**
*System neutral interface for intializing the windowing system application
*
*@version 1.0
@author Jim Crafton
*/

/*@interface*/
class APPKIT_API ApplicationPeer {
public:
	virtual ~ApplicationPeer(){};
	
	/**
	*initializes the application Peer. Any window specific system stuff can 
	*go in the implementation here.
	*/
	virtual bool initApp( const std::vector<String>& appCmdLine )=0;

	/**
	*terminates the application Peer
	*/
	virtual void terminateApp() = 0; 
	
	/**
	*get the Application instance associated with the 
	*application implmenter
	*/
	virtual Application* getApplication() = 0;

	/**
	*set the Application instance ot be associated with the 
	*application implmenter
	*/
	virtual void setApplication( Application* application ) = 0;	

	virtual ResourceBundle* getResourceBundle() = 0;

	virtual String getFileName() = 0;

	virtual void run() = 0;

	virtual String getCurrentDirectory() = 0;

	virtual void setCurrentDirectory( const String& currentDirectory ) = 0;
};


};
	
#endif