/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _BASICTABLEITEMEDITOR_H__
#define _BASICTABLEITEMEDITOR_H__

#include "Object.h"
#include "TableItemEditor.h"
#include "ItemEditorListener.h"
#include "ItemEditorEvent.h"
#include <map>

namespace VCF {

class TableCellItem;


#define BASICTABLEITEMEDITOR_CLASSID		"7E202381-F9EB-4585-9685-889FA9B07D47"

class APPKIT_API BasicTableItemEditor : public TableItemEditor{
public:

	BEGIN_CLASSINFO(BasicTableItemEditor, "VCF::BasicTableItemEditor", "VCF::TableItemEditor", BASICTABLEITEMEDITOR_CLASSID)
	END_CLASSINFO(BasicTableItemEditor)

	BasicTableItemEditor( TableCellItem* editingItem = NULL );

	virtual ~BasicTableItemEditor();

	virtual bool isCellEditable();

	LISTENER_LIST(ItemEditorListener);

	/**
	*This macro creates a method for adding a listener to the BasicTableItemEditor's TableItemEditor events
	*/
	ADD_LISTENER(ItemEditorListener);

	/**
	*This macro creates a method for removing a listener to the BasicTableItemEditor's TableItemEditor events
	*/
	REMOVE_LISTENER(ItemEditorListener);    

	NOTIFIER(ItemEditorListener, onChange, ItemEditorEvent);    

	NOTIFIER(ItemEditorListener, onValidateChange, ItemEditorEvent);    

	virtual void setItemToEdit( TableCellItem* itemToEdit );

	virtual TableCellItem* getItemToEdit(){
		return 	m_editingItem;
	}

	virtual void updateItem();

	virtual Control* getEditingControl();

	Control* getCurrentEditingControl( TableCellItem* currentItem );
private:
	TableCellItem* m_editingItem;
	/**
	*this is here just in case we get reused by multiple items
	*/
	std::map<TableCellItem*,Control*> m_editorMap;
};

}; //end of namespace VCF

#endif //_BASICTABLEITEMEDITOR_H__