/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//ClassInfo.h


#ifndef _CLASSINFO_H__ 
#define _CLASSINFO_H__

#include "VCF.h"
#include "Object.h"
#include "ClassRegistry.h"

#pragma warning (disable : 4786)

namespace VCF
{

template <class CLASS_TYPE> class FRAMEWORK_API ClassInfo : public Object {
public:
	ClassInfo( const String& className, 
		       const String& superClassName, const String& classID ){
		m_isClassRegistered = registerClass<CLASS_TYPE>( NULL, className, superClassName, classID );
	};
	
	virtual ~ClassInfo(){};
	
	bool isClassRegistered(){
		return m_isClassRegistered;
	}
	
private:
	bool m_isClassRegistered;
};	


template <class CLASS_TYPE> class FRAMEWORK_API AbstractClassInfo : public Object {
public:
	AbstractClassInfo( const String& className, 
		       const String& superClassName, const String& classID ){
		
		m_isClassRegistered = registerAbstractClass<CLASS_TYPE>( NULL, className, superClassName, classID );
		
	};
	
	virtual ~AbstractClassInfo(){};
	
	bool isClassRegistered(){
		return m_isClassRegistered;
	}
	
private:
	bool m_isClassRegistered;
};

#define BEGIN_CLASSINFO( ClassType, ClassName, superClassName, classID ) \
	class ClassType##Info : public ClassInfo<ClassType> { \
	public: \
		ClassType##Info(): \
			ClassInfo<ClassType>( ClassName, superClassName, classID ){ \
			String tmpClassName = ClassName; \
			if ( true == isClassRegistered()  ){ \
			\
			    		

#define BEGIN_ABSTRACT_CLASSINFO( ClassType, ClassName, superClassName, classID ) \
	class ClassType##Info : public AbstractClassInfo<ClassType> { \
	public: \
		ClassType##Info(): \
			AbstractClassInfo<ClassType>( ClassName, superClassName, classID ){ \
			String tmpClassName = ClassName; \
			if ( true == isClassRegistered()  ){ \
			\


#define EVENT( eventClassName,eventMethodName,listenerClassName,adapterClassName ) \
		registerEvent( tmpClassName, eventClassName, eventMethodName, listenerClassName, adapterClassName ); \
		\

#define PROPERTY( type, propName, getFunc, setFunc, propType ) \
	            registerProperty<type>( tmpClassName, propName, \
							           (TypedProperty<type>::GetFunction)getFunc, \
									   (TypedProperty<type>::SetFunction)setFunc, \
									    propType ); \
										\

#define READONLY_PROPERTY( type, propName, getFunc, propType ) \
	            registerReadOnlyProperty<type>( tmpClassName, propName, \
							                    (TypedProperty<type>::GetFunction)getFunc, \
									            propType ); \
										                    \

#define OBJECT_PROPERTY( type, propName, getFunc, setFunc ) \
				registerProperty<type>( tmpClassName, propName,  \
                            (TypedObjectProperty<type>::GetFunction)getFunc,  \
							(TypedObjectProperty<type>::SetFunction)setFunc );  \
							                                                   \


#define READONLY_OBJECT_PROPERTY( type, propName, getFunc ) \
				registerReadOnlyProperty<type>( tmpClassName, propName, \
                            (TypedObjectProperty<type>::GetFunction)getFunc ); \
																		       \

#define ENUM_PROPERTY(type,propName,getFunc,setFunc, lower, upper) \
				registerProperty<type>( tmpClassName, propName, \
									    (TypedEnumProperty<type>::GetFunction)getFunc, \
										(TypedEnumProperty<type>::SetFunction)setFunc, \
										lower, \
										upper); \
										 \

#define LABELED_ENUM_PROPERTY(type,propName,getFunc,setFunc, lower, upper, count, enumNames) \
				registerProperty<type>( tmpClassName, propName, \
									    (TypedEnumProperty<type>::GetFunction)getFunc, \
										(TypedEnumProperty<type>::SetFunction)setFunc, \
										lower, \
										upper, \
										count, \
										enumNames ); \
													\


#define READONLY_ENUM_PROPERTY(type,propName,getFunc,lower, upper) \
				registerReadOnlyProperty<type>( tmpClassName, propName, \
												(TypedEnumProperty<type>::GetFunction)getFunc, \
												lower, \
												upper); \
												        \


#define READONLY_LABELED_ENUM_PROPERTY(type,propName,getFunc,lower, upper, count, enumNames) \
				registerReadOnlyProperty<type>( tmpClassName, propName, \
									            (TypedEnumProperty<type>::GetFunction)getFunc, \
												lower, \
												upper, \
												count, \
												enumNames ); \
												             \


#define COLLECTION_PROPERTY( type,propName,getFunc, propType ) \
				registerCollectionProperty<type>( tmpClassName, propName, \
				                                  (TypedCollectionProperty<type>::GetFunction)getFunc, \
                                                  propType ); \
										            \

#define OBJECT_COLLECTION_PROPERTY( type,propName,getFunc, addFunc, insertFunc, deleteFunc1, deleteFunc2 ) \
				registerCollectionProperty<type>( tmpClassName, propName, \
				                                  (TypedObjectCollectionProperty<type>::GetFunction)getFunc, \
												  (TypedObjectCollectionProperty<type>::AddFunction)addFunc, \
												  (TypedObjectCollectionProperty<type>::InsertFunction)insertFunc, \
												  (TypedObjectCollectionProperty<type>::DeleteFunction1)deleteFunc1, \
												  (TypedObjectCollectionProperty<type>::DeleteFunction2)deleteFunc2 ); \
										            \

#define METHOD_VOID( methodName, classType, methType, methodPtr ) \
	typedef  void ( classType::* _T_##classType##methType )(void); \
	registerVoidMethodArg0<classType,_T_##classType##methType##>( NULL, tmpClassName, methodName, methodPtr, "" ); \
	  \

#define METHOD_1VOID( methodName, classType, methType, argType, methodPtr, argTypeDesc ) \
	typedef  void ( classType::* _T_##classType##methType )(argType); \
	registerVoidMethodArg1<classType,_T_##classType##methType##>( NULL, tmpClassName, methodName, methodPtr, argTypeDesc ); \
	  \

 #define METHOD_2VOID( methodName, classType, methType, argType1, argType2, methodPtr, argTypeDesc ) \
	typedef  void ( classType::* _T_##classType##methType )(argType1, argType2); \
	registerVoidMethodArg1<classType,_T_##classType##methType##>( NULL, tmpClassName, methodName, methodPtr, argTypeDesc ); \
	  \

#define END_CLASSINFO( ClassType ) \
  			} \
		}; \
		 \
		 \
		virtual ~ClassType##Info(){}; \
     \
	}; \
	\


#define REGISTER_CLASSINFO( ClassType ) \
	ClassType::ClassType##Info classInfoFor##ClassType; \
	                                           \


};





#endif

