/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef COMPONENT_H
#define COMPONENT_H


#include "Object.h"
#include "Persistable.h"
#include "ComponentListener.h"
#include "ComponentEvent.h"
#include "ClassInfo.h"
#include <map>
#include <vector>

namespace VCF{

class Rect;

class InvalidPeer;

class Color;

/**
 *The base class for all widgets in the Visual Component Framework.
 *Components are not actual widgets. They are the base class for 
 *any object that wants to be manipulated within the IDE
 */

class Border;

#define COMPONENT_CLASSID		"ED88C09B-26AB-11d4-B539-00C04F0196DA"

enum ComponentState {
	CS_UNKNOWN=0,
	CS_LOADING,
	CS_READING,
	CS_WRITING,
	CS_CREATED,
	CS_DESIGNING,
	CS_NORMAL
};

class APPKIT_API Component : public Object, public Persistable{
public:
	
	BEGIN_ABSTRACT_CLASSINFO(Component, "VCF::Component", "VCF::Object", COMPONENT_CLASSID)	
	
	PROPERTY( double, "left", Component::getLeft, Component::setLeft, PROP_DOUBLE );
	PROPERTY( double, "top", Component::getTop, Component::setTop, PROP_DOUBLE );
	PROPERTY( String, "name", Component::getName, Component::setName, PROP_STRING );		
	EVENT("VCF::ComponentEvent", "onComponentCreated", "VCF::ComponentListener","VCF::ComponentHandler" );
	EVENT("VCF::ComponentEvent", "onComponentDeleted", "VCF::ComponentListener","VCF::ComponentHandler" );
	END_CLASSINFO(Component)

	Component();	

	virtual ~Component();

	/**
	*This macro declares a std::vector<> of ComponentListener listeners
	*/
	LISTENER_LIST(ComponentListener);

	/**
	*This macro creates a method for adding a listener to the Component's events
	*/
	ADD_LISTENER(ComponentListener);

	/**
	*This macro creates a method for removing a listener to the Component's events
	*/
	REMOVE_LISTENER(ComponentListener);

	/**
	*Component events
	*/
	NOTIFIER(ComponentListener, onComponentCreated, ComponentEvent);

	NOTIFIER(ComponentListener, onComponentDeleted, ComponentEvent);

    virtual double getLeft() throw( InvalidPeer );

    virtual double getTop() throw( InvalidPeer );

	virtual void setLeft( const double& left ) throw( InvalidPeer );

	virtual void setTop( const double& top ) throw( InvalidPeer );

    virtual String getName();

    virtual void setName( const String& name );           

    virtual void saveToStream( OutputStream * stream );

    virtual void loadFromStream( InputStream * stream );

    virtual void beforeDestroy( ComponentEvent* event );

	virtual void afterCreate( ComponentEvent* event ) = 0;
	
	virtual String toString();

	void processComponentEvent( ComponentEvent* componentEvent );

	ComponentState getComponentState();

	void setComponentState( const ComponentState& componentState );

	virtual void addComponent( Component* component );

	virtual void removeComponent( Component* component );

	virtual Enumerator<Component*>* getComponents();

	virtual unsigned long getComponentCount();

	Component* Component::findComponent( const String& componentName );

	bool bindVariable( Component** variablePtr, const String& variableName );

	/**
	*registers a component in the system component map. The map is organized into 
	a series of keys called categories, with 1 or more entries in each category.
	The UIToolkit registers the default ApplicationKit components, and other developers may
	use this to extend the list with theor own components.
	*/
	static bool registerComponent( const String& componentClassName, const String& category );

	static Enumerator<Class*>* getComponentClassesForCategory( const String& category );

	static Enumerator<String>* getRegisteredComponentCategories();

	static void clearRegistedComponents();

protected:
	ComponentState m_componentState;	
private:
    String m_name;
	double m_left;
	double m_top;

	std::vector<Component*> m_components;
	EnumeratorContainer<std::vector<Component*>, Component*> m_componentContainer;
	/**
	*a map of component classes to categorys
	*/
	
	static std::map<String,std::vector<Class*>*> registeredComponentMap;
	static std::vector<String> registeredCategory;
	static EnumeratorContainer<std::vector<String>,String> registeredCategoryContainer;
	static EnumeratorContainer<std::vector<Class*>,Class*> registeredCompContainer;
	
};	




class APPKIT_API ComponentDTD : public Object {
public:
	ComponentDTD(){};

	virtual ~ComponentDTD(){};
	
	virtual String toString();
};

};

#endif //COMPONENT_H