/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#ifndef CONTAINER_H
#define CONTAINER_H



#include "Enumerator.h"
#include "Object.h"
#include "CommonDefines.h"
#include <vector>

namespace VCF{

class Control;

class ControlEvent;

class MouseEvent;

class GraphicsContext;

class ControlHandler;

class MouseHandler;

class Rect;


/** @interface */
class APPKIT_API Container {

public:
	virtual ~Container(){};
    
	virtual void add( Control * child ) = 0;
    
	virtual void add( Control * child, const AlignmentType& alignment ) = 0;	
	
	virtual void remove( Control* child ) = 0;

	virtual Enumerator<Control*>* getChildren() = 0;

	virtual unsigned long getChildCount() = 0;	

	virtual Control* findControl( const String& controlName ) = 0;

	virtual void resizeChildren() = 0;
};


/**
*Basic implementation of a container
*handles basic alignment,layout of child components
*/
class APPKIT_API AbstractContainer : public Container, public EnumeratorContainer<std::vector<Control*>, Control*>{
public:	
	/**
	*nested inner class to handle implementation of resizing events
	*/
	class ContainerResizeHandler : public Object {
	public:

		ContainerResizeHandler( AbstractContainer* container );
		virtual ~ContainerResizeHandler(){};

		void containerResized( ControlEvent* event );

	private:
		AbstractContainer* m_container;
	};

	class ContainerMouseEventHandler : public Object {
	public:
		ContainerMouseEventHandler( AbstractContainer* container ){
			m_container = container;
		};

		virtual ~ContainerMouseEventHandler(){};

		void onMouseEvent( MouseEvent* event ); 

	private:
		AbstractContainer* m_container;
	};

	AbstractContainer();
	
	virtual ~AbstractContainer();

	virtual void init();

	virtual void add( Control * child );

    virtual void add( Control * child, const AlignmentType& alignment );

	virtual void remove( Control* child );

	virtual void resizeChildren();
	
	void setContainer( Control* container );

	Control* getContainer();

	void paintChildren( GraphicsContext* context );

	virtual Enumerator<Control*>* getChildren();

	virtual unsigned long getChildCount();

	virtual Control* findControl( const String& controlName );
protected:
	std::vector<Control*> m_controls;	
	Control* m_container;
	ControlHandler* m_containerHandler;
	ContainerResizeHandler* m_handlerObject;
	ContainerMouseEventHandler* m_containerMouseHandler;
	MouseHandler* m_mouseHandler;

	bool insertBefore( Control* c1, Control* c2, const AlignmentType& alignment );
	void doPosition( Control* component, const AlignmentType& alignment, Rect* rect );
	void doAlign( const AlignmentType& alignment, Rect* rect );
	bool alignWork();
};

};
#endif //CONTAINER_H