/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _DEFAULTMENUITEM_H__
#define _DEFAULTMENUITEM_H__

#include "MenuItem.h"
#include "MenuItemListener.h"
#include "MenuItemEvent.h"
#include "ItemEvent.h"
#include "ItemListener.h"
#include <vector>
#include "VCF.h"

namespace VCF {

class MenuItemPeer;


class APPKIT_API DefaultMenuItem : public MenuItem {

public:
	DefaultMenuItem();	

	DefaultMenuItem( const String& caption, MenuItem* parent=NULL, Menu* menuOwner=NULL );

	virtual ~DefaultMenuItem();

	void init();

	virtual bool containsPoint( Point * pt );

	virtual Image* getImage();

    virtual unsigned long getIndex();

	virtual void setIndex( const unsigned long& index );

    virtual void* getData();

	virtual void setData( void* data );	

	virtual Model* getModel();

	virtual void setModel( Model* model );

	virtual void paint( GraphicsContext* context, Rect* paintRect );
	
	virtual bool isSelected();

	virtual void setSelected( const bool& selected );

	virtual Enumerator<MenuItem*>* getChildren();	

	virtual void addChild( MenuItem* child );

	virtual void insertChild( const unsigned long& index, MenuItem* child );
	
	virtual void deleteChild( MenuItem* child );

	virtual void deleteChild( const unsigned long& index );

	virtual void clearChildren();

	virtual bool isChecked();

	virtual void setChecked( const bool& checked );

	virtual bool hasParent();

	virtual bool hasChildren();

	virtual Menu* getMenuOwner();

	virtual void setMenuOwner( Menu* menuOwner );

	virtual MenuItem* getParent();

	virtual void setParent( MenuItem* parent );

	virtual MenuItem* getChildAt( const unsigned long& index );

	virtual bool isEnabled();

	virtual void setEnabled( const bool& enabled );

	virtual bool isVisible();

	virtual void setVisible( const bool& visible );

	virtual bool getRadioItem();

	virtual void setRadioItem( const bool& value );

	virtual void setCaption( const String& caption );
	
	virtual String getCaption();

	virtual MenuItemPeer* getPeer();

	virtual bool isSeparator();

	virtual void setSeparator( const bool& separator );

	virtual void click();

	virtual void update();
	/**
	*This macro declares a std::vector<> of ItemListener listeners
	*/
	LISTENER_LIST(ItemListener);

	/**
	*This macro declares a std::vector<> of MenuItemListener listeners
	*/
	LISTENER_LIST(MenuItemListener);

	/**
	*This macro creates a method for adding a listener to the DefaultMenuItem's Item events
	*/
	ADD_LISTENER(ItemListener);

	/**
	*This macro creates a method for removing a listener to the DefaultMenuItem's Item events
	*/
	REMOVE_LISTENER(ItemListener);    

	/**
	*This macro creates a method for adding a listener to the DefaultMenuItem's Item events
	*/
	ADD_LISTENER(MenuItemListener);

	/**
	*This macro creates a method for removing a listener to the DefaultMenuItem's Item events
	*/
	REMOVE_LISTENER(MenuItemListener);
    
	NOTIFIER(ItemListener, onItemPaint, ItemEvent);

	NOTIFIER(ItemListener, onItemChanged, ItemEvent);

	NOTIFIER(ItemListener, onItemSelected, ItemEvent);

	NOTIFIER(ItemListener, onItemAdded, ItemEvent);

	NOTIFIER(ItemListener, onItemDeleted, ItemEvent);

	NOTIFIER(MenuItemListener, onMenuItemClicked, MenuItemEvent);

	NOTIFIER(MenuItemListener, onMenuItemUpdate, MenuItemEvent);

private:
	MenuItemPeer* m_Peer;
	std::vector<MenuItem*> m_menuItems;
	EnumeratorContainer<std::vector<MenuItem*>, MenuItem*> m_container;
	String m_caption;
	bool m_visible;
	ulong32 m_index;
	bool m_selected;
	void* m_data;
	Menu* m_menuOwner;
	bool m_separator;
	MenuItem* m_parent;
	bool m_radioItem;
};

}; //end of namespace VCF

#endif //_DEFAULTMENUITEM_H__