/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//DefaultPropertyEditors.h

#ifndef _DEFAULT_PROPERTY_EDITORS_H__ 
#define _DEFAULT_PROPERTY_EDITORS_H__ 


#include "PropertyEditor.h"


namespace VCF
{

class Control;

class ItemEvent;

class ComboBoxAdapter;

class APPKIT_API AbstractPropertyEditor : public Object, public PropertyEditor {
public:
	AbstractPropertyEditor();

	virtual ~AbstractPropertyEditor();

	virtual Control* getCustomEditor();
	
	virtual void paintValue( GraphicsContext* context, const Rect& bounds );

	virtual VariantData* getValue();

	virtual void setSource( Object* source );

	virtual void setProperty( Property* property );

	virtual void setValue( VariantData* value );

	/**
	*is it OK to call getCustomEditor ?
	*/
	virtual bool hasCustomEditor();
	
	/**
	*is OK to call paintValue ?
	*/
	virtual bool canPaintValue();

	virtual String getValueAsText();

	virtual void setValueAsText( const String& textValue );

	/**
	*called by an IDE to get the C++ string representation of 
	the property to insert into C++ code.
	*/
	virtual String getCodeString();
	
protected:
	Property* m_property;
	Object* m_source;
	VariantData m_data;
private:
	
};


class APPKIT_API IntegerPropertyEditor : public AbstractPropertyEditor {
public:
	IntegerPropertyEditor(){};

	virtual ~IntegerPropertyEditor(){};	

private:
	
};


class APPKIT_API DoublePropertyEditor : public AbstractPropertyEditor {
public:
	DoublePropertyEditor(){};

	virtual ~DoublePropertyEditor(){};	

private:
	
};

class APPKIT_API StringPropertyEditor : public AbstractPropertyEditor {
public:
	StringPropertyEditor(){};

	virtual ~StringPropertyEditor(){};	

private:
	
};


class ComboBoxControl;

class APPKIT_API BoolPropertyEditor : public AbstractPropertyEditor {
public:
	BoolPropertyEditor();

	virtual ~BoolPropertyEditor();	

	virtual bool hasCustomEditor(){
		return true;	
	};

	virtual Control* getCustomEditor();

	void onSelectionChange( ItemEvent* event );
private:
	ComboBoxControl* m_combo;
	ComboBoxAdapter* m_comboBoxAdapter;
};

class APPKIT_API EnumPropertyEditor : public AbstractPropertyEditor{
public:
	EnumPropertyEditor();

	virtual ~EnumPropertyEditor();
	
	virtual bool hasCustomEditor(){
		return true;	
	};

	virtual Control* getCustomEditor();

	void onSelectionChange( ItemEvent* event );
private:
	ComboBoxControl* m_combo;
	ComboBoxAdapter* m_comboBoxAdapter;
};



class APPKIT_API ColorPropertyEditor : public AbstractPropertyEditor {
public:
	ColorPropertyEditor();

	virtual ~ColorPropertyEditor();
	
	virtual bool hasCustomEditor(){
		return true;	
	};

	virtual bool canPaintValue(){
		return true;
	}

	virtual void paintValue( GraphicsContext* context, const Rect& bounds );

	virtual Control* getCustomEditor();

	void showColorEditor( VariantData* data );
private:
	
};

class APPKIT_API FontPropertyEditor : public AbstractPropertyEditor {
public:
	FontPropertyEditor();

	virtual ~FontPropertyEditor();
	
	virtual bool hasCustomEditor(){
		return true;	
	};

	virtual bool canPaintValue(){
		return true;
	}

	virtual void paintValue( GraphicsContext* context, const Rect& bounds );

	virtual Control* getCustomEditor();

	void showFontEditor( VariantData* data );
private:
	
};


class APPKIT_API DefaultMenuItemPropertyEditor : public AbstractPropertyEditor {
public:
	DefaultMenuItemPropertyEditor();

	virtual ~DefaultMenuItemPropertyEditor();
	
	virtual bool hasCustomEditor(){
		return true;	
	};	

	virtual Control* getCustomEditor();

	void showDefaultMenuItemEditor( VariantData* data );
private:
	
};

class APPKIT_API DefaultListModelPropertyEditor : public AbstractPropertyEditor {
public:
	DefaultListModelPropertyEditor();

	virtual ~DefaultListModelPropertyEditor();
	
	virtual bool hasCustomEditor(){
		return true;	
	};	

	virtual Control* getCustomEditor();

	void showDefaultListModelEditor( VariantData* data );
private:
	
};

};


#endif