/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _DEFAULTTABMODEL_H__
#define _DEFAULTTABMODEL_H__

#include "TabModel.h"
#include "TabPage.h"
#include <vector>

namespace VCF {

#define DEFAULTTABMODEL_CLASSID		"86F02174-3E7F-11d4-8EA7-00207811CFAB"

class APPKIT_API DefaultTabModel : public TabModel {
public:
	BEGIN_CLASSINFO(DefaultTabModel, "VCF::DefaultTabModel", "VCF::TabModel", DEFAULTTABMODEL_CLASSID )
	END_CLASSINFO(DefaultTabModel)

	DefaultTabModel();

	virtual ~DefaultTabModel();

	/**
	*This macro declares a std::vector<> of ItemListener listeners
	*/
	LISTENER_LIST(TabModelListener);

	/**
	*This macro creates a method for adding a listener to the DefaultTreeItem's Item events
	*/
	ADD_LISTENER(TabModelListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTreeItem's Item events
	*/
	REMOVE_LISTENER(TabModelListener);   

	NOTIFIER(TabModelListener, onTabPageAdded, TabModelEvent);    

	NOTIFIER(TabModelListener, onTabPageRemoved, TabModelEvent);

	NOTIFIER(TabModelListener, onTabPageSelected, TabModelEvent);
	

	virtual void addTabPage( TabPage* page );

	virtual void insertTabPage( const ulong32& index, TabPage* page );
	
	virtual void deleteTabPage( TabPage* page );

	virtual void deleteTabPage( const ulong32& index );

	virtual void clearTabPages();

	virtual TabPage* getPageAt( const ulong32& index );

	virtual TabPage* getSelectedPage();

	virtual void setSelectedPage( TabPage* page );

	virtual void setSelectedPage( const ulong32& index );

	virtual Enumerator<TabPage*>* getPages();

	LISTENER_LIST(ModelValidationListener);

	ADD_LISTENER(ModelValidationListener);

	REMOVE_LISTENER(ModelValidationListener);   
    /**
     * validate the model. 
     * The implementation for this can vary widely, or even be nonexistant for model's that do not require validation.
     * The basic idea is to call all the listeners in the list , passing in a local variable to the
     * onModelValidate() methods of the listener's. The variable is initialized to true, and if it is
     * still true at the end of the listener iterations, then it is safe to apply the changes to the
     * model, other wise the changes are removed. 
     */
    virtual void validate(){};

    /**
     * clears out the model's data 
     */
    virtual void empty(){};
private:
	std::vector<TabPage*> m_pages;
	EnumeratorContainer<std::vector<TabPage*>, TabPage*> m_container;
};

}; //end of namespace VCF

#endif //_DEFAULTTABMODEL_H__