/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _DEFAULTTABPAGE_H__
#define _DEFAULTTABPAGE_H__

#include "VCF.h"
#include "TabPage.h"
#include "Rect.h"
#include "ItemListener.h"
#include "ItemEvent.h"

namespace VCF {

#define DEFAULTTABPAGE_CLASSID	"86F02173-3E7F-11d4-8EA7-00207811CFAB"

class APPKIT_API DefaultTabPage : public TabPage{
public:
	BEGIN_CLASSINFO(DefaultTabPage, "VCF::DefaultTabPage", "VCF::TabPage", DEFAULTTABPAGE_CLASSID)
	END_CLASSINFO(DefaultTabPage)

	DefaultTabPage( Control* component=NULL );

	virtual ~DefaultTabPage();

	virtual bool containsPoint( Point * pt );

	virtual Image* getImage();

    virtual unsigned long getIndex();

	virtual void setIndex( const unsigned long& index );

    virtual void* getData();

	virtual void setData( void* data );	

	virtual Model* getModel();

	virtual void setModel( Model* model );

	/**
	*This macro declares a std::vector<> of ItemListener listeners
	*/
	LISTENER_LIST(ItemListener);

	/**
	*This macro creates a method for adding a listener to the DefaultTreeItem's Item events
	*/
	ADD_LISTENER(ItemListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTreeItem's Item events
	*/
	REMOVE_LISTENER(ItemListener);    
    
	NOTIFIER(ItemListener, onItemPaint, ItemEvent);

	NOTIFIER(ItemListener, onItemChanged, ItemEvent);

	NOTIFIER(ItemListener, onItemSelected, ItemEvent);

	NOTIFIER(ItemListener, onItemAdded, ItemEvent);

	NOTIFIER(ItemListener, onItemDeleted, ItemEvent);

	virtual void setPageName( const String& name );

	virtual String getPageName();

	virtual Control* getPageComponent();

	virtual void setPageComponent( Control* component );

	virtual bool isSelected();

	virtual void setSelected( const bool& selected );

	virtual void paint( GraphicsContext* context, Rect* paintRect );

	virtual ulong32 getPreferredHeight();
private:
	Model* m_model;
	ulong32 m_index;
	String m_pageName;
	Control* m_component;
	bool m_selected;
	ulong32 m_preferredHeight;
	Rect m_bounds;
};

}; //end of namespace VCF

#endif //_DEFAULTTABPAGE_H__