/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _DEFAULTTABLECELLITEM_H__
#define _DEFAULTTABLECELLITEM_H__

#include "TableCellItem.h"
#include "ItemEvent.h"
#include "ItemListener.h"
#include "Rect.h"
#include "Basic3DBorder.h"
#include "BasicTableItemEditor.h"

namespace VCF {

#define DEFAULTTABLECELLITEM_CLASSID	"53EA0BA6-7068-11d4-8F12-00207811CFAB"

class APPKIT_API DefaultTableCellItem : public TableCellItem {
public:	
	BEGIN_CLASSINFO(DefaultTableCellItem, "VCF::DefaultTableCellItem", "VCF::TableCellItem", DEFAULTTABLECELLITEM_CLASSID)
	END_CLASSINFO(DefaultTableCellItem)

	DefaultTableCellItem();

	virtual ~DefaultTableCellItem();

	virtual bool containsPoint( Point * pt );

	/**
	*This macro declares a std::vector<> of ItemListener listeners
	*/
	LISTENER_LIST(ItemListener);

	/**
	*This macro creates a method for adding a listener to the DefaultTreeItem's Item events
	*/
	ADD_LISTENER(ItemListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTreeItem's Item events
	*/
	REMOVE_LISTENER(ItemListener);    
    
	NOTIFIER(ItemListener, onItemPaint, ItemEvent);

	NOTIFIER(ItemListener, onItemChanged, ItemEvent);

	NOTIFIER(ItemListener, onItemSelected, ItemEvent);

	NOTIFIER(ItemListener, onItemAdded, ItemEvent);

	NOTIFIER(ItemListener, onItemDeleted, ItemEvent);

	virtual Image* getImage();

    virtual unsigned long getIndex();

	virtual void setIndex( const unsigned long& index );

    virtual void* getData();

	virtual void setData( void* data );	

	virtual Model* getModel();

	virtual void setModel( Model* model );

	virtual void paint( GraphicsContext* context, Rect* paintRect );
	
	virtual bool isSelected();

	virtual void setSelected( const bool& selected );

	virtual uint32 getRow();
	
	virtual void setRow( const uint32& currentRow );

	virtual uint32 getColumn();

	virtual void setColumn( const uint32& currentColumn );

	virtual TableItemEditor* getItemEditor();

	virtual bool isItemEditable();

	virtual bool isFixed();

	virtual void setFixed( const bool& isFixed );

	virtual String getCaption();

	virtual void setCaption( const String& caption );
private:
	void init();
	Rect m_bounds;
	uint32 m_row;
	uint32 m_col;
	Image* m_image;
	void* m_data;
	Model* m_model;
	bool m_selected;
	Basic3DBorder m_border;
	bool m_isFixedItem;
	String m_caption;
	BasicTableItemEditor* m_basicItemEditor;
};

}; //end of namespace VCF

#endif //_DEFAULTTABLECELLITEM_H__