/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _DEFAULTTABLEMODEL_H__
#define _DEFAULTTABLEMODEL_H__

#include "TableModel.h"
#include <map>
#include <vector>
#include "ModelValidationListener.h"
#include "TableModelListener.h"
#include "Enumerator.h"
#include "TableModelEvent.h"

namespace VCF {

#define DEFAULTTABLEMODEL_CLASSID		"53EA0BA8-7068-11d4-8F12-00207811CFAB"

class TableCellItem;

class APPKIT_API DefaultTableModel : public TableModel{
public:
	BEGIN_CLASSINFO(DefaultTableModel, "VCF::DefaultTableModel", "VCF::TableModel", DEFAULTTABLEMODEL_CLASSID )
	END_CLASSINFO(DefaultTableModel)
	DefaultTableModel();

	virtual ~DefaultTableModel();

	/**
	*This macro declares a std::vector<> of ModelValidationListener listeners
	*/
	LISTENER_LIST(ModelValidationListener);
	
	/**
	*This macro creates a method for adding a listener to the DefaultTableModel's Item events
	*/
	ADD_LISTENER(ModelValidationListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTableModel's Item events
	*/
	REMOVE_LISTENER(ModelValidationListener);    

    /**
     * validate the model. 
     * The implementation for this can vary widely, or even be nonexistant for model's that do not require validation.
     * The basic idea is to call all the listeners in the list , passing in a local variable to the
     * onModelValidate() methods of the listener's. The variable is initialized to true, and if it is
     * still true at the end of the listener iterations, then it is safe to apply the changes to the
     * model, other wise the changes are removed. 
     */
    virtual void validate();

    /**
     * clears out the model's data 
     */
    virtual void empty();

	/**
	*This macro declares a std::vector<> of ModelValidationListener listeners
	*/
	LISTENER_LIST(TableModelListener);
	
	/**
	*This macro creates a method for adding a listener to the DefaultTableModel's Item events
	*/
	ADD_LISTENER(TableModelListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTableModel's Item events
	*/
	REMOVE_LISTENER(TableModelListener);    

	NOTIFIER(TableModelListener, onChange, TableModelEvent);

	virtual void addRow();

	virtual void insertRow( const uint32& afterRow );

	virtual void deleteRow( const uint32& row );

    virtual void addColumn();

	virtual void insertColumn( const uint32& afterColumn );

	virtual void deleteColumn( const uint32& column );

	virtual bool isCellEditable( const uint32& row, const uint32& column );

	virtual TableCellItem* getItem( const uint32& row, const uint32& column );
	
	virtual uint32 getRowCount();

	virtual uint32 getColumnCount();

	virtual TableRowItemEnumerator* getRowItemEnumerator( const uint32& row );

	virtual void createCell( const uint32& row, const uint32& column, TableCellItem** createdCell );
protected:
	/**
	*defines the a vector of TableCellItem's, or one row of data m_columnCount long
	*/
	typedef std::vector<TableCellItem*> TTableColumn;
private:
	void init();
	uint32 m_rowCount;
	uint32 m_columnCount;
	/*row/column data
	*access is by row column.
	*so m_tableData[0] is row 0, and (*m_tableData[0])[2] is the item in 
	*row 0 at column 3
	*/
	std::vector<TTableColumn*> m_tableData;

	EnumeratorContainer<std::vector<TableCellItem*>,TableCellItem*> m_rowEnumContainer;
};

}; //end of namespace VCF

#endif //_DEFAULTTABLEMODEL_H__