/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef DEFAULTTREEITEM_H
#define DEFAULTTREEITEM_H

#include "Object.h"
#include <vector>
#include "TreeItem.h"
#include "TreeModel.h"
#include "ItemEvent.h"
#include "ItemListener.h"
#include "Color.h"


namespace VCF
{



#define DEFAULTTREEITEM_CLASSID		"3126B224-2819-11d4-B53A-00C04F0196DA"

class APPKIT_API DefaultTreeItem : public TreeItem {
public:

	BEGIN_CLASSINFO(DefaultTreeItem, "VCF::DefaultTreeItem", "VCF::TreeItem", DEFAULTTREEITEM_CLASSID)	
	END_CLASSINFO(DefaultTreeItem)	
	
	DefaultTreeItem( const String& caption="", TreeModel* model=NULL );

	virtual ~DefaultTreeItem();

	virtual bool containsPoint( Point * pt );

	/**
	*This macro declares a std::vector<> of ItemListener listeners
	*/
	LISTENER_LIST(ItemListener);

	/**
	*This macro creates a method for adding a listener to the DefaultTreeItem's Item events
	*/
	ADD_LISTENER(ItemListener);

	/**
	*This macro creates a method for removing a listener to the DefaultTreeItem's Item events
	*/
	REMOVE_LISTENER(ItemListener);    
    
	NOTIFIER(ItemListener, onItemPaint, ItemEvent);

	NOTIFIER(ItemListener, onItemChanged, ItemEvent);

	NOTIFIER(ItemListener, onItemSelected, ItemEvent);

	NOTIFIER(ItemListener, onItemAdded, ItemEvent);

	NOTIFIER(ItemListener, onItemDeleted, ItemEvent);

	virtual Image* getImage();

    virtual unsigned long getIndex();

    virtual void* getData();

	virtual void setData( void* data );	

    virtual Color* getTextColor();

    virtual void setTextColor(Color* color);

    virtual void setTextBold(const bool& bold);

    virtual bool getTextBold();

    virtual bool isLeaf();

	virtual bool isRoot();

    virtual TreeItem* getParent();

	virtual void setParent( TreeItem* parent );

    virtual TreeItem* getNextChildNodeItem();

    virtual TreeItem* getPrevChildNodeItem();

    virtual String getCaption();

	virtual void setCaption( const String& caption );

	virtual Enumerator<TreeItem*>* getChildren();

	virtual void init();

	virtual void addChild( TreeItem* child );

	virtual void insertChild( const unsigned long& index, TreeItem* child );
	
	virtual void deleteChild( TreeItem* child );

	virtual void deleteChild( const unsigned long& index );

	virtual void clearChildren();

	virtual Model* getModel();

	virtual void setModel( Model* model );

	virtual void setIndex( const unsigned long& index );

	virtual void paint( GraphicsContext* context, Rect* paintRect );
	
	virtual bool isSelected();

	virtual void setSelected( const bool& selected );
private:    
    bool m_textBold;
    Color m_TextColor;
	String m_caption;
	TreeItem* m_parent;
	Image* m_image;
	void* m_userData;
	TreeModel* m_treeModel;
	unsigned long m_index;
	EnumeratorContainer<std::vector<TreeItem*>,TreeItem*> m_enumContainer; 
	std::vector<TreeItem*> m_childNodeItems;
	bool m_selected;
};

};

#endif //DEFAULTTREEITEM_H