/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef FRAME_H
#define FRAME_H

#include "Container.h"
#include "Control.h"
#include "Enumerator.h"


namespace VCF{


#define FRAME_CLASSID			"ED88C0A2-26AB-11d4-B539-00C04F0196DA"
	
enum FrameStyleType {
	FST_SIZEABLE = 0,
	FST_NOBORDER,	
	FST_FIXED,
	FST_NOBORDER_FIXED
};

static String FrameStyleTypeNames[] = { "FST_SIZEABLE", 
                                         "FST_NOBORDER", 
										 "FST_FIXED",
										 "FST_NOBORDER_FIXED"	 };

/**
 * A frame is a component that houses other components, 
 *   plus has special adornments used for interacting with it. 
 *   These can include minimize, maximize, restore, system, and help buttons on the caption bar,
 * which is also part of the frame. 
 */
class APPKIT_API Frame : public VCF::Control, public AbstractContainer{
public:
	BEGIN_ABSTRACT_CLASSINFO(Frame, "VCF::Frame", "VCF::Control", FRAME_CLASSID )
	PROPERTY( String, "caption", Frame::getCaption, Frame::setCaption, PROP_STRING );
	PROPERTY( bool, "isTopmost", Frame::isFrameTopmost, Frame::setFrameTopmost, PROP_BOOL );
	LABELED_ENUM_PROPERTY( FrameStyleType, "frameStyle", Frame::getFrameStyle, Frame::setFrameStyle, 
						   FST_SIZEABLE, FST_NOBORDER_FIXED, 4, FrameStyleTypeNames);	
	END_CLASSINFO(Frame)
	virtual ~Frame(){};	
	
	Frame();

    virtual void setCaption( const String& caption );

    virtual String getCaption();

	virtual Rect* getClientBounds() = 0;
	virtual void  setClientBounds( Rect* bounds ) = 0;
	
	virtual void paint( GraphicsContext * context );

	virtual void show();

	virtual void hide();

	virtual void close() = 0;

	virtual String toString();

	virtual bool isFrameTopmost();

	virtual void setFrameTopmost( const bool& isTopmost );

	virtual FrameStyleType getFrameStyle(){
		return 	m_frameStyle;
	};

	virtual void setFrameStyle( const FrameStyleType& frameStyle ) = 0;
protected:	
	String m_caption;
	bool m_isTopmostFrame;
	FrameStyleType m_frameStyle;
private:    
    
};

}
#endif //FRAME_H