/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
// Locales.h

#ifndef _LOCALES_H__
#define _LOCALES_H__

#include "Object.h"
#include <map>

namespace VCF {

/**
*These defines are based on the ISO 3166 country codes
*/
#define LOCALE_USA				"USA"
#define LOCALE_DEFAULT			"DEFAULT"

/**
*These defines are the country number codes also from ISO 3166 country codes
*/
#define LOCALE_USA_CODE			840
#define LOCALE_DEFAULT_CODE		-1

class FRAMEWORK_API Locale : public Object {
public:
	Locale( const String& language,
			const String& displayLang,
		    const String& country,
			const int& countryCode ){
		m_country = country;
		m_language = language;
		m_countryCode = countryCode;
		m_displayLang = displayLang;
	};

	virtual ~Locale(){};	

	String getCountry(){
		return m_language;
	};

	String getLanguage() {
		return m_country;
	};

	String getDisplayLang() {
		return m_displayLang;
	};

	int getCountryCode(){
		return m_countryCode;
	};

private:
	String m_country;
	String m_language;
	String m_displayLang;
	int m_countryCode;
};


/**
*Locales is just a list on locales key by the country code
*for  more info see the following links
*<a href="http://userpage.chemie.fu-berlin.de/diverse/doc/ISO_3166.html">ISO 3166 Codes (Countries)</a>
*<a href="http://www.ics.uci.edu/pub/ietf/http/related/iso639.txt">Technical contents of ISO 639:1988 (E/F)(Countries)</a>
*<a href="ftp://dkuug.dk/i18n/iso-639-2.txt">Codes for the representation of names of languages -Part 2: Alpha-3 code</a>
*/
class FRAMEWORK_API Locales : public Object  
{
public:
	Locales();	

	virtual ~Locales();

	static Locale* getLocale( const String& country );
private:
	std::map<String,Locale*> m_countryMap;

	static Locales localListing;	
};

};

#endif //_LOCALES_H__
