/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef OBJECT_H
#define OBJECT_H

#pragma warning (disable : 4786)


#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <string.h>
#include "FrameworkConfig.h"
#include "VCFString.h"
#include <list>
#include <map>
#ifdef USE_GARBAGE_COLLECTION
#include "gc_cpp.h"
#endif


#ifdef WIN32
#include <ostream>
#pragma warning (disable : 4786)
#endif




namespace VCF{


class Class;
class Listener;

#define OBJECT_CLASSID		"ED88C099-26AB-11d4-B539-00C04F0196DA"

/**
*Base class for entire Visual Component Framework 
*/
class FRAMEWORK_API Object {
public:    
	
	Object();
	virtual ~Object();
    /**
     * returns a string representation of the object 
     */
    virtual String toString();

    /**
     * returns the class name of the object. 
     */
    String getClassName();

	
    virtual void init();
	
	virtual bool isEqual( Object* object ){
		return ( object == this );
	};

	virtual void copy( Object* source ){};

	Class* getClass();

	/**
	*This is a utility method for adding event handlers to an object.
	*Event handlers are simple implementations of a specific Listener interface
	*Adding a handler provides a convenient way to store the various events
	*an object may wish to listen to. Handlers still need to have their method 
	*callbacks set, otherwise nothing will happen. When the object is destroyed
	*all the event handlers in this list are also destroyed.
	*/
	void addEventHandler( const String& handlerName, Listener* handler );

	/**
	*Finds a Listener handler for a given name. If the object has that 
	*handler then a pointer to the is returned otherwise null
	*is returned
	*/
	Listener* getEventHandler( const String& handlerName );

#ifdef USE_VCF_NEW
	
	void* operator new( size_t allocSize );
	
	void operator delete( void *memPtr );	

	static void dump();

private:
	static std::list<void*> allocatedObjects; 
#endif

private:
	std::map<String,Listener*> m_eventHandlers;
};

};

#ifdef WIN32

namespace VCFUtility {
    std::ostream& redirectCerr(std::string& fileName);
    std::ostream& redirectCout(std::string& fileName);
    std::ostream& redirect(std::ostream& str, std::string& fname) ;
    //void closeCerr();
    //void closeCout();
};

#endif 

#endif //OBJECT_H__