/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//PropertyEditor.h

#ifndef _PROPERTY_EDITOR_H__
#define _PROPERTY_EDITOR_H__

#include "VCFString.h"
#include "Rect.h"
namespace VCF
{

class Property;
class Control;
class GraphicsContext;

/**
*PropertyEditor
*/
class APPKIT_API PropertyEditor{
public:
	virtual ~PropertyEditor(){};

	/**
	*returns a new instance of the custom editor. The caller
	*is resposible for deleting the editor when it is finished 
	*with it.
	*Implementer of this interface should always make a new instance 
	*each time this is called.
	*/
	virtual Control* getCustomEditor() = 0;
	
	virtual void paintValue( GraphicsContext* context, const Rect& bounds ) = 0;

	/**
	*Sets the source for the Property editor
	*/
	virtual void setSource( Object* source ) = 0;

	/**
	*Sets the property of the Property editor
	*/
	virtual void setProperty( Property* property ) = 0;

	virtual VariantData* getValue() = 0;
	
	virtual void setValue( VariantData* value ) = 0;

	/**
	*is it OK to call getCustomEditor ?
	*/
	virtual bool hasCustomEditor() = 0;
	
	/**
	*is OK to call paintValue ?
	*/
	virtual bool canPaintValue() = 0;

	virtual String getValueAsText() = 0;

	virtual void setValueAsText( const String& textValue ) = 0;

	/**
	*called by an IDE to get the C++ string representation of 
	the property to insert into C++ code.
	*/
	virtual String getCodeString() = 0;
	
};


};

#endif