/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// PropertyEditorManager.h

#ifndef _PROPERTYEDITORMANAGER_H__
#define _PROPERTYEDITORMANAGER_H__


#include "Object.h"
#include "PropertyEditor.h"
#include <map>
namespace VCF
{


class APPKIT_API PropertyEditorManager : public Object  
{
public:
	PropertyEditorManager();

	virtual ~PropertyEditorManager();

	/**
	*find a suitable property editor for a given 
	*classname.
	*/
	static PropertyEditor* findEditor( const String& className );	
	
	/**
	*register a property editor for a given class name
	*/
	static void registerPropertyEditor( PropertyEditor* editor, const String& className );

	/**
	*initializes the Property editor for use - must be called at start up
	*/
	static void initPropertyEditorManager();

	/**
	*frees any memory used by initPropertyEditorManager(). Must be called before the 
	*runtime shuts down
	*/
	static void closePropertyEditorManager();
private:
	static PropertyEditorManager propertyEditorMgr;
	std::map<String,PropertyEditor*> m_propertEditorMap;

};


};

#endif 
