/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#ifndef TABLECONTROL_H
#define TABLECONTROL_H

#include "Container.h"
#include "CustomControl.h"
#include <map>
#include <vector>

namespace VCF{

#define DEFAULT_ROW_HEIGHT			20
#define DEFAULT_COLUMN_WIDTH		80

#define TABLECONTROL_CLASSID		"AC704858-753B-46a7-815B-72C09C884164"

class TableModel;
class TableModelEvent;
class TableCellItem;
class ItemHandler;
class ItemEvent;
class TableItemEditor;

class APPKIT_API TableControl : public CustomControl, public AbstractContainer{
public:
	BEGIN_CLASSINFO(TableControl, "VCF::TableControl", "VCF::CustomControl", TABLECONTROL_CLASSID );
	END_CLASSINFO(TableControl);

	TableControl( TableModel* model=NULL );

	virtual ~TableControl();

    virtual void paint( GraphicsContext * context );

	void init();

	uint32 getDefaultRowHeight();

	void setDefaultRowHeight( const uint32& defaultRowHeight );

	uint32 getDefaultColumnWidth();

	void setDefaultColumnWidth( const uint32& defaultColumnWidth );

	void setFixedColumn( const bool& isFixed, const uint32& col );

	void setFixedRow( const bool& isFixed, const uint32& row );
	
	void setColumnWidth( const uint32& column, const uint32& width );

	TableModel* getTableModel();

	void setTableModel( TableModel* model );

	void setColumnCount( const uint32& colCount );

	void setRowCount( const uint32& rowCount );

	TableCellItem* getSelectedItem(){
		return m_selectedCellItem;
	}

	void setSelectedItem( TableCellItem* selectedCellItem ){
		m_selectedCellItem = selectedCellItem;
	}

	virtual void mouseDown( MouseEvent* event );

	virtual void mouseMove( MouseEvent* event );

	virtual void mouseUp( MouseEvent* event );

	virtual void mouseDblClick(  MouseEvent* event );

	void OnMouseDblClick( MouseEvent* event );

	virtual void setBounds( Rect* rect ) throw( InvalidPeer );
	
	void setAutoResizeColumns( const bool& autoResizeColumns );

	bool getAutoResizeColumns(){
		return m_autoResizeColumns;
	}

protected:
	void onTableModelChanged( TableModelEvent* event );
	void onTableCellItemSelected( ItemEvent* event );
private:
	std::vector<uint32> m_columnWidths;
	std::vector<uint32> m_rowHeights;
	TableModel* m_model;
	uint32 m_defaultColumnWidth;
	uint32 m_defaultRowHeight;
	TableCellItem* m_selectedCellItem;
	ItemHandler* m_tabelCellHandler;
	Control* m_currentEditingControl;
	TableItemEditor* m_currentItemEditor;
	bool m_autoResizeColumns;
	void resetColumnWidths();
};

};
#endif //TABLECONTROL_H