/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _TABLEITEMEDITOR_H__
#define _TABLEITEMEDITOR_H__

#include "Object.h"
#include "ClassInfo.h"

namespace VCF {

class Control;

class ItemEditorListener;

class TableCellItem;

#define TABLEITEMEDITOR_CLASSID		"53EA0BA4-7068-11d4-8F12-00207811CFAB"

class TableItemEditor : public Object{
public:
	BEGIN_ABSTRACT_CLASSINFO(TableItemEditor, "VCF::TableItemEditor", "VCF::Object", TABLEITEMEDITOR_CLASSID)
	END_CLASSINFO(TableItemEditor)

	TableItemEditor(){};

	virtual ~TableItemEditor(){};
	
	virtual bool isCellEditable() = 0;

	virtual void addItemEditorListener( ItemEditorListener* listener ) = 0;

	virtual void removeItemEditorListener( ItemEditorListener* listener ) = 0;

	/**
	*this is called to initialize the table item editor with some value
	*from the TableCellItem, typically the caption.
	*/
	virtual void setItemToEdit( TableCellItem* itemToEdit ) = 0;

	virtual TableCellItem* getItemToEdit() = 0;

	/**
	*updates the item passed in during the setItemToEdit() call, to 
	*change to whatever new value has been set in the editor.
	*prior to the data transfer, all ItemEditorListeners should have their
	*ItemEditorListener::onValidateChange() method called. Implementers of
	*ItemEditorListener::onValidateChange() should throw an exception if 
	*invalid data is detected. This can done by simply calling the 
	*ItemEditrEvent's fireInvalidStateException() method.
	*the data is presumed to be stored in the Event's setUserData/getUserData
	*calls.
	*/
	virtual void updateItem() = 0;

	virtual Control* getEditingControl() = 0;
};

}; //end of namespace VCF

#endif //_TABLEITEMEDITOR_H__