/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 




#ifndef TEXTMODEL_H
#define TEXTMODEL_H


#include "Model.h"
#include "TextModelListener.h"

namespace VCF{

#define TEXTMODEL_CLASSID	"8B2CDC31-3CAD-11d4-B553-00C04F0196DA"

/** @interface */
/**
*TextModel is a model for working with text data.
*
*@version 1.0
*@author Jim Crafton
*/
class APPKIT_API TextModel : public Model {
public:

	BEGIN_ABSTRACT_CLASSINFO(TextModel, "VCF::TextModel", "VCF::Model", TEXTMODEL_CLASSID)	
	END_CLASSINFO(TextModel)

	TextModel(){
		
	};

	virtual ~TextModel(){};

	/**
	*adds a new TextModel listener
	*/
    virtual void addTextModelListener( TextModelListener * listener ) = 0;

	/**
	*removes a TextModel listener
	*/
    virtual void removeTextModelListener( TextModelListener * listener ) = 0;

	/**
	*sets the contents of the text model, completely changes what was previously 
	*in the model
	*/
    virtual void setText( const String& text ) = 0;

	/**
	*inserts text into the model at the given index
	*@param long the starting point to begin inserting the text. This number represents
	*a zero based index.
	*@param String the text to insert
	*/
    virtual void insertText( const unsigned long& index, const String& text ) = 0;

	/**
	*deletes text from the model, starting at index, and continuing for count characters, 
	*or until the text data is empty.
	*@param long the starting point. The index is zero based.
	*@param long the number of characters to delete
	*/
    virtual void deleteText( const unsigned long& index, const unsigned long& count ) = 0;

	/**
	*adds text to end of the current text data
	*/
    virtual void appendText( const String& text ) = 0;

	/**
	*returns all of the TextModel's text in a string.
	*/
	virtual String getText() = 0;

	/**
	*returns the size of the TextModel
	*/
	virtual unsigned long getSize() = 0;

};

};
#endif //TEXTMODEL_HTMODEL_H