/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _TOOLBARDOCK_H__
#define _TOOLBARDOCK_H__


namespace VCF {

enum ToolbarDockSide {
	TDS_TOP=0,
	TDS_LEFT,
	TDS_RIGHT,
	TDS_BOTTOM
};

static String ToolbarDockSideNames[] = { "TDS_TOP", 
                                         "TDS_LEFT", 
										 "TDS_RIGHT", 
										 "TDS_BOTTOM" };

enum ToolbarBackgroundStyle {
	TBKS_NONE=0,
	TBKS_TILE,
	TBKS_STRETCH
};


static String ToolbarBackgroundStyleNames[] = { "TBKS_NONE", 
											"TBKS_TILE", 
											"TBKS_STRETCH" };



class Toolbar;

#define TOOLBARDOCK_CLASSID			"54AE13E1-113D-4dcf-90F9-18FBBF713179"


class ToolbarDock : public CustomControl, public AbstractContainer {
public:
	
	BEGIN_CLASSINFO(ToolbarDock, "VCF::ToolbarDock", "VCF::CustomControl", TOOLBARDOCK_CLASSID);
	LABELED_ENUM_PROPERTY( ToolbarDockSide, "dockSide", ToolbarDock::getDockSide, ToolbarDock::setDockSide, 
						   TDS_TOP, TDS_BOTTOM, 4, ToolbarDockSideNames);	

	LABELED_ENUM_PROPERTY( ToolbarBackgroundStyle, "backgroundStyle", ToolbarDock::getBackgroundStyle, ToolbarDock::setBackgroundStyle, 
						   TBKS_NONE, TBKS_STRETCH, 3, ToolbarBackgroundStyleNames);	
	
	OBJECT_PROPERTY( Image, "background", ToolbarDock::getBackground, ToolbarDock::setBackground );
	
	PROPERTY( bool, "usingBackground", ToolbarDock::isUsingBackground, ToolbarDock::setUsingBackground, PROP_BOOL );	
	PROPERTY( bool, "allowsDragging", ToolbarDock::allowsDragging, ToolbarDock::setAllowsDragging, PROP_BOOL );	

	END_CLASSINFO(ToolbarDock);

	ToolbarDock();

	virtual ~ToolbarDock();
	

	/**
	*add overriden to prevent the adding of non-Toolbar Controls - this is 
	*the only type if control the ToolbarDock will allow - anythign will
	*cause an exception to be thrown
	*/
	virtual void add( Control * child );

	virtual void add( Control * child, const AlignmentType& alignment );
	
	ToolbarDockSide getDockSide();

	void setDockSide( const ToolbarDockSide& dockSide );

	Image* getBackground();

	void setBackground( Image* background );

	ToolbarBackgroundStyle getBackgroundStyle();

	void setBackgroundStyle( const ToolbarBackgroundStyle& backgroundStyle );

	bool isUsingBackground();

	void setUsingBackground( const bool& usingBackground );

	bool allowsDragging();

	void setAllowsDragging( const bool& allowsDrag );

	virtual void paint( GraphicsContext* context );

	virtual void mouseDown( MouseEvent* event );
	
	virtual void mouseMove( MouseEvent* event );

	virtual void mouseUp( MouseEvent* event );

	void addToolbar( Toolbar* toolbar );
protected:
	ToolbarDockSide m_dockSide;
	ToolbarBackgroundStyle m_backgroundStyle;
	Image* m_backgroundImage;
	bool m_isUsingBackground;
	bool m_allowsDockDragging;
	std::vector<Toolbar*> m_toolbars;

	Toolbar* findToolbar( Point* pt );
};

}; //end of namespace VCF

#endif //_TOOLBARDOCK_H__