/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#ifndef WINDOW_H
#define WINDOW_H

#include "Frame.h"
#include "WindowPeer.h"
#include "WindowListener.h"
#include "MenuBar.h"


namespace VCF
{

/**
 * A frame with min. max, restore, and system buttons. 
 * Has a caption of the standard height. 
 */

#define WINDOW_CLASSID				"ED88C0A1-26AB-11d4-B539-00C04F0196DA"



class APPKIT_API Window : public Frame {
public:

	BEGIN_CLASSINFO(Window, "VCF::Window", "VCF::Frame", WINDOW_CLASSID )
	OBJECT_PROPERTY( MenuBar, "menuBar", Window::getMenuBar, Window::setMenuBar );	
	END_CLASSINFO(Window)

	Window();
	virtual ~Window();

	virtual void setCaption( const String& caption );

	virtual void paint(GraphicsContext * context);

	/**
	*overridden to take into account client bounds
	**/
	virtual void resizeChildren();

	virtual Rect* getClientBounds();

	virtual void  setClientBounds( Rect* bounds );

	MenuBar* getMenuBar();

	void setMenuBar( MenuBar* menuBar );

	/**
	*This macro declares a std::vector<> of WindowListener listeners
	*/
	LISTENER_LIST(WindowListener);

	/**
	*This macro creates a method for adding a listener to the Window's Window events
	*/
	ADD_LISTENER(WindowListener);

	/**
	*This macro creates a method for removing a listener to the Window's Window events
	*/
	REMOVE_LISTENER(WindowListener);

	/**
	*macro adds event notifier methods. These are equivalent to the fireXXX methods
	*of java. The macro expands to code that automatically calls all the listeners and
	*fires off the appropriate listener method.
	*/
	/**
	*Mouse events
	*/
	NOTIFIER(WindowListener, onWindowClose, WindowEvent);    

	NOTIFIER(WindowListener, onWindowRestore, WindowEvent);

	NOTIFIER(WindowListener, onWindowMaximize, WindowEvent);    
	
	NOTIFIER(WindowListener, onWindowMinimize, WindowEvent);

	virtual void beforeDestroy( ComponentEvent* event );

	virtual void close();

	virtual void setFrameStyle( const FrameStyleType& frameStyle );

	virtual void setFrameTopmost( const bool& isTopmost );
private:    
    WindowPeer * m_windowPeer;	
	MenuBar* m_menuBar;
};

};
#endif //WINDOW_H_H