/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef DRAGSOURCE_H
#define DRAGSOURCE_H

#include "Object.h"
#include "DragSourceListener.h"
#include "DragEvent.h"

#include <vector>
namespace VCF{


class DragDropPeer;

class ClipboardDataObject;


class APPKIT_API DragSource : public Object {

public:
	DragSource();
	virtual ~DragSource();    	

	void startDragDrop( ClipboardDataObject* data );

    void copyDataToClipboard();

	void setActionType( const DragActionType& actionType );

	DragActionType getActionType();

	/**
	*This macro declares a std::vector<> of listeners
	*/
	LISTENER_LIST(DragSourceListener);
	
	/**
	*This macro creates a method for adding a listener to the DragSource
	*/
	ADD_LISTENER(DragSourceListener);

	/**
	*This macro creates a method for removing a listener to the DragSource
	*/
	REMOVE_LISTENER(DragSourceListener);
	
	/**
	*macro adds event notifier methods. These are equivalent to the fireXXX methods
	*of java. The macro expands to code that automatically calls all the listeners and
	*fires off the appropriate listener method.
	*/
	NOTIFIER(DragSourceListener, onDragSourceBegin, DragEvent);    

	NOTIFIER(DragSourceListener, onDragSourceCanContinueDragOp, DragEvent);

	NOTIFIER(DragSourceListener, onDragSourceGiveFeedback, DragEvent);    
	
	NOTIFIER(DragSourceListener, onDragSourceDropped, DragEvent);

	NOTIFIER(DragSourceListener, onDragSourceEnd, DragEvent);

private:    

    /**
     * @supplierCardinality 1 
     */
	DragActionType m_actionType;
    DragDropPeer * m_dragDropPeer;

    /**
     * @supplierCardinality 1 
     */
    ClipboardDataObject * m_clipboardDataObject;
};

};



#endif //DRAGSOURCE_H