/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef DRAGSOURCELISTENER_H
#define DRAGSOURCELISTENER_H

#include "Listener.h"
#include "DragEvent.h"


namespace VCF{

#define DRAGSOURCE_EVENT_DRAG_BEGUN		CUSTOM_EVENT_TYPES + 300
#define DRAGSOURCE_EVENT_DRAG_DROPPED	CUSTOM_EVENT_TYPES + 301
#define DRAGSOURCE_EVENT_DRAG_END		CUSTOM_EVENT_TYPES + 302


/** @interface */
class APPKIT_API DragSourceListener : public Listener {
public:
    virtual void onDragSourceBegin( DragEvent * event ) = 0;
    
    virtual void onDragSourceDropped( DragEvent * event ) = 0;

    virtual void onDragSourceEnd( DragEvent * event ) = 0;

    virtual void onDragSourceGiveFeedback( DragEvent* event ) = 0;

    virtual void onDragSourceCanContinueDragOp( DragEvent* event ) = 0;

};


typedef void (Object::* OnDragEvent)( DragEvent* event );


/**
*DragSourceAdapter is an adapter for DragSourceListener
*/
class APPKIT_API DragSourceHandler : public DragSourceListener, public Object {
public:
	
	DragSourceHandler( Object* source ){
		m_source = source;
		this->m_dragSourceBegin = NULL;
		this->m_dragSourceDropped = NULL;
		this->m_dragSourceEnd = NULL;
		this->m_dragSourceGiveFeedback = NULL;
		this->m_dragSourceCanContinueDragOp = NULL;
	};

	virtual ~DragSourceHandler(){};

	virtual void onDragSourceBegin( DragEvent * event ){
		if ( NULL != m_dragSourceBegin ){
			(m_source->*m_dragSourceBegin)( event );
		}
	};
    
    virtual void onDragSourceDropped( DragEvent * event ){
		if ( NULL != m_dragSourceDropped ){
			(m_source->*m_dragSourceDropped)( event );
		}
	};

    virtual void onDragSourceEnd( DragEvent * event ){
		if ( NULL != m_dragSourceEnd ){
			(m_source->*m_dragSourceEnd)( event );
		}
	};

    virtual void onDragSourceGiveFeedback( DragEvent* event ){
		if ( NULL != m_dragSourceGiveFeedback ){
			(m_source->*m_dragSourceGiveFeedback)( event );
		}
	};

    virtual void onDragSourceCanContinueDragOp( DragEvent* event ){
		if ( NULL != m_dragSourceCanContinueDragOp ){
			(m_source->*m_dragSourceCanContinueDragOp)( event );
		}
	};

	OnDragEvent m_dragSourceBegin;
	OnDragEvent m_dragSourceDropped;
	OnDragEvent m_dragSourceEnd;
	OnDragEvent m_dragSourceGiveFeedback;
	OnDragEvent m_dragSourceCanContinueDragOp;

private:
	 Object* m_source;
};

};
#endif //DRAGSOURCELISTENER_H