/**
*Visual Control Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Control Framework
*/ 
//ControlListener.h

#ifndef _CONTROL_LISTENER_H__
#define _CONTROL_LISTENER_H__

#include "Listener.h"
#include "ControlEvent.h"

namespace VCF {

class APPKIT_API ControlListener : Listener {
public:
	virtual ~ControlListener(){};

	virtual void onControlSized( ControlEvent* event ) = 0;

    virtual void onControlPositioned( ControlEvent* event ) = 0;
    
    virtual void onControlParentChanged( ControlEvent* event ) = 0;

};


typedef void (Object::* OnControlEvent)( ControlEvent* event );


class APPKIT_API ControlHandler : public ControlListener, public Object {
public:
	ControlHandler( Object* source ){
		m_source = source;
		m_controlSized = NULL;
		m_controlPositioned = NULL;		
		m_controlParentChanged = NULL;
	};
	
	virtual ~ControlHandler(){};

	virtual void onControlSized( ControlEvent* event ){
		if ( NULL != m_controlSized ){
			(m_source->*m_controlSized)( event );
		}	
	};

    virtual void onControlPositioned( ControlEvent* event ){
		if ( NULL != m_controlPositioned ){
			(m_source->*m_controlPositioned)( event );
		}
	};

    virtual void onControlParentChanged( ControlEvent* event ){
		if ( NULL != m_controlParentChanged ){
			(m_source->*m_controlParentChanged)( event );
		}
	};


	OnControlEvent m_controlSized;
	OnControlEvent m_controlPositioned;	
	OnControlEvent m_controlParentChanged;
private:
	Object* m_source;
};



};



#endif //_CONTROL_LISTENER_H__