/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef EVENT_H
#define EVENT_H

#include "Object.h"
#include <time.h>
#include "CommonDefines.h"

namespace VCF{

#define CUSTOM_EVENT_TYPES	500


/**
*The Event class is root object for events in the Framework. Events are defined as any action that is generated
*in response to some change in the system. Moving the mouse, typing on the keyboard, etc are all example of Window
*System events that are generated by the User, but events can also be generated by an objects change in state.
*For example, a text model can fire events if the state of it's text has changed.
*
*@version 1.0
*@author Jim Crafton
*/
class FRAMEWORK_API Event : public Object {

public:    

    Event( Object * source);
	Event( Object* source, const unsigned long& eventType );
	
	virtual void init();
	
	/**
	*shows whether or not the event has been consumed.
	*primarily used by the Listener list implementation
	*/
	bool m_consumed;

	virtual void consume() { m_consumed = true; }

	bool getConsumed() {        
        return m_consumed;
    }

	void setConsumed( const bool& isConsumed ){
		m_consumed = isConsumed;
	}

	bool isConsumed() { 
		return m_consumed; 
	}

	/**
	*retrieves the events source, in other words, who actually caused the
	*event to be generated
	*/
    Object* getSource();

	/**
	*gets the type of event. Some events may not put anything useful here, but events
	*associated with Listeners that have multiple methods (like MouseListener) will
	*definitely store a flag here. See the specific event class for the exact event types
	*the event types can be set to
	*/
    unsigned long getType();

	void setType( const unsigned long& type );

	/**
	*gets the time the event was created.  
	*/
    time_t getTime();
	
	/*
	*sets the user data. User data is simply a place to store any sort of user specific info
	*that the programmer wants to attach to the event object. Only used by the specific application,
	*the Framework ignores this.
	*/
	void setUserData( void* userData );

	void* getUserData();

private:
    Object* m_source;
	void* m_userData;
    unsigned long m_eventType;
    time_t m_time;
};

};
#endif //EVENT_H