/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ImageListListener.h

#if !defined(_IMAGELISTLISTENER_H__)
#define _IMAGELISTLISTENER_H__


#include "Listener.h"
#include "ImageListEvent.h"

namespace VCF
{

class ImageListListener : public Listener  
{
public:
	ImageListListener();

	virtual ~ImageListListener();

	virtual void onImageAdded( ImageListEvent* event ) = 0;

	virtual void onImageDeleted( ImageListEvent* event ) = 0;

	virtual void onImageListCreated( ImageListEvent* event ) = 0;

	virtual void onImageListDestroyed( ImageListEvent* event ) = 0;

	virtual void onImageListSizeChanged( ImageListEvent* event ) = 0;

};

typedef void (Object::*OnImageAdded)( ImageListEvent* event );
typedef void (Object::*OnImageDeleted)( ImageListEvent* event );
typedef void (Object::*OnImageListCreated)( ImageListEvent* event );
typedef void (Object::*OnImageListDestroyed)( ImageListEvent* event );
typedef void (Object::*OnImageListSizeChanged)( ImageListEvent* event );

class ImageListHandler : public Object, public ImageListListener {
public:
	ImageListHandler( Object* source ){
		m_source = source;

		m_imageAdded = NULL;
		m_imageDeleted = NULL;
		m_imageListCreated = NULL;
		m_imageListDestroyed = NULL;
		m_imageListSizeChanged = NULL;
	};

	virtual ~ImageListHandler(){};

	virtual void onImageAdded( ImageListEvent* event ){
		if ( (NULL != m_source) && (NULL != m_imageAdded) ){
			(m_source->*m_imageAdded)( event );
		}
	};

	virtual void onImageDeleted( ImageListEvent* event ){
		if ( (NULL != m_source) && (NULL != m_imageDeleted) ){
			(m_source->*m_imageDeleted)( event );
		}
	};

	virtual void onImageListCreated( ImageListEvent* event ){
		if ( (NULL != m_source) && (NULL != m_imageListCreated) ){
			(m_source->*m_imageListCreated)( event );
		}
	};

	virtual void onImageListDestroyed( ImageListEvent* event ){
		if ( (NULL != m_source) && (NULL != m_imageListDestroyed) ){
			(m_source->*m_imageListDestroyed)( event );
		}
	};

	virtual void onImageListSizeChanged( ImageListEvent* event ){
		if ( (NULL != m_source) && (NULL != m_imageListSizeChanged) ){
			(m_source->*m_imageListSizeChanged)( event );
		}
	};

	OnImageAdded m_imageAdded;
	OnImageDeleted m_imageDeleted;
	OnImageListCreated m_imageListCreated;
	OnImageListDestroyed m_imageListDestroyed;
	OnImageListSizeChanged m_imageListSizeChanged;

private:
	Object* m_source;
};

};

#endif 
