/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _ITEMEDITOREVENT_H__
#define _ITEMEDITOREVENT_H__

#include "Event.h"
#include "InvalidStateException.h"

namespace VCF {

#define ITEMEDITOR_EVENT_CONST				3000

#define ITEMEDITOR_VALIDATION				CUSTOM_EVENT_TYPES + ITEMEDITOR_EVENT_CONST + 1
#define ITEMEDITOR_CHANGED					CUSTOM_EVENT_TYPES + ITEMEDITOR_EVENT_CONST + 2

class TableCellItem;


class ItemEditorEvent : public Event {
public:
	ItemEditorEvent( Object* source ) : 
		Event( source ){
	}

	ItemEditorEvent( Object* source, void* dataToValidate ) : 
		Event( source, ITEMEDITOR_VALIDATION ){
		setUserData( dataToValidate );
	}
	ItemEditorEvent( Object* source, const unsigned long& eventType  ) : 
		Event( source, eventType ){
	}

	virtual ~ItemEditorEvent(){
	 
	}

	void fireInvalidStateException(){
		throw InvalidStateException(MAKE_ERROR_MSG(BAD_ITEMSTATE_EXCEPTION), __LINE__);
	}

	TableCellItem* getItemBeingEdited(){
		return m_itemBeingEdited;
	}

	void setItemBeingEdited( TableCellItem* itemBeingEdited ){
		m_itemBeingEdited = itemBeingEdited;
	}

private:
	TableCellItem* m_itemBeingEdited;
};

}; //end of namespace VCF

#endif //_ITEMEDITOREVENT_H__