/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _ITEMEDITORLISTENER_H__
#define _ITEMEDITORLISTENER_H__


#include "Listener.h"

namespace VCF {

class ItemEditorEvent;

/**
*ItemEditorListener
*/
class APPKIT_API ItemEditorListener : public Listener{

public:

	virtual ~ItemEditorListener(){};

	virtual void onChange( ItemEditorEvent* event ) = 0;

	virtual void onValidateChange( ItemEditorEvent* event ) = 0;
};


typedef void (Object::* OnItemEditorEvent)( ItemEditorEvent* event );


class APPKIT_API ItemEditorEventAdapter : public Object, public ItemEditorListener {
public:
	ItemEditorEventAdapter( Object* source, 
		                    OnItemEditorEvent change=NULL, 
							OnItemEditorEvent validateChange=NULL ){
		m_source = source;
		m_change = change;
		m_validateChange = validateChange;
	}
	
	virtual ~ItemEditorEventAdapter(){}

	virtual void onChange( ItemEditorEvent* event ){
		if ( (NULL != m_source) && (NULL != m_change) ){
			(m_source->*m_change)( event );
		}
	}

	virtual void onValidateChange( ItemEditorEvent* event ){
		if ( (NULL != m_source) && (NULL != m_validateChange) ){
			(m_source->*m_validateChange)( event );
		}
	}

	OnItemEditorEvent m_change;

	OnItemEditorEvent m_validateChange;
	Object* m_source;
};

}; //end of namespace VCF

#endif //_ITEMEDITORLISTENER_H__