/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//MenuItemListener.h

#include "Listener.h"
#include "MenuItemEvent.h"

namespace VCF
{

class APPKIT_API MenuItemListener : public Listener {
public:
	virtual ~MenuItemListener(){};
	
	virtual void onMenuItemClicked( MenuItemEvent* event ) = 0;

	virtual void onMenuItemUpdate( MenuItemEvent* event ) = 0;
};


typedef void (Object:: *OnMenuItemEvent)( MenuItemEvent* event );


class APPKIT_API MenuItemHandler : public Object, public MenuItemListener {
public:

	MenuItemHandler( Object* source, OnMenuItemEvent itemClicked=NULL, OnMenuItemEvent itemUpdate=NULL ){
		m_source = source;
		m_menuItemClicked = itemClicked;
		m_menuItemUpdate = itemUpdate;
	};

	virtual ~MenuItemHandler(){};


	virtual void onMenuItemClicked( MenuItemEvent* event ){
		if ( (NULL != m_source) && (NULL != m_menuItemClicked) ){
			(m_source->*m_menuItemClicked)( event );
		}
	};

	virtual void onMenuItemUpdate( MenuItemEvent* event ){
		if ( (NULL != m_source) && (NULL != m_menuItemUpdate) ){
			(m_source->*m_menuItemUpdate)( event );
		}
	};

	OnMenuItemEvent m_menuItemClicked;

	OnMenuItemEvent m_menuItemUpdate;

private:
	Object* m_source;
};

};