/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _TABLEMODELEVENT_H__
#define _TABLEMODELEVENT_H__

#include "VCF.h"
#include "Event.h"

namespace VCF {

#define TABLEMODEL_EVENT_CONST						2000
#define COLUMN_DELETED					CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 1
#define COLUMN_ADDED					CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 2
#define ROW_DELETED						CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 3
#define ROW_ADDED						CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 4
#define CELL_CHANGED					CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 5
#define ALL_COLUMNS_CHANGED				CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 6	
#define ALL_ROWS_CHANGED				CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 7	
#define ALL_ITEMS_DELETED				CUSTOM_EVENT_TYPES + TABLEMODEL_EVENT_CONST + 8

#define NO_ROW_CHANGED					-1
#define NO_COLUMN_CHANGED				-1


class APPKIT_API TableModelEvent : public Event {
public:

	TableModelEvent( Object* source, const ulong32& eventType,
		             const int& rowThatChanged =-1,
					 const int& columnThatChanged =-1 );

	virtual ~TableModelEvent();

	/**
	*get the row that changed. a return of -1 means no rows
	*changed for this event
	*/
	int getRowThatChanged();

	/**
	*get the column that changed. a return of -1 means no columns
	*changed for this event
	*/
	int getColumnThatChanged();

private:
	int m_rowThatChanged;
	int m_columnThatChanged;
};

}; //end of namespace VCF

#endif //_TABLEMODELEVENT_H__