/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 


#ifndef BASICEXCEPTION_H
#define BASICEXCEPTION_H
#include "VCFString.h"
#include "FrameworkConfig.h"
#include "ErrorStrings.h"
#include <exception>


namespace VCF{

/**
*The base exception class in the Framework. 
*All exceptions should try and describe what went wrong as clearly as possible.
*For convenience a basic set of error strings are defined in ErrorStrings.h.
*
*@version 1.0
*@author Jim Crafton
*/
class FRAMEWORK_API BasicException : public std::exception {
public:

	/**
	*Basic constructor
	*All Exceptions take a text message identifying (in human readable terms ) what
	*went wrong. 
	*/
	BasicException( const VCF::String & message );
	
	BasicException( const VCF::String & message, const int lineNumber );

	virtual ~BasicException(){};

	/**
	*returns the message text associated with the exception
	*@return String - the error text
	*/
    VCF::String getMessage();  

private:
    VCF::String m_message;
};

};

#endif //BASICEXCEPTION_HXCEPTION_H