/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef ABSTRACTIMAGE_H
#define ABSTRACTIMAGE_H

#include "VCF.h"
#include "Image.h"
#include "ImageListener.h"
#include <vector>

namespace VCF{

class ImageBits;
class GraphicsContext;

/**
*AbstractImage represents a base implementation of the Image interface. It implements
*common functions, such as getWidth, etc, but still requires actually image class GRAPHICSKIT_API to be 
*derived from it. It also provides basic support for ImageListeners, so derived classes 
*do not have to baother with it. See Image for more information on what the functions do.
*
*@version 1.0
*@author Jim Crafton
*/
class GRAPHICSKIT_API AbstractImage : public Image , public Object{
public:

	AbstractImage();

	virtual ~AbstractImage();

	virtual ImageBitDepth getBitDepth();    

    virtual void setWidth(const unsigned long & width);

    virtual unsigned long getWidth();

    virtual void setHeight(const unsigned long & height);

    virtual unsigned long getHeight();

    //virtual void addImageListener( ImageListener* listener );

    //virtual void removeImageListener( ImageListener* listener );

	LISTENER_LIST(ImageListener);

	/**
	*This macro creates a method for adding a listener to the AbstractImage's ImageListener events
	*/
	ADD_LISTENER(ImageListener);

	/**
	*This macro creates a method for removing a listener to the AbstractImage's ImageListener events
	*/
	REMOVE_LISTENER(ImageListener);

	NOTIFIER(ImageListener, onSizeChanged, ImageEvent);    

    GraphicsContext* getImageContext();

	virtual ImageBits* getImageBits();
protected:
	ImageBits* m_imageBits;
	int m_height;
    int m_width;
	GraphicsContext * m_context;
private:    	
	
	ImageBitDepth m_bitDepth;
};

};
#endif //ABSTRACTIMAGE_H