/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Basic3DBorder.h


#ifndef _BASIC3DBORDER_H__
#define _BASIC3DBORDER_H__


#include "Border.h"
#include "Object.h"

namespace VCF
{


#define BASIC3DBORDER_CLASSID			"3126B222-2819-11d4-B53A-00C04F0196DA"

/**
*Draws a basic 3D border around a component. The border is 2 pixels thick
*
*@version 1.0
*@author Jim Crafton
*/
class APPKIT_API Basic3DBorder : public Border {

public:
	BEGIN_CLASSINFO(Basic3DBorder, "VCF::Basic3DBorder", "VCF::Border", BASIC3DBORDER_CLASSID)
	END_CLASSINFO(Basic3DBorder)

	Basic3DBorder();

	virtual ~Basic3DBorder();

	/**
	*Paint the border on context
	*/
	virtual void paint( Control* control, GraphicsContext* context );

	virtual void paint( Rect* bounds, GraphicsContext* context );

	/**
	*returns a client rect in which contained components may live.
	*Thus if a particular implementation required a 3 pixel
	*border at the edges to draw itself, then it would retrieve the
	*component's border and then reduce it by 3 pixels. This new
	*value is the available area that the component could use to
	*place child components
	*
	*@param Control* - component that the border belongs to 
	*
	*@return Rect* the client rectangle 
	*/
	virtual Rect* getClientRect( Control* component );

};

};

#endif
