// bitmap.h: interface for the Bitmap class.
//
//	Creator : El Barto (ef00@luc.ac.be)
//	Location : http://www.luc.ac.be/~ef00/ebgfx
//	Date : 09-04-98
//////////////////////////////////////////////////////////////////////
//  Modified for use with SimpelSTEP
//  by Jim Crafton
//////////////////////////////////////////////////////////////////////

#ifndef _BITMAP_H__
#define _BITMAP_H__

#pragma warning (disable : 4786) //vector warnings about string too long...

#include "Object.h"
#include "BasicException.h"

namespace SimpleSTEP
{

struct RGBVals{
	uchar b;
	uchar g;
	uchar r;
};

struct RGBAVals{
	uchar b;
	uchar g;
	uchar r;
	uchar a;
};

struct RGBValsf{
	float b;
	float g;
	float r;
};

struct RGBAValsf{
	float b;
	float g;
	float r;
	float a;
};

class GRAPHICSKIT_API BitmapException : public BasicException{
public:
	BitmapException( const std::string& message ): BasicException(message){};
};



class GRAPHICSKIT_API Bitmap : public Object
{
public:
	Bitmap();
	virtual ~Bitmap();
	
	uint32 getWidth();
	uint32 getHeight();
	
	void setWidth( const uint32& newWidth );	
	void setHeight( const uint32& newHeight );

	COLORREF *bits;
	bool isSizeEqual( Bitmap* bitmap );

	bool create ( uint32 width, uint32 height ) throw( BitmapException );
	bool destroy ();

	void fill ( ubyte R, ubyte G, ubyte B );
	void fillRect ( uint32 x, uint32 y, uint32 w, uint32 h, ubyte R, ubyte G, ubyte B );
	void render ( int32 DCHandleID, uint32 width, uint32 height );
	void line( uint32 x1, uint32 y1, uint32 x2, uint32 y2, uint32 color, ubyte A );
private:
	
	BITMAPINFO info;
	int32 bitmapHandleID;
	uint32 height;
	uint32 width;
	
	//void SetBitmap ( CDC *pDC, CBitmap *pBitmap );
	
	

	/*
	void fillGlass ( ubyte R, ubyte G, ubyte , ubyte A );
	void fillStippledGlass ( ubyte R, intB G, ubyte B );
	*/
	void copy ( Bitmap *bitmap );
	void paste ( Bitmap *bitmap );

	void copyRect ( Bitmap *bitmap, uint32 x, uint32 y );
	void pasteRect ( Bitmap *bitmap, uint32 x, uint32 y );

	void blend ( Bitmap *bitmap, ubyte A );
	/*
	void darken ( Bitmap *bitmap );
	void difference ( Bitmap *bitmap );
	void lighten ( Bitmap *bitmap );
	void multiply ( Bitmap *bitmap );
	void screen ( Bitmap *bitmap );	

	
	void fillGlassRect ( ubyte x, ubyte y, ubyte w, ubyte h, ubyte R, ubyte G, ubyte B, ubyte A );
	void fillStippledGlassRect ( ubyte x, ubyte y, ubyte w, ubyte h, ubyte R, ubyte G, ubyte B );
	
	void blendRect ( Bitmap *bitmap, ubyte x, ubyte y, ubyte A );
	void darkenRect ( Bitmap *bitmap, ubyte x, ubyte y );
	void differenceRect ( Bitmap *bitmap, ubyte x, ubyte y );
	void lightenRect ( Bitmap *bitmap, ubyte x, ubyte y );
	void multiplyRect ( Bitmap *bitmap, ubyte x, ubyte y );
	void screenRect ( Bitmap *bitmap, ubyte x, ubyte y );
	*/
	/*
	void Line ( ubyte x1, ubyte y1, ubyte x2, ubyte y2, ubyte R, ubyte G, ubyte B );
	*/
	
	
	
};

};  //end of SimpleSTEP namespace
#endif 
