/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Border.h

#ifndef _BORDER_H__
#define _BORDER_H__


#include "Rect.h"
#include "GraphicsContext.h"
#include "Control.h"

namespace VCF
{

/*@interface*/

#define BORDER_CLASSID		"3126B221-2819-11d4-B53A-00C04F0196DA"		

class APPKIT_API Border : public Object{
public:
	BEGIN_ABSTRACT_CLASSINFO(Border, "VCF::Border", "VCF::Object", BORDER_CLASSID)
	END_CLASSINFO(Border)

	Border() {
		
	};

	virtual ~Border(){};
	
	/**
	*Paint the border on context
	*/
	virtual void paint( Control* component, GraphicsContext* context ) = 0;

	/**
	*paints a border in the given bounds on the context
	*@param Rect the rectangle that makes up the outer most region
	*of the border
	*@param GraphicsContext the context on which to paint the border
	*/
	virtual void paint( Rect* bounds, GraphicsContext* context ) = 0;

	/**
	*returns a client rect in which contained components may live.
	*Thus if a particular implementation required a 3 pixel
	*border at the edges to draw itself, then it would retrieve the
	*component's border and then reduce it by 3 pixels. This new
	*value is the available area that the component could use to
	*place child components
	*
	*@param Control* - component that the border belongs to 
	*
	*@return Rect* the client rectangle 
	*/
	virtual Rect* getClientRect( Control* component ) = 0;
	
};

};

#endif