/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef COLOR_H
#define COLOR_H

#include "Object.h"
#include <map>
#include "StringUtils.h"

namespace VCF{

/**
*standard system color defines
*/
#define		SYSCOLOR_SHADOW					0
#define		SYSCOLOR_FACE					1
#define		SYSCOLOR_HIGHLIGHT				2
#define		SYSCOLOR_ACTIVE_CAPTION			3
#define		SYSCOLOR_ACTIVE_BORDER			4
#define		SYSCOLOR_DESKTOP				5
#define		SYSCOLOR_CAPTION_TEXT			6
#define		SYSCOLOR_SELECTION				7
#define		SYSCOLOR_SELECTION_TEXT			8
#define		SYSCOLOR_INACTIVE_BORDER		9
#define		SYSCOLOR_INACTIVE_CAPTION		10
#define		SYSCOLOR_TOOLTIP				11
#define		SYSCOLOR_TOOLTIP_TEXT			12
#define		SYSCOLOR_MENU					13
#define		SYSCOLOR_MENU_TEXT				14
#define		SYSCOLOR_WINDOW					15
#define		SYSCOLOR_WINDOW_TEXT			16
#define		SYSCOLOR_WINDOW_FRAME			17


#define		WHITE							"white"
#define		BLACK							"black"
#define		GREY							"grey"
#define		YELLOW							"yellow"
#define		GREEN							"green"
#define		BLUE							"blue"
#define		RED								"red"
#define		PURPLE							"purple"
#define		LT_BLUE							"lt_blue"


class GRAPHICSKIT_API Color : public VCF::Object {

public:
	Color();
	Color( const Color& color );
    Color(const float & r, const float & g, const float & b);
    Color(const double & h, const double & s, const double & v);
    Color(const double & c, const double & m, const double & y, const double & k);
    Color(const unsigned char & r, const unsigned char & g, const unsigned char & b);
    Color(const unsigned long & color);
	
	virtual ~Color(){};

    void getHSV(double & h, double & s, double & v);
    void getHLS(double & h, double & l, double & s);
    void getCMYK(double & c, double & m, double & y, double & k);
    void getRGB(unsigned char & r, unsigned char & g, unsigned char & b);
    void getRGB(double & r, double & g, double & b);
    unsigned long getRGB();    
    void getLab();
    void getYUV();

    void setHSV( const double & h, const double & s, const double & v);
    void setHLS( const double & h, const double & l, const double & s);
    void setCMYK( const double & c, const double & m, const double & y, const double & k);
    void setRGB( const unsigned char & r, const unsigned char & g, const unsigned char & b);
    void setRGB( const double & r, const double & g, const double & b);
    void setRGB( const unsigned long& rgb );
    void setLab();
    void setYUV();

    void brighter();

    void darker();

	double getRed();

	double getGreen();

	double getBlue();

	virtual void copy( Object* source );

	operator=( const Color& newValue ){
		copy( (Object*)&newValue );
	};

	virtual bool isEqual( Object* object ){
		bool result = false;
		if ( NULL != object ){
			Color* color = dynamic_cast<Color*>(object);
			if ( NULL != color ){
				result = (color->m_b == m_b) && (color->m_g == m_g) && (color->m_r == m_r);
			}
		}
		return result;
	};

	virtual String toString(){
		char tmp[256];
		memset( tmp, 0, 256 );
		sprintf( tmp, "#%02X%02X%02X", (int)(m_r*255.0), (int)(m_g*255.0), (int)(m_b*255.0) );
		return String(tmp);
	};

	static Color* getColor( const String& colorName );
private:
	double m_r;
	double m_g;
	double m_b;

	static std::map<String,Color*> colorMap;
	static Color white;

	class GRAPHICSKIT_API ColorMapInitializer {
	public:
		ColorMapInitializer();

		virtual ~ColorMapInitializer();
	};
	
	friend class GRAPHICSKIT_API ColorMapInitializer;

	static ColorMapInitializer colorMapInitializer;
};



};

#endif //COLOR_H