/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _CONTROLGRAPHICSCONTEXT_H__
#define _CONTROLGRAPHICSCONTEXT_H__

#include "GraphicsContext.h"

namespace VCF {

class Control;

class APPKIT_API ControlGraphicsContext : public GraphicsContext{

public:
	/**
	*Creates a context based on the component. The context is attached to the component
	*for the duration of it's lifetime.
	*/
	ControlGraphicsContext( Control* control );

	virtual ~ControlGraphicsContext();

	void init();

	/**
	*returns the component the Graphics context belongs to. This is almost always a heavweight
	*component. If the context was created from scratch (in other words, not as the result of 
	*a component being created, then it will return NULL.
	*@return Control* the component that owns this context or NULL for memory contexts
	*/
	Control* getOwningControl(){
		return 	m_owningControl;
	};

	void setOwningControl( Control* owningComponent );
private:
	Control* m_owningControl;
};

}; //end of namespace VCF

#endif //_CONTROLGRAPHICSCONTEXT_H__