/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#ifndef FONT_H
#define FONT_H

#include "Object.h"
#include "Color.h"
#include "GlyphCollection.h"

namespace VCF{

#define DEFAULT_FONT_PT_SIZE	8
#define DEFAULT_FONT_NAME		"Arial"

class GRAPHICSKIT_API FontPeer;

class GRAPHICSKIT_API Font : public Object {

public:    
    Font();

	Font( const String& fontName );

	Font( const String& fontName, const double& pointSize );

	virtual ~Font();

	bool isTrueType();

	/**
	*returns the Color the Font will be rendered in
	*/
	Color* getColor();

	/**
	*sets the Color to render the Font in
	*/
    void setColor( Color* color );

	/**
	*return the point size of the Font. One point is 1/72 of an inch
	*( or 0.0352552 cm for our more civilized friends !), so to figure 
	*out the pixels involved, find out the Pixels per Inch and then apply
	*the following formula
	*<code>
	* (PointSize / 72) * PPI
	*</code>
	*where PPI represents the Pixels Per Inch
	*/
    double getPointSize();

	/**
	*sets the point size of the Font
	*/
	void setPointSize( const double pointSize );

	double getPixelSize();

	void setPixelSize( const double pixelSize );

    void setBold( const bool& bold );    

    bool getBold();

    bool getItalic();

    void setItalic( const bool& italic );

    bool getUnderlined();

    void setUnderlined( const bool& underlined );

    bool getStrikeOut();

    void setStrikeOut( const bool& strikeout );

    double getShear();

    void setShear(const double& shear );

    double getAngle();

    void setAngle( const double& angle );

	String getName();

	void setName( const String& name );

    GlyphCollection* getGlyphCollection( const String& text );

	FontPeer* getFontPeer();

	virtual void copy( Object* source );

private:
	FontPeer* m_Peer;
};

};

#endif //FONT_H