/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//FontPeer.h

#ifndef _FONT_PEER_H__
#define _FONT_PEER_H__

#include "VCF.h"
#include "Font.h"
#include "GlyphCollection.h"
#include "Color.h"

namespace VCF
{

/**
*Represents the native windowing system's representation of a Font. 
*/
class GRAPHICSKIT_API FontPeer {
public:
	virtual ~FontPeer(){};

	/**
	*returns a integer representing some native handle to a font structure.
	*What this actually is depends on the Windowing system implementation.
	*Under Win32 this represents a pointer to a LOGFONT structure.
	*/
	virtual ulong32 getFontHandleID() = 0;

	virtual String getName() = 0;

	virtual void setName( const String& name ) = 0;

	/**
	*is this font a TrueType font ?
	*/
	virtual bool isTrueType() = 0;

	/**
	*returns the Color the Font will be rendered in
	*/
	virtual Color* getColor() = 0;

	/**
	*sets the Color to render the FontImplmenter in
	*/
    virtual void setColor( Color* color ) = 0;

	/**
	*return the point size of the Font. One point is 1/72 of an inch
	*( or 0.0352552 cm for our more civilized friends !), so to figure 
	*out the pixels involved, find out the Pixels per Inch and then apply
	*the following formula
	*<code>
	* (PointSize / 72) * PPI
	*</code>
	*where PPI represents the Pixels Per Inch
	*/
    virtual double getPointSize() = 0;

	/**
	*sets the point size of the FontPeer
	*/
	virtual void setPointSize( const double pointSize ) = 0;

	virtual double getPixelSize() = 0;

	virtual void setPixelSize( const double pixelSize ) = 0;

    virtual void setBold( const bool& bold ) = 0;    

    virtual bool getBold() = 0;

    virtual bool getItalic() = 0;

    virtual void setItalic( const bool& italic ) = 0;

    virtual bool getUnderlined() = 0;

    virtual void setUnderlined( const bool& underlined ) = 0;

    virtual bool getStrikeOut() = 0;

    virtual void setStrikeOut( const bool& strikeout ) = 0;

    virtual double getShear() = 0;

    virtual void setShear(const double& shear ) = 0;

    virtual double getAngle() = 0;

    virtual void setAngle( const double& angle ) = 0;

	/**
	*returns a GlyphCollection that represents the set of shapes 
	*for the specified text string.
	*@param String text - the group of characters to put in the GlyphCollection
	*@return GlyphCollection the collection of Glyphs, or vectorized shapes that
	*represent the text string.
	*/
    virtual GlyphCollection* getGlyphCollection( const String& text ) = 0;
};

};

#endif//_FONT_PEER_H__