/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//ImageBits.h

#ifndef _IMAGE_BITS_H__
#define _IMAGE_BITS_H__

#include "VCF.h"
#include "Object.h"
#include "ClassInfo.h"

namespace VCF
{

struct RGBVals{
	char b;
	char g;
	char r;
};

struct RGBAVals{
	char b;
	char g;
	char r;
	char a;
};

struct RGBValsf{
	float b;
	float g;
	float r;
};

struct RGBAValsf{
	float b;
	float g;
	float r;
	float a;
};


/**
*Basic class for image bit twiddling 
*default behaviour is to allow for RGBA channels
*/

#define IMAGEBITS_CLASSID		"ED88C098-26AB-11d4-B539-00C04F0196DA"

class GRAPHICSKIT_API ImageBits : public Object {
public :

	BEGIN_CLASSINFO(ImageBits, "VCF::ImageBits", "VCF::Object", IMAGEBITS_CLASSID)		
	END_CLASSINFO(ImageBits)

	ImageBits(){
		REGISTER_CLASSINFO( ImageBits );
		m_width = 0;
		m_height = 0;
		m_bits = NULL;
		this->init();
	};

	ImageBits( const unsigned long& width, const unsigned long& height ){		
		m_width = width;
		m_height = height;
		m_bits = NULL;
		this->init();
	};

	virtual ~ImageBits(){
		delete [] m_bits;	
	};

	void init(){
		if ( NULL != m_bits ){
			delete [] m_bits;
		}
		unsigned long size = getSize();
		if ( size > 0 ){
			m_bits = new RGBAVals[size];
		}
		else{
			m_bits = NULL;
		}
		
	};

	void setWidth( const unsigned long& width ){
		m_width = width;
		this->init();
	};

	void setHeight( const unsigned long& height ){
		m_height = height;
		this->init();
	};

	unsigned long m_width;

	unsigned long m_height;
	
	unsigned long getSize(){
		return m_height * m_width;
	};

	RGBAVals* m_bits;
};

};
	
#endif