/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _PATHENUMERATOR_H__
#define _PATHENUMERATOR_H__

#include "VCF.h"
#include "Enumerator.h"


namespace VCF {


class GRAPHICSKIT_API PathEnumerator : public Interface {
public:
	enum SegmentType{
		SEGMENT_UNKNOWN = 0,
		SEGMENT_MOVETO,
		SEGMENT_LINETO,
		SEGMENT_QUADTO,
		SEGMENT_CUBICTO,
		SEGMENT_CLOSE
	};

	enum WindingType {
		WINDING_EVENODD=0,
		WINDING_NONZERO
	};	

	virtual ~PathEnumerator(){};

	/**
	*returns the winding rule for this path
	*/
	virtual WindingType getWindingRule() = 0;

	/**
	*does this PathEnumerator have any more segments ?
	*@return bool true if the path has more segments
	*false otherwise.
	*/
	virtual bool hasMoreSegments() = 0;

	/**
	*iterates to the next segment in the path, or does nothing 
	*if the path has no more segments
	*/
	virtual void next() = 0;

	/**
	*returns the number of points in this segment.
	*The rules are:
	*1 point for SEGMENT_MOVETO and SEGMENT_LINETO
	*3 points for SEGMENT_CUBICTO
	*and 0 points for SEGMENT_CLOSE
	*/
	virtual ulong32 getPointCount() = 0;

	/**
	*retrieves the points for this segment
	*@param Point* an array of Point's. The array 
	*must be allocated by the caller and be of the size 
	*specified in getPointCount() for the segment. The values
	*will be copied into the array by the implementer of a 
	*PathEnumerator.
	*@return SegmentType the type of the segment returned
	*/
	virtual SegmentType getSegment( Point* points ) = 0;

	/**
	*returns an enumerator of all the points for this PathEnumerator
	*If any of the point segments are curves (either Cubic or Quadric),
	*then this will cause the curves to be interpolated and returned as 
	*a series of points as well.
	*/
	virtual Enumerator<Point*>* flattenPoints() = 0;

	virtual Rect* getBounds() = 0;

};

}; //end of namespace VCF

#endif //_PATHENUMERATOR_H__