/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/**
*borrowed from 
*art_render.h & Libart_LGPL - library of basic graphic primitives
* Copyright (C) 2000 Raph Levien
*/

#ifndef _RENDERABLEAREA_H__
#define _RENDERABLEAREA_H__

#include "VCF.h"

namespace VCF {


class AlphaGamma;


#if RENDERABLE_MAX_DEPTH == 16
	typedef ushort RenderPixelMaxDepthType;
	#define RENDER_PIX_MAX_FROM_8(x) ((x) | ((x) << 8))
	#define RENDER_PIX_8_FROM_MAX(x) (((x) + 0x80 - (((x) + 0x80) >> 8)) >> 8)
#else
	#if RENDERABLE_MAX_DEPTH == 8
		typedef uchar RenderPixelMaxDepthType;
		#define RENDER_PIX_MAX_FROM_8(x) (x)
		#define RENDER_PIX_8_FROM_MAX(x) (x)
	#else
		#error RENDERABLE_MAX_DEPTH must be either 8 or 16
	#endif //RENDERABLE_MAX_DEPTH == 8
#endif //RENDERABLE_MAX_DEPTH == 16


#define RENDERABLE_MAX_CHAN			16


enum RenderAlphaType{
	RA_NONE=0,
	RA_SEPERATE,
	RA_PRE_MULTIPLY
};

enum RenderCompositeMode{
	RCM_NONE=0,
	RCM_NORMAL,
	RCM_ADDITIVE,
	RCM_SUBBTRACTIVE,
	RCM_MULTIPLY,
	RCM_DIVIDE
	//more to come
};

class GRAPHICSKIT_API RenderMaskRun {
public:
	int32 m_x;
	int32 m_alpha;
};

class GRAPHICSKIT_API RenderableArea {

public:
	RenderableArea( const int32& x0, const int32& y0, const int32& x1, const int32& y1,
					const int32& rowstride, uchar *pixels=NULL,
					int n_chan=4, const int32& depth, 
					const RenderAlphaType& alpha_type=RA_NONE,
					AlphaGamma *alphaGamma=NULL ){};
	
	virtual ~RenderableArea(){};
	
	virtual void invoke();

	virtual void clear();

	virtual void clearRGB( const uint32& rgb );

	virtual void maskSolid( const int32& opacity );


protected:
	int32 m_x0;
	int32 m_y0;
	int32 m_x1;
	int32 m_y1;
	uchar* m_pixels;
	int32 m_rowstride;
	int32	m_numberOfChannels;
	int32	m_depth;
	RenderAlphaType m_alphaType;
	
	bool m_isClear;

	long m_clearColors[RENDERABLE_MAX_CHAN + 1];
	uint32 m_opacity; /* [0..0x10000] */
	
	RenderCompositeMode compositing_mode;
	
	AlphaGamma* m_alphaGamma;
	
	uchar* alphaBuffer;
	
	/* parameters of intermediate buffer */
	int m_bufferDepth;
	RenderAlphaType m_bufferAlpha;
	uchar* m_imageBuffer;
	
	/* driving alpha scanline data */
	/* A "run" is a contiguous sequence of x values with the same alpha value. */
	int m_runNumber;
	
	RenderMaskRun* m_run;
	
	/* A "span" is a contiguous sequence of x values with non-zero alpha. */
	int m_spanNumber;
	int* m_spanX;
	
	bool m_needSpan;

private:
	
	
};

}; //end of namespace VCF

#endif //_RENDERABLEAREA_H__