/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//CommonDialogPeer.h
#ifndef _COMMON_DIALOG_PEER_H__
#define _COMMON_DIALOG_PEER_H__

#include "VCF.h"
#include "VCFString.h"
#include "Interface.h"
#include "Enumerator.h"

namespace VCF {

class Color;

class Font;

/**
*
*/
class APPKIT_API CommonDialogPeer : public Interface{ 
public:
	virtual ~CommonDialogPeer(){};

	virtual void setTitle( const String& title ) = 0;

	virtual bool execute() = 0;
};


/**
*
*/
class APPKIT_API CommonFileDialogPeer : public CommonDialogPeer{ 
public:
	virtual ~CommonFileDialogPeer(){};

	virtual void addFilter( const String & description, const String & extension ) = 0;

    virtual void setDirectory( const String & directory ) = 0;
    
	virtual void setFileName( const String & filename ) = 0;

    virtual String getFileName() = 0;

    virtual String getDirectory() = 0;

    virtual String getFileExtension() = 0;

	virtual uint32 getSelectedFileCount() = 0;

	virtual Enumerator<String>* getSelectedFiles() = 0;
	
	virtual void setAllowsMultiSelect( const bool& allowsMultiSelect ) = 0;
	
};

/**
*
*/
class APPKIT_API CommonFolderBrowseDialogPeer : public CommonDialogPeer{ 
public:
	virtual ~CommonFolderBrowseDialogPeer(){};

	virtual void setDirectory( const String & directory ) = 0;

	virtual String getDirectory() = 0;
	
};

/**
*
*/
class APPKIT_API CommonColorDialogPeer : public CommonDialogPeer {
public:
	virtual ~CommonColorDialogPeer(){};

	virtual Color* getSelectedColor() = 0;

	virtual void setSelectedColor( Color* selectedColor ) = 0;

};


/**
*
*/
class APPKIT_API CommonPrintDialogPeer : public CommonDialogPeer {
public:
	virtual ~CommonPrintDialogPeer(){};
	
};


/**
*
*/
class APPKIT_API CommonFontDialogPeer : public CommonDialogPeer {
public:
	virtual ~CommonFontDialogPeer(){};
	
	virtual Font* getSelectedFont() = 0;

	virtual void setSelectedFont( Font* selectedFont ) = 0;
};

};


#endif //_COMMON_DIALOG_PEER_H__