/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef LISTBOXPEER_H
#define LISTBOXPEER_H

#include "ControlPeer.h"

namespace VCF{

class ListItem;

class Rect;

/** @interface */

class APPKIT_API ListBoxPeer : public ControlPeer {
public:    

    /**
     * on Win32 see LB_SETCARETINDEX 
     */
    virtual void setFocusedItem(ListItem * item) = 0;

    /**
     * on Win32 see LB_RESETCONTENT 
     */
    virtual void clear() = 0;

    /**
     * on Win32 see LB_SELITEMRANGEEX 
     */
    virtual void rangeSelect(const bool & isSelected, ListItem * first, ListItem * last) = 0;

    /**
     * on Win32 see LB_SETCURSEL 
     */
    virtual void selectItem(ListItem * item) = 0;

    /**
     * on Win32 see LB_GETITEMRECT 
     */
    virtual Rect* getItemRect() = 0;

    /**
     * on Win32 see LB_ADDSTRING 
     */
    virtual void addItem(ListItem* item) = 0;

    virtual void insertItem( const unsigned long& index, ListItem * item) = 0;

	virtual void removeItem( ListItem* item ) = 0;

    /**
     * on Win32 see LB_GETCURSEL 
     */
    virtual bool isItemSelected(ListItem* item) = 0;

    /**
     * returns NULL if the point passed in is not over a list item 
     */
    virtual ListItem* isPtOverItem(Point* point) = 0;    

    /**
     * returns a the item that has the focus in a multiple selection range.
     * On Win32 see LB_GETCARETINDEX 
     */
    virtual ListItem* getFocusedItem() = 0;    

    /**
     * on  Win32 see LB_SETITEMHEIGHT 
     */
    virtual void setItemHeight( const unsigned long& itemHeight ) = 0;

    /**
     * on Win32 see LB_GETCURSEL 
     */
    virtual ListItem* getSelectedItem() = 0;

    /**
     * on Win32 see LB_GETITEMHEIGHT 
     */
    virtual unsigned long getItemHeight() = 0;
};

}
#endif //LISTBOXPEER_H