/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef TEXTPEER_H
#define TEXTPEER_H


#include "Point.h"
#include "ControlPeer.h"


namespace VCF{

/** @interface */

/**
*A interface for supporting text controls
*This results in a native text control widget being 
*created, with all painting responsibilities being 
*controlled by the native widget. This means that
*some advanced graphics features and Font capabilities
*may not appear in the control. FOr example, setting 
*the Font's setShear() method will have no effect
*on the widget's rendering of it's text.
*@version 1.0
*author Jim Crafton
*/
class APPKIT_API TextPeer : public ControlPeer {
public:    
    virtual void setRightMargin( const double & rightMargin ) = 0;
    
	virtual void setLeftMargin( const double & leftMargin ) = 0;

    virtual unsigned long getLineCount() = 0;

    virtual void getCurrentLinePosition() = 0;

    virtual double getLeftMargin() = 0;

    virtual double getRightMargin() = 0;

    virtual Point* getPositionFromCharIndex( const unsigned long& index ) = 0;

    virtual unsigned long getCharIndexFromPosition( Point* point ) = 0;

	/**
	*returns the current caret position with in the text control
	*this is specified by a zero based number representing the 
	*insertion point with the text control's text (stored in the text
	*control's Model).
	*@return long the index of the current insertion point in the Model's text
	*/
	virtual unsigned long getCaretPosition() = 0;
};

};

#endif //TEXTPEER_H