/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ThreadPeer.h


#ifndef _THREADPEER_H__
#define _THREADPEER_H__

//#include "Object.h"
#include "vcf.h"


namespace VCF
{

/**
*@interface ThreadPeer
*Basic Thread interface to wrap native thread OS calls.
*
*@author Jim Crafton
*@version 1.0
*/
class FRAMEWORK_API ThreadPeer {
public:
	virtual ~ThreadPeer() {};

	virtual bool start() = 0;

	virtual bool stop( int32 waitMillisec, bool forcekill, uint32 * pThreadExitCode ) = 0;
	
	virtual void sleep( uint32 msec ) = 0;

	virtual	bool checkForStop() = 0;
};


/**
*@interface ThreadMutexPeer
*Basic ThreadMutex interface to wrap native thread mutex OS calls.
*
*@author Jim Crafton
*@version 1.0
*/
enum ThreadMutexAcquire {
	ACQUIRED_YES = 0,	// got the mutex
	ACQUIRED_NO,		// didn't get the mutex
	ACQUIRED_ERROR,		// something wrong - mutex invalid, etc.
};

class FRAMEWORK_API ThreadMutexPeer {
public:
	virtual ~ThreadMutexPeer() {};

	virtual ThreadMutexAcquire acquire( int32 msec, bool* pStopNow ) = 0;

	virtual bool release() = 0;
};


/**
*@interface ThreadSignalPeer
*Basic ThreadSignal interface to wrap native thread signal OS calls.
*
*@author Jim Crafton
*@version 1.0
*/
enum ThreadSignalState {
	SIGNALSTATE_TRUE = 0,	// signalled
	SIGNALSTATE_FALSE,		// un-signalled
	SIGNALSTATE_ERROR,		// something wrong - threadsignal invalid, etc.
};

class FRAMEWORK_API ThreadSignalPeer {
public:
	virtual ~ThreadSignalPeer() {};

	virtual bool setSignalState( bool signalState ) = 0;

	virtual ThreadSignalState waitOnSignalState( bool desiredState, int32 msec, bool* pStopNow ) = 0;
};

};

#endif
