/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32CustomControl.h

#ifndef _WIN32CUSTOMCONTROL_H__
#define _WIN32CUSTOMCONTROL_H__

#include "Win32Peer.h"
#include "Object.h"
#include "Control.h"
#include "ControlPeer.h"
#include "Rect.h"

namespace VCF
{

class APPKIT_API Win32CustomControl : public Object, public ControlPeer  
{
public:
	Win32CustomControl( Control* component );
	virtual ~Win32CustomControl();
	
	static bool m_windowClassRegistered;

	/**
     * returns a ID that identifies this control. On Win32 this corresponds to an HWND. 
     */
    virtual long getHandleID();

    /**
     * returns a text associated with the component. This usually gets used in the Component::getCaption() method. 
     */
    virtual VCF::String getText();

    /**
     * sets the text for the widget 
     */
    virtual void setText( const VCF::String& text );

    /**
     * sets the bounds for the component. Bounds are specified in the coordinate system of the componenents parent. 
     */
    virtual void setBounds( VCF::Rect* rect );

    /**
     * returns the bounds of the component in the coordinate system of the parent. 
     */
    virtual VCF::Rect* getBounds();

    /**
     * shows or hides the component. 
     * This does NOT close the component (if invoked on a frame based component ). 
     */
    virtual void setVisible( const bool& visible );

    /**
     * returns wether or not the component is currently visible. 
     */
    virtual bool getVisible();

    /**
     * returns a bit-masked unsigned long that contains style constants. 
     *  These style constants are defined in the VCF, and must 
     * be translated to the particular windowing system being used. 
     */
    virtual unsigned long getStyleMask();

    /**
     * sets the current style mask.
     *  Should cause a repaint of the component, if neccessary. 
     */
    virtual void setStyleMask( const unsigned long& styleMask );

    /**
     * returns the component that this Peer is attached to. 
     */
    virtual VCF::Control* getControl();

    /**
     * attahces the Peer to a particular component. This should only be done once. 
     */
    virtual void setControl( VCF::Control* component );

    virtual void getCursor();

    virtual void setCursor();    

	virtual void setParent( VCF::Control* parent );

	virtual VCF::Control* getParent();

	virtual bool isFocused();

	virtual void setFocus( const bool& focused );

	virtual bool isEnabled();

	virtual void setEnabled( const bool& enabled );

	virtual void repaint( Rect* repaintRect=NULL );

	virtual void keepMouseEvents();

	virtual void releaseMouseEvents();
private:
	void registerWndClass();

	HWND m_wndHandle;
	VCF::Control* m_component;
	VCF::Rect m_bounds;	
	VCF::Point m_clickPt;	
};

};

#endif
