/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _WIN32FONT_H__
#define _WIN32FONT_H__

#include "VCF.h"
#include "Object.h"
#include "FontPeer.h"
#include "Win32Peer.h"

namespace VCF {

class GRAPHICSKIT_API Win32Font : public FontPeer , public Object {

public:
	Win32Font( const String& fontName );
	
	Win32Font( const String& fontName, const double& pointSize );

	virtual ~Win32Font();

	void init();

	/**
	*returns a integer representing some native handle to a font structure.
	*What this actually is depends on the Windowing system implementation.
	*Under Win32 this represents a pointer to a LOGFONT structure.
	*/
	virtual ulong32 getFontHandleID();

	virtual String getName();

	virtual void setName( const String& name );

	/**
	*is this font a TrueType font ?
	*/
	virtual bool isTrueType();

	/**
	*returns the Color the Font will be rendered in
	*/
	virtual Color* getColor();

	/**
	*sets the Color to render the FontImplmenter in
	*/
    virtual void setColor( Color* color );

	/**
	*return the point size of the Font. One point is 1/72 of an inch
	*( or 0.0352552 cm for our more civilized friends !), so to figure 
	*out the pixels involved, find out the Pixels per Inch and then apply
	*the following formula
	*<code>
	* (PointSize / 72) * PPI
	*</code>
	*where PPI represents the Pixels Per Inch
	*/
    virtual double getPointSize();

	/**
	*sets the point size of the FontPeer
	*/
	virtual void setPointSize( const double pointSize );

	virtual double getPixelSize();

	virtual void setPixelSize( const double pixelSize );

    virtual void setBold( const bool& bold );    

    virtual bool getBold();

    virtual bool getItalic();

    virtual void setItalic( const bool& italic );

    virtual bool getUnderlined();

    virtual void setUnderlined( const bool& underlined );

    virtual bool getStrikeOut();

    virtual void setStrikeOut( const bool& strikeout );

    virtual double getShear();

    virtual void setShear(const double& shear );

    virtual double getAngle();

    virtual void setAngle( const double& angle );

	/**
	*returns a GlyphCollection that represents the set of shapes 
	*for the specified text string.
	*@param String text - the group of characters to put in the GlyphCollection
	*@return GlyphCollection the collection of Glyphs, or vectorized shapes that
	*represent the text string.
	*/
    virtual GlyphCollection* getGlyphCollection( const String& text );

	virtual bool isEqual( Object* object );
private:
	double m_shear;
	Color m_color;
	LOGFONT m_logFont;
	double m_pointSize;
	String m_fontName;
};

}; //end of namespace VCF

#endif //_WIN32FONT_H__