/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _WIN32FONTMANAGER_H__
#define _WIN32FONTMANAGER_H__

#include "Object.h"
#include "Win32Font.h"
#include <map>

namespace VCF {

#define DEFAULT_VCF_FONT_NAME		"Arial"


/**
*used to hold a HFONT and a refcount
*/
class GRAPHICSKIT_API Win32FontHandle : public Object {
public:
	Win32FontHandle( HFONT font );
	
	virtual ~Win32FontHandle();

	void incRef();

	/**
	*decrement the ref count. If the count goes to 0 then
	*delete the HFONT object
	*/
	void decRef();

	HFONT getFontHandle();

	bool isRefCountZero();
private:
	int m_refCount;
	HFONT m_fontHandle;
};


/**
*Uses to manage fonts for controls
*/
class GRAPHICSKIT_API Win32FontManager : public Object{

public:
	Win32FontManager();

	virtual ~Win32FontManager();
	
	/**
	*this is called by the Win32Font class to add the
	*the font peer to the font manager.
	*/
	static void initializeFont( Win32Font* font );

	static void removeFont( Win32Font* font ); 
	
	static HFONT getFontHandleFromFontPeer( Win32Font* font );
private:
	/**
	*adds the font to the manager called internally
	*/
	void addFont( Win32Font* font );

	std::map<String, Win32FontHandle*> m_fontMap;
	Win32Font* m_defaultFont;
	
	/**
	*makes a string id (like a hash code - there probably is a better way to do this.....
	*of the Win32Font.
	*the ID is the Name + Color + 
	*/
	String makeStringID( Win32Font* font );
	
	static Win32FontManager win32FontMgr;
};

}; //end of namespace VCF

#endif //_WIN32FONTMANAGER_H__