/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Image.h

#ifndef _WIN32IMAGE_H__
#define _WIN32IMAGE_H__

#include "VCF.h"
#include "AbstractImage.h"
#include "Win32Peer.h"
#include "ImageLoader.h"
#include "GraphicsContext.h"
#include "Win32Context.h"

namespace VCF
{

class GRAPHICSKIT_API Win32Image : public AbstractImage  
{
public:
	Win32Image();
	
	Win32Image( const String& fileName );
	
	Win32Image( const unsigned long& width, const unsigned long& height );
	
	Win32Image( GraphicsContext* context, Rect* rect );

	Win32Image( HBITMAP bitmap );

	virtual ~Win32Image();

	void init();

	virtual void setWidth( const unsigned long & width );

	virtual void setHeight( const unsigned long & height );

	void createBMP();

	HBITMAP getBitmap();
	
	HDC getDC();

	BITMAPINFO m_bmpInfo;
	HPALETTE m_palette;//for 256 or fewer color images
protected:
	void loadFromFile( const String& fileName );

	void loadFromBMPHandle( HBITMAP bitmap );

	void saveToFile( const String& fileName );

private:
	HBITMAP m_hBitmap;
	HDC m_dc;
	
};

class GRAPHICSKIT_API BMPLoader : public Object, public ImageLoader {
public:
	BMPLoader();
	
	virtual ~BMPLoader();

	virtual Image* loadImageFromFile( const String& fileName );
	

	virtual void saveImageToFile( const String& fileName, Image* image );

	static ImageLoader* getLoader(){
		return &bmpLoaderInstance;
	}
private:
	static BMPLoader bmpLoaderInstance;
};

};

#endif 
