/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32MenuItem.h: interface for the Win32MenuItem class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _WIN32MENUITEM_H__
#define _WIN32MENUITEM_H__

#include "VCF.h"
#include "MenuItemPeer.h"
#include "Win32Peer.h"
#include <map>

namespace VCF
{

class APPKIT_API Win32MenuItem : public MenuItemPeer  
{
public:
	Win32MenuItem();

	Win32MenuItem( MenuItem* item );

	virtual ~Win32MenuItem();

	void init();

	virtual ulong32 getMenuID();

	virtual void addChild( MenuItem* child );

	virtual void insertChild( const unsigned long& index, MenuItem* child );
	
	virtual void deleteChild( MenuItem* child );

	virtual void deleteChild( const unsigned long& index );

	virtual void clearChildren();

	virtual bool isChecked();

	virtual void setChecked( const bool& checked );

	virtual MenuItem* getParent();

	virtual bool hasParent();

	virtual MenuItem* getChildAt( const unsigned long& index );

	virtual bool isEnabled();

	virtual void setEnabled( const bool& enabled );

	virtual bool isVisible();

	virtual void setVisible( const bool& visible );

	virtual bool getRadioItem();

	virtual void setRadioItem( const bool& value );

	virtual void setMenuItem( MenuItem* item );

	virtual void setCaption( const String& caption ); 

	static MenuItem* getMenuItemFromID( const uint32 id );	
private:
	HMENU m_itemHandle;
	uint32 m_itemId;
	MenuItem* m_menuItem;
	void insertSimpleMenuItem( MenuItem* child, HMENU menu );
	static std::map<uint32,MenuItem*> menuItemMap;
};

};
#endif // !defined(AFX_WIN32MENUITEM_H__080448FD_2C01_11D4_B53C_00C04F0196DA__INCLUDED_)
