/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Object.h

#ifndef _WIN32OBJECT_H__
#define _WIN32OBJECT_H__



#include "Object.h"
#include "Win32Peer.h"
#include <vector>
#include <map>
#include "Rect.h"

namespace VCF
{

#define SIMPLE_VIEW				WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN
#define BORDERED_VIEW			WS_VISIBLE | WS_CHILD | WS_BORDER | WS_CLIPSIBLINGS | WS_CLIPCHILDREN
#define SIMPLE_DIALOG			WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN
#define FRAME_WINDOW			WS_OVERLAPPEDWINDOW

class Control;

class APPKIT_API Win32Object : public Object  
{
public:
	Win32Object();
	virtual ~Win32Object();

	void registerWin32Class( const String& className, WNDPROC wndProc );

	static void addRegisterWin32Class( const String& className );
	
	/**
	*Checks whether the class has been registered. Performs a
	*std::find() using the m_registeredWindowClasses static list
	*/
	bool isRegistered();

	/**
	*called before the create window is called. Overide this to change the 
	*m_styleMask and m_exStyleMask values for determing the initial look of the window
	*/
	virtual void createParams();

	/**
	*This is where the window is actually created. Called from constructors
	*/
	virtual void init();

	virtual LRESULT handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc = NULL);

	static Win32Object* getWin32ObjectFromHWND( HWND hwnd );

	static void registerWin32Object( Win32Object* wndObj );

	HWND getHwnd();

	LRESULT defaultWndProcedure( UINT message, WPARAM wParam, LPARAM lParam );
	
	void setCreated( const bool& creationComplete );

	bool isCreated();

	void setPeerControl( Control* control );

	Control* getPeerControl();
protected:
	static std::vector< std::string > m_registeredWindowClasses;
	static std::map<HWND, Win32Object*> m_windowMap;
	DWORD m_styleMask;
	DWORD m_exStyleMask;
	Rect m_bounds;
	/**
	*this actually contains the current window proc
	*/
	WNDPROC m_wndProc;
	/**
	*this contains the window proc used for window
	*subclassing or just performing the default
	*behaviour. By default it is set
	*to point to ::DefWindowProc()
	*/
	WNDPROC m_defaultWndProc;
	HWND m_hwnd;
	String m_windowCaption;
	bool m_created;
	Control* m_peerControl;
};


};

#endif 
