/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Win32Peer.h
/**
*Main header for implementation object that Win32 headers
*/


#ifndef WIN32_PEER_H__
#define WIN32_PEER_H__

#include <windows.h>
#include <commctrl.h> //make sure to link with comctl32.lib
#include <shlobj.h>
#include "VCF.h"
#include "VCFString.h"
#include "Locales.h"



namespace VCFWin32 {

struct KeyboardData {
	int repeatCount;
	int scanCode;
	bool isExtendedKey;
	bool altKeyDown;
	unsigned short character;
	int VKeyCode;
	UINT keyMask;
};

#define KB_CONTEXT_CODE			29
#define KB_PREVIOUS_STATE		30
#define KB_IS_EXTENDED_KEY		24

#define SHIFT_KEY_DOWN \
	((GetKeyState( VK_SHIFT) & 15 ) == 1)

#define VCF_MESSAGE				WM_USER	+ 100
#define VCF_CONTROL_CREATE		VCF_MESSAGE + 1


/**
*what follows are a set of standard utility functions for Win32
*/
class FRAMEWORK_API Win32Utils {

public:
	static unsigned long translateKeyMask( UINT win32KeyMask );

	static unsigned long translateButtonMask( UINT win32ButtonMask );

	static KeyboardData translateKeyData( HWND wndHandle, LPARAM keyData );

	static DWORD translateStyle( unsigned long style );

	static DWORD translateExStyle( unsigned long style );

	static void trace( const VCF::String& text );

	static WORD	getWin32LangID( VCF::Locale* locale );

	static int getXFromLParam( LPARAM lParam );
	
	static int getYFromLParam( LPARAM lParam );

	static VCF::ulong32 translateVKCode( UINT vkCode );

	static bool fileExists( const VCF::String& fileName );

	static VCF::String getErrorString( const DWORD& errorCode );
};


};


#endif