/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _WIN32REGISTRY_H__
#define _WIN32REGISTRY_H__

#include "Object.h"
#include "Enumerator.h"
#include "RegistryPeer.h"
#include "Win32Peer.h"
#include <vector>

namespace VCF {

class FRAMEWORK_API Win32Registry : public Object, public RegistryPeer{

public:
	Win32Registry();

	virtual ~Win32Registry();

	virtual void setRoot( const RegistryKeyType& rootKey );
	
	virtual RegistryKeyType getRoot();

	virtual bool openKey( const String& keyname, const bool& createIfNonExistant );

	virtual bool setValue( const String& value, const String& valuename ); 

	virtual bool setValue( const uint32& value, const String& valuename ); 
	
	virtual bool setValue( const bool& value, const String& valuename ); 

	virtual bool setValue( void* dataBuffer, const uint32& dataBufferSize, const String& valuename ); 

	virtual String getStringValue( const String& valuename );

	virtual uint32 getIntValue( const String& valuename );

	virtual bool getBoolValue( const String& valuename );

	virtual void getDataBufValue( const String& valuename, uint32& dataBufferSize, void** dataBuffer );

	virtual Enumerator<String>* getKeyNames();

	virtual Enumerator<RegistryValueInfo*>* getValues();

	virtual String getCurrentKey();
private:
	std::vector<RegistryValueInfo*> m_values;
	std::vector<String> m_keys;
	EnumeratorContainer<std::vector<RegistryValueInfo*>,RegistryValueInfo*> m_valuesContainer;
	EnumeratorContainer<std::vector<String>,String> m_keysContainer;
	RegistryKeyType m_rootKeyType;
	String m_currentRegKey;

	HKEY m_currentKeyHandle;
	HKEY m_rootKeyHandle;
};

}; //end of namespace VCF

#endif //_WIN32REGISTRY_H__