/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32SocketImplementer.h

#ifndef _WIN32SOCKETIMPLEMENTER_H__
#define _WIN32SOCKETIMPLEMENTER_H__

#include <winsock2.h> //makes sure to link with Ws2_32.lib
#include "Object.h"
#include "SocketImplementer.h"
#include "Socket.h"
#include "Win32Implementer.h"
#include <memory.h>
#include <process.h>
#include <vector>

namespace VCFNet
{

#define VCFNET_SOCKET_NAME_ERROR			10
#define VCFNET_SOCKET_OK					11
#define VCFNET_SOCKET_CONNECT_ERROR		12
#define VCFNET_SOCKET_CLOSED_ERROR		13
#define VCFNET_SOCKET_READ_ERROR			14
#define VCFNET_SOCKET_WRITE_ERROR		15
#define VCFNET_SOCKET_CREATE_ERROR		16
#define VCFNET_SOCKET_BIND_ERROR			17
#define VCFNET_SOCKET_THREAD_ERROR		18
#define VCFNET_SOCKET_LISTEN_ERROR		19


#define MAX_PENDING_CONNECTS SOMAXCONN

class FRAMEWORK_API Win32SocketImplementer : public VCF::Object, public SocketImplementer  
{
public:
	Win32SocketImplementer();
	virtual ~Win32SocketImplementer();

	virtual int getLocalPort();
    
	virtual void setLocalPort( const int& localPort );

	virtual VCF::String getLocalHost();

	virtual void setLocalHost( const VCF::String& localHost );

	virtual int getPort();

	virtual void setPort( const int& port );

	virtual VCF::String getHost();

	virtual void setHost( const VCF::String& host );	

	/** 
     * Receives data from a socket. Blocks until data has
     * been received.
     *
     * @param data - the buffer to hold the message (allocated by caller).
     * @param size - set to the length of the message that was read. 
     *
     * N.B. The caller is responsible for making sure that -buf- is large
     * enough to receieve messages.     
     */
    virtual int receive( char* data, VCF::uint32& size );
    
    
    /**
	*Sends a data down a socket. Blocks until the message has been sent.
	*/
    virtual int send( const char* data, const VCF::uint32& size );

	void *m_clientData;

	static void startup();

	static void closedown();

	int getErrorCode() {
        return WSAGetLastError();
    };
    
    /** 
     * If there is a message in the queue, this returns the size of
     * the message.  If there is no message in the queue, this returns
     * 0.
     */
    int hasMessage();
	
    /** 
     * Clients call this to connect to a particular host on the
     * server.  They should then call sendMessage/receiveMessage to
     * talk with the server.
     */
    
    long connectTo( const char *hostName, const u_short port );
    
    /**
     * Closes a socket connection.
     */
    virtual void close();

	virtual Socket* getSocket();
	
	virtual void setSocket( Socket* socket );
	
	long startAsServer( u_short port );

	void runAsServer(int thread) ;

	std::vector<Win32SocketImplementer*> m_clients;

	void addClient();

	void closeClient( Win32SocketImplementer *client ) ;
protected:
    
    SOCKET  m_sock;
    /**
	*Which tcp port are we going to use?
	*/
    u_short m_port; 
    
    static bool m_ready;		// = FALSE;
    
    static int  m_instances; 
    
    WSAEVENT m_eventObj;
    
    char m_sendBuffer[512];

private:
	VCF::String m_host;
	VCF::String m_localHost;
	Socket* m_socket;
	Socket::SocketState m_state;
};


};

#endif 
