/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32SocketPeer.h

#ifndef _WIN32SOCKETPeer_H__
#define _WIN32SOCKETPeer_H__

#include <winsock2.h> //makes sure to link with Ws2_32.lib
#include "Object.h"
#include "SocketPeer.h"
#include "Socket.h"
#include "Win32Peer.h"
#include <memory.h>
#include <process.h>
#include <vector>

namespace VCFNet
{


#define MAX_PENDING_CONNECTS SOMAXCONN

class NETKIT_API Win32SocketPeer : public VCF::Object, public SocketPeer  
{
public:
	/**
	*if host = VCFNET_NOHOST, and port doesn't = VCFNET_NOPORT,
	*then start as a server socket.
	*If host is valid (!= VCFNET_NOHOST) and port is valid (!=VCFNET_NOPORT)
	*then assume the socket is a client socket, connecting to 
	*the specified host/port
	*/
	Win32SocketPeer( const VCF::String& host=VCFNET_NOHOST, 
		             const int& port=VCFNET_NOPORT);

	virtual ~Win32SocketPeer();

	virtual int getPort();

	virtual void setPort( const int& port );

	virtual VCF::String getHost();

	virtual void setHost( const VCF::String& host );	

	/** 
     * Receives data from a socket. Blocks until data has
     * been received.
     *
     * @param data - the buffer to hold the message (allocated by caller).
     * @param size - set to the length of the message that was read. 
     *
     * N.B. The caller is responsible for making sure that -buf- is large
     * enough to receieve messages.     
     */
    virtual int receive( char* data, VCF::uint32& size );
    
    
    /**
	*Sends a data down a socket. Blocks until the message has been sent.
	*/
    virtual int send( const char* data, const VCF::uint32& size );	

	static void startup();
	
    /** 
     * Clients call this to connect to a particular host on the
     * server.  They should then call sendMessage/receiveMessage to
     * talk with the server.
     */
    
    virtual long connectTo( const VCF::String& hostName, const VCF::ushort port );
    
    /**
     * Closes a socket connection.
     */
    virtual void close();

	virtual long startAsServer( VCF::ushort port );

	/**
	*returns the number of clients waiting.
	*If there are no clients then it returns 0.
	*/
	virtual int clientWaiting ();

	void runAsServer(int thread);
	
	void runServerLoop();

	/**
	*returns the socket identifying the client - the socket id may not be the 
	*same as the socket id on the client machine/process
	*/
	virtual int getClient();
protected:    
    SOCKET  m_sock;    
    VCF::int32 m_port;
	struct sockaddr_in m_sin;
	std::vector<int> m_clientList;

	void checkForPendingData();
private:
	VCF::String m_host;
	VCF::String m_localHost;
	Socket* m_socket;
	Socket::SocketState m_state;
};


};

#endif 
