/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32ToolKit.h

#ifndef _WIN32TOOLKIT_H__
#define _WIN32TOOLKIT_H__

#include "Win32Peer.h"
#include "UIToolkit.h"

namespace VCF
{


class APPKIT_API Win32ToolKit : public UIToolkit  
{
public:
	Win32ToolKit();
	virtual ~Win32ToolKit();
	
	virtual Color* getSystemColor( const unsigned long& systemColor );
	
	virtual ApplicationPeer* createApplicationPeer();
	
	virtual TextPeer* createTextPeer( TextControl* component, ComponentType componentType=CT_DEFAULT);

	virtual TreePeer* createTreePeer( TreeControl* component, ComponentType componentType=CT_DEFAULT);

	virtual ListBoxPeer* createListboxPeer( ListBoxControl* component, ComponentType componentType=CT_DEFAULT);

	virtual ListviewPeer* createListViewPeer( ListViewControl* component, ComponentType componentType=CT_DEFAULT);

	virtual DialogPeer* createDialogPeer( Frame* owner, Dialog* component, ComponentType componentType=CT_DEFAULT );

	virtual DialogPeer* createDialogPeer();

	HWND getDummyParent();
	
	virtual MenuItemPeer* createMenuItemPeer( MenuItem* item );

	virtual MenuBarPeer* createMenuBarPeer( MenuBar* menuBar );

	virtual PopupMenuPeer* createPopupMenuPeer( PopupMenu* popupMenu );

	virtual ComboBoxPeer* createComboBoxPeer( ComboBoxControl* component, ComponentType componentType=CT_DEFAULT);

	virtual ButtonPeer* createButtonPeer( CommandButton* component, ComponentType componentType);

	virtual ContextPeer* createContextPeer( Control* component );

	virtual CommonFileDialogPeer* createCommonFileOpenDialogPeer( Control* owner );

	virtual CommonFileDialogPeer* createCommonFileSaveDialogPeer( Control* owner );

	virtual CommonColorDialogPeer* createCommonColorDialogPeer( Control* owner );

	virtual CommonFolderBrowseDialogPeer* createCommonFolderBrowseDialogPeer( Control* owner );

	virtual CommonFontDialogPeer* createCommonFontDialogPeer( Control* owner );

	virtual DragDropPeer* createDragDropPeer(); 

	virtual DataObjectPeer* createDataObjectPeer();

	virtual DropTargetPeer* createDropTargetPeer();

private:
	void loadSystemColors();
	void createDummyParentWindow();
	HWND m_dummyParentWnd;	
};

};

#endif 
