/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// BasicInputStream.h: interface for the BasicInputStream class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _BASICINPUTSTREAM_H__
#define _BASICINPUTSTREAM_H__


#include "Stream.h"
#include "MemStreamUtils.h"

namespace VCF
{

class FRAMEWORK_API BasicInputStream : public InputStream
{
public:
	BasicInputStream();

	BasicInputStream( const String& textBuffer );

	BasicInputStream( char* dataBuffer, const unsigned long& dataBufferSize ); 

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the BasicInputStream to the inStream.
	*So if a read is called on the BasicInputStream, it simply
	*passes it to the inStream's read() method
	*/
	BasicInputStream( InputStream* inStream );

	virtual ~BasicInputStream();

	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

	virtual unsigned long getSize();

	virtual char* getBuffer();

	virtual void read( char* bytesToRead, unsigned long sizeOfBytes );	

	void init();
private:	
	CharMemStream m_inStream;
	InputStream* m_inputStream;
	unsigned long m_size;
};

};

#endif 

