/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// BasicOutputStream.h

#ifndef _BASICOUTPUTSTREAM_H__
#define _BASICOUTPUTSTREAM_H__


#include "Stream.h"
#include "MemStreamUtils.h"

namespace VCF
{

class FRAMEWORK_API BasicOutputStream : public OutputStream  
{
public:
	BasicOutputStream();

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the BasicOutputStream to the outStream.
	*So if a write is called on the BasicOutputStream, it simply
	*passes it to the outStream's write() method
	*/
	BasicOutputStream( OutputStream* outStream );

	virtual ~BasicOutputStream();

	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

	virtual unsigned long getSize();

	virtual char* getBuffer();

	virtual void write( char* bytesToRead, unsigned long sizeOfBytes );

	void init();
private:
	CharMemStream m_outStream;
	OutputStream* m_outputStream;
	unsigned long m_size;
};


};

#endif 
