/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ComponentOutputStream.h

#ifndef _COMPONENTOUTPUTSTREAM_H__
#define _COMPONENTOUTPUTSTREAM_H__

#include "Stream.h"
#include "Persistable.h"

namespace VCF
{

class Control;

class Persistable;

class Property;

class VariantData;

class APPKIT_API ComponentOutputStream : public OutputStream  
{
public:
	ComponentOutputStream( OutputStream* outStream );

	virtual ~ComponentOutputStream();

	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

	virtual unsigned long getSize();

	virtual char* getBuffer();

	virtual void write( char* bytesToWrite, unsigned long sizeOfBytes );	
	
	virtual void write( Persistable* persistableObject ){
		OutputStream::write( persistableObject );
	};

	virtual void write( const short& val );
	
	virtual void write( const long& val );

	virtual void write( const int& val );

	virtual void write( const bool& val );

	virtual void write( const char& val );

	virtual void write( const float& val );

	virtual void write( const double& val );

	virtual void write( const String& val );

	/**
	*Writing a component out to a ComponentOutputStream takes 
	*the following steps:
	*	1.	Open a <object> tag and write out the class name (<object class="component::ClassName" )
	*	2.	Get the Class for the component and then write out the properties
	*	3.	For each property write out a <property> tag with the name of the 
	*		property (<property name="propertyName" ). If the value of the property
	*		is a basic type (i.e. int, double, etc ), then write out the string value
	*		of the property. If the property value is a Object* then the value should be the 
	*		class name of the Object* ( <property name="listModel" value="DefaultListModel"> ).
	*	4.	If the value of the property is a Object* then open a new <objectProperty> tag. Follow
	*		steps 2 & 3. Close out the tag with a </objectProperty> tag.
	*	5.	Close the <property> tag with a </property> tag.
	*	6.	Close the <object> tag with </object>.
	*/
	void writeControl( Control * component );
private:
	OutputStream* m_outStream;
	void writeProperty( Property* property, Object* source );
	void writePropertyValue( const String& propName, VariantData* value );
};

};

#endif 
