/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef FILESTREAM_H
#define FILESTREAM_H

#include "Stream.h"
#include "FileStreamPeer.h"
namespace VCF

{

/**
*special file stream that uses petal-to-the-metal native OS calls
*for file IO as opposed to the more generic C++ filestream class.
*This class uses a FileStreamPeer to do the actual work
*
*@author Jim Crafton
*@version 1.0
*/
class FRAMEWORK_API FileStream : public InputStream, public OutputStream, public Object {
public:
	FileStream( const String& filename, const FileStreamAccessType& accessType );	

	FileStream( const String& filename );

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the FileStream to the outStream.
	*So if a write is called on the FileStream, it simply
	*passes it to the outStream's write() method
	*/
	FileStream( const String& filename, OutputStream* outStream );

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the FileStream to the inStream.
	*So if a read is called on the FileStream, it simply
	*passes it to the inStream's read() method
	*/
	FileStream( const String& filename, InputStream* inStream );

	virtual ~FileStream();


	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

	/**
	*returns the size of the stream. The size represents the 
	*number of bytes that have been <b>written</b>, or <b>read</b> to/from the stream
	*/
    virtual unsigned long getSize();

	/**
	*returns a pointer to the buffer that holds the stream memory
	*/
	virtual char* getBuffer();

	//input
	virtual void read( char* bytesToRead, unsigned long sizeOfBytes );

	//output
	virtual void write( char* bytesToWrite, unsigned long sizeOfBytes );

	void init();

	/**
	*closes the file stream
	*/
	void close();

	/**
	*opens a file stream
	*/
	void open( const String& filename, const FileStreamAccessType& accessType );

	void open( const String& filename );
private:
	FileStreamPeer* m_fsPeer;
	String m_filename;
	FileStreamAccessType m_access;
	OutputStream* m_outputStream;
	InputStream* m_inputStream;

};


};
#endif //FILESTREAM_HFILESTREAM_H