/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//MemStreamUtils.h

/**
* @author  Jim Crafton
* @date 3/7/2000
* @version 1.0
*/

#ifndef _MEMSTREAMUTILS_H__
#define _MEMSTREAMUTILS_H__
#include "FrameworkConfig.h"
#include <iostream>

/**
*BasicMemoryBuffer
*/
template <class Ch> class FRAMEWORK_API BasicMemoryBuffer : public std::basic_streambuf<Ch> 
{
public:

	BasicMemoryBuffer( unsigned long allocationSize = 65536 ) : 
		std::basic_streambuf<Ch>(),
		m_allocationSize( allocationSize ),
		m_currentSize(0)
	{
	};

	virtual ~BasicMemoryBuffer()
	{
		Ch *pBase = pbase();
		if ( NULL != pBase) {
			delete pBase;
		}
	};

    Ch* getBuffer()
	{
		return pbase();
	};

protected:

	virtual int overflow( int nCh )
	{
		int res = (int) std::char_traits<Ch>::eof;
		unsigned long lastSize = m_currentSize;
		m_currentSize += m_allocationSize;
		Ch *p = new Ch[m_currentSize];
		if ( NULL != p) {
			memset( p, NULL, sizeof(Ch) * m_currentSize);
			Ch *pBase = pbase();
			if ( NULL != pBase) {
				memcpy( p, pBase, (sizeof(Ch) * lastSize) );
				delete pBase;
			}
			//setp ( p, p + (sizeof(Ch) * lastSize), p + (sizeof(Ch) * m_currentSize) );
			setp ( p, p + (sizeof(Ch) * m_currentSize) );
			pbump(sizeof(Ch) * lastSize);
			sputc( nCh);
			setg ( p, p, p + (sizeof(Ch) * m_currentSize));
			res = (int) std::char_traits<Ch>::not_eof;
		}
		return res;
	};
	
private:

	unsigned long m_allocationSize;
	unsigned long m_currentSize;
};

/**
*MemStream
*/
template <class Ch> class FRAMEWORK_API MemStream : public std::basic_iostream<Ch>
{
public:

	MemStream( unsigned long allocationSize = 65536) :
		m_membuf ( allocationSize),
		std::basic_iostream<Ch> ((std::basic_streambuf<Ch> *) &m_membuf )
	{
	};
	virtual ~MemStream()
	{
	};

	Ch* getBuffer()
	{
		return m_membuf.getBuffer();
	};

private:

	BasicMemoryBuffer<Ch> m_membuf;
};



/**
*CharMemStream
*/
class FRAMEWORK_API CharMemStream : public MemStream<char>
{
public:
	CharMemStream( unsigned long allocationSize = 65536 ) :
		MemStream<char> ( allocationSize )
	{
	};
};



#endif // __MemStream_h__
