/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef MEMORYSTREAM_H
#define MEMORYSTREAM_H

#include "Stream.h"
#include "MemStreamUtils.h"

namespace VCF{

class FRAMEWORK_API MemoryStream : public VCF::InputStream, public VCF::OutputStream, public VCF::Object {	
public:
	MemoryStream();

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the MemoryStream to the outStream.
	*So if a write is called on the MemoryStream, it simply
	*passes it to the outStream's write() method
	*/
	MemoryStream( OutputStream* outStream );

	/**
	*This constructor is used in stream chaining.
	*The flow goes from the MemoryStream to the inStream.
	*So if a read is called on the MemoryStream, it simply
	*passes it to the inStream's read() method
	*/
	MemoryStream( InputStream* inStream );

	virtual ~MemoryStream();

	virtual char* getBuffer(){ 
		return NULL;
	};

	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

    virtual unsigned long getSize();

	virtual void write( Persistable* persistableObject );

	virtual void write( char* bytesToWrite, unsigned long sizeOfBytes );

	virtual void read( char* bytesToRead, unsigned long sizeOfBytes );	

	void init();
private:
	CharMemStream m_stream;
	OutputStream* m_outputStream;
	InputStream* m_inputStream;
	unsigned long m_size;
};

};

#endif //MEMORYSTREAM_H