/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// TextInputStream.h

#ifndef _TEXTINPUTSTREAM_H__
#define _TEXTINPUTSTREAM_H__


#include "Stream.h"

namespace VCF
{

/**
*TextInputStream is used for interpreting data from another stream as text.
*This assumes the data in the stream was originally written as text. For example,
*a FileStream that has been used to write a binary file, and is then passed into
*a new instance of TextInputStream to read the same file, will cause unpredicatable
*results in the decoding of the stream, since the TextInputStream is taking advantage of
*certain layout assumptions in how the source stream is organized.
*So consider yourself warned !
*/
class FRAMEWORK_API TextInputStream : public InputStream  
{
public:	
	TextInputStream( InputStream* inStream );

	virtual ~TextInputStream();

	virtual void seek(const unsigned long& offset, const SeekType& offsetFrom);

	virtual unsigned long getSize();

	virtual char* getBuffer();

	virtual void read( char* bytesToRead, unsigned long sizeOfBytes );
	
	/**
	*all numbers are written out as the number converted to text, followed by a 
	*space character ( ' ' ), so the value 2 becomes "2 "
	*/

	TextInputStream& operator>>( short& val );

	TextInputStream& operator>>( long& val );

	TextInputStream& operator>>( int& val );

	TextInputStream& operator>>( bool& val );

	TextInputStream& operator>>( char& val );

	TextInputStream& operator>>( float& val );

	TextInputStream& operator>>( double& val );

	virtual void read( short& val );
	
	virtual void read( long& val );

	virtual void read( int& val );

	virtual void read( bool& val );	

	virtual void read( float& val );

	virtual void read( double& val );

	virtual void read( String& val );

	void init();

	void readLine( String& line );
private:
	InputStream* m_inStream;
	unsigned long m_size;
	unsigned long m_totCharRead;
	String readTillWhiteSpace();

	/**
	*reads to the starting point of the token and then accumalates anything in the buffer
	*till the next occurence of the token
	*@return String the accumalted text
	*/
	String readTillTokenPair( const char& token );
};


};

#endif 

