/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */
//Enumerator.h

#ifndef ENUMERATOR_H
#define ENUMERATOR_H

#include <algorithm>
#include "FrameworkConfig.h"

#pragma warning (disable : 4786)

namespace VCF{
/**
 * This is a template class that provides an interface for 
 * iterating through a collection, and provides a standard set of methods. 
 * Either multiply inherited by classes that want to public expose a single set of children,
 *or as  a returned object for safe enumeration of data collections.
 */

template <class COLLECTION_TYPE> class FRAMEWORK_API Enumerator{
public:
	Enumerator(){
		m_supportsEditing = false;
	};

	virtual ~Enumerator(){};
	
	virtual bool hasMoreElements(const bool& backward=false)=0;
	
	virtual COLLECTION_TYPE nextElement()=0;
	
	virtual COLLECTION_TYPE prevElement()=0;
	
	virtual void reset(const bool& backward=false)=0;	

	bool supportsEditing(){
		return m_supportsEditing;
	};
protected:
	bool m_supportsEditing;
};



/**
*EnumeratorContainer is based on some COLLECTION class that represents a collection
*class of some sort, i.e. std::vector<>, std::map, et al.
*The internal implementation counts on the collection having a common interface to 
*STL's collection classes
*/

template <class COLLECTION, class COLLECTION_TYPE> class FRAMEWORK_API EnumeratorContainer : public Enumerator<COLLECTION_TYPE>{
public:
	
	EnumeratorContainer( COLLECTION &container ){
		m_container = &container;
		this->reset();
		m_supportsEditing = true;
	};

	EnumeratorContainer(){
		m_supportsEditing = true;
	};

	virtual ~EnumeratorContainer(){};

	virtual void initContainer( COLLECTION &container ){
		m_container = &container;
		this->reset();	
	};

	virtual bool hasMoreElements(const bool& backward=false){
		bool result = false;
		if ( true == backward ){
			result = m_containerIterator != m_container->begin();
		}
		else{
			result = m_containerIterator != m_container->end();
		}
		return result;
	};

	virtual COLLECTION_TYPE nextElement(){
		if (m_containerIterator != m_container->end() ){			
			return *m_containerIterator++;
		}
		else {
			return NULL;
		}
	};

	virtual COLLECTION_TYPE prevElement(){
		if (m_containerIterator != m_container->begin() ){			
			return *--m_containerIterator;
		}		
		else {
			return NULL;
		}
	};

	virtual void reset(const bool& backward=false){
		if ( true != backward ){
			m_containerIterator = m_container->begin();
		}
		else {
			m_containerIterator = m_container->end();			
		}
	};

	virtual Enumerator<COLLECTION_TYPE>* getEnumerator(){
		this->reset();
		return (Enumerator<COLLECTION_TYPE>*)this;
	};
private:
	COLLECTION::iterator m_containerIterator;	
	COLLECTION* m_container;
};


template <class COLLECTION, class COLLECTION_TYPE> class FRAMEWORK_API EnumeratorMapContainer : public Enumerator<COLLECTION_TYPE>{
public:
	EnumeratorMapContainer( COLLECTION &container ){
		m_container = &container;
		this->reset();
	};

	EnumeratorMapContainer(){};

	virtual ~EnumeratorMapContainer(){};

	virtual void initContainer( COLLECTION &container ){
		m_container = &container;
		this->reset();	
	};

	virtual Enumerator<COLLECTION_TYPE>* getEnumerator(){
		this->reset();
		return (Enumerator<COLLECTION_TYPE>*)this;
	};

	virtual bool hasMoreElements(const bool& backward=false){
		bool result = false;
		if ( true == backward ){
			result = m_containerIterator != m_container->begin();
		}
		else{
			result = m_containerIterator != m_container->end();
		}
		return result;
	};

	virtual COLLECTION_TYPE nextElement(){
		if (m_containerIterator != m_container->end() ){
			COLLECTION_TYPE result = m_containerIterator->second;
			m_containerIterator ++;
			return result;
		}
		else {
			return NULL;
		}
	};

	virtual COLLECTION_TYPE prevElement(){
		if (m_containerIterator != m_container->begin() ){
			COLLECTION_TYPE result = m_containerIterator->second;
			m_containerIterator --;
			return result;
		}
		else {
			return NULL;
		}
	};

	virtual void reset(const bool& backward=false){
		if ( true != backward ){
			m_containerIterator = m_container->begin();
		}
		else {
			m_containerIterator = (m_container->end());			
		}
	};

private:
	COLLECTION::iterator m_containerIterator;	
	COLLECTION* m_container;
};


};

#endif //ENUMERATOR_H