/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _REGISTRY_H__
#define _REGISTRY_H__

#include "VCF.h"
#include "Enumerator.h"
#include "Object.h"
#include "Property.h"

namespace VCF {

enum RegistryKeyType{
	RKT_ROOT=0,
	RKT_CURRENT_USER,
	RKT_LOCAL_MACHINE
};

enum RegistryDataType{
	RDT_STRING = 0,
	RDT_INTEGER,
	RDT_BOOL,
	RDT_BINARY
};


class FRAMEWORK_API RegistryValueInfo : public Object {
public:

	RegistryValueInfo( VariantData* data, const RegistryDataType& dataType, 
		               const String& valueName, void* dataBuffer=NULL, const uint32& dataBufferSize=0  ){		
		
		m_dataType = dataType;

		m_valueName = valueName;
		
		m_dataBufferSize = dataBufferSize;

		m_dataBuffer = NULL;

		if ( NULL != data ){
			switch ( m_dataType ){
				case RDT_STRING :{
					m_data = (String)*data;
				}
				break;

				case RDT_INTEGER :{
					m_data = (ulong32)*data;
				}
				break;

				case RDT_BOOL :{
					m_data = (bool)*data;
				}
				break;

				case RDT_BINARY :{
					m_dataBuffer = dataBuffer;
				}
				break;
			}	
		}
	}


	virtual ~RegistryValueInfo(){
		if ( NULL != m_dataBuffer ){
			delete m_dataBuffer;
		}
	}

	RegistryDataType m_dataType;

	VariantData m_data; 

	String m_valueName;

	void* m_dataBuffer;

	uint32 m_dataBufferSize;
};



class RegistryPeer;


class FRAMEWORK_API Registry  : public Object{

public:
	Registry();

	virtual ~Registry();

	void setRoot( const RegistryKeyType& rootKey );
	
	RegistryKeyType getRoot();

	bool openKey( const String& keyname, const bool& createIfNonExistant=false );

	bool setValue( const String& value, const String& valuename ); 

	bool setValue( const uint32& value, const String& valuename ); 
	
	bool setValue( const bool& value, const String& valuename ); 

	bool setValue( void* dataBuffer, const uint32& dataBufferSize, const String& valuename ); 

	String getStringValue( const String& valuename );

	uint32 getIntValue( const String& valuename );

	bool getBoolValue( const String& valuename );

	void getDataBufValue( const String& valuename, uint32& dataBufferSize, void** dataBuffer );

	Enumerator<String>* getKeyNames();

	Enumerator<RegistryValueInfo*>* getValues();

	String getCurrentKey();
private:
	RegistryPeer* m_peer;
};

}; //end of namespace VCF

#endif //_REGISTRY_H__