/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ResourceBundle.h

#ifndef _RESOURCEBUNDLE_H__
#define _RESOURCEBUNDLE_H__


#include "Object.h"

namespace VCF
{

class Resource;
class Image;
/**
*A ResourceBundle is used to represent a collection of resources. ResourceBundle's can 
*be attached to external files or applications. 
*/
class APPKIT_API ResourceBundle : public Object  
{
public:
	ResourceBundle(){};

	virtual ~ResourceBundle(){};

	/**
	*returns a resource string given a resource name
	*/
	virtual String getString( const String& resourceName ) = 0;

	/**
	*returns the text that represents a Form. This text is standard XML and contains
	*the component hierarchy and properties for creating a frame.
	*/
	virtual String getVFF( const String& resourceName ) = 0;

	/**
	*returns an Image based on a given resource name
	*/
	virtual Image* getImage( const String& resourceName ) = 0;

	/**
	* returns a Resource object given the resource's name.
	*/ 
	virtual Resource* getResource( const String& resourceName ) = 0;

};


};
#endif 
