/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */



#ifndef UIGRAPHICSTOOLKIT_H
#define UIGRAPHICSTOOLKIT_H


#include "Object.h"
#include "ContextPeer.h"
#include "FontPeer.h"
#include "Image.h"
#include "ImageLoader.h"
#include "GraphicsContext.h"
#include "Rect.h"
#include <map>

namespace VCF{

class OpenGLPeer;

class OpenGLContext;

class GRAPHICSKIT_API UIGraphicsToolkit : public Object{
public:
	UIGraphicsToolkit();

	virtual ~UIGraphicsToolkit();

    static UIGraphicsToolkit* getDefaultGraphicsToolkit();

	virtual ContextPeer* createContextPeer( const unsigned long& width, const unsigned long& height );	

	virtual ContextPeer* createContextPeer( const long& contextID );

	virtual FontPeer* createFontPeer( const String& fontName );

	virtual FontPeer* createFontPeer( const String& fontName, const double& pointSize );
	
	virtual OpenGLPeer* createOpenGLPeer( GraphicsContext* glContext );
	/**
	*Create a blank image - this will currently return a bitmap
	*/
	virtual Image* createImage();

	virtual Image* createImage( const unsigned long& width, const unsigned long& height );

	virtual Image* createImage( GraphicsContext* context, Rect* rect );
	/**
	*Create a image from a filename. The file is loaded into a Bitmap instance.
	*The toolkit looks up the file extension and matches the type to 
	*an ImageLoader instance. The ImageLoader does the work of actually decoding the image
	*/
	virtual Image* createImage( const String& fileName );

	void registerImageLoader( const String& name, ImageLoader* imageLoader );

	ImageLoader* getImageLoader( const String& contentType );
private:
	std::map<String,String> m_contentTypes;
	
	void initContentTypes();

	static UIGraphicsToolkit graphicsToolkitInstance;
	/**
	*ImageLoaders are stored in map, keyed by the MIME content type
	*Thus JPEG loaders are stored under the key: 
	*<p><pre>
	*	"image/jpeg"
	*</pre></p>
	*/
	static std::map<String, ImageLoader*> graphicsToolkitImageLoaders;
};

};
#endif //UIGRAPHICSTOOLKIT_HSTOOLKIT_H