/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#ifndef UITOOLKIT_H
#define UITOOLKIT_H                         

#include "Object.h"
#include "NoToolKitFoundException.h"
#include <map>


enum ComponentType{
        CT_DEFAULT=0,
        CT_LIGHTWEIGHT,
        CT_HEAVYWEIGHT
};

namespace VCF{

#define STANDARD_CATEGORY		"Standard"
#define ADDITIONAL_CATEGORY		"Additional"
#define SYSTEM_CATEGORY			"System"
#define DIALOG_CATEGORY			"Dialogs"
#define NETWORKING_CATEGORY		"Networking"

class ListviewPeer;
class ListBoxPeer;
class ComboBoxPeer;
class TextPeer;
class RichTextPeer;
class ControlPeer;
class TreePeer;
class ButtonPeer;
class ScrollbarPeer;
class TabbedPagesPeer;
class HeaderPeer;
class DialogPeer;
class WindowPeer;
class ToolbarPeer;
class ClipboardPeer;
class DragDropPeer;
class DataObjectPeer;
class SystemPeer;
class System;
class DropTargetPeer;
class Component;
class ContextPeer;
class ApplicationPeer;
class TextControl;
class TreeControl;
class ListBoxControl;
class ListViewControl;
class Dialog;
class MenuItem;
class MenuItemPeer;
class MenuBar;
class MenuBarPeer;
class ComboBoxControl;
class CommandButton;
class CommonFileDialogPeer;
class CommonColorDialogPeer;
class Color;
class Clipboard;
class UIGraphicsToolkit;
class PopupMenuPeer;
class PopupMenu;
class Frame;
class Control;
class CommonFolderBrowseDialogPeer;
class CommonFontDialogPeer;

class APPKIT_API UIToolkit : public Object{


public:

    UIToolkit();

	virtual ~UIToolkit();

	virtual void init();
	/**
	*Initializes the toolkit. the default toolkit is created and any other initialization
	*functions called
	*/
	static void initToolKit();
	
	/**
	*cleans up the toolkit instance and deletes it
	*/
	static void closeToolkit();
	/**
	*creates a new instance of a ControlPeer
	*the component passed in represents the component the implmenter will get attached to.
	*/
    virtual ControlPeer* createControlPeer( Control* component, ComponentType componentType=CT_DEFAULT);

	virtual ContextPeer* createContextPeer( Control* component );
	
	virtual ContextPeer* createContextPeer( const long& contextID );

	virtual ContextPeer* createContextPeer( const unsigned long& width, const unsigned long& height );

    virtual ListviewPeer* createListViewPeer( ListViewControl* component, ComponentType componentType=CT_DEFAULT);

    virtual TreePeer* createTreePeer( TreeControl* component, ComponentType componentType=CT_DEFAULT);

    virtual ListBoxPeer* createListboxPeer( ListBoxControl* component, ComponentType componentType=CT_DEFAULT);

    virtual ComboBoxPeer* createComboBoxPeer( ComboBoxControl* component, ComponentType componentType=CT_DEFAULT);

    virtual TextPeer* createTextPeer( TextControl* component, ComponentType componentType=CT_DEFAULT);

    virtual RichTextPeer* createRTFPeer( ComponentType componentType=CT_DEFAULT);

    virtual ScrollbarPeer* createScrollbarPeer( ComponentType componentType=CT_DEFAULT);

    virtual ButtonPeer* createButtonPeer( CommandButton* component, ComponentType componentType=CT_DEFAULT);
	
    virtual HeaderPeer* createHeaderPeer( ComponentType componentType=CT_DEFAULT);

    virtual TabbedPagesPeer* createTabsheetsPeer( ComponentType componentType=CT_DEFAULT);

    virtual DialogPeer* createDialogPeer( Frame* owner, Dialog* component, ComponentType componentType=CT_DEFAULT);

	virtual DialogPeer* createDialogPeer();    

    virtual WindowPeer* createWindowPeer( Control* component, ComponentType componentType=CT_DEFAULT);
    
	virtual MenuItemPeer* createMenuItemPeer( MenuItem* item );

	virtual MenuBarPeer* createMenuBarPeer( MenuBar* menuBar );

	virtual PopupMenuPeer* createPopupMenuPeer( PopupMenu* popupMenu );

    virtual ClipboardPeer* createClipboardPeer();

    virtual DragDropPeer* createDragDropPeer(); 

	virtual DataObjectPeer* createDataObjectPeer();

	virtual DropTargetPeer* createDropTargetPeer();

	virtual SystemPeer* getSystemPeer();

	virtual ApplicationPeer* createApplicationPeer();

	virtual Clipboard* getSystemClipboard();

	virtual System* getSystem();		

	virtual Color* getSystemColor( const unsigned long& systemColor );
	
	virtual CommonFileDialogPeer* createCommonFileOpenDialogPeer( Control* owner );

	virtual CommonFolderBrowseDialogPeer* createCommonFolderBrowseDialogPeer( Control* owner );

	virtual CommonFileDialogPeer* createCommonFileSaveDialogPeer( Control* owner );

	virtual CommonColorDialogPeer* createCommonColorDialogPeer( Control* owner );

	virtual CommonFontDialogPeer* createCommonFontDialogPeer( Control* owner );
	/*
	*gets t
	*/
	unsigned long getSystemTickCount();

	static UIToolkit* getDefaultUIToolkit() throw(NoToolKitFoundException);

protected:
	std::map<unsigned long,Color*> m_systemColors;

private:    
	static UIToolkit* toolKitInstance;
    /**
     * @supplierCardinality 1 
     */
	//static UIToolkit toolKitInstance;	

	Clipboard* m_systemClipboard;
	
    UIGraphicsToolkit * m_graphicsToolKit;

	System* m_system;
};

};
#endif //UITOOLKIT_H
