/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#include "ClipboardDataObject.h"
#include "DataType.h"
#include "UIToolkit.h"
#include <algorithm>

using namespace VCF;


ClipboardDataObject::ClipboardDataObject( const char* dataToInitWith,  const unsigned long& dataSize, const unsigned long& dataType )
{
	this->m_dataSize = dataSize;
	this->m_rawData = new char[ m_dataSize ];
	strncpy( m_rawData, dataToInitWith, m_dataSize );
	
	this->init();

	this->addSupportedDataType( dataType );
}


ClipboardDataObject::~ClipboardDataObject()
{
	delete[] this->m_rawData;
	for ( std::vector<DataType*>::iterator it = m_types.begin(); it != m_types.end(); it++ ){
		delete (*it);		
	}
	m_types.clear();
}

void ClipboardDataObject::init()
{

}

void ClipboardDataObject::addSupportedDataType( const unsigned long & dataType )
{
	this->m_types.push_back( new DataType( dataType ) );	
}
	
bool ClipboardDataObject::isTypeSupported( const unsigned long & dataType )
{
	bool result = false;
	
	for (std::vector<DataType*>::iterator it = m_types.begin(); it != m_types.end(); it ++ ){
		DataType* dataTypeObj = *it;
		if ( dataType == dataTypeObj->getRawType() ){
			result = true;
			break;
		}
	}

	return result;
}

void ClipboardDataObject::saveToStream( OutputStream * stream )
{
	stream->write( m_rawData, m_dataSize );
}

void ClipboardDataObject::loadFromStream( InputStream * stream )
{
	delete[]m_rawData;
	m_rawData = new char[m_dataSize];
	stream->read( m_rawData, m_dataSize );
}

Enumerator<DataType*> * ClipboardDataObject::getSupportedDataTypes()
{
	m_typesContainer.initContainer( m_types );
	return m_typesContainer.getEnumerator();	
}