/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Basic3DBorder.cpp

#ifdef WIN32
#pragma warning (disable : 4786)
#endif

#include "Basic3DBorder.h"
#include "UIToolkit.h"
#include "StrokeState.h"
#include "FillState.h"	

using namespace VCF;


Basic3DBorder::Basic3DBorder()
{
	
}

Basic3DBorder::~Basic3DBorder()
{
	
}

Rect* Basic3DBorder::getClientRect( Control* component )
{
	Rect* result = NULL;
	if ( NULL != component ){
		result = component->getClientBounds();
		result->inflate( -2.0, -2.0 );
	}
	return result;
}

void Basic3DBorder::paint( Rect* bounds, GraphicsContext* context )
{
	if ( NULL != bounds ){ 
		UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();
		Color* hilight = toolkit->getSystemColor( SYSCOLOR_HIGHLIGHT );
		
		Color* shadow = toolkit->getSystemColor( SYSCOLOR_SHADOW );
		
		Color* face = toolkit->getSystemColor( SYSCOLOR_FACE );
		
		StrokeState stroke;
		
		FillState fill;
		fill.m_Color = *face;
		context->rectangle( bounds->m_left, bounds->m_top, bounds->m_right, bounds->m_bottom );
		context->setCurrentFillState( &fill );
		context->fillPath();
		
		stroke.m_Color = *hilight;
		
		context->moveTo( bounds->m_left+1, bounds->m_bottom-2 );	
		context->lineTo( bounds->m_left+1, bounds->m_top+1 );	
		context->lineTo( bounds->m_right-2, bounds->m_top+1 );
		
		context->setCurrentStrokeState( &stroke );
		
		context->strokePath();
		
		context->moveTo( bounds->m_right-2, bounds->m_top+1 );	
		context->lineTo( bounds->m_right-2, bounds->m_bottom-2 );	
		context->lineTo( bounds->m_left+1, bounds->m_bottom-2 );
		
		stroke.m_Color = *shadow;
		context->setCurrentStrokeState( &stroke );
		context->strokePath();
		
		context->moveTo( bounds->m_right-1, bounds->m_top );	
		context->lineTo( bounds->m_right-1, bounds->m_bottom-1 );	
		context->lineTo( bounds->m_left, bounds->m_bottom-1 );
		
		stroke.m_Color.setRGB( 0.0, 0.0, 0.0 );
		context->setCurrentStrokeState( &stroke );
		
		context->strokePath();
	}
}

void Basic3DBorder::paint( Control* component, GraphicsContext* context )
{
	if ( NULL == component ){
		//throw exception
	}
	Rect* bounds = component->getClientBounds();
	if ( NULL != bounds ){
		paint( bounds, context );
	}
}