/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "BasicStroke.h"
#include "GraphicsContext.h"

using namespace VCF;


BasicStroke::BasicStroke()
{

}

BasicStroke::~BasicStroke()
{

}

void BasicStroke::init()
{
	m_context = NULL;
	m_width  = 0.0;
	m_color = *Color::getColor( BLACK );	
}

void BasicStroke::setContext( GraphicsContext* context )
{
	m_context = context;
}

void BasicStroke::render( Path * path )
{
	if ( (NULL != m_context) && (NULL != path) ){
		m_strokeState.m_Color = m_color;
		m_strokeState.m_width = m_width;
		
		m_context->setCurrentStrokeState( &m_strokeState );	
		Matrix2D* currentXFrm = m_context->getCurrentTransform();
		PathEnumerator* pathEnum = path->getPathEnumerator( currentXFrm );
		if ( NULL != pathEnum ){
			while ( true == pathEnum->hasMoreSegments() ){
				ulong32 count = pathEnum->getPointCount();
				Point* pts = NULL;
				if ( count > 0 ){
					pts = new Point[count];
				}
				PathEnumerator::SegmentType type = pathEnum->getSegment( pts );
				switch ( type ){
					case PathEnumerator::SEGMENT_MOVETO: {
						m_context->moveTo( pts[0].m_x, pts[0].m_y );
					}
					break;

					case PathEnumerator::SEGMENT_LINETO: {
						m_context->lineTo( pts[0].m_x, pts[0].m_y );
					}
					break;

					case PathEnumerator::SEGMENT_QUADTO: {
						
					}
					break;

					case PathEnumerator::SEGMENT_CUBICTO: {
						
					}
					break;

					case PathEnumerator::SEGMENT_CLOSE: {
						
					}
					break;
				}
				pathEnum->next();
				delete [] pts;
			}
		}
	}
	m_context->strokePath();
}

double BasicStroke::getWidth()
{
	return m_width;
}

void BasicStroke::setWidth( double width )
{
	m_width = width;
}

Color* BasicStroke::getColor()
{
	return &m_color;
}

void BasicStroke::setColor( Color* color )
{
	m_color = *color;
}

void BasicStroke::line( const double& x1, const double& y1,
		                const double& x2, const double& y2 )
{

}