/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#include "Color.h"

using namespace VCF;


std::map<String,Color*> Color::colorMap;
Color::ColorMapInitializer Color::colorMapInitializer;


Color::ColorMapInitializer::ColorMapInitializer()
{
	Color::colorMap[WHITE]		=	new Color( 1.0f, 1.0f, 1.0f );
	Color::colorMap[BLACK]		=	new Color( 0.0f, 0.0f, 0.0f );
	Color::colorMap[GREY]		=	new Color( 0.5f, 0.5f, 0.5f );
	Color::colorMap[YELLOW]		=	new Color( 1.0f, 1.0f, 0.0f );
	Color::colorMap[GREEN]		=	new Color( 0.0f, 1.0f, 0.0f );
	Color::colorMap[BLUE]		=	new Color( 0.0f, 0.0f, 1.0f );
	Color::colorMap[RED]		=	new Color( 1.0f, 0.0f, 0.0f );
	Color::colorMap[PURPLE]		=	new Color( 0.5f, 0.0f, 1.0f );
	Color::colorMap[LT_BLUE]	=	new Color( 0.0f, 1.0f, 1.0f );
}


Color::ColorMapInitializer::~ColorMapInitializer()
{
	std::map<String,Color*>::iterator it = Color::colorMap.begin();
	while ( it != Color::colorMap.end() ){
		Color* c = it->second;
		delete c;
		c = NULL;
		it ++;
	}		
	Color::colorMap.clear();
}

Color::Color()
{
	m_r = 0.0;	
	m_g = 0.0;
	m_b = 0.0;
}

Color::Color( const Color& color )
{
	this->m_b = color.m_b;
	this->m_g = color.m_g;
	this->m_r = color.m_r;
}

Color::Color(const float & r, const float & g, const float & b)
{
	this->m_b = b;
	this->m_g = g;
	this->m_r = r;
}

Color::Color(const double & h, const double & s, const double & v)
{
	
}

Color::Color(const double & c, const double & m, const double & y, const double & k)
{

}

Color::Color(const unsigned char & r, const unsigned char & g, const unsigned char & b)
{
	m_r = ((double)r) / 255.0; 
	m_g = ((double)g) / 255.0; 
	m_b = ((double)b) / 255.0; 
}

Color::Color(const unsigned long & color)
{	
#ifdef WIN32 //windows stores its color format in the following scheme 0x00BBGGRR
	m_r = ((unsigned char*)&color)[0] / 255.0;	
	m_g = ((unsigned char*)&color)[1] / 255.0;
	m_b = ((unsigned char*)&color)[2] / 255.0;
#endif
}



void Color::getHSV(double & h, double & s, double & v)
{

}

void Color::getHLS(double & h, double & l, double & s)
{

}

void Color::getCMYK(double & c, double & m, double & y, double & k)
{

}

void Color::getRGB(unsigned char & r, unsigned char & g, unsigned char & b)
{
	r = (unsigned char)(m_r * 255);
	g = (unsigned char)(m_g * 255);
	b = (unsigned char)(m_b * 255);
}

void Color::getRGB(double & r, double & g, double & b)
{
	r = this->m_r;
	g = this->m_g;
	b = this->m_b;
}

unsigned long Color::getRGB()   
{	
	unsigned long rgb = 0;
#ifdef WIN32 //windows stores its color format in the following scheme ox00BBGGRR
	((unsigned char*)(&rgb))[0] = (unsigned char)(m_r * 255);
	((unsigned char*)(&rgb))[1] = (unsigned char)(m_g * 255);
	((unsigned char*)(&rgb))[2] = (unsigned char)(m_b * 255);

#endif
	return rgb;
}

void Color::getLab()
{

}

void Color::getYUV()
{

}


void Color::setHSV( const double & h, const double & s, const double & v)
{

}

void Color::setHLS( const double & h, const double & l, const double & s)
{

}

void Color::setCMYK( const double & c, const double & m, const double & y, const double & k)
{

}

void Color::setRGB( const unsigned char & r, const unsigned char & g, const unsigned char & b)
{
	m_r = ((double)r) / 255.0; 
	m_g = ((double)g) / 255.0; 
	m_b = ((double)b) / 255.0; 
}

void Color::setRGB( const double & r, const double & g, const double & b)
{
	this->m_b = b;
	this->m_g = g;
	this->m_r = r;
}

void Color::setRGB( const unsigned long& rgb )
{
#ifdef WIN32 //windows stores its color format in the following scheme ox00BBGGRR
	m_r = ((unsigned char*)&rgb)[0] / 255.0;	
	m_g = ((unsigned char*)&rgb)[1] / 255.0;
	m_b = ((unsigned char*)&rgb)[2] / 255.0;
#endif
}

void Color::setLab()
{

}

void Color::setYUV()
{

}


void Color::brighter()
{

}


void Color::darker()
{

}

double Color::getRed()
{
	return m_r;
}

double Color::getGreen()
{
	return m_g;
}

double Color::getBlue()
{
	return m_b;
}

void Color::copy( Object* source )
{
	if ( NULL != source ){
		Color* color = dynamic_cast<Color*>( source );
		if ( NULL != color ){
			m_r = color->getRed();
			m_g = color->getGreen();
			m_b = color->getBlue();
		}
	}
}

Color* Color::getColor( const String& colorName )
{
	Color* result = NULL;
	std::map<String,Color*>::iterator found = Color::colorMap.find( colorName );
	if ( found != Color::colorMap.end() ){
		result = found->second;
	}
	return result;
}