/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Font.cpp

#include "Font.h"
#include "FontPeer.h"
#include "UIGraphicsToolkit.h"
#include "InvalidPeer.h"
#include "NoGraphicsToolkitFoundException.h"
using namespace VCF;

Font::Font()
{
	UIGraphicsToolkit* toolkit = UIGraphicsToolkit::getDefaultGraphicsToolkit();
	if ( NULL == toolkit ){	
		throw NoGraphicsToolkitFoundException( MAKE_ERROR_MSG(NO_GFX_TOOLKIT_ERROR), __LINE__ );
	}
	m_Peer = NULL;
	m_Peer = toolkit->createFontPeer( DEFAULT_FONT_NAME ); 
	if ( NULL == m_Peer ){
		throw InvalidPeer( MAKE_ERROR_MSG(NO_PEER), __LINE__ );
	}
}

Font::Font( const String& fontName )
{
	UIGraphicsToolkit* toolkit = UIGraphicsToolkit::getDefaultGraphicsToolkit();
	if ( NULL == toolkit ){
		throw NoGraphicsToolkitFoundException( MAKE_ERROR_MSG(NO_GFX_TOOLKIT_ERROR), __LINE__ );
	}
	m_Peer = NULL;
	m_Peer = toolkit->createFontPeer( fontName ); 
	if ( NULL == m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
}

Font::Font( const String& fontName, const double& pointSize )
{
	UIGraphicsToolkit* toolkit = UIGraphicsToolkit::getDefaultGraphicsToolkit();
	if ( NULL == toolkit ){
		throw NoGraphicsToolkitFoundException( MAKE_ERROR_MSG(NO_GFX_TOOLKIT_ERROR), __LINE__ );
	}
	m_Peer = NULL;
	m_Peer = toolkit->createFontPeer( fontName, pointSize ); 
	if ( NULL == m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
}
Font::~Font()
{
	delete this->m_Peer;
}

bool Font::isTrueType()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->isTrueType();
}

Color* Font::getColor()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getColor();
}

void Font::setColor( Color* color )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setColor( color );
}

double Font::getPointSize()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getPointSize();
}

void Font::setPointSize( const double pointSize )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setPointSize( pointSize );
}

double Font::getPixelSize()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getPixelSize();
}

void Font::setPixelSize( const double pixelSize )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setPixelSize( pixelSize );
}

void Font::setBold( const bool& bold )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setBold( bold );
}

bool Font::getBold()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getBold();
}	

bool Font::getItalic()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getItalic();
}

void Font::setItalic( const bool& italic )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setItalic( italic );
}

bool Font::getUnderlined()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getUnderlined();
}

void Font::setUnderlined( const bool& underlined )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setUnderlined( underlined );
}

bool Font::getStrikeOut()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getStrikeOut();
}

void Font::setStrikeOut( const bool& strikeout )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setStrikeOut( strikeout );
}

double Font::getShear()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getShear();
}

void Font::setShear(const double& shear )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setShear( shear );
}

double Font::getAngle()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	return m_Peer->getAngle();
}

void Font::setAngle( const double& angle )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	m_Peer->setAngle( angle );
}

GlyphCollection* Font::getGlyphCollection( const String& text )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return m_Peer->getGlyphCollection( text );
}

FontPeer* Font::getFontPeer()
{
	return this->m_Peer;
}

String Font::getName()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return m_Peer->getName();
}

void Font::setName( const String& name )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_Peer->setName( name );
}

void Font::copy( Object* source )
{
	if ( NULL != source ){
		Font* srcFont = dynamic_cast<Font*>( source );
		if ( NULL != srcFont ){
			this->setAngle( srcFont->getAngle() );
			this->setShear( srcFont->getShear() );
			this->setStrikeOut( srcFont->getStrikeOut() );
			this->setUnderlined( srcFont->getUnderlined() );
			this->setBold( srcFont->getBold() );
			this->setItalic( srcFont->getItalic() );
			this->setPointSize( srcFont->getPointSize() );
			this->setName( srcFont->getName() );

			Color* srcColor = srcFont->getColor();
			this->getColor()->copy( srcColor );
		}
	}
}