/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#include "GraphicsContext.h"
#include "UIGraphicsToolkit.h"
#include "Fill.h"
#include "Stroke.h"
#include "StrokeState.h"
#include "FillState.h"

using namespace VCF;


GraphicsContext::GraphicsContext()
{
	this->init();
}

GraphicsContext::GraphicsContext( const unsigned long& width, const unsigned long& height )
{
	this->init();
	
	this->m_contextPeer = UIGraphicsToolkit::getDefaultGraphicsToolkit()->createContextPeer( width, height );
	if ( NULL == m_contextPeer ){
		//throw exception
	}

	m_contextPeer->setContext( this );
}

/*

*/

GraphicsContext::GraphicsContext( const long& contextID )
{
	this->init();
	this->m_contextPeer = UIGraphicsToolkit::getDefaultGraphicsToolkit()->createContextPeer( contextID );
	if ( NULL == m_contextPeer ){
		//throw exception
	}
	m_contextPeer->setContext( this );
}

GraphicsContext::~GraphicsContext()
{
	if ( NULL != m_contextPeer ){
		delete m_contextPeer;
	}
	m_contextPeer = NULL;

	if ( NULL != m_currentFill ){
		delete m_currentFill;
	}
	m_currentFill = NULL;

	if ( NULL != m_currentStroke ){
		delete m_currentStroke;
	}
	m_currentStroke = NULL;

	if ( NULL != m_currentFont ){
		delete m_currentFont;
	}
	m_currentFont = NULL;
}

void GraphicsContext::init()
{	
	this->m_contextPeer = NULL;
	this->m_currentFill = NULL;
	this->m_currentStroke = NULL;
	this->m_currentFont = NULL;
	
	m_currentFont = new Font();

	this->m_theta = 0.0;
	this->m_xScale = 1.0;
	this->m_yScale = 1.0;
	this->m_xTranslate = 0.0;
	this->m_yTranslate = 0.0;
	this->m_xShear = 0.0;
	this->m_yShear = 0.0;
	this->m_transformMatrix.identity();
}

void GraphicsContext::setRenderingHint(const String & hintName, const unsigned long & hintValue)
{

}

void GraphicsContext::setRenderingHints(const std::map<String, unsigned long> &renderingHints )
{

}

void GraphicsContext::setCurrentFont(Font * font)
{	
	m_currentFont->copy( font );
}

void GraphicsContext::setCurrentFill(Fill * fill)
{
	m_currentFill = fill;
	if ( NULL != m_currentFill ){
		m_currentFill->setContext( this );
	}
}

void GraphicsContext::setCurrentStroke(Stroke * stroke)
{
	m_currentStroke = stroke;
	if ( NULL != m_currentStroke ){
		m_currentStroke->setContext( this );
	}
}

void GraphicsContext::draw(Path * path)
{
	if ( NULL != m_currentFill ){
		m_currentFill->render( path );
	}
	if ( NULL != m_currentStroke ){
		m_currentStroke->render( path );
	}
}

void GraphicsContext::drawImage( const double& x, const double& y, Image * image)
{
	this->m_contextPeer->drawImage( x, y, image );
}

void GraphicsContext::getCapabilities()
{

}

void GraphicsContext::drawString( const double& x, const double& y, const String& text )
{
	this->m_contextPeer->textAt( x, y, text );
}

void GraphicsContext::setCurrentFontState(FontState * state)
{
	this->m_contextPeer->setCurrentFontState( state );
}

void GraphicsContext::textAt(const double & x, const double & y, const String & text)
{
	this->m_contextPeer->textAt( x, y, text );
}

void GraphicsContext::rectangle(const double & x1, const double & y1, const double & x2, const double & y2)
{
	this->m_contextPeer->rectangle( x1, y1, x2, y2 );
}

void GraphicsContext::circle(const double & x, const double & y, const double & radius)
{
	this->m_contextPeer->circle( x, y, radius );
}

void GraphicsContext::ellipse(const double & x1, const double & y1, const double & x2, const double & y2, const double & angle)
{
	this->m_contextPeer->ellipse( x1, y1, x2, y2, angle );
}

void GraphicsContext::setCurrentStrokeState(StrokeState * state)
{
	this->m_contextPeer->setCurrentStrokeState( state );
}

void GraphicsContext::setCurrentFillState(FillState * state)
{
	this->m_contextPeer->setCurrentFillState( state );
}

void GraphicsContext::polyline(std::vector < Point * > & pts)
{
	this->m_contextPeer->polyline( pts );
}

void GraphicsContext::curve(const double & x1, const double & y1, const double & x2, const double & y2,
                            const double & x3, const double & y3, const double & x4, const double & y4)
{
	this->m_contextPeer->curve( x1, y1, x2, y2, x3, y3, x4, y4 );
}

void GraphicsContext::lineTo(const double & x, const double & y)
{
	this->m_contextPeer->lineTo( x, y );
}

void GraphicsContext::moveTo(const double & x, const double & y)
{
	this->m_contextPeer->moveTo( x, y );
}

void GraphicsContext::fillPath()
{
	this->m_contextPeer->fillPath();
}

void GraphicsContext::strokePath()
{
	this->m_contextPeer->strokePath();
}

ContextPeer* GraphicsContext::getPeer()
{
	return this->m_contextPeer;
}

void GraphicsContext::setOrigin( const double& x, const double& y )
{
	this->m_contextPeer->setOrigin( x, y );
}

Point* GraphicsContext::getOrigin()
{
	return this->m_contextPeer->getOrigin();
}


void GraphicsContext::copyContext( const double& x, const double& y, GraphicsContext* context )
{
	ContextPeer* ctxPeer = context->getPeer();
	this->m_contextPeer->copyContext( x, y, ctxPeer );
}

void GraphicsContext::setCurrentTransform( Matrix2D* transform )
{
	m_transformMatrix = *transform;
}

Matrix2D* GraphicsContext::getCurrentTransform()
{
	compositeMatrices();
	return &m_transformMatrix;
}

void GraphicsContext::setRotation( const double& theta )
{
	this->m_theta = theta;
}

void GraphicsContext::setTranslation( const double transX, const double& transY )
{
	this->m_xTranslate = transX;
	this->m_yTranslate = transY;
}

void GraphicsContext::setShear( const double& shearX, const double& shearY )
{
	this->m_xShear = shearX;
	this->m_yShear = shearY;
}

void GraphicsContext::setScale( const double& scaleX, const double& scaleY )
{
	this->m_xScale = scaleX;
	this->m_yScale = scaleY;
}

void GraphicsContext::compositeMatrices()
{
	Matrix2D rot;
	rot.rotate( this->m_theta );
	Matrix2D trans;
	trans.translate( this->m_xTranslate, this->m_yTranslate );
	Matrix2D scale;
	scale.scale( this->m_xScale, this->m_yScale );
	Matrix2D shear;
	shear.shear( this->m_xShear, this->m_yShear );
	Matrix2D tmp;
	tmp.multiply( &rot, &trans );
	tmp.multiply( &tmp, &scale );
	tmp.multiply( &tmp, &shear );
	this->m_transformMatrix = tmp;
}

Font* GraphicsContext::getCurrentFont()
{
	return this->m_currentFont;
}

void GraphicsContext::setColor( Color* color )
{
	if ( NULL != color ){
		m_color.copy( color );
		FillState* fs = m_contextPeer->getCurrentFillState();
		fs->m_Color.copy( color );
		
		StrokeState* ss = m_contextPeer->getCurrentStrokeState();
		ss->m_Color.copy( color );
	}
}

Color* GraphicsContext::getColor()
{
	return &m_color;
}