/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//ImageList.cpp

#include "ImageList.h"
#include "Image.h"
#include "GraphicsContext.h"
#include <algorithm>

using namespace VCF;

ImageList::ImageList()
{

}

ImageList::ImageList( Image* listOfImages, const unsigned long& imageWidth, const unsigned long& imageHeight )
{

}

ImageList::~ImageList()
{

}

void ImageList::init()
{
	this->m_imageContainer.initContainer( m_images );
	m_imageWidth = 32;
	m_imageHeight = 32;
}

unsigned long ImageList::getImageWidth()
{
	return m_imageWidth;
}

unsigned long ImageList::getImageHeight()
{
	return m_imageHeight;
}

void ImageList::setImageWidth( const unsigned long& width )
{
	m_imageWidth = 	width;
}

void ImageList::setImageHeight( const unsigned long& height )
{
	m_imageHeight = height;
}

void ImageList::setTransparentColor( Color* color )
{
	m_transparentColor.copy( color );
}

Color* ImageList::getTransparentColor()
{
	return &m_transparentColor;
}

void ImageList::addImage( Image* newImage )
{
	m_images.push_back( newImage );
	ImageListEvent event( this, IMAGELIST_EVENT_ITEM_ADDED );
	notify_onImageAdded( &event );
}
	
void ImageList::insertImage( const unsigned long & index, Image* newImage )
{
	m_images.insert( m_images.begin() + index, newImage );
}

void ImageList::deleteImage( const unsigned long & index )
{
	Image* image = m_images.at( index );
	
	m_images.erase( m_images.begin() + index );

	if ( NULL != image ){
		delete image;
	}
}

void ImageList::deleteImage( Image* imageToDelete )
{
	if ( NULL != imageToDelete ){
		std::vector<Image*>::iterator it = std::find( m_images.begin(), m_images.end(), imageToDelete );
		if ( it != m_images.end() ){
			m_images.erase( it );
		}
		delete imageToDelete;
	}
}

Enumerator<Image*>* ImageList::getImages()
{
	return this->m_imageContainer.getEnumerator();
}

Image* ImageList::getImage( const unsigned long& index )
{
	return m_images.at( index );
}

void ImageList::draw( GraphicsContext* context, const unsigned long& index )
{

}