/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// BasicInputStream.cpp

#include "BasicInputStream.h"
#include "Persistable.h"

using namespace VCF;

BasicInputStream::BasicInputStream()
{
	this->init();
}

BasicInputStream::BasicInputStream( const String& textBuffer )
{
	this->init();
	this->m_inStream.write( textBuffer.c_str(), textBuffer.size() );
	m_size = textBuffer.size();
}

BasicInputStream::BasicInputStream( char* dataBuffer, const unsigned long& dataBufferSize )
{	
	this->init();
	this->m_inStream.write( dataBuffer, dataBufferSize );
	m_size = dataBufferSize;
}

BasicInputStream::BasicInputStream( InputStream* inStream )
{
	this->init();
	m_inputStream = inStream;	
}


void BasicInputStream::init()
{
	m_size = 0;
	this->m_inputStream = NULL;
}

BasicInputStream::~BasicInputStream()
{
	
}

void BasicInputStream::seek(const unsigned long& offset, const SeekType& offsetFrom)
{
	
}

unsigned long BasicInputStream::getSize()
{
	return m_size;
}

char* BasicInputStream::getBuffer()
{
	return m_inStream.getBuffer();
}

void BasicInputStream::read( char* bytesToRead, unsigned long sizeOfBytes )
{
	if ( NULL != m_inputStream ){
		m_inputStream->read( bytesToRead, sizeOfBytes );
	}
	else {
		this->m_inStream.read( bytesToRead, sizeOfBytes );
	}
	m_size += sizeOfBytes;
}


