/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ComponentOutputStream.cpp


#include "ComponentOutputStream.h"
#include "Container.h"
#include "Control.h"
#include "Property.h"
#include "Persistable.h"

using namespace VCF;


ComponentOutputStream::ComponentOutputStream( OutputStream* outStream )
{
	this->m_outStream = outStream;
}

ComponentOutputStream::~ComponentOutputStream()
{

}

void ComponentOutputStream::seek(const unsigned long& offset, const SeekType& offsetFrom)
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->seek( offset, offsetFrom );
	}
}

unsigned long ComponentOutputStream::getSize()
{
	if ( NULL != this->m_outStream ){
		return this->m_outStream->getSize();
	}

	return 0;
}

char* ComponentOutputStream::getBuffer()
{
	if ( NULL != this->m_outStream ){
		return this->m_outStream->getBuffer();
	}
	else{
		return NULL;
	}	
}

void ComponentOutputStream::write( char* bytesToWrite, unsigned long sizeOfBytes )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( bytesToWrite, sizeOfBytes );
	}
}	
	
void ComponentOutputStream::write( const short& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}
	
void ComponentOutputStream::write( const long& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}

void ComponentOutputStream::write( const int& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}

void ComponentOutputStream::write( const bool& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}

void ComponentOutputStream::write( const char& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}

void ComponentOutputStream::write( const float& val )
	{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}

void ComponentOutputStream::write( const double& val )
{
	if ( NULL != this->m_outStream ){
		this->m_outStream->write( val );
	}
}	

void ComponentOutputStream::write( const String& val )
{
	if ( NULL != this->m_outStream ){
		String tmp = val + "\n";
		this->m_outStream->write( tmp );
	}
}

void ComponentOutputStream::writeControl( Control* component )
{
	if ( NULL != component ){
		Class* clazz = component->getClass();
		Control* parent = component->getParent();
		if ( NULL == parent ){
			//dump the DTD
			String hdr = "<?xml version=\"1.0\" standalone=\"no\"?>";
			this->write( hdr );
			ComponentDTD dtd;
			this->write( dtd.toString() );
		}
		if ( NULL != clazz ){
			String tmp = "";
			String strVal = "";
			String propName = "";
			Enumerator<Property*>* props = clazz->getProperties();
			tmp = "<object class=\"" + component->getClassName() + "\" classID=\"" + clazz->getID() + "\">";
			this->write( tmp );		
			
			if ( NULL != props ){
				while ( props->hasMoreElements() ){
					Property* prop = props->nextElement();
					writeProperty( prop, component );					
				}
			}//NULL != props
			//now check for children
			Container* container = dynamic_cast<Container*>( component );
			if ( NULL != container ){
				
				Enumerator<Control*>* children = container->getChildren();
				if ( NULL != children ){
					tmp = "<children>";
					this->write( tmp );

					while ( children->hasMoreElements() ){
						Control* child = children->nextElement();
						if ( NULL != child ){
							this->writeControl( child );
						}
					}

					tmp = "</children>";
					this->write( tmp );
				}				
			}
			tmp = "</object>";
			this->write( tmp );
		}//NULL != clazz
	}
}

void ComponentOutputStream::writePropertyValue( const String& propName, VariantData* value )
{
	String tmp = "";
	if ( NULL != value ){
		tmp = "\t<property name=\"" + propName + "\" ";
		String strVal = "null";
		if ( PROP_OBJECT != value->type ){
			strVal = value->toString();
			tmp += "value=\"" + strVal + "\"/>";
			this->write( tmp );
		}
		else {
			Object* object = *value;
			if ( NULL != object ){
				strVal = object->getClassName();
				tmp += "value=\"" + strVal + "\">";	
				this->write( tmp );
				Class* objectClass = object->getClass();
				if ( NULL != objectClass ){
					Enumerator<Property*>* objectProps = objectClass->getProperties();
					tmp = "\t<objectProperty>";
					this->write( tmp );
					if ( NULL != objectProps ){						
						while ( objectProps->hasMoreElements() ){
							Property* objectProp = objectProps->nextElement();
							writeProperty( objectProp, object );
						}						
					}
					tmp = "\t</objectProperty>";
					this->write( tmp );
				}
				else{
					tmp = "\t<objectProperty></objectProperty>";
					this->write( tmp );
				}
				tmp = "</property>";
				this->write( tmp );
			}
			else {
				tmp += "value=\"null\"/>";	
				this->write( tmp );
			}
		}
	}
	else{
		tmp = "\t<property name=\"" + propName + " \"null\"";
		this->write( tmp );
	}	
}

void ComponentOutputStream::writeProperty( Property* property, Object* source )
{
	if ( NULL != property ){
		String tmp = "";
		String propName = property->getName();
		if ( true == property->isCollection() ){
			property->startCollection( source );
			while ( property->hasMoreElements( source ) ){
				VariantData* collectionVal = property->nextElement( source );
				writePropertyValue( propName, collectionVal );
			}
		}
		else {
			VariantData* val = property->get();
			this->writePropertyValue( propName, val );
		}
	}
}
