/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//FileStream.cpp

#include "FileStream.h"
#include "InvalidPeer.h"

#ifdef WIN32
#include "Win32FileStream.h"
#endif

using namespace VCF;

FileStream::FileStream( const String& filename, const FileStreamAccessType& accessType )
{
	m_filename = filename;
	m_access = accessType;
	try {
		this->init();
	}
	catch (...){

	}
}

FileStream::FileStream( const String& filename )
{
	m_filename = filename;
	m_access = FS_READ_WRITE;
	try {
		this->init();
	}
	catch (...){

	}
}

FileStream::FileStream( const String& filename, OutputStream* outStream )
{
	if ( NULL != outStream ){
		m_filename = filename;
		m_access = FS_WRITE;
		try {
			this->init();
			this->m_outputStream = outStream;
		}
		catch (...){
			
		}
	}
	//else throw exception ?
}

FileStream::FileStream( const String& filename, InputStream* inStream )
{
	if ( NULL != inStream ){
		m_filename = filename;
		m_access = FS_READ;
		try {
			this->init();
			this->m_inputStream = inStream;
		}
		catch (...){
			
		}
	}
}

FileStream::~FileStream()
{
	close();
}

void FileStream::init()
{
	m_fsPeer = NULL;
#ifdef WIN32
	m_fsPeer = new Win32FileStream( m_filename, m_access ); 
#endif
	this->m_inputStream = NULL;
	this->m_outputStream = NULL;
}

void FileStream::close()
{
	delete m_fsPeer;
	m_fsPeer = NULL;
}

void FileStream::open( const String& filename, const FileStreamAccessType& accessType )
{
	m_filename = filename;
	m_access = accessType;
	if ( NULL != m_fsPeer ){
		delete m_fsPeer;
	}
	m_fsPeer = NULL;

	this->init();
}

void FileStream::open( const String& filename )
{
	this->open( filename, FS_READ_WRITE );
}

void FileStream::seek(const unsigned long& offset, const SeekType& offsetFrom )
{
	if ( NULL == this->m_fsPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	m_fsPeer->seek( offset, offsetFrom );
}

unsigned long FileStream::getSize()
{
	if ( NULL == this->m_fsPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return m_fsPeer->getSize();
}

char* FileStream::getBuffer()
{
	char* result = NULL;

	return result;
}

void FileStream::read( char* bytesToRead, unsigned long sizeOfBytes )
{
	if ( NULL != this->m_inputStream ){
		m_inputStream->read( bytesToRead, sizeOfBytes );
	}
	else {
		if ( NULL == this->m_fsPeer ){
			throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
		};	
		
		m_fsPeer->read( bytesToRead, sizeOfBytes );
	}
}

void FileStream::write( char* bytesToWrite, unsigned long sizeOfBytes )
{
	if ( NULL != this->m_outputStream ){
		m_outputStream->write( bytesToWrite, sizeOfBytes );
	}
	else {
		if ( NULL == this->m_fsPeer ){
			throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
		};	
		
		m_fsPeer->write( bytesToWrite, sizeOfBytes );
	}
}
