/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Stream.cpp

#include "Stream.h"
#include "Persistable.h"

using namespace VCF;


void InputStream::read( Persistable* persistableObject ){
	if ( NULL != persistableObject ){
		persistableObject->loadFromStream( this );
	}	
}

void InputStream::read( short& val )
{
	this->read( (char*)&val, sizeof(val) );
}
	
void InputStream::read( long& val )
{	
	this->read( (char*)&val, sizeof(val) );
}	

void InputStream::read( int& val )
{
	this->read( (char*)&val, sizeof(val) );
}

void InputStream::read( bool& val )
{
	this->read( (char*)&val, sizeof(val) );
}

void InputStream::read( char& val )
{
	this->read( (char*)&val, sizeof(val) );
}

void InputStream::read( float& val )
{
	this->read( (char*)&val, sizeof(val) );
}

void InputStream::read( double& val )
{
	this->read( (char*)&val, sizeof(val) );
}

void InputStream::read( String& val )
{
	val = "";
	char c = '\0';
	unsigned long count = 0;
	unsigned long size = this->getSize();

	this->read( &c, sizeof(c) );
	while ( (c != '\0') && (count < size) ){
		val += c;
		this->read( &c, sizeof(c) );
		count ++;
	}
}

InputStream& InputStream::operator>>( short& val ){
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( long& val ){
	this->read( val );
	return *this;	
}

InputStream& InputStream::operator>>( int& val ){
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( bool& val ){
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( char& val ){
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( float& val ){
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( double& val ){
	this->read( val );
	return *this;
}	

InputStream& InputStream::operator>>( String& val )
{
	this->read( val );
	return *this;
}

InputStream& InputStream::operator>>( Persistable* val ){
	this->read( val );
	return *this;
}


void OutputStream::write( Persistable* persistableObject ){
	if ( NULL != persistableObject ){
		persistableObject->saveToStream( this );
	}
}

OutputStream& OutputStream::operator<<( const short& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const long& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const int& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const bool& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const char& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const float& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const double& val ){
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( const String& val )
{
	this->write( val );
	return *this;
}

OutputStream& OutputStream::operator<<( Persistable* val ){
	this->write( val );
	return *this;
}

void OutputStream::write( const short& val )
{
	this->write( (char*)&val, sizeof(val) );
}
	
void OutputStream::write( const long& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const int& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const bool& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const char& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const float& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const double& val )
{
	this->write( (char*)&val, sizeof(val) );
}

void OutputStream::write( const String& val )
{
	int size = val.size()+1;
	char* buf = new char[size];
	memset(buf, 0, size );
	val.copy( buf, size-1 );	
	this->write( buf, size );
	delete [] buf;
}
