/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// TextInputStream.cpp


#include "TextInputStream.h"

using namespace VCF;

TextInputStream::TextInputStream( InputStream* inStream )
{
	this->init();
	this->m_inStream = inStream;
	if ( NULL != m_inStream ){
		this->m_size = this->m_inStream->getSize();
	}	
}

TextInputStream::~TextInputStream()
{

}

void TextInputStream::seek(const unsigned long& offset, const SeekType& offsetFrom )
{
	if ( NULL != m_inStream ){
		m_inStream->seek( offset, offsetFrom );
	}
}

unsigned long TextInputStream::getSize()
{
	return m_size;
}

char* TextInputStream::getBuffer()
{
	return NULL;
}

void TextInputStream::read( char* bytesToRead, unsigned long sizeOfBytes )
{
	if ( NULL != m_inStream ){
		m_inStream->read( bytesToRead, sizeOfBytes );
	}
}

void TextInputStream::read( short& val )
{
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();		
		int i = val = 0;
		sscanf( tmp.c_str(), STR_INT_CONVERSION, &i );
		val = (short)i;		
	}
}
	
void TextInputStream::read( long& val )
{
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();		
		int i = val = 0;
		sscanf( tmp.c_str(), STR_INT_CONVERSION, &i );
		val = i;		
	}
}

void TextInputStream::read( int& val )
{
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();		
		int i = val = 0;
		sscanf( tmp.c_str(), STR_INT_CONVERSION, &i );
		val = i;
	}
}

void TextInputStream::read( bool& val )
{
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();
		if ( STR_BOOL_CONVERSION_TRUE == tmp ){
			val = true;
		}
		else if ( STR_BOOL_CONVERSION_FALSE == tmp ){
			val = false;
		}
	}
}

void TextInputStream::read( float& val )
{	
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();
		val = 0.0;
		float f = 0.0;
		sscanf( tmp.c_str(), STR_DOUBLE_CONVERSION, &f );
		val = f;
	}
}

void TextInputStream::read( double& val )
{
	if ( NULL != this->m_inStream ){
		String tmp = readTillWhiteSpace();
		val = 0.0;
		float f = 0.0;
		sscanf( tmp.c_str(), STR_DOUBLE_CONVERSION, &f );
		val = f;
	}
}

void TextInputStream::read( String& val )
{
	val = readTillTokenPair( '\n' );
}

TextInputStream& TextInputStream::operator>>( short& val )
{
	this->read( val );
	return *this;
}

TextInputStream& TextInputStream::operator>>( long& val )
{
	this->read( val );
	return *this;
}

TextInputStream& TextInputStream::operator>>( int& val )
{
	this->read( val );
	return *this;
}

TextInputStream& TextInputStream::operator>>( bool& val )
{
	this->read( val );
	return *this;
}

TextInputStream& TextInputStream::operator>>( char& val )
{
	if ( NULL != this->m_inStream ){
		m_inStream->read( &val, sizeof(val) );
	}
	return *this;
}

TextInputStream& TextInputStream::operator>>( float& val )
{
	this->read( val );
	return *this;
}

TextInputStream& TextInputStream::operator>>( double& val )
{
	this->read( val );
	return *this;
}	

void TextInputStream::init()
{
	this->m_inStream = NULL;
	this->m_size = 0;
	m_totCharRead = 0;
}

void TextInputStream::readLine( String& line )
{
	if ( NULL != this->m_inStream ){
		char c = '\0';
		m_inStream->read( &c, sizeof(c) );
		m_totCharRead++;
		while ( (c == '\n') && (m_totCharRead < m_size) ){ 
			line += c;
			m_inStream->read( &c, sizeof(c) );			
			m_totCharRead++;
		}
	}
}

String TextInputStream::readTillWhiteSpace()
{	
	String result = "";
	if ( NULL != this->m_inStream ){				
		char c = '\0';
		m_inStream->read( &c, sizeof(c) );
		m_totCharRead++;
		
		while ( ((c == ' ') || (c == '\0')) && (m_totCharRead < m_size) ){			
			m_inStream->read( &c, sizeof(c) );			
			m_totCharRead++;
		}
		//get all the nonwhitespace characters
		while ( (c != ' ') && (c != '\0')  && (m_totCharRead < m_size) ){
			result += c;
			m_inStream->read( &c, sizeof(c) );
			m_totCharRead++;
		}				
	}
	return result;
}

String TextInputStream::readTillTokenPair( const char& token )
{
	String result = "";
	if ( NULL != this->m_inStream ){				
		char c = '\0';
		m_inStream->read( &c, sizeof(c) );
		m_totCharRead++;
		
		while ( (c != token) && (c != ' ') && (m_totCharRead < m_size) ){
			m_inStream->read( &c, sizeof(c) );
			m_totCharRead++;
		}
		
		while ( (c != token) && (m_totCharRead < m_size) ){
			result += c;
			m_inStream->read( &c, sizeof(c) );
			m_totCharRead++;
		}
	}
	return result;
}