/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// TextOutputStream.cpp

#include "TextOutputStream.h"

using namespace VCF;

#define NUMBER_TXT_SIZE		25


TextOutputStream::TextOutputStream( OutputStream* outStream )
{
	this->init();
	m_outStream = outStream;
}

TextOutputStream::TextOutputStream()
{
	this->init();
}

TextOutputStream::~TextOutputStream()
{

}

void TextOutputStream::init()
{
	m_size = 0;
	m_outStream = NULL;	
}

void TextOutputStream::seek(const unsigned long& offset, const SeekType& offsetFrom )
{
	if ( NULL != m_outStream ){
		m_outStream->seek( offset, offsetFrom );
	}
}

unsigned long TextOutputStream::getSize()
{
	unsigned long result = 0;
	if ( NULL != m_outStream ){
		result = m_outStream->getSize();
	}
	else {
		result = this->m_textBuffer.size();
	}
	return result;
}

char* TextOutputStream::getBuffer()
{
	return NULL;
}

void TextOutputStream::write( char* bytesToRead, unsigned long sizeOfBytes )
{
	if ( NULL != this->m_outStream ){
		m_outStream->write( bytesToRead, sizeOfBytes );
	}
	m_textBuffer += bytesToRead;
}

void TextOutputStream::write( const short& val )
{
	char tmpText[NUMBER_TXT_SIZE];
	memset( tmpText, 0, NUMBER_TXT_SIZE );
	sprintf( tmpText, SHORT_STR_CONVERSION, val );
	this->write( tmpText, strlen(tmpText) );
}
	
void TextOutputStream::write( const long& val )
{
	char tmpText[NUMBER_TXT_SIZE];
	memset( tmpText, 0, NUMBER_TXT_SIZE );
	sprintf( tmpText, LONG_STR_CONVERSION, val );
	this->write( tmpText, strlen(tmpText) );
}

void TextOutputStream::write( const int& val )
{
	char tmpText[NUMBER_TXT_SIZE];
	memset( tmpText, 0, NUMBER_TXT_SIZE );
	sprintf( tmpText, INT_STR_CONVERSION, val );
	this->write( tmpText, strlen(tmpText) );
}

void TextOutputStream::write( const bool& val )
{
	char* tmpText = (char*)(val ? BOOL_STR_CONVERSION_TRUE : BOOL_STR_CONVERSION_FALSE);
	this->write( tmpText, strlen(tmpText) );
}

void TextOutputStream::write( const float& val )
{
	char tmpText[NUMBER_TXT_SIZE];
	memset( tmpText, 0, NUMBER_TXT_SIZE );
	sprintf( tmpText, FLOAT_STR_CONVERSION, val );
	this->write( tmpText, strlen(tmpText) );
}

void TextOutputStream::write( const double& val )
{
	char tmpText[NUMBER_TXT_SIZE];
	memset( tmpText, 0, NUMBER_TXT_SIZE );
	sprintf( tmpText, DOUBLE_STR_CONVERSION, val );
	this->write( tmpText, strlen(tmpText) );
}

void TextOutputStream::write( const String& val )
{
	String tmp = "\n" + val + "\n";
	OutputStream::write( tmp );
}

TextOutputStream& TextOutputStream::operator<<( const short& val )
{
	this->write( val );	
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const long& val )
{
	this->write( val );	
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const int& val )
{
	this->write( val );	
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const bool& val ){	
	this->write( val );
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const char& val )
{
	this->write( (char*)&val, sizeof(val) );
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const float& val )
{
	this->write( val );	
	return *(this);
}

TextOutputStream& TextOutputStream::operator<<( const double& val )
{
	this->write( val );	
	return *(this);
}

void TextOutputStream::writeLine( const String& line )
{
	*((OutputStream*)this) << line;
	*this << '\n';
}

String TextOutputStream::getTextBuffer()
{
	return this->m_textBuffer;
}

String TextOutputStream::toString()
{
	return m_textBuffer;	
}