/**
*Visual Control Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Control Framework
*/ 

// AbstractWin32Component.cpp


#include "AbstractWin32Component.h"
#include "Win32ToolKit.h"
#include "StringUtils.h"
#include "Win32MenuItem.h"
#include "MenuItemEvent.h"
#include "Win32FontManager.h"
#include "Control.h"
#include "GraphicsContext.h"
#include "MenuItem.h"
#include "ContextPeer.h"
#include "Frame.h"

using namespace VCF;

using namespace VCFWin32;

AbstractWin32Component::AbstractWin32Component()
{

}

AbstractWin32Component::AbstractWin32Component( Control* component )
{
	this->m_component = component;
	this->setPeerControl( component );
}

AbstractWin32Component::~AbstractWin32Component()
{

}

long AbstractWin32Component::getHandleID()
{
	long result = 0;
	result = (long)this->m_hwnd;
	return result;
}

VCF::String AbstractWin32Component::getText()
{
	VCF::String result = "";
	
	int size = ::GetWindowTextLength( this->m_hwnd );
	TCHAR* tmpText = new TCHAR[size];
	memset( tmpText, 0, size );
	::GetWindowText( this->m_hwnd, tmpText, size );
	result = tmpText;
	delete [] tmpText;
	return result;
}

void AbstractWin32Component::setText( const VCF::String& text )
{
	::SetWindowText( this->m_hwnd, text.c_str() );
}

void AbstractWin32Component::setBounds( VCF::Rect* rect )
{
	/*
	Control* parent = this->m_component->getParent();
	if ( !parent->isLighweight() ){
		Container* c = dynamic_cast<Container*>(parent);
		if ( NULL != c ){
			c->getChildCount()
			ControlPeer* p = parent->getPeer();
			
		}
		
	}
	*/
	::MoveWindow( this->m_hwnd, rect->m_left, rect->m_top, 
                    rect->getWidth(), rect->getHeight(), TRUE );
	/*
	::SetWindowPos( this->m_hwnd, NULL, rect->m_left, rect->m_top, 
                    rect->getWidth(), rect->getHeight(), SWP_NOCOPYBITS | SWP_NOACTIVATE | SWP_NOOWNERZORDER| SWP_NOZORDER );
	*/
}

VCF::Rect* AbstractWin32Component::getBounds()
{
	VCF::Rect* result = NULL;
	RECT r;
	::GetWindowRect( this->m_hwnd, &r );
	
	HWND parent = ::GetParent( this->m_hwnd );

	if ( NULL != parent ){ 		
		POINT pt = {0};
		pt.x = r.left;	
		pt.y = r.top;
		::ScreenToClient( parent, &pt );
		r.left = pt.x;
		r.top = pt.y;

		pt.x = r.right;	
		pt.y = r.bottom;
		::ScreenToClient( parent, &pt );

		r.right = pt.x;
		r.bottom = pt.y;

		m_bounds.setRect( r.left, r.top, r.right, r.bottom );
		result = &m_bounds;
	}
	else{
		m_bounds.setRect( r.left, r.top, r.right, r.bottom );
		result = &m_bounds;	
	}
	return result;
}

void AbstractWin32Component::setVisible( const bool& visible )
{
	if ( true == visible ){
		::ShowWindow( this->m_hwnd, SW_SHOW );
	}
	else{
		::ShowWindow( this->m_hwnd, SW_HIDE );
	}
}

bool AbstractWin32Component::getVisible()
{
	bool result = false;
	DWORD style = GetWindowLong( this->m_hwnd, GWL_STYLE );
	result =  (style & WS_VISIBLE ) != 0;
	return result;
}

unsigned long AbstractWin32Component::getStyleMask()
{
	unsigned long result = 0;
	result = GetWindowLong( this->m_hwnd, GWL_STYLE );
	return result;
}

void AbstractWin32Component::setStyleMask( const unsigned long& styleMask )
{

}

VCF::Control* AbstractWin32Component::getControl()
{
	return m_component;
}

void AbstractWin32Component::setControl( VCF::Control* component )
{
	this->m_component = component;
	this->setPeerControl( component );
}

void AbstractWin32Component::getCursor()
{

}

void AbstractWin32Component::setCursor()
{

}

void AbstractWin32Component::setParent( VCF::Control* parent )
{
	VCF::ControlPeer* Peer = parent->getPeer();
	if ( NULL == dynamic_cast<Frame*>(this->m_component) ){
		HWND wndParent = (HWND)Peer->getHandleID();
		
		if ( NULL == wndParent ){
			//throw exception !!!
		}
		
		::SetParent( this->m_hwnd, wndParent );
	}
}

VCF::Control* AbstractWin32Component::getParent()
{
	return NULL;
}

bool AbstractWin32Component::isFocused()
{
	HWND focusedHandle = ::GetFocus();
	bool result = (NULL != focusedHandle) && (focusedHandle == this->m_hwnd);
	return result;
}

void AbstractWin32Component::setFocus( const bool& focused )
{
	::SetFocus( this->m_hwnd );
}

bool AbstractWin32Component::isEnabled()
{	
	return (::IsWindowEnabled( this->m_hwnd ) != 0);
}

void AbstractWin32Component::setEnabled( const bool& enabled )
{
	::EnableWindow( this->m_hwnd, enabled );
}

void AbstractWin32Component::setFont( Font* font )
{	
	if ( NULL != font ){
		Win32Font* win32FontPeer = NULL;
		FontPeer* fontPeer = font->getFontPeer();
		win32FontPeer = dynamic_cast<Win32Font*>( fontPeer );
		if ( NULL != win32FontPeer ){
			HFONT fontHandle = Win32FontManager::getFontHandleFromFontPeer( win32FontPeer );
			if ( NULL != fontHandle ){
				::SendMessage( this->m_hwnd, WM_SETFONT, (WPARAM)fontHandle, (LPARAM)TRUE );
			}
		}
		else {
			//what the hell is a non Win32Font doing here !!!! throw exception !!!!!
		}
	}
}

LRESULT AbstractWin32Component::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;
	HDC contextID = 0;
	PAINTSTRUCT ps;
	WNDPROC oldProc = this->m_defaultWndProc;
	if ( NULL != defaultWndProc ){
		m_defaultWndProc = defaultWndProc;
	}
	switch ( message ) {

		case WM_ERASEBKGND :{
			result = 1;	
		}
		break;

		case WM_PAINT:{
			if ( true == this->isCreated() ){
				contextID = ::BeginPaint( this->m_hwnd, &ps);	
				if ( true == this->m_component->isDoubleBuffered() ){

					RECT r = {0};
					
					GetWindowRect( this->m_hwnd, &r );

					VCF::GraphicsContext* ctx = m_component->getContext();

					VCF::GraphicsContext memCtx( r.left + (r.right - r.left), r.top + (r.bottom - r.top) );
					
					memCtx.setOrigin( r.left, r.top );
					
					this->m_component->paint( &memCtx );

					ctx->copyContext( 0, 0, &memCtx );
					
				}
				else {
					VCF::GraphicsContext* ctx = m_component->getContext();

					ctx->getPeer()->setContextID( (long)ps.hdc );

					this->m_component->paint( ctx );
				}

				::EndPaint( this->m_hwnd, &ps);
			}
		};
		break;

		case WM_SIZE:{
			VCF::Size sz( LOWORD(lParam), HIWORD(lParam) );

			VCF::ControlEvent event( this->getControl(), sz );
	
			this->m_component->processControlEvent( &event );
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;

		case WM_LBUTTONDOWN: case WM_MBUTTONDOWN: case WM_RBUTTONDOWN:{			
			m_clickPt.m_x = Win32Utils::getXFromLParam( lParam );
			m_clickPt.m_y = Win32Utils::getYFromLParam( lParam );
			
			VCF::Point pt( m_clickPt.m_x, m_clickPt.m_y );
			
			VCF::MouseEvent event( this->getControl(), MOUSE_EVENT_DOWN,
				Win32Utils::translateButtonMask( wParam ),
				Win32Utils::translateKeyMask( wParam ), &pt );
			
			this->m_component->processMouseEvent( &event );
			
			result = defaultWndProcedure( message, wParam, lParam );			
		}
		break;

		case WM_LBUTTONUP: case WM_MBUTTONUP: case WM_RBUTTONUP:{			
			VCF::Point pt( Win32Utils::getXFromLParam( lParam ) , 
				           Win32Utils::getYFromLParam( lParam ) );
			VCF::Point tmpPt( Win32Utils::getXFromLParam( lParam ) , 
				              Win32Utils::getYFromLParam( lParam ) );
			
			WPARAM tmpWParam = wParam;
			switch ( message ){
				case WM_LBUTTONUP:{
					tmpWParam |= MK_LBUTTON;
				}
				break;

				case WM_MBUTTONUP:{
					tmpWParam |= MK_MBUTTON;
				}
				break;

				case WM_RBUTTONUP:{
					tmpWParam |= MK_RBUTTON;
				}
				break;
			}
			VCF::MouseEvent event( this->getControl(), MOUSE_EVENT_UP,
				Win32Utils::translateButtonMask( tmpWParam ),
				Win32Utils::translateKeyMask( tmpWParam ), &pt );
			
			this->m_component->processMouseEvent( &event );
			
			/**
			*determine if we have to fire off a click event
			*/
			if ( m_clickPt.closeTo( tmpPt.m_x, tmpPt.m_y, 2 ) ){
				
				VCF::MouseEvent event( this->getControl(), MOUSE_EVENT_CLICK,
					Win32Utils::translateButtonMask( tmpWParam ),
					Win32Utils::translateKeyMask( tmpWParam ), &pt );
				
				this->m_component->processMouseEvent( &event );
			}
			result = defaultWndProcedure( message, wParam, lParam );			
		}
		break;

		case WM_CREATE: {
			VCF::ComponentEvent event( this->getControl(), COMPONENT_EVENT_CREATED );
			
			this->m_component->processComponentEvent( &event );
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;	

		case VCF_CONTROL_CREATE: {
			VCF::ComponentEvent event( this->getControl(), COMPONENT_EVENT_CREATED );
			
			this->m_component->processComponentEvent( &event );
		}
		break;	

		case WM_DESTROY: {	
			result = defaultWndProcedure( message, wParam, lParam );

			VCF::ComponentEvent event( this->getControl(), COMPONENT_EVENT_DELETED );
			this->m_component->processComponentEvent( &event );			
		}
		break;

		case WM_LBUTTONDBLCLK: case WM_MBUTTONDBLCLK: case WM_RBUTTONDBLCLK:{			
			VCF::Point pt( Win32Utils::getXFromLParam( lParam ) , 
				           Win32Utils::getYFromLParam( lParam ) );
			VCF::MouseEvent event( this->getControl(), MOUSE_EVENT_DBLCLICK,
				Win32Utils::translateButtonMask( wParam ),
				Win32Utils::translateKeyMask( wParam ), &pt );
			
			this->m_component->processMouseEvent( &event );	
			result = defaultWndProcedure( message, wParam, lParam );			
		}
		break;

		case WM_MOVE: {
			VCF::Point pt( Win32Utils::getXFromLParam( lParam ) , 
				           Win32Utils::getYFromLParam( lParam ) );
			VCF::ControlEvent event( this->getControl(), pt );
			
			this->m_component->processControlEvent( &event );
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;
		
		case WM_MOUSEMOVE: {			
			VCF::Point pt( Win32Utils::getXFromLParam( lParam ) , 
				           Win32Utils::getYFromLParam( lParam ) );
			
			
			VCF::MouseEvent event( this->getControl(), MOUSE_EVENT_MOVE,
				Win32Utils::translateButtonMask( wParam ),
				Win32Utils::translateKeyMask( wParam ), &pt );
			
			this->m_component->processMouseEvent( &event );
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;

		case WM_CHAR: case WM_KEYDOWN: case WM_KEYUP: {			
			KeyboardData keyData = Win32Utils::translateKeyData( this->m_hwnd, lParam );
			unsigned long eventType = 0;
			switch ( message ){
			case WM_CHAR:{
				eventType = KEYBOARD_EVENT_DOWN;
						 }
				break;
				
			case WM_KEYDOWN:{
				eventType = KEYBOARD_EVENT_PRESSED;
							}
				break;
				
			case WM_KEYUP:{
				eventType = KEYBOARD_EVENT_UP;
						  }
				break;
			}			
			
			unsigned long keyMask = Win32Utils::translateKeyMask( keyData.keyMask );
			ulong32 virtKeyCode = Win32Utils::translateVKCode( keyData.VKeyCode );
			VCF::KeyboardEvent event( this->m_component, eventType, keyData.repeatCount, 
				keyMask, (VCF::VCFChar)keyData.character, virtKeyCode );
			
			this->m_component->processKeyboardEvent( &event );
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;
		
		case WM_NOTIFY : {
			NMHDR* notificationHdr = (LPNMHDR)lParam;
			Win32Object* win32Obj = Win32Object::getWin32ObjectFromHWND( notificationHdr->hwndFrom );
			if ( NULL != win32Obj ){
				win32Obj->handleEventMessages( notificationHdr->code, wParam, lParam );
			}
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;

		case WM_COMMAND:{
			UINT notifyCode = HIWORD(wParam); 
			HWND hwndCtl = (HWND) lParam;
			Win32Object* win32Obj = NULL;
			if ( NULL != hwndCtl ){
				win32Obj = Win32Object::getWin32ObjectFromHWND( hwndCtl );
			}
			if ( NULL != win32Obj ){
				if ( NULL != win32Obj ){
					win32Obj->handleEventMessages( notifyCode, wParam, lParam );						
				}
				else {
					StringUtils::trace( "win32Obj == NULL!\n" );
					UINT wID = LOWORD(wParam);
					char tmp2[256];
					memset( tmp2, 0, 256 );
					sprintf( tmp2, "command ID: %d\n", wID );
					
					StringUtils::trace( String(tmp2) );
					
					defaultWndProcedure( message, wParam, lParam );
				}
			}
			else {
				switch ( notifyCode ) {
					case 0 : { //menu item command
						UINT wID = LOWORD(wParam);				
						
						MenuItem* item = Win32MenuItem::getMenuItemFromID( wID );
						if ( NULL != item ){						
							item->click();
						}
					}
					break;

					case 1 : { //accellerator
						
					}
					break;
				}
			}			
		}
		break;

		default:{
			result = defaultWndProcedure( message, wParam, lParam );
		}
		break;
	}
	if ( NULL != defaultWndProc ){
		m_defaultWndProc = oldProc;
	}
	return result;
}
void AbstractWin32Component::repaint( Rect* repaintRect )
{
	if ( NULL == repaintRect ){
		::InvalidateRect( this->m_hwnd, NULL, TRUE );
	}
	else {
		RECT rect = {0};
		rect.left = repaintRect->m_left;
		rect.top = repaintRect->m_top;
		rect.right = repaintRect->m_right;
		rect.bottom = repaintRect->m_bottom;
		::InvalidateRect( this->m_hwnd, &rect, TRUE );
	}
}

void AbstractWin32Component::keepMouseEvents()
{
	::SetCapture( this->m_hwnd );
}

void AbstractWin32Component::releaseMouseEvents()
{
	::ReleaseCapture();
}


String AbstractWin32Component::toString()
{
	String result = Win32Object::toString();
	char tmp[256];
	memset(tmp,0,256);
	sprintf(tmp, "\n\tm_component=%p\n\tHWND=%p\n", m_component, this->m_hwnd );
	result += tmp;
	return result;
}