/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// LightweightComponent.cpp


#include "LightweightComponent.h"
#include "Control.h"

using namespace VCF;

LightweightComponent::LightweightComponent( Control* component )
{
	this->m_component = component;
}

LightweightComponent::~LightweightComponent()
{

}

long LightweightComponent::getHandleID()
{	
	return 0;
}

VCF::String LightweightComponent::getText()
{
	VCF::String result = "";

	return result;
}

void LightweightComponent::setText( const VCF::String& text )
{
	
}

void LightweightComponent::setBounds( VCF::Rect* rect )
{
	m_bounds.inflate(1,1);

	VCF::Control* parent = getHeavyWeightParent();
	if ( NULL != parent ){
		parent->repaint( &m_bounds );
	}
	this->m_bounds.setRect( rect->m_left, rect->m_top, rect->m_right, rect->m_bottom );
}

VCF::Rect* LightweightComponent::getBounds()
{	
	return &m_bounds;
}

void LightweightComponent::setVisible( const bool& visible )
{
	this->m_visible = visible;
}

bool LightweightComponent::getVisible()
{	
	return this->m_visible;
}

unsigned long LightweightComponent::getStyleMask()
{	
	return 0;
}

void LightweightComponent::setStyleMask( const unsigned long& styleMask )
{

}

VCF::Control* LightweightComponent::getControl()
{
	return m_component;
}

void LightweightComponent::setControl( VCF::Control* component )
{
	this->m_component = component;
}

void LightweightComponent::getCursor()
{

}

void LightweightComponent::setCursor()
{

}

void LightweightComponent::setParent( VCF::Control* parent )
{
	
}

VCF::Control* LightweightComponent::getParent()
{
	return m_component->getParent();
}

bool LightweightComponent::isFocused()
{	
	return false;
}

void LightweightComponent::setFocus( const bool& focused )
{
	
}

bool LightweightComponent::isEnabled()
{	
	return false;
}

void LightweightComponent::setEnabled( const bool& enabled )
{
	
}

void LightweightComponent::setFont( Font* font )
{
	
}

VCF::Control* LightweightComponent::getHeavyWeightParent()
{
	Control* result = NULL;

	VCF::Control* parent = this->getParent();
	if ( NULL != parent ){
		bool heavyWeightParent = !parent->isLighweight();
		result = parent;
		while ( (false == heavyWeightParent) && (parent!=NULL) ){
			parent = result->getParent();
			if ( NULL != parent ){
				heavyWeightParent = !parent->isLighweight();
				if ( false == heavyWeightParent ){
					result = parent;
					parent = NULL;
				}
			}
		}
	}
	result = parent;
	return result;
}

void LightweightComponent::repaint( Rect* repaintRect )
{
	VCF::Control* tmp = NULL;
	VCF::Control* parent = getHeavyWeightParent();
	if ( NULL != parent ){
		Rect tmpRect;
		if ( NULL == repaintRect ){
			tmpRect = *this->m_component->getBounds();
			tmpRect.inflate( 2, 2 );
			repaintRect = &tmpRect;
		}
		parent->repaint( repaintRect );
	}
}

void LightweightComponent::keepMouseEvents()
{
	Control* parent = this->getHeavyWeightParent();
	if ( NULL != parent ){
		parent->keepMouseEvents();
		Control::setCapturedMouseControl( this->m_component );
	}
}

void LightweightComponent::releaseMouseEvents()
{
	Control* parent = this->getHeavyWeightParent();
	if ( NULL != parent ){
		parent->releaseMouseEvents();
	}
}