
#include "MFCImplementer.h"

#include "SimpleDragDropImplementer.h"
#include "DragSource.h"
#include "ClipboardDataObject.h"
#include "DataObject.h"
#include "DragEvent.h"
#include "COMUtils.h"
#include "MemoryStream.h"
#include <iostream>



using namespace ImprovUI;
using namespace ImprovCOM;

class CustomDropSource : public COleDropSource
{
public:
	CustomDropSource( ImprovUI::DragSource* dragSource, ImprovUI::ClipboardDataObject* data ):
	  COleDropSource(){
		m_dragSource = dragSource;
		m_data = data;
	 };
	
	virtual ~CustomDropSource(){};

	virtual BOOL OnBeginDrag( CWnd* pWnd ){
		std::cout << "CustomDropSource::OnBeginDrag" << std::endl;
		
		return COleDropSource::OnBeginDrag( pWnd );
	};

	virtual SCODE GiveFeedback( DROPEFFECT dropEffect ){
		std::cout << "CustomDropSource::GiveFeedback" << std::endl;
		return COleDropSource::GiveFeedback( dropEffect );
	};

	virtual SCODE QueryContinueDrag( BOOL bEscapePressed, DWORD dwKeyState ){
		std::cout << "CustomDropSource::QueryContinueDrag" << std::endl;
		
		DragEvent event( m_dragSource, m_data );
		
		m_dragSource->notify_onDragSourceCanContinueDragOp( &event );

		return COleDropSource::QueryContinueDrag( bEscapePressed, dwKeyState );
	};

private:
	ImprovUI::DragSource* m_dragSource;
	ImprovUI::ClipboardDataObject* m_data;
};



SimpleDragDropImplementer::SimpleDragDropImplementer()
{
	m_COMDragSource = new ImprovCOM::DragSource();
}

SimpleDragDropImplementer::~SimpleDragDropImplementer()
{
	delete m_COMDragSource;
}

void SimpleDragDropImplementer::startDragDrop(  ClipboardDataObject* data  )
{	
	CustomDropSource dropSource( m_dragSource, data ) ;

	MemoryStream memStream;
	memStream.write( data );
	
	//AfxOleInit() needs to be called for this to work

	COleDataSource dataSrc;
	HGLOBAL hGlobal = ::GlobalAlloc(  GMEM_MOVEABLE | GMEM_ZEROINIT | GMEM_SHARE , memStream.getSize() );
	char* charMem = (char*)::GlobalLock( hGlobal );
	memStream.read( charMem, memStream.getSize() );
	

	Enumerator<DataType*> *enumerator = data->getSupportedDataTypes();
	while ( enumerator->hasMoreElements() ){
		DataType* dataTypeObj = enumerator->nextElement();
		if ( NULL != dataTypeObj ){
			UINT clipFmt =  COMUtils::translateFrameworkFormat( dataTypeObj->getRawType() );
			if ( clipFmt != 0 ){
				dataSrc.CacheGlobalData( clipFmt, hGlobal );					
			}
		}
	}

	DWORD effect = COMUtils::translateActionType( m_dragSource->getActionType() );

	dataSrc.DoDragDrop( effect, NULL, &dropSource );

	GlobalUnlock( hGlobal );

	/*

	//translate the action types into COM types;
	DWORD effects = DROPEFFECT_COPY;
	DWORD resultingEffect = 0;

	DataObject* dataObject = NULL;

	DataObjectImplementer* dataObjImpl = data->getDataObjectImplementer();

	dataObject = dynamic_cast<ImprovCOM::DataObject*>( dataObjImpl );
	//dataObject->AddRef();
	if ( NULL != dataObject ){
		
	}
	else{
		dataObject = new DataObject();	
		//dataObject->addSupportedData( 	
	}	
	//m_COMDragSource->AddRef();
	//dataObject->AddRef();

	::DoDragDrop( dataObject, m_COMDragSource, effects, &resultingEffect );
	
	//dataObject->Release();
	*/
}

ImprovUI::DragSource* SimpleDragDropImplementer::getDragSource()
{
	return m_dragSource;
}

void SimpleDragDropImplementer::setDragSource( ImprovUI::DragSource*  dragSource )
{
	m_dragSource = dragSource;
}

DWORD SimpleDragDropImplementer::translateActionType( DragActionType action )
{
	DWORD result = 0;
	switch ( action ){
		case DRAG_ACTION_COPY:{
			result = DROPEFFECT_COPY;	
		}
		break;
		
		case DRAG_ACTION_MOVE:{
			result = DROPEFFECT_MOVE;	
		}
		break;

		case DRAG_ACTION_LINK:{
			result = DROPEFFECT_LINK;	
		}
		break;

	};

	return result;
}