/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Application.cpp

#include "Win32Application.h"
#include "Win32Peer.h"
#include "Win32ResourceBundle.h"
#include "BasicException.h"

//#define _WIN32_IE 0x0400 //this might be overkill
#include <commctrl.h> //make sure to link with comctl32.lib


using namespace VCF;

Win32Application::Win32Application()
{
	m_resBundle = NULL;
	m_resBundle = new Win32ResourceBundle();
}

Win32Application::~Win32Application()
{
	delete m_resBundle;
	m_resBundle = NULL;
}

bool Win32Application::initApp( const std::vector<VCF::String>& appCmdLine )
{
	INITCOMMONCONTROLSEX controlsToInit = {0};
	controlsToInit.dwSize = sizeof(controlsToInit);
	controlsToInit.dwICC = ICC_WIN95_CLASSES;
	InitCommonControlsEx( &controlsToInit );
	
	
	if ( S_OK != OleInitialize(NULL) ){
		throw BasicException("OleInitialize failed");
	}
	
	return true;
}

void Win32Application::terminateApp()
{
	OleFlushClipboard();
	OleUninitialize();	
}

VCF::Application* Win32Application::getApplication()
{
	return this->m_app;
}

void Win32Application::setApplication( VCF::Application* application )
{
	this->m_app = application;
}

ResourceBundle* Win32Application::getResourceBundle()
{
	return m_resBundle;
}	

String Win32Application::getFileName()
{
	TCHAR fileName[MAX_PATH];
	memset( fileName, 0, MAX_PATH );
	::GetModuleFileName( NULL, fileName, MAX_PATH );

	return String( fileName );
}

void Win32Application::run()
{
	MSG msg;
	

	HACCEL hAccelTable = NULL;

	while ( GetMessage( &msg, NULL, 0, 0 ) ) {
		if (!TranslateAccelerator( msg.hwnd, hAccelTable, &msg ) ) {
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
	}
}

String Win32Application::getCurrentDirectory()
{
	String result = "";
	VCFChar	dirPath[MAX_PATH+1];
	memset( dirPath, 0, MAX_PATH+1 );
	if ( 0 != ::GetCurrentDirectory( MAX_PATH, dirPath ) ){
		result = dirPath;
	}
	return result;
}

void Win32Application::setCurrentDirectory( const String& currentDirectory )
{
	::SetCurrentDirectory( currentDirectory.c_str() );
}