/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Button.cpp


#include "Win32Button.h"
#include "Win32ToolKit.h"

using namespace VCF;


Win32Button::Win32Button( CommandButton* component ):
	AbstractWin32Component( component )
{	
	this->init();

	this->createParams();
	/*
	if ( true != this->isRegistered() ){
		registerWin32Class( this->getClassName(), m_wndProc  );
	}
	*/
	this->m_commandButton = component;
	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();

	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             "BUTTON",
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){	
		Win32Object::registerWin32Object( this );
		m_oldButtonWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 
		this->m_defaultWndProc = NULL;
				
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32Button::~Win32Button()
{

}

void Win32Button::createParams()
{
	this->m_styleMask = SIMPLE_VIEW | BS_DEFPUSHBUTTON;
}

Image* Win32Button::getImage()
{
	return NULL;
}


void Win32Button::setImage( Image* image )
{

}

void Win32Button::getState()
{

}

void Win32Button::setState()
{

}

LRESULT Win32Button::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;

	switch ( message ) {
		case WM_PAINT:{
			result = CallWindowProc( m_oldButtonWndProc, m_hwnd, message, wParam, lParam );
		}
		break;
		
		case BN_CLICKED :{

			m_commandButton->click();

			HWND wnd = (HWND)this->m_component->getParent()->getPeer()->getHandleID();
			WPARAM msgWParam = 0;
			switch ( this->m_commandButton->getCommandType() ){
				case BC_NONE : {
					
				}
				break;

				case BC_OK : {
					msgWParam = MAKEWPARAM(0,IDOK);
				}
				break;

				case BC_CANCEL : {
					msgWParam = MAKEWPARAM(0,IDCANCEL);
				}
				break;

				case BC_YES : {
					msgWParam = MAKEWPARAM(0,IDYES);
				}
				break;

				case BC_NO : {
					msgWParam = MAKEWPARAM(0,IDNO);
				}
				break;

				case BC_MAYBE : {
					
				}
				break;
			}
			PostMessage( wnd, WM_COMMAND, msgWParam, NULL );
		}
		break;

		case WM_COMMAND: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( m_oldButtonWndProc, m_hwnd, message, wParam, lParam );
			
		}

		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( m_oldButtonWndProc, m_hwnd, message, wParam, lParam );
			
		}
	}
	return result;
}
