/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
//Win32ColorDialog.cpp

#include "Win32ColorDialog.h"
#include "Control.h"
#include "ControlPeer.h"
#include "Win32Peer.h"

using namespace VCF;



Win32ColorDialog::Win32ColorDialog( Control* owner )
{
	m_owner = owner;
}

Win32ColorDialog::~Win32ColorDialog()
{

}

Color* Win32ColorDialog::getSelectedColor()
{
	return &this->m_color;
}

void Win32ColorDialog::setSelectedColor( Color* selectedColor )
{
	this->m_color.copy( selectedColor );
}

void Win32ColorDialog::setTitle( const String& title )
{
	this->m_title = title;
}

bool Win32ColorDialog::execute()
{
	bool result = false;

	CHOOSECOLOR color = {0};
	color.lStructSize = sizeof(CHOOSECOLOR);
	COLORREF customColors[16];
	color.lpCustColors = 
		customColors;
	color.hInstance = NULL;
	color.hwndOwner = NULL;

	if ( NULL != m_owner ){
		ControlPeer* impl = m_owner->getPeer();
		color.hwndOwner = (HWND) impl->getHandleID();
	}

	color.rgbResult = this->m_color.getRGB();

	color.Flags = CC_ANYCOLOR | CC_FULLOPEN;

	if ( TRUE == ChooseColor( &color ) ){
		result = true;
		uchar r = GetRValue( color.rgbResult );
		uchar g = GetGValue( color.rgbResult );
		uchar b = GetBValue( color.rgbResult );
		this->m_color.setRGB( r, g, b );
	}

	return result;
}