/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32ComboBox.cpp


#include "Win32ComboBox.h"
#include "Win32ToolKit.h"
#include "ComboBoxControl.h"
#include "ListItem.h"

using namespace VCF;


Win32ComboBox::Win32ComboBox( ComboBoxControl* component ):
	AbstractWin32Component( component )
{
	this->createParams();	

	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();
	
	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             "COMBOBOX",
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){	
		Win32Object::registerWin32Object( this );		
		m_oldComboBoxWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 		
		this->m_defaultWndProc = NULL;
				
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32ComboBox::~Win32ComboBox()
{

}

void Win32ComboBox::createParams()
{
	this->m_styleMask = SIMPLE_VIEW | CBS_DROPDOWN | WS_VSCROLL | CBS_AUTOHSCROLL | CBS_HASSTRINGS;
}

void Win32ComboBox::addItem( ListItem * item )
{
	String caption = item->getCaption();
	char* tmp = new char[caption.size()];
	memset( tmp, 0, caption.size() );
	caption.copy( tmp, caption.size() );
	int index = ::SendMessage( this->m_hwnd, CB_ADDSTRING, 0, (LPARAM)caption.c_str() );
	item->setIndex( index );
	delete [] tmp;
}

void Win32ComboBox::insertItem( const unsigned long& index, ListItem * item )
{

}

LRESULT Win32ComboBox::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;

	switch ( message ) {
		case WM_PAINT:{
			result = CallWindowProc( m_oldComboBoxWndProc, m_hwnd, message, wParam, lParam );
		}
		break;
		
		case WM_COMMAND: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( m_oldComboBoxWndProc, m_hwnd, message, wParam, lParam );
		}
		break;
		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( m_oldComboBoxWndProc, m_hwnd, message, wParam, lParam );			
		}
	}
	return result;
}

void Win32ComboBox::setBounds( VCF::Rect* rect )
{
	double h = rect->getHeight() * 4;
	::SetWindowPos( this->m_hwnd, NULL, rect->m_left, rect->m_top, 
                    rect->getWidth(), h, SWP_NOACTIVATE );
}