/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
//Win32DropTargetPeer.cpp

#include "Win32DropTargetPeer.h"
#include "ControlPeer.h"
#include "Control.h"
#include "StringUtils.h"

using namespace VCF;


Win32DropTargetPeer::Win32DropTargetPeer()
{
	m_dropTarget = NULL;
	m_comDropTarget = new VCFCOM::DropTarget();
	m_comDropTarget->setPeer( this );
}

Win32DropTargetPeer::~Win32DropTargetPeer()
{
	delete m_comDropTarget;
}

void Win32DropTargetPeer::registerTarget( Control* control )
{
	

	if ( NULL != control ){
		ControlPeer* peer = control->getPeer();
		if ( NULL != peer ){
			HWND hwnd = (HWND)peer->getHandleID();
			if ( NULL != hwnd ){
				IDropTarget* dt = static_cast<IDropTarget*>(this->m_comDropTarget);
				HRESULT result = RegisterDragDrop( hwnd, dt );
				switch ( result ){
					case S_OK :{
						StringUtils::trace("RegisterDragDrop returned S_OK\n");
					}
					break;

					case DRAGDROP_E_INVALIDHWND :{
						StringUtils::trace("RegisterDragDrop returned DRAGDROP_E_INVALIDHWND\n");
					}
					break;

					case DRAGDROP_E_ALREADYREGISTERED:{
						StringUtils::trace("RegisterDragDrop returned DRAGDROP_E_ALREADYREGISTERED\n");
					}
					break;

					default :{
						StringUtils::trace("RegisterDragDrop returned some other error\n");
					}
					break;
				}
			}
		}
	}
}

void Win32DropTargetPeer::unregisterTarget( Control* control )
{
	if ( NULL != control ){
		ControlPeer* peer = control->getPeer();
		if ( NULL != peer ){
			HWND hwnd = (HWND)peer->getHandleID();
			if ( NULL != hwnd ){
				RevokeDragDrop( hwnd );
			}
		}
	}
}

DropTarget* Win32DropTargetPeer::getDropTarget()
{
	return this->m_dropTarget;
}

void Win32DropTargetPeer::setDropTarget( DropTarget* dropTarget )
{
	this->m_dropTarget = dropTarget;
}
