/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Edit.cpp


#include "Win32Edit.h"
#include "Win32ToolKit.h"
#include "TextModel.h"
#include "TextModelListener.h"
#include "TextEvent.h"


using namespace VCFWin32;
using namespace VCF;


Win32Edit::Win32Edit( TextControl* component ):
	AbstractWin32Component( component ),
	m_oldEditWndProc(NULL)
{
	m_OKToResetControlText = true;
	m_textControl = component;
	if ( NULL == m_textControl ){
		//throw exception !!!!!!
	}
	this->createParams();
	/*
	if ( true != this->isRegistered() ){
		registerWin32Class( this->getClassName(), m_wndProc  );
	}
	*/
	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();

	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             "EDIT",//className.c_str(), 
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask | ES_AUTOHSCROLL, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 CW_USEDEFAULT, 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){
		
		Win32Object::registerWin32Object( this );
		m_oldEditWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 
		this->m_defaultWndProc = NULL;
		
		TextModelHandler* tml = new TextModelHandler( this );
		this->addEventHandler( "Win32TextModelHandler", tml );
		tml->m_textChanged = (OnTextEvent)onTextModelTextChanged;
		TextModel* tm = m_textControl->getTextModel();
		tm->addTextModelListener( tml );

	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32Edit::~Win32Edit()
{

}

void Win32Edit::setRightMargin( const double & rightMargin )
{
	::SendMessage( this->m_hwnd, EM_SETMARGINS, EC_RIGHTMARGIN, (long)rightMargin );
}

void Win32Edit::setLeftMargin( const double & leftMargin )
{
	::SendMessage( this->m_hwnd, EM_SETMARGINS, EC_LEFTMARGIN, (long)leftMargin );
}

unsigned long Win32Edit::getLineCount()
{
	return ::SendMessage( this->m_hwnd, EM_GETLINECOUNT, 0, 0 );
}

void Win32Edit::getCurrentLinePosition()
{

}

double Win32Edit::getLeftMargin()
{
	double result = 0.0;
	
	DWORD margin = ::SendMessage( this->m_hwnd, EM_GETMARGINS, 0, 0 ); 
	result = LOWORD(margin);
	return result;
}

double Win32Edit::getRightMargin()
{
	double result = 0.0;
	
	DWORD margin = ::SendMessage( this->m_hwnd, EM_GETMARGINS, 0, 0 ); 
	result = HIWORD(margin);
	return result;
}

VCF::Point* Win32Edit::getPositionFromCharIndex( const unsigned long& index )
{
	DWORD pos = ::SendMessage( this->m_hwnd, EM_POSFROMCHAR, index, 0 );
	m_posAtChar.m_x = LOWORD(pos);
	m_posAtChar.m_y = HIWORD(pos);
	return &m_posAtChar;
}

unsigned long Win32Edit::getCharIndexFromPosition( VCF::Point* point )
{
	DWORD pos = 0;
	
	pos = MAKEWORD( (long)point->m_x, (long)point->m_y );

	unsigned long result = ::SendMessage( this->m_hwnd, EM_CHARFROMPOS, 0, pos );

	return result;
}

unsigned long Win32Edit::getCaretPosition()
{
	unsigned long result = 0;
	DWORD pos = 0;
	::SendMessage( this->m_hwnd, EM_GETSEL, (LPARAM)&pos, 0 );
	result = pos;
	return result;
}

void Win32Edit::createParams()
{
	this->m_exStyleMask = WS_EX_CLIENTEDGE;
	this->m_styleMask = SIMPLE_VIEW;
}

void Win32Edit::processTextEvent( VCFWin32::KeyboardData keyData, WPARAM wParam, LPARAM lParam )
{
	VCF::TextModel* model = this->m_textControl->getTextModel();
	if ( NULL != model ){
		//this is the braindead way - needs to be reworked in the future
		int length = ::GetWindowTextLength( this->m_hwnd );
		char* text = new char[length+1];
		::GetWindowText( this->m_hwnd, text, length+1 );  

		VCF::String newText = text;
		
		m_OKToResetControlText = false;
		
		model->setText( newText );

		m_OKToResetControlText = true;

		delete[] text;

		/**
		*for the future we need to be able to insertText and deleteText from the
		*model according to the characters pressed. At first glance this is that
		*big of a deal: determine if we have a selection, delete the selected
		*text, and then insert the appropriate character. The problem comes in 
		*determing what causes a delete and what to insert. Normally the only things
		*that should cause an delete/insert are the set of characters [a..z,A..Z,0..9], 
		*back space, space, and delete. This is a valid assumption assuming en/US language
		*but for other languages this totally falls down...
		*/

		/*
		//figure out if anything is selected
		int selStart = 0;
		int selEnd = 0;
		::SendMessage( this->m_hwnd, EM_GETSEL, (WPARAM)&selStart, (LPARAM)&selEnd );
		int delta = abs( selStart - selEnd );
		if ( 0 != delta ){
			//text is being deleted

			if ( (keyData.VKeyCode != VK_BACK) || (keyData.VKeyCode == VK_BACK) ){
				
			}		
		}

		
		unsigned long index = this->m_component->getCaretPosition(); 
		VCF::String text;
		char ch[2] = {0};
		ch[0] = (VCF::VCFChar)keyData.character;
		text += ch;
		model->insertText( index, text );
		*/
	}
}

LRESULT Win32Edit::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc)
{
	LRESULT result = 0;

	switch ( message ) {
		case WM_CHAR: case WM_KEYDOWN: case WM_KEYUP:{
			KeyboardData keyData = Win32Utils::translateKeyData( this->m_hwnd, lParam );
			unsigned long eventType = 0;
			switch ( message ){
				case WM_CHAR:{
					eventType = KEYBOARD_EVENT_DOWN;									
				}
				break;

				case WM_KEYDOWN:{
					eventType = KEYBOARD_EVENT_PRESSED;
				}
				break;

				case WM_KEYUP:{
					eventType = KEYBOARD_EVENT_UP;
				}
				break;
			}
			
			unsigned long keyMask = Win32Utils::translateKeyMask( keyData.keyMask );
			ulong32 virtKeyCode = Win32Utils::translateVKCode( keyData.VKeyCode );
			VCF::KeyboardEvent event( this->m_component, eventType, keyData.repeatCount, 
				                 keyMask, (VCF::VCFChar)keyData.character,
								 virtKeyCode );

			processTextEvent( keyData, wParam, lParam );

			this->m_component->processKeyboardEvent( &event );			
			
			result = CallWindowProc( this->m_oldEditWndProc, m_hwnd, message, wParam, lParam );			
		}
		break;

		case WM_PAINT:{
			//result = (m_oldEditWndProc)(m_hwnd, message, wParam, lParam );
			result = CallWindowProc( this->m_oldEditWndProc, m_hwnd, message, wParam, lParam );
		}
		break;

		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( this->m_oldEditWndProc, m_hwnd, message, wParam, lParam );
		}
		break;
	}
	return result;
}

void Win32Edit::onTextModelTextChanged( TextEvent* event )
{
	if ( NULL != event ){ 
		String text = event->getChangeText();
		if ( true == m_OKToResetControlText ){
			this->setText( text );
		}
	}
}

void Win32Edit::setText( const VCF::String& text )
{
	AbstractWin32Component::setText( text );
}