/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "VCF.h"
#include "Win32Peer.h"

#include "Win32FilePeer.h"
#include "BasicFileError.h"
#include "StringUtils.h"

using namespace VCF;

Win32FilePeer::Win32FilePeer( const String& fileName )
{
	m_fileHandle = NULL;
	
	setName( fileName );
}

Win32FilePeer::~Win32FilePeer()
{
	if ( NULL != m_fileHandle ){
		::CloseHandle( m_fileHandle );
	}
}

void Win32FilePeer::remove()
{
	if ( NULL != m_fileHandle ){
		::CloseHandle( m_fileHandle );
		m_fileHandle = NULL;
	}

	::DeleteFile( m_filename.c_str() );
}

void Win32FilePeer::create()
{
	m_fileHandle = ::CreateFile( m_filename.c_str(), 
		                         GENERIC_READ | GENERIC_WRITE, 
								 FILE_SHARE_READ | FILE_SHARE_WRITE,
								 NULL,
								 OPEN_ALWAYS,
								 FILE_ATTRIBUTE_NORMAL,
								 NULL );

	if ( (NULL == m_fileHandle) || (INVALID_HANDLE_VALUE == m_fileHandle) ){
		m_fileHandle = NULL;
		//throw exception
		throw BasicFileError("Unable to create or attach to the specified file");
	}

}


uint32 Win32FilePeer::getSize()
{
	uint32 result = 0;
	if ( NULL != m_fileHandle ){
		GetFileSize( m_fileHandle, (DWORD*)&result );
	}
	return result;
}

void Win32FilePeer::setName( const String& fileName )
{	
	this->m_filename = fileName;
	
	if ( false == m_filename.empty() ){
		if ( NULL != m_fileHandle ){
			::CloseHandle( m_fileHandle );
		}
		//attach to the file
		String fileDir = StringUtils::getFileDir( m_filename );
		if ( true == fileDir.empty() ){
			TCHAR currentDir[MAX_PATH];
			memset( currentDir, 0 , MAX_PATH );
			GetCurrentDirectory( MAX_PATH, currentDir );
			m_filename = "\\" + m_filename;
			m_filename = currentDir +  m_filename;
		}

		m_fileHandle = ::CreateFile( m_filename.c_str(), 
									GENERIC_READ | GENERIC_WRITE, 
									FILE_SHARE_READ | FILE_SHARE_WRITE,
									NULL,
									OPEN_EXISTING,
									FILE_ATTRIBUTE_NORMAL,
									NULL );
		
		if ( (NULL == m_fileHandle) || (INVALID_HANDLE_VALUE == m_fileHandle) ){
			m_fileHandle = NULL;
			//throw exception
			throw BasicFileError("Unable to create or attach to the specified file");
		}
	}
}