/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32FileStream.cpp


#include "Win32FileStream.h"

using namespace VCF;


Win32FileStream::Win32FileStream( const String& filename, const FileStreamAccessType& accessType )
{
	m_filename = filename;
	m_fileHandle = CreateFile( filename.c_str(), 
		                       this->translateAccessType( accessType ),
                               0, //for now
							   NULL, //for now
							   translateAccessTypeToCreationType( accessType ),
							   FILE_ATTRIBUTE_NORMAL,
							   NULL );
	
	if ( INVALID_HANDLE_VALUE == m_fileHandle ){

		throw FileIOError( CANT_ACCESS_FILE + filename );
	}
}

Win32FileStream::~Win32FileStream()
{
	if ( INVALID_HANDLE_VALUE != m_fileHandle ){
		CloseHandle( m_fileHandle );
	}
}

void Win32FileStream::seek(const unsigned long& offset, const SeekType& offsetFrom)
{
	SetFilePointer( m_fileHandle, offset, NULL, translateSeekTypeToMoveType( offsetFrom ) );
}

unsigned long Win32FileStream::getSize()
{
	return GetFileSize( m_fileHandle, NULL );
}

void Win32FileStream::read( char* bytesToRead, unsigned long sizeOfBytes )
{
	DWORD bytesRead = 0;
	ReadFile( m_fileHandle, bytesToRead, sizeOfBytes, &bytesRead, NULL );
	if ( bytesRead != sizeOfBytes ){ //error if we are not read /writing asynchronously !
		//throw exception ?
	}
}
 
void Win32FileStream::write( char* bytesToWrite, unsigned long sizeOfBytes )
{
	DWORD bytesWritten = 0;
	WriteFile( m_fileHandle, bytesToWrite, sizeOfBytes, &bytesWritten, NULL );
	if ( bytesWritten != sizeOfBytes ){//error if we are not read /writing asynchronously !
		//throw exception ?
		throw new FileIOError( CANT_WRITE_TO_FILE + m_filename );
	}
}

char* Win32FileStream::getBuffer()
{
	return NULL;
}

DWORD Win32FileStream::translateAccessType( const FileStreamAccessType& accessType )
{
	DWORD result = 0;
	switch ( accessType ){
		case FS_READ : {
			result = GENERIC_READ;
		}
		break;

		case FS_WRITE : {
			result = GENERIC_WRITE;
		}
		break;

		case FS_READ_WRITE : case FS_DONT_CARE : {
			result = GENERIC_WRITE | GENERIC_READ;
		}
		break;
	}
	return result;
}

DWORD Win32FileStream::translateAccessTypeToCreationType( const FileStreamAccessType& accessType )
{
	DWORD result = 0;
	switch ( accessType ){
		case FS_READ : {
			result = OPEN_EXISTING;
		}
		break;

		case FS_WRITE : {
			result = CREATE_ALWAYS;
		}
		break;

		case FS_READ_WRITE : case FS_DONT_CARE : {
			result = OPEN_ALWAYS;
		}
		break;
	}
	return result;
}

DWORD Win32FileStream::translateSeekTypeToMoveType( const SeekType& offsetFrom )
{
	DWORD result = 0;
	switch ( offsetFrom ){
		case SEEK_FROM_START : {
			result = FILE_BEGIN;
		}
		break;

		case SEEK_FROM_RELATIVE : {
			result = FILE_CURRENT;
		}
		break;

		case SEEK_FROM_END : {
			result = FILE_END;
		}
		break;
	}
	return result;
}