/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32FontDialog.cpp

#include "Win32FontDialog.h"
#include "Control.h"
#include "ControlPeer.h"
#include "Color.h"
#include "Font.h"
#include "FontPeer.h"


using namespace VCF;


Win32FontDialog::Win32FontDialog( Control* owner )
{
	m_owner = owner;
}

Win32FontDialog::~Win32FontDialog()
{

}

void Win32FontDialog::setTitle( const String& title )
{

}

bool Win32FontDialog::execute()
{
	bool result = false;
	CHOOSEFONT chooseFont = {0};
	chooseFont.lStructSize = sizeof(CHOOSEFONT);
	if ( NULL != m_owner ){
		chooseFont.hwndOwner = (HWND)m_owner->getPeer()->getHandleID();
	}
	else {
		chooseFont.hwndOwner = GetActiveWindow();
	}
	LOGFONT logFont = {0};
	memcpy( &logFont, (LOGFONT*)m_font.getFontPeer()->getFontHandleID(), sizeof(LOGFONT) ); 
	
	chooseFont.lpLogFont = &logFont;
	chooseFont.Flags = CF_SCREENFONTS | CF_EFFECTS | CF_INITTOLOGFONTSTRUCT;
	Color* c = m_font.getColor();
	unsigned char r;
	unsigned char g;
	unsigned char b;
	c->getRGB( r, g, b );
	chooseFont.rgbColors = RGB( c->getRed()*255, c->getGreen()*255, c->getBlue()*255 );

	if ( TRUE == ChooseFont( &chooseFont ) ){
		r = GetRValue( chooseFont.rgbColors );
		g = GetGValue( chooseFont.rgbColors );
		b = GetBValue( chooseFont.rgbColors );
		c->setRGB( r,g,b);
		LOGFONT* fontLogFont = (LOGFONT*)m_font.getFontPeer()->getFontHandleID();
		memcpy( fontLogFont, chooseFont.lpLogFont, sizeof(LOGFONT) );
		result = true;
	}
	return result;
}

Font* Win32FontDialog::getSelectedFont()
{
	return &m_font;
}

void Win32FontDialog::setSelectedFont( Font* selectedFont )
{
	m_font.copy( selectedFont );
}
