/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Win32FontManager.h"
#include "StringUtils.h"

using namespace VCF;

Win32FontManager Win32FontManager::win32FontMgr;


/**
*Win32FontHandle code
*/
Win32FontHandle::Win32FontHandle( HFONT font )
{
	this->m_refCount = 0;
	this->m_fontHandle = NULL;
	this->m_fontHandle = font;
	this->incRef();
}

Win32FontHandle::~Win32FontHandle()
{
	if ( this->m_fontHandle != NULL ){
		DeleteObject( this->m_fontHandle );
	}
}

void Win32FontHandle::incRef()
{
	this->m_refCount ++;
}

void Win32FontHandle::decRef()
{
	if ( m_refCount > 0 ){
		this->m_refCount --;
		if ( (this->m_refCount == 0) && (this->m_fontHandle != NULL) ){
			DeleteObject( this->m_fontHandle );
		}
	}
}

HFONT Win32FontHandle::getFontHandle()
{
	return this->m_fontHandle;
}

bool Win32FontHandle::isRefCountZero()
{
	return 0 == m_refCount;
}
/**
*Win32FontManager code
*/

Win32FontManager::Win32FontManager()
{	
	m_defaultFont = new Win32Font( DEFAULT_VCF_FONT_NAME );
	addFont( m_defaultFont );
}

Win32FontManager::~Win32FontManager()
{
	std::map<String,Win32FontHandle*>::iterator it = m_fontMap.begin();
	
	while ( it != m_fontMap.end() ){
		Win32FontHandle* fh = it->second;
		if ( NULL != fh ){
			delete fh;
		}	
		fh = NULL;
		it ++;
	}
	
	m_fontMap.clear();

	delete m_defaultFont;
}

void Win32FontManager::addFont( Win32Font* font )
{	
	String fontID = makeStringID( font );
	if ( fontID.size() > 0 ){
		
		std::map<String,Win32FontHandle*>::iterator found = this->m_fontMap.find( fontID );
		if ( found != m_fontMap.end() ){ 
			Win32FontHandle* win32FontHandle = found->second;
			win32FontHandle->incRef();
		}
		else {// only add if no entry
			/**
			*no entry found so go ahead and create a new HFONT object
			*and store the entry
			*/
			HFONT fontHandle = CreateFontIndirect( (LOGFONT*)font->getFontHandleID() );
			if ( NULL != fontHandle ){
				m_fontMap[fontID] = new Win32FontHandle( fontHandle );
			}//otherwise throw exception ?????			
		}			
	}
}

String Win32FontManager::makeStringID(  Win32Font* font )
{
	String result = "";
	if ( NULL != font ){
		result = font->getName() + font->getColor()->toString() + 
			StringUtils::toString( (int)font->getPixelSize() ) + 
			StringUtils::toString( (int)font->getAngle() ) +
			StringUtils::toString( font->getBold() ) +
			StringUtils::toString( font->getUnderlined() ) +
			StringUtils::toString( font->getStrikeOut() ) +
			StringUtils::toString( font->getItalic() );
	}
	return result;
}

void Win32FontManager::initializeFont( Win32Font* font )
{
	Win32FontManager::win32FontMgr.addFont( font );
}

HFONT Win32FontManager::getFontHandleFromFontPeer( Win32Font* font )
{
	HFONT result = NULL;

	String fontID = Win32FontManager::win32FontMgr.makeStringID( font );

	std::map<String,Win32FontHandle*>::iterator found = 
		Win32FontManager::win32FontMgr.m_fontMap.find( fontID );
	
	Win32FontHandle* win32FontHandle = NULL;

	if ( found != Win32FontManager::win32FontMgr.m_fontMap.end() ){
		win32FontHandle = found->second;
	}
	else { 
	/**
	*this font should have called Win32FontManager::initializeFont()
	*but somehow it didn't get added - let's add it now
	*/
		Win32FontManager::win32FontMgr.addFont( font );
		found = Win32FontManager::win32FontMgr.m_fontMap.find( fontID );
		if ( found != Win32FontManager::win32FontMgr.m_fontMap.end() ){
			win32FontHandle = found->second;
		}
		//otherwise we're fucked - throw an exception !!!!!
	}

	if ( NULL != win32FontHandle ){
		result = win32FontHandle->getFontHandle();
	}
	return result;
}

void Win32FontManager::removeFont( Win32Font* font )
{
	String fontID = Win32FontManager::win32FontMgr.makeStringID( font );

	std::map<String,Win32FontHandle*>::iterator found = 
		Win32FontManager::win32FontMgr.m_fontMap.find( fontID );

	if ( found != Win32FontManager::win32FontMgr.m_fontMap.end() ){
		Win32FontHandle* fontHandle = found->second;
		fontHandle->decRef();

		if ( true == fontHandle->isRefCountZero() ){
			delete fontHandle;
			fontHandle = NULL;
			Win32FontManager::win32FontMgr.m_fontMap.erase( found );
		}
	}
}