/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Image.cpp


#include "Win32Image.h"
#include "ImageBits.h"
#include "UIGraphicsToolkit.h"
#include "GraphicsContext.h"

using namespace VCF;

BMPLoader BMPLoader::bmpLoaderInstance;


Win32Image::Win32Image()
{
	this->init();
}

Win32Image::Win32Image( const unsigned long& width, const unsigned long& height )	
{
	this->setWidth( width );
	this->setHeight( height );
	this->init();
}

Win32Image::Win32Image( const String& fileName )
{
	this->init();
	loadFromFile( fileName );
}

Win32Image::Win32Image( GraphicsContext* context, Rect* rect  )
{
	if ( (NULL != context) && (NULL != rect) ){
		Win32Context* ctx = dynamic_cast<Win32Context*>( context->getPeer() );
		if ( NULL != ctx ){
			//set up the bitmap data
			this->setWidth( rect->getWidth() );
			this->setHeight( rect->getHeight() );
			this->init();
			//HDC dc = ctx->getDC();			
			this->createBMP();

			context->copyContext( rect->m_left, rect->m_top, this->m_context );
			/*
			BitBlt( this->m_dc,
				    0,
					0,
					rect->getWidth(),
					rect->getHeight(),
					dc, 
					rect->m_left,
					rect->m_top,
					SRCCOPY );
			*/
		}
		else {
			//throw exception !!!	
		}
	}
	else {
		//throw exception !!!
	}
}

Win32Image::Win32Image( HBITMAP bitmap )
{
	init();
	loadFromBMPHandle( bitmap );
}

Win32Image::~Win32Image()
{
	if ( NULL != m_hBitmap ){
		DeleteObject( m_hBitmap );
		this->m_imageBits->m_bits = NULL;
	}
}

void Win32Image::init()
{
	HDC desktopDC = ::GetDC( ::GetDesktopWindow() );

	m_dc = ::CreateCompatibleDC( desktopDC );
	
	ReleaseDC( ::GetDesktopWindow(), desktopDC );

	this->m_context = new GraphicsContext( (long)m_dc );
}

void Win32Image::setWidth( const unsigned long & width )
{
	AbstractImage::setWidth( width );	
}

void Win32Image::setHeight( const unsigned long & height )
{
	AbstractImage::setHeight( height );	
}

void Win32Image::createBMP()
{	
	
	memset( &m_bmpInfo, 0, sizeof (BITMAPINFOHEADER) ); 
	this->m_bmpInfo.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
	this->m_bmpInfo.bmiHeader.biWidth = this->getWidth();
	this->m_bmpInfo.bmiHeader.biHeight = -this->getHeight(); // Win32 DIB are upside down - do this to filp it over
	this->m_bmpInfo.bmiHeader.biPlanes = 1;
	this->m_bmpInfo.bmiHeader.biBitCount = 32; 
	this->m_bmpInfo.bmiHeader.biCompression = BI_RGB;
	this->m_bmpInfo.bmiHeader.biSizeImage = m_bmpInfo.bmiHeader.biWidth * m_bmpInfo.bmiHeader.biHeight * 4;
	
	this->m_hBitmap = CreateDIBSection ( NULL, &this->m_bmpInfo, DIB_RGB_COLORS, (void **)&m_imageBits->m_bits, NULL, NULL ); 
	if ( NULL != m_hBitmap ) {
		HGDIOBJ oldObj = ::SelectObject( this->m_dc, m_hBitmap );
		::DeleteObject( oldObj );//clear out the old object
	}
}

HBITMAP Win32Image::getBitmap()
{
	return this->m_hBitmap;
}
	
HDC Win32Image::getDC()
{
	return this->m_dc;
}

void Win32Image::loadFromFile( const String& fileName )
{	
	HBITMAP hBMP = (HBITMAP)LoadImage( NULL, fileName.c_str(), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
	loadFromBMPHandle( hBMP );
}

void Win32Image::loadFromBMPHandle( HBITMAP bitmap )
{
	m_hBitmap = bitmap;
	BITMAP bmp = {0};
	GetObject( m_hBitmap, sizeof(bmp), &bmp );
	this->setHeight( bmp.bmHeight );
	this->setWidth( bmp.bmWidth );
	memset( &m_bmpInfo, 0, sizeof (BITMAPINFOHEADER) );
	this->m_bmpInfo.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
	HDC dc = CreateCompatibleDC( GetDC(GetDesktopWindow() ) );

	GetDIBits( dc, m_hBitmap, 0, 0, NULL, &m_bmpInfo, DIB_RGB_COLORS );
	GetDIBits( dc, m_hBitmap, 0, bmp.bmHeight, (void **)m_imageBits->m_bits, &m_bmpInfo, DIB_RGB_COLORS );
	
	DeleteDC( dc );
}


void Win32Image::saveToFile( const String& fileName )
{

}

BMPLoader::BMPLoader()
{
	UIGraphicsToolkit::getDefaultGraphicsToolkit()->registerImageLoader( "image/bmp", this );
}
	
BMPLoader::~BMPLoader()
{

}

Image* BMPLoader::loadImageFromFile( const String& fileName )
{
	return new Win32Image( fileName );
}
	
void BMPLoader::saveImageToFile( const String& fileName, Image* image )
{

}
