/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Object.h"
#include "LibraryPeer.h"
#include "Win32LibraryPeer.h"
#include "RuntimeException.h"
#include "InvalidPointerException.h"

using namespace VCF;

Win32LibraryPeer::Win32LibraryPeer()
{
	m_libHandle = NULL;
}

Win32LibraryPeer::~Win32LibraryPeer()
{
	m_libHandle = NULL;
}


void Win32LibraryPeer::load( const String& libraryFilename )
{
	m_libHandle = LoadLibrary( libraryFilename.c_str() );
	
	if ( NULL == m_libHandle ){
		String error = VCFWin32::Win32Utils::getErrorString( GetLastError() );
		throw RuntimeException( error );
	}
}

void* Win32LibraryPeer::getFunction( const String& functionName )
{
	void* result = NULL;
	
	if ( NULL == m_libHandle ){
		throw InvalidPointerException(MAKE_ERROR_MSG(INVALID_POINTER_ERROR), __LINE__);
	}

	result = GetProcAddress( m_libHandle, functionName.c_str() );

	if ( NULL == result ){
		String error = VCFWin32::Win32Utils::getErrorString( GetLastError() );
		throw RuntimeException( error );
	}
	
	return result;
}

void Win32LibraryPeer::unload()
{
	if ( NULL == m_libHandle ){
		throw InvalidPointerException(MAKE_ERROR_MSG(INVALID_POINTER_ERROR), __LINE__);
	}

	BOOL freeLibResult = FreeLibrary( m_libHandle );

	if ( FALSE == freeLibResult ){
		String error = VCFWin32::Win32Utils::getErrorString( GetLastError() );
		throw RuntimeException( error );
	}
}
