/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32ListBox.cpp


#include "Win32ListBox.h"
#include "Win32ToolKit.h"
#include "ListItem.h"
#include "ListModel.h"

using namespace VCFWin32;
using namespace VCF;


Win32ListBox::Win32ListBox( ListBoxControl* listboxControl ):
	AbstractWin32Component( listboxControl ),
	m_listboxControl( listboxControl )
{
	this->createParams();
	/*
	if ( true != this->isRegistered() ){
		registerWin32Class( this->getClassName(), m_wndProc  );
	}
	*/

	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();

	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             "ListBox",//className.c_str(), 
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){	
		Win32Object::registerWin32Object( this );
		m_oldListboxWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 
		this->m_defaultWndProc = NULL;
				
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32ListBox::~Win32ListBox()
{

}

void Win32ListBox::createParams()
{
	this->m_exStyleMask = WS_EX_CLIENTEDGE;
	this->m_styleMask = BORDERED_VIEW | LBS_EXTENDEDSEL | LBS_STANDARD | LBS_NOINTEGRALHEIGHT;
}

LRESULT Win32ListBox::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;
	
	switch ( message ) {		
		case WM_PAINT:{
			result = CallWindowProc( m_oldListboxWndProc, m_hwnd, message, wParam, lParam );
			//result = this->defaultWndProcedure( message, wParam, lParam );
			//result =  ::DefWindowProc( this->m_hwnd, message, wParam, lParam );// (m_oldTreeWndProc)(this->m_hwnd, message, wParam, lParam );
		}
		break;

		case WM_CREATE:{				
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
		}
		break;

		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( this->m_oldListboxWndProc, m_hwnd, message, wParam, lParam );
			
		}
		break;
	}
	return result;
}

void Win32ListBox::setFocusedItem(ListItem * item)
{
	//LB_SETCARETINDEX
	if ( NULL != item ){		
		::SendMessage( this->m_hwnd, LB_SETCARETINDEX, item->getIndex(), MAKELPARAM(TRUE, 0) );
	}
}

void Win32ListBox::clear()
{
	::SendMessage( this->m_hwnd, LB_RESETCONTENT, 0, 0 );
}

void Win32ListBox::rangeSelect(const bool & isSelected, ListItem * first, ListItem * last)
{
	ListModel* model = this->m_listboxControl->getListModel();
	if ( NULL != model ){
		
		Enumerator<ListItem*>* items = model->getItems();
		if ( NULL != items ){
			while ( items->hasMoreElements() ){
				ListItem* item = items->nextElement();
				if ( NULL != item ){
					unsigned long index = item->getIndex();
					/**this will handle setting items for 
					*first, last, and anything with an index
					*in between
					*/
					if ( (index >= first->getIndex()) && (  index <= last->getIndex()) ){
						::SendMessage( this->m_hwnd, LB_SETSEL, TRUE, index );
					}
				}
			}
		}
	}
	else {
		//throw exception
	}
}

void Win32ListBox::selectItem(ListItem * item)
{
	if ( NULL != item ){		
		::SendMessage( this->m_hwnd, LB_SETCURSEL, item->getIndex(), 0 );
	}
}

Rect* Win32ListBox::getItemRect()
{	
	//LB_GETITEMRECT
	RECT rect = {0};
	::SendMessage( this->m_hwnd, LB_GETITEMRECT, 0, (LPARAM)&rect );
	m_itemRect.m_left = rect.left;
	m_itemRect.m_top = rect.top;
	m_itemRect.m_right = rect.right;
	m_itemRect.m_bottom = rect.bottom;

	return &m_itemRect;
}

void Win32ListBox::addItem(ListItem * item)
{
	//LB_ADDSTRING
	if ( NULL != item ){
		String caption = item->getCaption();
		int index = ::SendMessage( this->m_hwnd, LB_ADDSTRING, 0, (LPARAM)caption.c_str() );
		item->setIndex( index );
	}
}

void Win32ListBox::insertItem( const unsigned long& index, ListItem * item)
{
	if ( NULL != item ){
		String caption = item->getCaption();
		::SendMessage( this->m_hwnd, LB_INSERTSTRING, index, (LPARAM)caption.c_str() );
		item->setIndex( index );
	}
}

bool Win32ListBox::isItemSelected(ListItem* item)
{
	return false;
}

ListItem* Win32ListBox::isPtOverItem(Point* point)
{
	return NULL;
}

ListItem* Win32ListBox::getFocusedItem()
{
	unsigned long index = ::SendMessage( this->m_hwnd, LB_GETCARETINDEX, 0, 0 );
	return NULL;
}

void Win32ListBox::setItemHeight( const unsigned long& itemHeight )
{
	::SendMessage( this->m_hwnd, LB_SETITEMHEIGHT, 0, MAKELPARAM(itemHeight, 0) ); 
}

ListItem* Win32ListBox::getSelectedItem()
{	
	ListItem* result = NULL;
	unsigned long index = ::SendMessage( this->m_hwnd, LB_GETCURSEL, 0, 0 );
	
	ListModel* model = this->m_listboxControl->getListModel();
	if ( NULL != model ){
		result = model->getItemFromIndex( index );
	}

	return result;
}

unsigned long Win32ListBox::getItemHeight()
{
	return ::SendMessage( this->m_hwnd, LB_GETITEMHEIGHT, 0, 0 ); 
}

void Win32ListBox::removeItem( ListItem* item )
{
	if ( NULL != item ){
		int index = item->getIndex();
		::SendMessage( this->m_hwnd, LB_DELETESTRING, index, 0 );
	}		
}