/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Listview.cpp


#include "Win32Listview.h"
#include "Win32ToolKit.h"
#include "ListModel.h"
#include "ListItem.h"

using namespace VCFWin32;
using namespace VCF;


Win32Listview::Win32Listview( ListViewControl* listviewControl ):
	AbstractWin32Component( listviewControl ),
	m_listviewControl( listviewControl )
{
	this->createParams();
	/*
	if ( true != this->isRegistered() ){
		registerWin32Class( this->getClassName(), m_wndProc  );
	}
	*/

	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();

	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             WC_LISTVIEW,//"SysListView32",//className.c_str(), 
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){	
		Win32Object::registerWin32Object( this );
		m_oldListviewWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 
		this->m_defaultWndProc = NULL;
				
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32Listview::~Win32Listview()
{

}

void Win32Listview::createParams()
{
	this->m_exStyleMask = WS_EX_CLIENTEDGE;
	this->m_styleMask = BORDERED_VIEW | LVS_ALIGNLEFT | LVS_AUTOARRANGE | LVS_ICON | LVS_SHOWSELALWAYS | LVS_SORTASCENDING | LVS_EDITLABELS;
}

LRESULT Win32Listview::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;
	
	switch ( message ) {		
		case WM_PAINT:{
			result = CallWindowProc( m_oldListviewWndProc, m_hwnd, message, wParam, lParam );
		}
		break;

		case WM_CREATE:{				
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
		}
		break;
		
		case LVN_BEGINDRAG:{				
			NMLISTVIEW* lvNotificationHdr = (LPNMLISTVIEW)lParam;
			
		}
		break;

		case LVN_BEGINLABELEDIT:{
			
		}
		break;

		case LVN_BEGINRDRAG:{
			
		}
		break;

		case LVN_COLUMNCLICK:{
			
		}
		break;

		case LVN_DELETEALLITEMS:{
			
		}
		break;

		case LVN_DELETEITEM:{
			
		}
		break;

		case LVN_ENDLABELEDIT:{
			NMLVDISPINFO* lvDispInfo = (NMLVDISPINFO*)lParam;
			result = (NULL != lvDispInfo->item.pszText) ? TRUE : FALSE;
			if ( NULL != lvDispInfo->item.pszText ){
				String newCaption = lvDispInfo->item.pszText;			
				ListModel* model = this->m_listviewControl->getListModel();
				
				ListItem* item = model->getItemFromIndex( lvDispInfo->item.iItem );
				if ( NULL != item ){
					if ( lvDispInfo->item.iItem == item->getIndex() ) {	
						item->setCaption( newCaption );
						item->setSelected( true );
					}
				}
			}
		}
		break;

		case LVN_GETDISPINFO:{
			
		}
		break;

		case LVN_GETINFOTIP:{
			
		}
		break;

		case LVN_HOTTRACK:{
			
		}
		break;

		case LVN_INSERTITEM:{
			
		}
		break;

		case LVN_ITEMACTIVATE:{
			
		}
		break;

		case LVN_ITEMCHANGED:{
			NMLISTVIEW* lvNotificationHdr = (LPNMLISTVIEW)lParam;
			
			if ( -1 != lvNotificationHdr->iItem ){
				ListModel* model = this->m_listviewControl->getListModel();
			
				ListItem* item = model->getItemFromIndex( lvNotificationHdr->iItem );
				if ( NULL != item ){
					if ( (lvNotificationHdr->uChanged & LVIF_STATE) != 0 ) {
						if ( (lvNotificationHdr->uNewState & LVIS_SELECTED) != 0 ){
							if ( lvNotificationHdr->iItem == item->getIndex() ){
								/*POINT tmpPt = {0};
								GetCursorPos( &tmpPt );
								::ScreenToClient( this->m_hwnd, &tmpPt );
								ItemEvent event( m_listviewControl, ITEM_EVENT_SELECTED );
								Point pt( tmpPt.x, tmpPt.y );
								event.setPoint( &pt );
								*/
								item->setSelected( true );
							}
						}
					}
				}
			}

		}
		break;

		case LVN_ITEMCHANGING:{
			
		}
		break;

		case LVN_KEYDOWN:{
			
		}
		break;

		case LVN_MARQUEEBEGIN:{
			
		}
		break;

		case LVN_ODCACHEHINT:{
				
		}
		break;

		case LVN_ODFINDITEM:{
				
		}
		break;

		case LVN_ODSTATECHANGED:{
				
		}
		break;

		case LVN_SETDISPINFO:{
				
		}
		break;

		case NM_CUSTOMDRAW:{
			
		}
		break;

		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( m_oldListviewWndProc, m_hwnd, message, wParam, lParam );
			
		}
		break;
	}	
	return result;
}

void Win32Listview::addItem( ListItem * item )
{
	//LVM_INSERTITEM
	if ( NULL != item ){
		LVITEM lvItem = {0};
		lvItem.mask = LVIF_TEXT | LVIF_PARAM;
		lvItem.pszText = const_cast<char*>( item->getCaption().c_str() );
		lvItem.cchTextMax = item->getCaption().size();
		lvItem.lParam = (LPARAM)item;
		int index = ListView_InsertItem( this->m_hwnd, &lvItem );
		if ( index != -1 ) {
			item->setIndex( index );
		}
		else{
			//throw exception
		}
		item->addItemListener( this );
	}
}

void Win32Listview::insertItem( const unsigned long& index, ListItem * item )
{
	if ( NULL != item ){
		LVITEM lvItem = {0};
		lvItem.mask = LVIF_TEXT | LVIF_PARAM;
		lvItem.iItem = index;
		lvItem.pszText = const_cast<char*>( item->getCaption().c_str() );
		lvItem.cchTextMax = item->getCaption().size();
		lvItem.lParam = (LPARAM)item;
		int index = ListView_InsertItem( this->m_hwnd, &lvItem );
		if ( index != -1 ) {
			item->setIndex( index );
		}
		else{
			//throw exception
		}

		item->addItemListener( this );
	}
}

void Win32Listview::clear()
{
	ListView_DeleteAllItems( this->m_hwnd );
}
		
void Win32Listview::deleteItem( ListItem* item )
{
	if ( NULL != item ){
		ListView_DeleteItem( this->m_hwnd, item->getIndex() );
	}
}

void Win32Listview::setFocusedItem(ListItem * item)
{
	if ( NULL != item ){
		ListView_SetItemState( this->m_hwnd, item->getIndex(),
			                   LVIS_FOCUSED, LVIS_FOCUSED );
	}
}

void Win32Listview::selectItem(ListItem * item)
{
	if ( NULL != item ){
		ListView_SetItemState( this->m_hwnd, item->getIndex(),
			                   LVIS_SELECTED, LVIS_SELECTED );
	}
}

Rect* Win32Listview::getItemRect( ListItem* item )
{
	RECT rect = {0};
	if ( NULL != item ){
		ListView_GetItemRect( this->m_hwnd, item->getIndex(), 
		                      &rect, LVIR_BOUNDS );
		m_itemRect.setRect( rect.left, rect.top, rect.right, rect.bottom );
	}
	else {
		return NULL;
	}
	return &m_itemRect;
}

bool Win32Listview::isItemSelected(ListItem* item)
{
	bool result = false;
	if ( NULL != item ){
		result = ( LVIS_SELECTED == 
			           ListView_GetItemState( this->m_hwnd, item->getIndex(), LVIS_SELECTED ) );
			                                              
	}
	return result;
}

ListItem* Win32Listview::isPtOverItem(Point* point)
{
	return NULL;
}

ListItem* Win32Listview::getFocusedItem()
{
	return NULL;
}

ListItem* Win32Listview::getSelectedItem()
{
	ListItem* result = NULL;
	int index = ListView_GetSelectionMark( this->m_hwnd );
	if ( -1 > index ){
		ListModel* model = this->m_listviewControl->getListModel();
		if ( NULL != model ){
			result = model->getItemFromIndex( index );
		}
	}
	return result;
}

Enumerator<ListItem*>* Win32Listview::getSelectedItems()
{
	int index = ListView_GetSelectionMark( this->m_hwnd );
	if ( -1 > index ){
		ListModel* model = this->m_listviewControl->getListModel();
		if ( NULL != model ){
			m_selectedItems.clear();
			ListItem* item = NULL;
			int i = 0;
			int count = ListView_GetSelectedCount( this->m_hwnd );
			while ( i < count ){
				if ( LVIS_SELECTED == ListView_GetItemState( this->m_hwnd, index, LVIS_SELECTED ) ){
					item = 	model->getItemFromIndex( index );
					m_selectedItems.push_back( item );
				}
				index++;
				i++;
			}
		}
	}
	else {
		return NULL;
	}

	m_enumContainer.initContainer( m_selectedItems );
	return m_enumContainer.getEnumerator();
}

void Win32Listview::rangeSelect( Rect* selectionRect )
{

}

void Win32Listview::addHeaderColumn( const String& columnName )
{
	LVCOLUMN column = {0};
	column.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	column.cx = 200;
	column.fmt = LVCFMT_LEFT;
	column.pszText = const_cast<char*>( columnName.c_str() );
	column.cchTextMax = columnName.size();
	ListView_InsertColumn( this->m_hwnd, 0, &column );
}

void Win32Listview::insertHeaderColumn( const unsigned long& index, const String& columnName )
{

}

void Win32Listview::deleteHeaderColumn( const unsigned long& index )
{

}

IconStyleType Win32Listview::getIconStyle()
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	return translateStyleToIconStyle( style );
}

void Win32Listview::setIconStyle( const IconStyleType& iconStyle )
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	if ( IA_NONE == iconStyle ){
		style = style ^ LVS_ALIGNLEFT;
		style = style ^ LVS_ALIGNTOP;
		style = style ^ LVS_AUTOARRANGE;
	}
	else {
		style |= translateIconStyleToStyleBit( iconStyle );
	}

	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
}

bool Win32Listview::getAllowsMultiSelect()
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	return ((style & LVS_SINGLESEL) == 0);
}

void Win32Listview::setAllowsMultiSelect( const bool& allowsMultiSelect )
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	if ( true == allowsMultiSelect ){
		style = style ^ LVS_SINGLESEL;
	}
	else {		
		style |= LVS_SINGLESEL;
	}
	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
}

IconAlignType Win32Listview::getIconAlignment()
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	return this->translateStyleToIconAlignment( style );	
}

void Win32Listview::setIconAlignment( const IconAlignType& iconAlignType )
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	style |= this->translateIconAlignmentToStyleBit( iconAlignType );
	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
}

bool Win32Listview::getAllowLabelEditing()
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	return ((style & LVS_EDITLABELS) == 0);
}

void Win32Listview::setAllowLabelEditing( const bool& allowLabelEditing )
{
	DWORD style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	if ( true == allowLabelEditing ){
		style |= LVS_EDITLABELS;
	}
	else {		
		style = style ^ LVS_EDITLABELS;		
	}
	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
}

IconStyleType Win32Listview::translateStyleToIconStyle( const DWORD& wsStyle )
{
	IconStyleType result;
	if ( (wsStyle & LVS_ICON) != 0 ){
		result = IS_LARGE_ICON;	
	}
	else if ( (wsStyle & LVS_LIST) != 0 ){
		result = IS_LIST;	
	}
	else if ( (wsStyle & LVS_SMALLICON) != 0 ){
		result = IS_SMALL_ICON;	
	}
	else if ( (wsStyle & LVS_REPORT) != 0 ){
		result = IS_DETAILS;	
	}
	return result;
}

DWORD Win32Listview::translateIconStyleToStyleBit( const IconStyleType& iconStyle )
{
	DWORD result = 0;
	switch ( iconStyle ){
		case IS_LARGE_ICON:{
			result = LVS_ICON;
		}
		break;

		case IS_SMALL_ICON:{
			result = LVS_SMALLICON;
		}
		break;

		case IS_LIST:{
			result = LVS_LIST;
		}
		break;

		case IS_DETAILS:{
			result = LVS_REPORT;
		}
		break;
	}
	return result;
}

IconAlignType Win32Listview::translateStyleToIconAlignment( const DWORD& wsStyle )
{
	IconAlignType result;
	if ( (wsStyle & LVS_ALIGNTOP) != 0 ){
		result = IA_TOP;	
	}
	else if ( (wsStyle & LVS_ALIGNLEFT) != 0 ){
		result = IA_LEFT;	
	}
	else if ( (wsStyle & LVS_AUTOARRANGE) != 0 ){
		result = IA_AUTO_ARRANGE;	
	}
	else {
		result = IA_NONE;	
	}
	return result;
}

DWORD Win32Listview::translateIconAlignmentToStyleBit( const IconAlignType& iconAlignment )
{
	DWORD result = 0;
	switch ( iconAlignment ){
		case IA_TOP:{
			result = LVS_ALIGNTOP;
		}
		break;

		case IA_LEFT:{
			result = LVS_ALIGNLEFT;
		}
		break;

		case IA_AUTO_ARRANGE:{
			result = LVS_AUTOARRANGE;
		}
		break;		
		case IA_NONE:{
			result = 0;
		}
		break;
	}
	return result;
}

void Win32Listview::onItemPaint( ItemEvent* event )
{

}

void Win32Listview::onItemChanged( ItemEvent* event )
{	
	switch ( event->getType() ){
			case ITEM_EVENT_TEXT_CHANGED:{
				Object* source = event->getSource();
				if ( NULL != source ){
					ListItem* item = dynamic_cast<ListItem*>(source);
					if ( NULL != item ){						
						LVITEM lvItem = {0};
						lvItem.mask = LVIF_TEXT;						
						String caption = item->getCaption();
						lvItem.cchTextMax = caption.size();
						lvItem.iItem = item->getIndex();
						char* tmp = new char[lvItem.cchTextMax];
						memset( tmp, 0, lvItem.cchTextMax );
						caption.copy( tmp, lvItem.cchTextMax );
						lvItem.pszText = tmp;							
						ListView_SetItem( this->m_hwnd, &lvItem );
						delete [] tmp;
					}
				}				
			}
			break;
		}
}

void Win32Listview::onItemSelected( ItemEvent* event )
{

}

void Win32Listview::onItemAdded( ItemEvent* event )
{

}
	
void Win32Listview::onItemDeleted( ItemEvent* event )
{

}