/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 


#include "Win32Peer.h"
#include "Win32OpenGLPeer.h"
#include "GraphicsContext.h"
#include "Win32Context.h"
#include "VCFOpenGL.h"

//gl stuff


using namespace VCF;

Win32OpenGLPeer::Win32OpenGLPeer( GraphicsContext* glContext )
{
	this->m_glContext = glContext;
	m_isInitialized = false;
}

Win32OpenGLPeer::~Win32OpenGLPeer()
{
	if ( NULL != m_glContext ){
		Win32Context* win32Ctx = dynamic_cast<Win32Context*>(m_glContext->getPeer());
		if ( NULL == win32Ctx ){
			//throw exception - we are screwed !!!!
		}
		HDC dc = (HDC)win32Ctx->getContextID();
		wglMakeCurrent( dc, NULL );
		wglDeleteContext( m_hrc );
	}
}

void Win32OpenGLPeer::initGL()
{
	if ( true == m_isInitialized ){
		return;
	}
	if ( NULL != m_glContext ){
		Win32Context* win32Ctx = dynamic_cast<Win32Context*>(m_glContext->getPeer());
		if ( NULL == win32Ctx ){
			//throw exception - we are screwed !!!!
		}
		HDC dc = (HDC)win32Ctx->getContextID();
		if ( NULL != dc ){
			PIXELFORMATDESCRIPTOR pfd =
			{
				sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
					1,                              // version number
					PFD_DRAW_TO_WINDOW |            // support window
					PFD_SUPPORT_OPENGL |          // support OpenGL
					PFD_DOUBLEBUFFER,             // double buffered
					PFD_TYPE_RGBA,                  // RGBA type
					24,                             // 24-bit color depth
					0, 0, 0, 0, 0, 0,               // color bits ignored
					0,                              // no alpha buffer
					0,                              // shift bit ignored
					0,                              // no accumulation buffer
					0, 0, 0, 0,                     // accum bits ignored
					32,                             // 32-bit z-buffer
					0,                              // no stencil buffer
					0,                              // no auxiliary buffer
					PFD_MAIN_PLANE,                 // main layer
					0,                              // reserved
					0, 0, 0                         // layer masks ignored
			};
			
			int pixelformat = ChoosePixelFormat( dc, &pfd );
			if ( pixelformat == 0 ){
				//throw exception
			}
			
			if ( FALSE == SetPixelFormat( dc, pixelformat, &pfd ) ){
				//throw exception
			}
			
			m_hrc = wglCreateContext( dc );
			m_isInitialized = ( TRUE == wglMakeCurrent( dc, m_hrc ) ) ? true : false;
		}
	}
}

void Win32OpenGLPeer::swapBuffers()
{
	if ( NULL != m_glContext ){
		Win32Context* win32Ctx = dynamic_cast<Win32Context*>(m_glContext->getPeer());
		if ( NULL == win32Ctx ){
			//throw exception - we are screwed !!!!
		}
		HDC dc = (HDC)win32Ctx->getContextID();
		::SwapBuffers( dc );
	}
}