/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Win32Peer.cpp

#include "Win32Peer.h"
#include "CommonDefines.h"
#include <bitset>


using namespace VCFWin32;


unsigned long Win32Utils::translateKeyMask( UINT win32KeyMask )
{	
	unsigned long result = KEY_UNDEFINED;

	if ( (win32KeyMask & MK_CONTROL) != 0 ){
		result |= KEY_CTRL;
	}

	if ( (win32KeyMask & MK_SHIFT) != 0 ){
		result |= KEY_SHIFT;
	}
	
	if ( ::GetAsyncKeyState( VK_MENU ) < 0 ) {
		result |= KEY_ALT;
	}
	
	return result;
}

unsigned long Win32Utils::translateButtonMask( UINT win32ButtonMask )
{
	unsigned long result = MOUSE_UNDEFINED;

	if ( (win32ButtonMask & MK_LBUTTON) != 0 ){
		result |= MOUSE_LEFT_BUTTON;
	}

	if ( (win32ButtonMask & MK_MBUTTON) != 0 ){
		result |= MOUSE_MIDDLE_BUTTON;
	}

	if ( (win32ButtonMask & MK_RBUTTON) != 0 ){
		result |= MOUSE_RIGHT_BUTTON;
	}

	return result;
}

DWORD Win32Utils::translateStyle( unsigned long style )
{
	DWORD result = 0;
	//if ( (style & ) > 0 ){
		
	//};
	return result;
}

DWORD Win32Utils::translateExStyle( unsigned long style )
{
	DWORD result = 0;

	return result;
}

/**
*translates the Win32 key code into an actually usable 
*structure.
*Specifies the repeat count, scan code, extended-key flag, context code, previous key-state flag, 
*and transition-state flag, as shown in the following table. 
*Value Description 
*	015 Specifies the repeat count for the current message. 
*	     The value is the number of times the keystroke is autorepeated 
*		 as a result of the user holding down the key. If the keystroke is held 
*		 long enough, multiple messages are sent. However, the repeat count is not cumulative. 
*
*	1623 Specifies the scan code. The value depends on the original equipment manufacturer (OEM). 
*
*	24   Specifies whether the key is an extended key, such as the right-hand ALT and CTRL keys that 
*	     appear on an enhanced 101- or 102-key keyboard. The value is 1 if it is an extended key; 
*		 otherwise, it is 0. 
*
*	2528 Reserved; do not use. 
*
*	29   Specifies the context code. The value is 1 if the ALT key is held down while the 
*        key is pressed; otherwise, the value is 0. 
*
*	30   Specifies the previous key state. The value is 1 if the key is down before
*        the message is sent, or it is 0 if the key is up. 
*	31   Specifies the transition state. The value is 1 if the key is being released, or it 
*	     is 0 if the key is being pressed. 
*/
KeyboardData Win32Utils::translateKeyData( HWND wndHandle, LPARAM keyData )
{
	KeyboardData result = {0};	
	int sum = 0;

	sum = ((BYTE*)&keyData)[2]; //gets bits 16-23
	
	BYTE keyState[256];
	if ( TRUE == GetKeyboardState( &keyState[0] ) ){
		result.scanCode = sum;
		result.repeatCount = keyData & 0xFFFF; //mask out the upper 16 bits
		result.altKeyDown = ( keyData & KB_CONTEXT_CODE ) != 0;//replace with #define
		result.isExtendedKey = ( keyData & KB_IS_EXTENDED_KEY ) != 0;
		result.character = 0;
		result.VKeyCode = MapVirtualKey( result.scanCode, 1);
		HKL keyboardLayout = GetKeyboardLayout( GetWindowThreadProcessId( wndHandle, NULL ) );

		ToAsciiEx( result.VKeyCode, result.scanCode, &keyState[0], &result.character, 1, keyboardLayout );
		
		std::bitset<16> keyBits;
		keyBits = GetAsyncKeyState( VK_SHIFT );		
		if ( keyBits[15] == 1 ){
			result.keyMask |= MK_SHIFT;
		}

		keyBits = GetAsyncKeyState( VK_CONTROL );
		if ( keyBits[15] == 1 ){
			result.keyMask |= MK_CONTROL;
		}

		keyBits = GetAsyncKeyState( VK_MENU );
		if ( keyBits[15] == 1 ){
			result.altKeyDown = true;
		}
	}
	return result;
}

void Win32Utils::trace( const VCF::String& text )
{
#ifdef _DEBUG
	OutputDebugString( TEXT( text.c_str() ) );
#endif
}

WORD Win32Utils::getWin32LangID( VCF::Locale* locale )
{
	WORD result = MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT );
	
	if ( NULL != locale ){
		switch( locale->getCountryCode() ){
			case LOCALE_USA_CODE :{
				if ( locale->getLanguage() == "en" ){
					result = MAKELANGID( LANG_ENGLISH, SUBLANG_ENGLISH_US );
				}
			}
			break;
		}
	}
	
	return result;
}

int Win32Utils::getXFromLParam( LPARAM lParam )
{
	return (int)(short) LOWORD(lParam);
}
	
int Win32Utils::getYFromLParam( LPARAM lParam )
{
	return (int)(short) HIWORD(lParam);
}

VCF::ulong32 Win32Utils::translateVKCode( UINT vkCode )
{
	VCF::ulong32 result = 0;

	switch ( vkCode ){
		case VK_F1 :{
			result = VIRT_KEY_F1;
		}
		break;

		case VK_F2 :{
			result = VIRT_KEY_F2;
		}
		break;

		case VK_F3 :{
			result = VIRT_KEY_F3;
		}
		break;

		case VK_F4 :{
			result = VIRT_KEY_F4;
		}
		break;

		case VK_F5 :{
			result = VIRT_KEY_F5;
		}
		break;

		case VK_F6 :{
			result = VIRT_KEY_F6;
		}
		break;

		case VK_F7 :{
			result = VIRT_KEY_F7;
		}
		break;

		case VK_F8 :{
			result = VIRT_KEY_F8;
		}
		break;

		case VK_F9 :{
			result = VIRT_KEY_F9;
		}
		break;

		case VK_F10 :{
			result = VIRT_KEY_F10;
		}
		break;

		case VK_F11 :{
			result = VIRT_KEY_F11;
		}
		break;

		case VK_F12 :{
			result = VIRT_KEY_F12;
		}
		break;

		case VK_UP :{
			result = VIRT_KEY_UP_ARROW;
		}
		break;

		case VK_DOWN :{
			result = VIRT_KEY_DOWN_ARROW;
		}
		break;

		case VK_LEFT :{
			result = VIRT_KEY_LEFT_ARROW;
		}
		break;

		case VK_RIGHT :{
			result = VIRT_KEY_RIGHT_ARROW;
		}
		break;
	}

	return result;
}

bool Win32Utils::fileExists( const VCF::String& fileName )
{
	bool result = false;
	HANDLE fileHandle = CreateFile( fileName.c_str(), 
									0,
									FILE_SHARE_READ | FILE_SHARE_WRITE, //for now
									NULL, //for now
									OPEN_EXISTING,
									FILE_ATTRIBUTE_NORMAL,
									NULL );
	result = (fileHandle != INVALID_HANDLE_VALUE) ? true : false;
	CloseHandle( fileHandle );
	return result;
}

VCF::String Win32Utils::getErrorString( const DWORD& errorCode )
{
	VCF::String result = "";
	
	TCHAR* msgBuf;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				   NULL,
				   errorCode,
				   0, // Default language
				   (LPTSTR) &msgBuf,
				   0,
				   NULL );
	
	result = msgBuf;
	// Free the buffer.
	LocalFree( msgBuf );
	

	return result;
}