/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32ResourceBundle.cpp

#include "Win32Peer.h"
#include "Win32ResourceBundle.h"
#include "Win32Image.h"

using namespace VCF;


BOOL CALLBACK Win32ResourceBundle_EnumResTypeProc( HMODULE hModule, LPTSTR lpszType, LPARAM lParam );
BOOL CALLBACK Win32ResourceBundle_EnumResNameProc( HMODULE hModule, LPCTSTR lpszType, LPTSTR lpszName, LPARAM lParam );

static bool foundResName = false;
static String foundResType="Hello";

Win32ResourceBundle::Win32ResourceBundle()
{

}

Win32ResourceBundle::~Win32ResourceBundle()
{
	
}

String Win32ResourceBundle::getString( const String& resourceName )
{	
	String result;
	HRSRC resHandle = ::FindResource( ::GetModuleHandle(NULL), resourceName.c_str(), RT_RCDATA );
	if ( NULL != resHandle ){
		HGLOBAL	data = ::LoadResource( NULL, resHandle );
		if ( NULL != data ){
			TCHAR* strData = (TCHAR*)::LockResource( data );
			result = strData;
			::FreeResource( data );
		}
	}
	else {
		//throw exception- resource not found !!!!
	}
	return result;
}

String Win32ResourceBundle::getVFF( const String& resourceName )
{
	String result;
	HRSRC resHandle = ::FindResource( ::GetModuleHandle(NULL), resourceName.c_str(), "VFF" );
	if ( NULL != resHandle ){
		HGLOBAL	data = ::LoadResource( NULL, resHandle );
		if ( NULL != data ){
			void* dataPtr = ::LockResource( data );
			TCHAR* strData = (TCHAR*)dataPtr;
			int size = SizeofResource( ::GetModuleHandle(NULL), resHandle );			
			result = strData;
			int resSize = result.size();
			void *tmp = dataPtr;
			/**
			*this is here to properly skip over '\0' characters in the stream
			*/
			while ( resSize < size ){
				tmp = (void*)((char*)dataPtr + resSize + 1);
				strData = (TCHAR*)tmp;
				result += "\n";
				result += strData;
				resSize = result.size();
			}
			::FreeResource( data );
		}
	}
	else {
		//throw exception- resource not found !!!!
	}
	return result;
}

Image* Win32ResourceBundle::getImage( const String& resourceName )
{
	HBITMAP resBMP = LoadBitmap( ::GetModuleHandle(NULL), resourceName.c_str() );
	if ( NULL != resBMP ){
		return new Win32Image( resBMP );
	}
	else {
		//throw exception- resource not found !!!!
	}
	return NULL;
}

Resource* Win32ResourceBundle::getResource( const String& resourceName )
{
	foundResName = false;
	foundResType = "\0";
	::EnumResourceTypes( ::GetModuleHandle(NULL), 
		                 Win32ResourceBundle_EnumResTypeProc,
						 (LPARAM)resourceName.c_str() );
	if ( true == foundResName ){
		HRSRC resHandle = ::FindResource( ::GetModuleHandle(NULL), 
			                              resourceName.c_str(), 
										  foundResType.c_str() );
		if ( NULL != resHandle ){
			HGLOBAL	dataHandle = ::LoadResource( NULL, resHandle );
			if ( NULL != dataHandle ){
				void* data = ::LockResource( dataHandle );
				int size = ::SizeofResource( ::GetModuleHandle(NULL), resHandle );
				return new Resource( data, size, resourceName );				
			}
		}
		else {
			//throw exception- resource not found !!!!
		}
	}
	return NULL;
}

BOOL CALLBACK Win32ResourceBundle_EnumResTypeProc( HMODULE hModule, LPTSTR lpszType, LPARAM lParam )
{
	return ::EnumResourceNames( hModule,
		                        lpszType,
								Win32ResourceBundle_EnumResNameProc,
								lParam );
}

BOOL CALLBACK Win32ResourceBundle_EnumResNameProc( HMODULE hModule, LPCTSTR lpszType, LPTSTR lpszName, LPARAM lParam )
{
	BOOL result = TRUE;
	const String resName( (TCHAR*)lParam );
	if ( resName == lpszName ){
		foundResName = true;
		foundResType = lpszType;
		result = FALSE;
	}
	return result;
}