// Win32SocketImplementer.cpp


#include "Win32SocketImplementer.h"
#include <iostream>
#include <algorithm>

using namespace VCF;
using namespace VCFNet;


bool Win32SocketImplementer::m_ready = FALSE;
int  Win32SocketImplementer::m_instances = 0;


/*
*   Structure passed into the accept thread
*/
typedef struct {
    int    nThread;				// Thread number.
    Win32SocketImplementer *improvSocket;	// Pointer to owner
} THREADPACK, *PTHREADPACK;


void __cdecl AcceptThreadProc(void *ptr) {
    PTHREADPACK ptp = (PTHREADPACK)ptr;
    ptp->improvSocket->runAsServer(ptp->nThread);
}


static long fillAddress(PSOCKADDR_IN psin, const char *host, u_short port)
{
    PHOSTENT phe;

    psin->sin_family = AF_INET;
    psin->sin_port = htons(port);        /* Convert to network ordering */


    if (host != NULL) {
        phe = gethostbyname(host);
        if (phe == NULL) {
            return VCFNET_SOCKET_NAME_ERROR;
        }
        memcpy((char FAR *)&(psin->sin_addr), phe->h_addr, phe->h_length);

    } else { // server

        // gethostname(buff, dwSize);

        psin->sin_addr.s_addr = INADDR_ANY;
    }    
	return VCFNET_SOCKET_OK;
}


Win32SocketImplementer::Win32SocketImplementer()
{
	m_instances++;
	m_clientData = NULL;
}

Win32SocketImplementer::~Win32SocketImplementer()
{
	m_instances--;
    if ( m_instances <= 0 ) {
		closedown();
	}
}

int Win32SocketImplementer::getLocalPort()
{
	return 0;
}
    
void Win32SocketImplementer::setLocalPort( const int& localPort )
{

}

VCF::String Win32SocketImplementer::getLocalHost()
{
	return this->m_localHost;
}

void Win32SocketImplementer::setLocalHost( const VCF::String& localHost )
{
	this->m_localHost = localHost;
}

int Win32SocketImplementer::getPort()
{
	return this->m_port;
}

void Win32SocketImplementer::setPort( const int& port )
{
	this->m_port = (u_short)port;
}

VCF::String Win32SocketImplementer::getHost()
{
	return this->m_host;
}

void Win32SocketImplementer::setHost( const VCF::String& host )
{
	this->m_host = host;
}

void Win32SocketImplementer::startup()
{
	WSADATA WSAData;
    int status;
    
    
    if (m_ready) {
        // Do nothing
    } else if ((status = WSAStartup(MAKEWORD(2,2), &WSAData)) == 0) {
        m_ready = true;
    } else {
        //result = VCFNET_SOCKET_STARTUP_ERROR;
    }
}

void Win32SocketImplementer::closedown()
{
	if (m_ready) {
        WSACleanup();
        m_ready = FALSE;
    }
}

int Win32SocketImplementer::receive( char* data, uint32& size )
{
	char sz[4];
    int status;
	
    status = recv(m_sock, sz, 4, 0);

    if (status == 0) { // Socket was closed
        data[0] = 0;
        size = 0;
        return VCFNET_SOCKET_CLOSED_ERROR; //throw exception
    } else if (status != 4) {
        data[0] = 0;
        size = 0;
        return VCFNET_SOCKET_READ_ERROR; //throw exception ?
    } 

    size = (sz[0] | (sz[1]<<8) | (sz[2]<<16) | (sz[3]<<24));
	
    status = recv(m_sock, data, size, 0);

	m_state = (status == size ? Socket::SOCKET_READING : (status == 0 ? Socket::SOCKET_CLOSED_ERROR 
                                                               : Socket::SOCKET_READ_ERROR));	

    return (status == size ? VCFNET_SOCKET_OK : (status == 0 ? VCFNET_SOCKET_CLOSED_ERROR 
                                        : VCFNET_SOCKET_READ_ERROR));	
}

int Win32SocketImplementer::send( const char* data, const uint32& size )
{
	int status;

    char *buf;

    if (size < 500) {
        buf = m_sendBuffer;
    } else {
        buf = new char[size + 4];
    }
    buf[0] = (size) & 0xff;
    buf[1] = (size >> 8) & 0xff;
    buf[2] = (size >> 16) & 0xff;
    buf[3] = (size >> 24) & 0xff;
    memcpy((char FAR *)(buf + 4), data, size);

    status = ::send(m_sock, buf, size + 4, 0);

    if (buf != m_sendBuffer) delete buf;
	m_state = (status == (size + 4) ? Socket::SOCKET_WRITING : Socket::SOCKET_WRITE_ERROR);
    return (status == (size + 4) ? VCFNET_SOCKET_OK : VCFNET_SOCKET_WRITE_ERROR);	
}

int Win32SocketImplementer::hasMessage()
{
	int len = 0;
    char sz[4];
    if (recv(m_sock, sz, 4, MSG_PEEK) == 4) {
        len =(sz[0] | (sz[1]<<8) | (sz[2]<<16) | (sz[3]<<24));
    }
    return len;
}

long Win32SocketImplementer::connectTo( const char *hostName, const u_short port )
{
	SOCKADDR_IN dest_sin;  /* DESTination Socket INternet */
 
    m_sock = socket(AF_INET, SOCK_STREAM, 0);
    if (m_sock == INVALID_SOCKET) {
		m_state = Socket::SOCKET_CREATE_ERROR;
        return VCFNET_SOCKET_CREATE_ERROR;
    }
    ioctlsocket(m_sock, FIONBIO, NULL); // ensure we are using blocking mode

    /*
     *    Retrieve the IP address and TCP Port number
     *    Global variable szBuff contains the remote host name.
     */
    int status = fillAddress(&dest_sin, hostName, port);
	 
    if ( status != VCFNET_SOCKET_OK ) {
        closesocket(m_sock);
        return status;
    }

    if (connect(m_sock, (PSOCKADDR) &dest_sin, sizeof(dest_sin)) < 0) {
        closesocket(m_sock);
		m_state = Socket::SOCKET_CONNECT_ERROR;
        return VCFNET_SOCKET_CONNECT_ERROR;
    }

	return VCFNET_SOCKET_OK;
}

void Win32SocketImplementer::close()
{
	if (m_sock != NULL) {
        closesocket(m_sock);
    }
    m_sock = NULL;
}

Socket* Win32SocketImplementer::getSocket()
{
	return this->m_socket;
}
	
void Win32SocketImplementer::setSocket( Socket* socket )
{
	this->m_socket = socket;
}

long Win32SocketImplementer::startAsServer( u_short port )
{	
	static THREADPACK tp;

    //m_listener = listener;
    m_sock = socket( AF_INET, SOCK_STREAM, 0);
    ioctlsocket(m_sock, FIONBIO, NULL); // ensure we are using blocking mode

    if (m_sock == INVALID_SOCKET) {
        closesocket(m_sock);
        return VCFNET_SOCKET_CREATE_ERROR;
    }

    /*
     *   Retrieve the IP address and TCP Port number
     */

    SOCKADDR_IN local_sin;  /* Local socket - internet style */
    int status = fillAddress(&local_sin, NULL, port);
    if (status != VCFNET_SOCKET_OK)
        return status;

    /*
     *   Associate an address with a socket. (bind)
     */
    if (bind(m_sock, (struct sockaddr FAR *) &local_sin, 
             sizeof(local_sin)) == SOCKET_ERROR) {
        return VCFNET_SOCKET_BIND_ERROR;
    }

    if (listen(m_sock, MAX_PENDING_CONNECTS) < 0) {
        return VCFNET_SOCKET_LISTEN_ERROR;
    }

	
    tp.nThread = 0;
    tp.improvSocket = this;


    if (_beginthread(AcceptThreadProc, 0, (void*)&tp) == -1) {
        closesocket(m_sock);
        m_sock = NULL;
        return VCFNET_SOCKET_THREAD_ERROR;
    }    

	return VCFNET_SOCKET_OK;
}

void Win32SocketImplementer::runAsServer(int thread) 
{
    m_eventObj = WSACreateEvent();

    if (WSA_INVALID_EVENT == m_eventObj) {
        return; // Problem with creating an event object??
    }

    if (WSAEventSelect(m_sock, m_eventObj, FD_ACCEPT) == SOCKET_ERROR) {
        return; // Problem selecting event??
    }

    WSAEVENT FAR *events = NULL;
    int eventSize = 0;
    int numEvents = 0;

    while (TRUE) {

        numEvents = m_clients.size() + 1; // One addnl event needed for the server's socket

        // Grow event array if needed
        if (numEvents > eventSize) {
            if (events != NULL) delete [] events;
            eventSize = numEvents * 2;
            events = new WSAEVENT[eventSize];
        }

        // Setup event array        
        std::vector<Win32SocketImplementer*>::iterator iter = m_clients.begin(); 
        for(int i = 0; iter != m_clients.end(); iter++, i++) {
            events[i] = (*iter)->m_eventObj;
        }
        events[numEvents-1] = m_eventObj;

        // Wait for an event from any client
        DWORD n = WSAWaitForMultipleEvents(numEvents, (WSAEVENT FAR *)events,
                                           FALSE, WSA_INFINITE, TRUE);


        if (n >= WSA_WAIT_EVENT_0 && n < (WSA_WAIT_EVENT_0 + numEvents)) {
            // Got an event
            WSAEVENT e = events[n - WSA_WAIT_EVENT_0];
            if (e == this->m_eventObj) { // Server got a new client
                WSANETWORKEVENTS netEvents;

                // Discover what event took place (this also resets
                // event object)
                WSAEnumNetworkEvents(m_sock, m_eventObj, (WSANETWORKEVENTS FAR *)&netEvents);
                if ((netEvents.lNetworkEvents & FD_ACCEPT) != 0) {
                    //acceptClient();
					addClient();
                }
            } else {
                // Must be a client event Search for client to handle event
                for(iter = m_clients.begin(); iter != m_clients.end(); iter++) {
                    Win32SocketImplementer *receiver = *iter;
                    if ( receiver->m_eventObj == e ) {
                        WSANETWORKEVENTS netEvents;

                        // Discover which events have occurred
                        WSAEnumNetworkEvents( receiver->m_sock,
                                              receiver->m_eventObj, (WSANETWORKEVENTS FAR *)&netEvents );
                        if ((netEvents.lNetworkEvents & FD_CLOSE) != 0) {
                            // Got a close - close client's socket
                            closeClient( receiver );
                        } else {
                            // client has to handle event
							uint32 len = receiver->hasMessage();
							if ( len > 0 ){
								char buf[256];
								memset( buf, 0, 256 );
								len = 0;
								if ( VCFNET_SOCKET_OK == receiver->receive( buf, len ) ){
									String s = buf;
									std::cout << "received data: \"" << s << "\"" << std::endl;	
									std::cout << "data len = " << len << std::endl;
								}	
							}
                            //m_listener->handleMessage(receiver);
                        }
                        break;
                    }
                }
            }
        }

    }
    // _endthread();

}

void Win32SocketImplementer::addClient() 
{
    SOCKADDR_IN acc_sin;    /* Accept socket address - internet style */
    int acc_sin_len;        /* Accept socket address length */

    acc_sin_len = sizeof(acc_sin);

	std::cout << "accepting client" << std::endl;

    SOCKET sock = accept(m_sock,(struct sockaddr FAR *) &acc_sin,
                         (int FAR *) &acc_sin_len );
    
	std::cout << "getting client" << std::endl;

    ioctlsocket(m_sock, FIONBIO, NULL); // ensure we are using blocking mode

    if (sock == INVALID_SOCKET) {
        // strange?
        return;
    }

    Win32SocketImplementer *client = new Win32SocketImplementer();
    client->m_sock = sock;
    client->m_port = 0;
    client->m_eventObj = WSACreateEvent();
	
    if (WSA_INVALID_EVENT == client->m_eventObj) {
        return; // Problem creating event obj??
    }

    if (SOCKET_ERROR ==
        WSAEventSelect(client->m_sock, client->m_eventObj, FD_READ|FD_CLOSE)) {
        return; // Problem accepting event obj??
    }
	
    m_clients.push_back(client);

    //m_listener->handleClientCreated(client);

    std::cout << "getting client done!" << std::endl;

}

void Win32SocketImplementer::closeClient( Win32SocketImplementer *client ) 
{

    shutdown(client->m_sock, SD_SEND);
    closesocket(client->m_sock);
    client->m_sock = NULL;

    // Remove client from list

    std::vector<Win32SocketImplementer*>::iterator iter = 
		std::find( m_clients.begin(), m_clients.end(), client );
    
	if ( iter != m_clients.end() ) {        
		m_clients.erase(iter);        
    }
	std::cout << "client removed !!!" << std::endl;

    //m_listener->handleClientDeleted(client);

    // delete client storage
    delete client;
}