// Win32SocketPeer.cpp


#include "Win32SocketPeer.h"
#include <iostream>
#include <algorithm>
#include "SocketException.h"
#include "StringUtils.h"

using namespace VCF;
using namespace VCFNet;

struct MemStruct{
	int size;
	char* data;
};

Win32SocketPeer::Win32SocketPeer( const VCF::String& host, const int& port )
{	
	try {
		Win32SocketPeer::startup();
		m_host = host;
		m_port = port;
		
		if ( (VCFNET_NOPORT != m_port) && (m_host == VCFNET_NOHOST) ){
			//go ahead and start as server
			startAsServer( m_port );
			StringUtils::trace( "started as server !!!\n" );
			runServerLoop();
		}
		else if ( (VCFNET_NOPORT != m_port) && (m_host != VCFNET_NOHOST) ){
			//go ahead and try and start as client
			connectTo( m_host, m_port );
		}
	}
	catch (...){
		
	}
}

Win32SocketPeer::~Win32SocketPeer()
{
	WSACleanup();
}

int Win32SocketPeer::getPort()
{
	return m_port;	
}

void Win32SocketPeer::setPort( const int& port )
{
	m_port = port;
}

VCF::String Win32SocketPeer::getHost()
{
	return m_host;
}

void Win32SocketPeer::setHost( const VCF::String& host )
{
	m_host = host;
}

int Win32SocketPeer::receive( char* data, VCF::uint32& size )
{
	return 0;
}

int Win32SocketPeer::send( const char* data, const VCF::uint32& size )
{
	return ::send( m_sock, data, size, 0 );
}

void Win32SocketPeer::startup()
{	
	WORD wVersionRequested = MAKEWORD(1, 1);
	WSADATA wsaData; 
	int err = WSAStartup(wVersionRequested, &wsaData); 
	if (err != 0) {
		//MessageBox(NULL, "Winsock dll load failed.", "FATAL ERROR", MB_OK);
		//throw exception
		WSACleanup();
		throw SocketException( "Winsock dll load failed." );
	}
	
	if ( LOBYTE( wsaData.wVersion ) != 1 || 
        HIBYTE( wsaData.wVersion ) != 1 ) 	{ 
		WSACleanup(); 
		throw SocketException( "Winsock version 1.1 not found." );
	}	
}


long Win32SocketPeer::connectTo( const VCF::String& hostName, const ushort port )
{
	m_host = hostName;
	const char* tmpHostName = hostName.c_str();
    struct hostent *hp;
	if ( 0 != m_sock ){ //attempt to close the previous connection
		closesocket( m_sock );
	}

    m_sock = -1;
    m_port = -1;
    
	m_sock = socket (AF_INET, SOCK_STREAM, 0);
    if ( m_sock < 0 ) {		
		return VCFNET_CANT_OPEN_SOCKET;
    }

    /* Initialize the socket address to the server's address. */    
	memset( &m_sin, 0, sizeof(m_sin) );
    m_sin.sin_family = AF_INET;
    hp = gethostbyname( tmpHostName );    /* to get host address */
    
	if (hp == NULL) {
		return VCFNET_HOSTNAME_ERROR;
    }
    
	memcpy( &(m_sin.sin_addr.s_addr), hp->h_addr, hp->h_length );
    m_sin.sin_port = htons(port);
	
    /* Connect to the server. */

	int connectErr = connect(m_sock, (sockaddr*)&m_sin,sizeof(m_sin));
    if ( connectErr < 0) {
			//printf( "error is : %s\n", getWSAErrorString( WSAGetLastError() ) );
		closesocket(m_sock);

		return VCFNET_CONNECT_ERROR;
    }
    //printf("Connection to \"%s\" at %d established\n", hostname, portNum) ;

	return VCFNET_CONNECT_SUCCESS;
}

void Win32SocketPeer::close()
{
	closesocket(m_sock);
}

long Win32SocketPeer::startAsServer( ushort port )
{
	int length = 0;
	int msgsock = 0;

    m_sock = -1 ;
    m_port = -1 ;

    /* Create a socket */

    if ((m_sock = ::socket(AF_INET,SOCK_STREAM,0)) < 0) 
    {		
		return VCFNET_CANT_OPEN_SOCKET;
    }

    /* Initialize the socket's address structure */

    m_sin.sin_family = AF_INET;
    m_sin.sin_addr.s_addr = INADDR_ANY;
    m_sin.sin_port = htons(port);
    m_port = port ;

    /* Assign an address to this socket */
	int bindErr =  bind( m_sock, (sockaddr*)&m_sin, sizeof(m_sin) );
    if ( bindErr < 0)  {
		closesocket( m_sock );
		m_sock = -1 ;
		return VCFNET_BIND_FAILED;
    }

    //printf("Listening for connect requests on port %d\n", portNum);

    /* Prepare the socket queue for connection requests */
    int listenErr = listen( m_sock, 5 );

	return VCFNET_SERVER_STARTED;
}

int Win32SocketPeer::clientWaiting()
{
	struct timeval timeout ;
	fd_set readfd ;
	
	FD_ZERO(&readfd);
	
	FD_SET ( m_sock, &readfd) ;
	timeout.tv_sec = 0 ;
	timeout.tv_usec = 0 ;
	int selectResult = select( m_sock + 1, &readfd, NULL, NULL, &timeout );
	if ( SOCKET_ERROR == selectResult ){
		//throw exception
	}
	return selectResult;
}

void Win32SocketPeer::runServerLoop()
{
	StringUtils::trace( "Starting server loop...\n" );
	while ( true ){
		Sleep( 50 );
		while ( 0 == clientWaiting() ) {
			Sleep( 50 );
			checkForPendingData();
		}
		
		int clientSock = getClient();
		if ( clientSock > 0 ){
			StringUtils::trace( "Client connected!\n" );
			m_clientList.push_back( clientSock );
		}
		
		Sleep( 10 );

		checkForPendingData();
	}
}

int Win32SocketPeer::getClient()
{
    int length = 0;
	int msgsock = 0;
    /* Initialize the socket's address structure */

    m_sin.sin_family = AF_INET;
    m_sin.sin_addr.s_addr = INADDR_ANY;
    m_sin.sin_port = htons(m_port);

	StringUtils::trace("Listening for connect requests on port " + StringUtils::toString(m_port) + "\n" );

    /* Prepare the socket queue for connection requests */

    length = sizeof(m_sin);
    msgsock = accept( m_sock, (sockaddr*)&m_sin, &length );
    if (msgsock < 0) {
		StringUtils::trace("Accept() failed on m_sock = " + StringUtils::toString((int)m_sock) + "\n" );
		return (0) ;
    }

	StringUtils::trace("Connection from host " + m_host + " port: " + StringUtils::toString(m_port) + "\n" );
    //printf("Connection from host %s, port %u, sock %d\n",
	//inet_ntoa(mSin.sin_addr), ntohs(mSin.sin_port), msgsock);

   return (msgsock) ;
}

void Win32SocketPeer::checkForPendingData()
{
	StringUtils::trace( "Starting checkForPendingData()...\n" );
	if ( !m_clientList.empty() ){
		StringUtils::trace( "checkForPendingData() has clients - looping...\n" );
		std::vector<int>::iterator it = m_clientList.begin();

		fd_set readfd;			
		FD_ZERO(&readfd);
		
		struct timeval timeout;		
		timeout.tv_sec = 0;
		timeout.tv_usec = 0;
		
		int maxSock = 0;
		while ( it != m_clientList.end() ){
			FD_SET ( *it, &readfd) ;		
			if ( *it > maxSock ){
				maxSock = *it;	
			}
			it++;
		}
		//printf( "maxSock = %d\n", maxSock );

		int i = select( maxSock + 1, &readfd, NULL, NULL, NULL );//&timeout);

		if ( SOCKET_ERROR != i ){
			it = m_clientList.begin();
			
			while ( it != m_clientList.end() ){
				if (FD_ISSET (*it, &readfd)) {
					StringUtils::trace( StringUtils::toString(*it) +  " sent data...\n" );
					// this guy sent something
					int clientSock = *it;
					//printf( "clientSock = %d\n", clientSock );
					
					char buf[VCFNET_MAX_SOCKET_SIZE];
					char* totalDataBuf = NULL;
					int totalSize = VCFNET_MAX_SOCKET_SIZE;
					std::vector<MemStruct*> dataList;
					memset( buf, 0 , VCFNET_MAX_SOCKET_SIZE );
					
					//read first chunk of data
					int recvErr = recv( clientSock, buf, VCFNET_MAX_SOCKET_SIZE, 0 );
					
					if ( recvErr != SOCKET_ERROR ){						
						//check to see if more data is still out there
						u_long res = 0;
						ioctlsocket( clientSock, FIONREAD, &res );
						char* tmp = NULL;
						while ( res > 0 ){ //continue to pull data
							MemStruct* memStruct = new MemStruct;
							memStruct->data = new char[res];
							memset( memStruct->data, 0, res );
							memStruct->size = res;
							recvErr = recv( clientSock, memStruct->data, res, 0 );
							if ( recvErr != SOCKET_ERROR ){														
								dataList.push_back( memStruct );
								totalSize += res;
								ioctlsocket( clientSock, FIONREAD, &res );
							}
							else {
								res = 0;
							}
						}
						if ( ! dataList.empty() ){
							char* totalDataBuf = new char[totalSize];
							memset( totalDataBuf, 0, totalSize );
							char* tmpBuf = totalDataBuf;
							int insertData = 0;
							std::vector<MemStruct*>::iterator dataIt = dataList.begin();
							while ( dataIt != dataList.end() ){
								MemStruct* memStruct = (*dataIt);
								tmp = memStruct->data;
								memcpy( tmpBuf, tmp, memStruct->size );
								tmpBuf += memStruct->size;
								delete [] memStruct->data;
								memStruct->data = NULL;
								delete memStruct;
								memStruct = NULL;
								dataIt++;
							}
						}
						else {
							totalDataBuf = buf;
						}					
					}
				}
				
				it++;
			}
		}
		else {
			//printf( "select() failed - returned %d\n\terror was : %s", i, getWSAErrorString( WSAGetLastError() ) );
		}
	}
}

void Win32SocketPeer::runAsServer(int thread) 
{

}

