/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32ToolKit.cpp

#include "Dialog.h"

#include "PopupMenuPeer.h"
#include "PopupMenu.h"

#include "Win32ToolKit.h"
#include "Win32Application.h"
#include "Win32Edit.h"
#include "Win32Tree.h"
#include "Win32ListBox.h"
#include "Win32Listview.h"
#include "Win32Dialog.h"
#include "Win32ComboBox.h"
#include "Win32Button.h"
#include "Win32MenuItem.h"
#include "Win32MenuBar.h"
#include "Win32FileOpenDialog.h"
#include "Win32FileSaveDialog.h"
#include "Win32ColorDialog.h"
#include "Win32DropTargetPeer.h"
#include "Win32ControlContext.h"
#include "Win32PopupMenu.h"
#include "PopupMenu.h"
#include "Win32FolderBrowseDialog.h"
#include "Win32FontDialog.h"

using namespace VCF;

/**
*used by the ToolKitHookWndProc andthe construcor for Win32ToolKit()
*/
static HHOOK hookHandle;

LRESULT CALLBACK ToolKitHookWndProc( int nCode, WPARAM wParam, LPARAM lParam );


/**
*this is a weird case that Win32 gives us, so here's the deal:
*WM_NOTIFY messages are sent to parent windows, so normally, all the 
*handling is done there. This is fine and well if you are writing a
*big fat SDK switch statement (like the one you're looking at now :),
*but really sucks if you have your code broken up into a well organized 
*framework (like I am attempting to build here ! ). What this means is 
*that control objects can never handle their own notifications! So to 
*facilitate this we'll do our processing here (which at the moment is nothing)
*and then pass the message right back to the child that sent it !
*The reason this code is here, instead of in AbstractWin32Component::handleEventMessages(),
*is due to the way windows are created in the framework. A hidden dummy window, 
*whose parent is the Desktop, is created and made available as a temporary parent
*to Peers before the created window is actually hooked into into the real
*parent. What seems to happen with the common controls is the first parent hwnd sent 
*in to the control when it is first created is the one to which all future WM_NOTIFY
*messages are sent to. So the code below exists to route those messages back to 
*the child Win32Object where they are handled.
*First the lParam is typecasted into a NMHDR structure. This gives us access to 
*the hwnd and hopefully the Win32Object. If we have a valid Win32Object then we 
*call the Win32Object::handleEventMessages(), passing the NMHDR.code field as the 
*reflected window message, the original wParam, which gets ignored anyways, and
*the original lParam, which upon arriving back at the child control will again
*get typcast back to some other notification structure. Gotta love M$ !!!!!
*PS: Yes, this is one of the biggest hacks I've seen  ;-)
*PPS This also works for WM_COMMAND messages as well
*/
LRESULT CALLBACK DummyWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	LRESULT result = 0;
	switch ( message ){
		case WM_CREATE:	{
			/**
			*set up the creation hook procedure
			*/
			/*
			hookHandle = ::SetWindowsHookEx( WH_CALLWNDPROCRET, 
					                         ToolKitHookWndProc,
											 NULL,//::GetModuleHandle( NULL ),
											 GetWindowThreadProcessId( hWnd, NULL ) );
			*/
		}
		break;
		case WM_NOTIFY:{			
			NMHDR* notificationHdr = (LPNMHDR)lParam;
			Win32Object* win32Obj = Win32Object::getWin32ObjectFromHWND( notificationHdr->hwndFrom );
			if ( NULL != win32Obj ){
				result = win32Obj->handleEventMessages( notificationHdr->code, wParam, lParam );
			}
		}
		break;

		case WM_COMMAND:{
			UINT notifyCode = HIWORD(wParam); 
			//wID = LOWORD(wParam); // item, control, or accelerator identifier , ignored for now        
			HWND hwndCtl = (HWND) lParam;
			Win32Object* win32Obj = Win32Object::getWin32ObjectFromHWND( hwndCtl );
			if ( NULL != win32Obj ){
				result = win32Obj->handleEventMessages( notifyCode, wParam, lParam );
			}
			result = ::DefWindowProc( hWnd, message, wParam, lParam );
		}
		break;

		default : {
			result = ::DefWindowProc( hWnd, message, wParam, lParam );
		}
		break;
	}
	return result;
}

ATOM RegisterDummyWndClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)DummyWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "DummyWnd";
	wcex.hIconSm		= NULL;

	return RegisterClassEx(&wcex);
}

/*

LRESULT CALLBACK ToolKitHookWndProc( int nCode, WPARAM wParam, LPARAM lParam ) 
{	
	if ( NULL != lParam ){
		CWPRETSTRUCT* msgStruct = NULL;
		msgStruct = (CWPRETSTRUCT*)lParam;
		Win32Object* win32Obj = Win32Object::getWin32ObjectFromHWND( msgStruct->hwnd );
		if ( NULL != win32Obj ){
			switch ( msgStruct->message ){
				case WM_CREATE:{
					Control* control = win32Obj->getPeerControl();
					if ( NULL != control ){

						ComponentEvent event( control, COMPONENT_EVENT_CREATED );
						control->processComponentEvent( &event );
					}
				}
				break;
			}
			//result = win32Obj->handleEventMessages( msgStruct->message, wParam, lParam );
		}
	}

	return ::CallNextHookEx( hookHandle, nCode, wParam, lParam );
}
*/

Win32ToolKit::Win32ToolKit():
	UIToolkit()
{
	m_dummyParentWnd = NULL;
	this->loadSystemColors();

}	


Win32ToolKit::~Win32ToolKit()
{
	::UnhookWindowsHookEx( hookHandle );
}

Color* Win32ToolKit::getSystemColor( const unsigned long& systemColor )
{
	Color* result = NULL;
	std::map<unsigned long,Color*>::iterator found = this->m_systemColors.find( systemColor );
	if ( found != m_systemColors.end() ){
		result = found->second;
	}
	return result;
}

void Win32ToolKit::loadSystemColors()
{
	Color* sysColor = NULL;
	sysColor = new Color( ::GetSysColor( COLOR_BTNSHADOW ) );
	m_systemColors[SYSCOLOR_SHADOW] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_3DFACE ) );
	m_systemColors[SYSCOLOR_FACE] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_3DHILIGHT ) );
	m_systemColors[SYSCOLOR_HIGHLIGHT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_ACTIVECAPTION ) );
	m_systemColors[SYSCOLOR_ACTIVE_CAPTION] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_ACTIVEBORDER ) );
	m_systemColors[SYSCOLOR_ACTIVE_BORDER] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_DESKTOP ) );
	m_systemColors[SYSCOLOR_DESKTOP] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_CAPTIONTEXT ) );
	m_systemColors[SYSCOLOR_CAPTION_TEXT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_HIGHLIGHT ) );
	m_systemColors[SYSCOLOR_SELECTION] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_HIGHLIGHTTEXT ) );
	m_systemColors[SYSCOLOR_SELECTION_TEXT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_INACTIVEBORDER ) );
	m_systemColors[SYSCOLOR_INACTIVE_BORDER] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_INACTIVECAPTION ) );
	m_systemColors[SYSCOLOR_INACTIVE_CAPTION] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_INFOBK ) );
	m_systemColors[SYSCOLOR_TOOLTIP] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_INFOTEXT ) );
	m_systemColors[SYSCOLOR_TOOLTIP_TEXT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_MENU ) );
	m_systemColors[SYSCOLOR_MENU] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_MENUTEXT ) );
	m_systemColors[SYSCOLOR_MENU_TEXT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_WINDOW ) );
	m_systemColors[SYSCOLOR_WINDOW] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_WINDOWTEXT ) );
	m_systemColors[SYSCOLOR_WINDOW_TEXT] = sysColor;

	sysColor = new Color( ::GetSysColor( COLOR_WINDOWFRAME ) );
	m_systemColors[SYSCOLOR_WINDOW_FRAME] = sysColor;

}

ApplicationPeer* Win32ToolKit::createApplicationPeer()
{
	return new Win32Application();
}

TextPeer* Win32ToolKit::createTextPeer( TextControl* component, ComponentType componentType)
{
	return new Win32Edit( component );
}

TreePeer* Win32ToolKit::createTreePeer( TreeControl* component, ComponentType componentType )
{
	return new Win32Tree( component );
}

ListBoxPeer* Win32ToolKit::createListboxPeer( ListBoxControl* component, ComponentType componentType)
{
	return new Win32ListBox( component );
}

ListviewPeer* Win32ToolKit::createListViewPeer( ListViewControl* component, ComponentType componentType )
{
	return new Win32Listview( component );
}

DialogPeer* Win32ToolKit::createDialogPeer( Frame* owner, Dialog* component, ComponentType componentType )
{
	return new Win32Dialog( owner, component );
}

DialogPeer* Win32ToolKit::createDialogPeer()
{
	return new Win32Dialog();
}

void Win32ToolKit::createDummyParentWindow()
{	
	RegisterDummyWndClass( ::GetModuleHandle( NULL ) );

	HWND parent = ::GetDesktopWindow();
	m_dummyParentWnd = ::CreateWindow( "DummyWnd", NULL, WS_CHILD , 0, 0, 0, 0, parent, NULL, ::GetModuleHandle( NULL ), NULL ); 
}

HWND Win32ToolKit::getDummyParent()
{
	if ( NULL == m_dummyParentWnd ){
		this->createDummyParentWindow();
	}
	return this->m_dummyParentWnd;
}

MenuItemPeer* Win32ToolKit::createMenuItemPeer( MenuItem* item )
{
	return new Win32MenuItem( item );
}

MenuBarPeer* Win32ToolKit::createMenuBarPeer( MenuBar* menuBar )
{
	return new Win32MenuBar( menuBar );
}

ComboBoxPeer* Win32ToolKit::createComboBoxPeer( ComboBoxControl* component, ComponentType componentType)
{
	return new Win32ComboBox( component );
}

ButtonPeer* Win32ToolKit::createButtonPeer( CommandButton* component, ComponentType componentType)
{
	return new Win32Button( component );
}

ContextPeer* Win32ToolKit::createContextPeer( Control* component )
{
	return new Win32ControlContext( component->getPeer() );
}

CommonFileDialogPeer* Win32ToolKit::createCommonFileOpenDialogPeer( Control* owner )
{
	return new Win32FileOpenDialog( owner );
}

CommonFileDialogPeer* Win32ToolKit::createCommonFileSaveDialogPeer( Control* owner )
{
	return new Win32FileSaveDialog( owner );
}

CommonColorDialogPeer* Win32ToolKit::createCommonColorDialogPeer( Control* owner )
{
	return new Win32ColorDialog( owner );
}

CommonFolderBrowseDialogPeer* Win32ToolKit::createCommonFolderBrowseDialogPeer( Control* owner )
{
	return new Win32FolderBrowseDialog();
}

DragDropPeer* Win32ToolKit::createDragDropPeer()
{
	return NULL;
}

DataObjectPeer* Win32ToolKit::createDataObjectPeer()
{
	return NULL;
}

DropTargetPeer* Win32ToolKit::createDropTargetPeer()
{
	return new Win32DropTargetPeer();
}

PopupMenuPeer* Win32ToolKit::createPopupMenuPeer( PopupMenu* popupMenu )
{
	return new Win32PopupMenu( popupMenu );
}

CommonFontDialogPeer* Win32ToolKit::createCommonFontDialogPeer( Control* owner )
{
	return new Win32FontDialog( owner );
}