/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Tree.cpp


#include "Win32Tree.h"
#include <commctrl.h> //make sure to link with comctl32.lib
#include "Win32ToolKit.h"
#include "Enumerator.h"
#include "TreeItem.h"


using namespace VCFWin32;
using namespace VCF;

Win32Tree::Win32Tree( TreeControl* tree ):
	AbstractWin32Component( tree ),
	m_treeControl( tree )
{	
	this->init();

	this->createParams();

	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = toolkit->getDummyParent();	

	String className = this->getClassName();

	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             WC_TREEVIEW,
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );


	if ( NULL != this->m_hwnd ){	
		Win32Object::registerWin32Object( this );
		m_oldTreeWndProc = (WNDPROC)::SetWindowLong( this->m_hwnd, GWL_WNDPROC, (LONG)m_wndProc ); 
		this->m_defaultWndProc = NULL;m_oldTreeWndProc;
				
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32Tree::~Win32Tree()
{

}

void Win32Tree::init()
{	
	m_oldTreeWndProc = NULL;
}

void Win32Tree::createParams()
{
	this->m_exStyleMask = WS_EX_CLIENTEDGE;
	this->m_styleMask = BORDERED_VIEW | TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT | TVS_NOTOOLTIPS | TVS_SHOWSELALWAYS;
}

TreeModel* Win32Tree::getTreeModel()
{
	return NULL;
}

void Win32Tree::setTreeModel( TreeModel* model )
{

}

double Win32Tree::getItemIndent()
{
	return 0.0;
}

void Win32Tree::setItemIndent( const double& indent )
{

}

ImageList* Win32Tree::getImageList()
{
	return NULL;
}

void Win32Tree::setImageList( ImageList* imageList )
{

}
bool first = true;

LRESULT Win32Tree::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;
	
	switch ( message ) {		
		case WM_PAINT:{
			result = CallWindowProc( m_oldTreeWndProc, m_hwnd, message, wParam, lParam );
		}
		break;

		case WM_CREATE:{				
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
		}
		break;

		case TVN_BEGINDRAG:{
			
		}
		break;

		case TVN_BEGINLABELEDIT:{
			
		}
		break;

		case TVN_BEGINRDRAG:{
			
		}
		break;

		case TVN_DELETEITEM:{
			
		}
		break;


		case TVN_ENDLABELEDIT:{
			
		}
		break;

		case TVN_GETDISPINFO:{
			
		}
		break;

		case TVN_GETINFOTIP:{
			
		}
		break;

		case TVN_ITEMEXPANDED:{
			
		}
		break;

		case TVN_ITEMEXPANDING:{
			
		}
		break;

		case TVN_KEYDOWN:{
			
		}
		break;

		case TVN_SELCHANGED:{
			
		}
		break;

		case TVN_SELCHANGING:{
			
		}
		break;

		case TVN_SETDISPINFO:{
			
		}
		break;

		case TVN_SINGLEEXPAND:{
			
		}
		break;

		case NM_CUSTOMDRAW:{
			
		}
		break;
 
		default: {
			AbstractWin32Component::handleEventMessages( message, wParam, lParam );
			result = CallWindowProc( this->m_oldTreeWndProc, m_hwnd, message, wParam, lParam );
			
		}
		break;
	}
	return result;
}

void Win32Tree::addItem( TreeItem* item )
{
	TVINSERTSTRUCT insert = {0};
	TVITEM tvItem = {0};
	TreeItem* parent = item->getParent();
	HTREEITEM itemParent = NULL;
	if ( NULL != parent ){
		std::map<TreeItem*,HTREEITEM>::iterator it =
		m_treeItems.find( parent );
		if ( it != m_treeItems.end() ){
			itemParent = it->second;			
		}
		else{
			//throw exception;
		}
	}
	tvItem.mask = TVIF_TEXT;
	tvItem.pszText = const_cast<char*>( item->getCaption().c_str() );
	tvItem.cchTextMax = item->getCaption().size();
	insert.hParent = itemParent;
	insert.hInsertAfter = TVI_FIRST;
	insert.item = tvItem;
	HTREEITEM addedItem = TreeView_InsertItem( this->m_hwnd, &insert );
	m_treeItems[item] = addedItem;

	item->addItemListener( this );

	//now check the children
	
	Enumerator<TreeItem*>* children = item->getChildren();
	if ( NULL != children ){
		while ( children->hasMoreElements() ){
			TreeItem* child = children->nextElement();
			if ( NULL != child ){
				addItem( child );
			}
		}
	}
}

void Win32Tree::clear()
{
	TreeView_DeleteAllItems( this->m_hwnd );
}

void Win32Tree::onItemPaint( ItemEvent* event )
{

}

void Win32Tree::onItemChanged( ItemEvent* event )
{
	if ( NULL != event ){
		switch ( event->getType() ){
			case ITEM_EVENT_TEXT_CHANGED:{
				Object* source = event->getSource();
				if ( NULL != source ){
					TreeItem* item = dynamic_cast<TreeItem*>(source);
					if ( NULL != item ){
						std::map<TreeItem*,HTREEITEM>::iterator it =
							m_treeItems.find( item );
						if ( it != m_treeItems.end() ){
							TVITEM tvItem = {0};
							tvItem.mask = TVIF_TEXT | TVIF_HANDLE;
							tvItem.hItem = it->second;	
							String caption = item->getCaption();
							tvItem.cchTextMax = caption.size();
							char* tmp = new char[tvItem.cchTextMax];
							memset( tmp, 0, tvItem.cchTextMax );
							caption.copy( tmp, tvItem.cchTextMax );
							tvItem.pszText = tmp;							
							TreeView_SetItem( this->m_hwnd, &tvItem );
							delete [] tmp;
						}
					}
				}				
			}
			break;
		}
	}
}

void Win32Tree::onItemSelected( ItemEvent* event )
{

}

void Win32Tree::onItemAdded( ItemEvent* event )
{

}

	
void Win32Tree::onItemDeleted( ItemEvent* event )
{

}