/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Object.h"
#include "LibraryPeer.h"
#include "Library.h"

#ifdef WIN32
#include "Win32LibraryPeer.h"
#endif

#include "StringUtils.h"



using namespace VCF;

Library::Library( const String& libraryName, const bool& autoUnloadLib )
{
	m_currentLibName = libraryName;
	m_autoUnloadLib = autoUnloadLib;
	init();
}

Library::~Library()
{
	if ( NULL != m_libPeer ){
		if ( true == m_autoUnloadLib ){
			try {
				m_libPeer->unload();
			}
			catch (...){
				StringUtils::trace( "failed to unload the libary - probably alread unloaded" ); 
			}
		}		
		delete m_libPeer;
		m_libPeer = NULL;
	}
}


void Library::init()
{
	m_libPeer = NULL;

#ifdef WIN32 
	m_libPeer = new Win32LibraryPeer();
#endif

	if ( "" != m_currentLibName ){
		m_libPeer->load( m_currentLibName ); 
	}
}

void Library::load( const String& libraryFilename )
{
	m_libPeer->load( libraryFilename );
	m_currentLibName = libraryFilename;
}

void Library::unload()
{
	m_libPeer->unload();
}

void* Library::getFunction( const String& functionName )
{
	return m_libPeer->getFunction( functionName );
}