/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Enumerator.h"
#include "Registry.h"
#include "RegistryPeer.h"
#ifdef WIN32
	#include "Win32Registry.h"
#endif

using namespace VCF;

Registry::Registry()
{
	m_peer = NULL;
#ifdef WIN32
	m_peer = new Win32Registry();
#endif
}

Registry::~Registry()
{
	delete m_peer;
}

void Registry::setRoot( const RegistryKeyType& rootKey )
{
	m_peer->setRoot( rootKey );
}

RegistryKeyType Registry::getRoot()
{
	return m_peer->getRoot();
}

bool Registry::openKey( const String& keyname, const bool& createIfNonExistant )
{
	return m_peer->openKey( keyname, createIfNonExistant );
}

bool Registry::setValue( const String& value, const String& valuename )
{
	return m_peer->setValue( value, valuename );
}

bool Registry::setValue( const uint32& value, const String& valuename )
{
	return m_peer->setValue( value, valuename );
}

bool Registry::setValue( const bool& value, const String& valuename )
{
	return m_peer->setValue( value, valuename );
}

bool Registry::setValue( void* dataBuffer, const uint32& dataBufferSize, const String& valuename )
{
	return m_peer->setValue( dataBuffer, dataBufferSize, valuename );
}

String Registry::getStringValue( const String& valuename )
{
	return m_peer->getStringValue( valuename );
}

uint32 Registry::getIntValue( const String& valuename )
{
	return m_peer->getIntValue( valuename );
}

bool Registry::getBoolValue( const String& valuename )
{
	return m_peer->getBoolValue( valuename );
}

void Registry::getDataBufValue( const String& valuename, uint32& dataBufferSize, void** dataBuffer )
{
	m_peer->getDataBufValue( valuename, dataBufferSize, dataBuffer );
}

Enumerator<String>* Registry::getKeyNames()
{
	return m_peer->getKeyNames();
}

Enumerator<RegistryValueInfo*>* Registry::getValues()
{
	return m_peer->getValues();
}

String Registry::getCurrentKey()
{
	return m_peer->getCurrentKey();
}