/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// StringUtils.cpp


#include "StringUtils.h"

#ifdef macintosh
	#include <stdio.h>
	#include <string.h>
	#include <extras.h>
#endif

#ifdef WIN32
#include "Win32Peer.h"
#include "Rpcdce.h" //make sure to link with Rpcrt4.lib
#endif

#define TO_STRING_TXT_SIZE		50

using namespace VCF;


StringUtils::StringUtils()
{

}

StringUtils::~StringUtils()
{

}

String StringUtils::getFileDir( const VCF::String& fileName )
{
	int pos = fileName.find_last_of( "\\" );
	if ( pos == -1 ){
		return "";
	}
	return fileName.substr( 0, pos );
}

String StringUtils::getFileName( const VCF::String& fileName )
{
	int pos = fileName.find_last_of( "\\" );
	if ( pos == -1 ){
		return "";
	}
	return fileName.substr( pos+1 );
}

String StringUtils::getFileExt( const VCF::String& fileName )
{
	int pos = fileName.find_last_of( "." );
	if ( pos == -1 ){
		return "";
	}
	return fileName.substr( pos+1 );
}

void StringUtils::traceWithArgs( const String& text,... )
{
#ifdef _DEBUG

	/*  Arghhhh!!!!!!!!!! t this doesn't wor k!!!!!!!
	va_list argList;

	va_start( argList, text );     // Initialize variable arguments. 

	char* buf = new char[MAX_TRACE_STRING];
	memset( buf, 0, MAX_TRACE_STRING );

	_vsnprintf( buf, MAX_TRACE_STRING, text.c_str(), argList );

	va_end( argList );              // Reset variable arguments.      

	StringUtils::trace( String(buf) );

	delete [] buf;

	*/


#endif   
}


void StringUtils::trace( const String& text )
{
#ifdef _DEBUG
#ifdef WIN32
	VCFWin32::Win32Utils::trace( text );
#endif
#endif
}

String StringUtils::lowerCase( const String& text )
{
	String result = "";
	
	char* copyText = new char[text.size()+1];
	memset(copyText, 0, text.size()+1 );
	text.copy( copyText, text.size() );

	_strlwr( copyText ); // not in ANSI standard library


	result = copyText;
	delete [] copyText;
	return result;
}

String StringUtils::upperCase( const VCF::String& text )
{
	String result = "";
	char* copyText = new char[text.size()+1];
	memset(copyText, 0, text.size()+1 );
	text.copy( copyText, text.size() );

	_strupr( copyText );

	result = copyText;
	delete [] copyText;
	return result;
}

VCF::String StringUtils::toString( const int& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%d", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const long& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%d", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const float& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%.5f", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const double& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%.5f", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const ulong32& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%d", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const uint32& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%d", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const char& value )
{
	char tmp[TO_STRING_TXT_SIZE];
	memset( tmp, 0, TO_STRING_TXT_SIZE );
	sprintf( tmp, "%c", value );
	return String( tmp );
}

VCF::String StringUtils::toString( const bool& value )
{
	return ( value == true ) ? "true" : "false";
}

VCF::String StringUtils::newUUID()
{
	VCF::String result = "";
#ifdef WIN32
	UUID id;
	if ( RPC_S_OK == ::UuidCreate( &id ) ){
		unsigned char *tmpid = NULL;
		
		RPC_STATUS rpcresult = UuidToString(  &id, &tmpid );
		
		if ( RPC_S_OUT_OF_MEMORY != rpcresult ) {
			result = VCF::String( (VCFChar*)tmpid );
			
			RpcStringFree( &tmpid );
		}
	}
#endif
	return result;
}