/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Thread.h"
#include "ThreadEvent.h"
#include "InvalidPeer.h"
// #include "StringUtils.h"


#ifdef WIN32 
#include "Win32Thread.h"
#endif


using namespace VCF;


// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Thread
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/


Thread::Thread()
{
	this->init();
}


void Thread::init()
{
	m_threadPeer = NULL;
#ifdef WIN32
	m_threadPeer = new Win32Thread();
#endif
}


Thread::~Thread()
{
//	this->stop();
	assert(0);	// todo - implement this properly
}


// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Publics

bool
Thread::start()
{
	if ( this->m_threadPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadPeer->start();
}


bool
Thread::stop( int32 waitMillisec, bool forcekill, uint32 * pThreadExitCode )
{
	if ( this->m_threadPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadPeer->stop(waitMillisec, forcekill, pThreadExitCode);
}


// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Protecteds

void
Thread::sleep( uint32 msec  )
{
	if ( this->m_threadPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	this->m_threadPeer->sleep(msec);
}


bool
Thread::checkForStop()
{
	if ( this->m_threadPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadPeer->checkForStop();
}




// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Thread Mutex
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

ThreadMutex::ThreadMutex()
{
	m_threadMutexPeer = NULL;
#ifdef WIN32
	m_threadMutexPeer = new Win32ThreadMutex();
#endif
}

ThreadMutex::~ThreadMutex()
{
	assert(0);	// todo - implement this properly
}

ThreadMutexAcquire
ThreadMutex::acquire( int32 msec, bool* pStopNow )
{
	if ( this->m_threadMutexPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadMutexPeer->acquire( msec, pStopNow );
}


bool
ThreadMutex::release()
{
	if ( this->m_threadMutexPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadMutexPeer->release( );
}




// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Thread Signal
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

ThreadSignal::ThreadSignal()
{
	m_threadSignalPeer = NULL;
#ifdef WIN32
	m_threadSignalPeer = new Win32ThreadSignal();
#endif
}

ThreadSignal::~ThreadSignal()
{
	assert(0);	// todo - implement this properly
}

bool
ThreadSignal::setSignalState( bool signalState )
{
	if ( this->m_threadSignalPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadSignalPeer->setSignalState( signalState );
}


ThreadSignalState
ThreadSignal::waitOnSignalState( bool desiredState, int32 msec, bool* pStopNow )
{
	if ( this->m_threadSignalPeer == NULL ) {
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_threadSignalPeer->waitOnSignalState( desiredState, msec, pStopNow );
}


