/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//UIGraphicsToolKit.cpp

#include "UIGraphicsToolKit.h"
#include "Control.h"
#include "StringUtils.h"
#include "OpenGLPeer.h"
#include "OpenGLContext.h"

#ifdef WIN32
	//Win32 specific stuff here
	#include "Win32Context.h"
	#include "Win32Image.h"
	#include "Win32Font.h"
	#include "Win32OpenGLPeer.h"
#endif

using namespace VCF;

UIGraphicsToolkit UIGraphicsToolkit::graphicsToolkitInstance;

std::map<String, ImageLoader*> UIGraphicsToolkit::graphicsToolkitImageLoaders;


UIGraphicsToolkit::UIGraphicsToolkit()
{
	initContentTypes();	
}

UIGraphicsToolkit::~UIGraphicsToolkit()
{

}

UIGraphicsToolkit* UIGraphicsToolkit::getDefaultGraphicsToolkit()
{
	if ( NULL != &UIGraphicsToolkit::graphicsToolkitInstance ){
		//throw exception
	}

	return &UIGraphicsToolkit::graphicsToolkitInstance;
}

ContextPeer* UIGraphicsToolkit::createContextPeer( const long& contextID )
{
	ContextPeer* result = NULL;
	
	result = new Win32Context( contextID );

	return result;
}

ContextPeer* UIGraphicsToolkit::createContextPeer( const unsigned long& width, const unsigned long& height )
{
	return new Win32Context( width, height );
}

FontPeer* UIGraphicsToolkit::createFontPeer( const String& fontName )
{
	return new Win32Font( fontName );
}

FontPeer* UIGraphicsToolkit::createFontPeer( const String& fontName, const double& pointSize )
{
	return new Win32Font( fontName, pointSize );
}

OpenGLPeer* UIGraphicsToolkit::createOpenGLPeer( GraphicsContext* glContext )
{
#ifdef WIN32
	return new Win32OpenGLPeer( glContext );
#else
	return NULL;
#endif
}

Image* UIGraphicsToolkit::createImage()
{
	return NULL;
}

Image* UIGraphicsToolkit::createImage( const unsigned long& width, const unsigned long& height )
{
	return NULL;
}

Image* UIGraphicsToolkit::createImage( const String& fileName )
{
	Image* result = NULL;
	String ext = StringUtils::getFileExt( fileName );	
	ext = StringUtils::lowerCase( ext );
	std::map<String,String>::iterator it = m_contentTypes.find( ext );
	if ( it != m_contentTypes.end() ){
		ImageLoader* imageLoader = getImageLoader( it->second );
		if ( NULL != imageLoader ){
			result = imageLoader->loadImageFromFile( fileName );
		}
	}
	return result;
}

void UIGraphicsToolkit::registerImageLoader( const String& name, ImageLoader* imageLoader )
{
	graphicsToolkitImageLoaders[name] = imageLoader;
}                                   

void UIGraphicsToolkit::initContentTypes()
{
	m_contentTypes["jpeg"] = "image/jpeg";
	m_contentTypes["jpe"] = "image/jpeg";
	m_contentTypes["jpg"] = "image/jpeg";
	m_contentTypes["gif"] = "image/gif";
	m_contentTypes["bmp"] = "image/bmp";
	m_contentTypes["rle"] = "image/bmp";
	m_contentTypes["dib"] = "image/bmp";
	m_contentTypes["wmf"] = "image/emf";
	m_contentTypes["emf"] = "image/emf";
	m_contentTypes["tif"] = "image/tiff";
	m_contentTypes["tiff"] = "image/tiff";
	m_contentTypes["tga"] = "image/targa";
	m_contentTypes["jfif"] = "image/jpeg";
	m_contentTypes["png"] = "image/png";
	m_contentTypes["pcx"] = "image/pcx";
	m_contentTypes["sgi"] = "image/sgi";
}

ImageLoader* UIGraphicsToolkit::getImageLoader( const String& contentType )
{
	ImageLoader* result = NULL;
	std::map<String,ImageLoader*>::iterator it = graphicsToolkitImageLoaders.find( contentType );
	if ( it != graphicsToolkitImageLoaders.end() ){
		result = it->second;
	}
	return result;
}

Image* UIGraphicsToolkit::createImage( GraphicsContext* context, Rect* rect  )
{	
	if ( NULL != context ){

#ifdef WIN32
		return new Win32Image( context, rect );
#endif

	}
	else {
		return NULL;
	}
}