/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#include "Dialog.h"

#include "UIToolkit.h"
#include "ClassRegistry.h"
#include "ClassInfo.h"
#include "Component.h"
#include "ControlPeer.h"
#include "ListviewPeer.h"
#include "TreePeer.h"
#include "ListBoxPeer.h"
#include "ComboBoxPeer.h"
#include "TextPeer.h"
#include "RichTextPeer.h"
#include "ScrollbarPeer.h"
#include "ButtonPeer.h"
#include "HeaderPeer.h"
#include "TabbedPagesPeer.h"
#include "DialogPeer.h"
#include "ToolbarPeer.h"
#include "WindowPeer.h"
#include "ListBoxPeer.h"
#include "ClipboardPeer.h"
#include "DragDropPeer.h"
#include "DataObjectPeer.h"
#include "SystemPeer.h"
#include "System.h" 
#include "DropTargetPeer.h"
#include "ContextPeer.h"
#include "TextControl.h"
#include "TreeControl.h"
#include "ListBoxControl.h"
#include "ListViewControl.h"
#include "Clipboard.h"
#include "UIGraphicsToolkit.h"
#include "PropertyEditorManager.h"

#include "DefaultPropertyEditors.h"
#include "ComboBoxControl.h"
#include "CustomControl.h"
#include "TabbedPages.h"
#include "Panel.h"
#include "CommandButton.h"
#include "TextModel.h"
#include "Label.h"
#include "CheckBoxControl.h"
#include <iostream>
#include "CommonDialogPeer.h"
#include "ListModel.h"
#include "ListItem.h"

//Peers


#include "LightweightComponent.h"
#include "Win32SystemPeer.h"
#include "Win32ToolKit.h"
#include "Win32Component.h"
#include "Win32Window.h"
#include "Locales.h"
#include "MenuBar.h"

using namespace VCF;


UIToolkit* UIToolkit::toolKitInstance;

//Locales VCF::Locales::localListing;


UIToolkit::UIToolkit()
{	
	
}

UIToolkit::~UIToolkit()
{
	std::map<unsigned long,Color*>::iterator it = m_systemColors.begin();
	while ( it != m_systemColors.end() ){
		delete it->second;
		it++;
	}
	m_systemColors.clear();
	delete m_systemClipboard;
	delete m_system;
}

void UIToolkit::init()
{
	this->m_graphicsToolKit = UIGraphicsToolkit::getDefaultGraphicsToolkit();
	this->m_systemClipboard = new Clipboard();
	m_system = new System();
	
	/**
	*register basic property editors
	*/
	
	PropertyEditorManager::registerPropertyEditor( new IntegerPropertyEditor(), CLASS_INTEGER );
	PropertyEditorManager::registerPropertyEditor( new DoublePropertyEditor(), CLASS_DOUBLE );
	PropertyEditorManager::registerPropertyEditor( new BoolPropertyEditor(), CLASS_BOOL );
	PropertyEditorManager::registerPropertyEditor( new StringPropertyEditor(), CLASS_STRING );
	PropertyEditorManager::registerPropertyEditor( new EnumPropertyEditor(), CLASS_ENUM );
	PropertyEditorManager::registerPropertyEditor( new ColorPropertyEditor(), "VCF::Color" );
	PropertyEditorManager::registerPropertyEditor( new FontPropertyEditor(), "VCF::Font" );
	PropertyEditorManager::registerPropertyEditor( new DefaultMenuItemPropertyEditor(), "VCF::DefaultMenuItem" );
	PropertyEditorManager::registerPropertyEditor( new DefaultListModelPropertyEditor(), "VCF::DefaultListModel" );	

	/**
	*register the components in categorys
	*/

	Component::registerComponent( "VCF::TabbedPages", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::Panel", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::CommandButton", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::ListBoxControl", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::ListViewControl", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::TreeControl", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::TextControl", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::Label", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::CheckBoxControl", STANDARD_CATEGORY );
	Component::registerComponent( "VCF::MenuBar", STANDARD_CATEGORY );

}

void UIToolkit::initToolKit()
{	
	UIToolkit::toolKitInstance = NULL;
	UIToolkit::toolKitInstance = new Win32ToolKit();
	((UIToolkit*)UIToolkit::toolKitInstance)->init();
}

void UIToolkit::closeToolkit()
{
	delete UIToolkit::toolKitInstance;
	UIToolkit::toolKitInstance = NULL;
}

UIToolkit* UIToolkit::getDefaultUIToolkit() throw (NoToolKitFoundException)
{
	if ( NULL == UIToolkit::toolKitInstance ){
		throw new NoToolKitFoundException();
	}
	return UIToolkit::toolKitInstance;
}

ControlPeer* UIToolkit::createControlPeer( Control* component, ComponentType componentType)
{
	ControlPeer* result = NULL;

	switch ( componentType ){
		case CT_LIGHTWEIGHT:{
			result = new LightweightComponent( component );
		}
		break;

		case CT_DEFAULT: case CT_HEAVYWEIGHT:{
			result = new Win32Component( component );
		}
		break;
	}
	return result;
}

ListviewPeer* UIToolkit::createListViewPeer( ListViewControl* component, ComponentType componentType)
{
	return NULL;
}

TreePeer* UIToolkit::createTreePeer( TreeControl* component, ComponentType componentType)
{
	return NULL;
}

ComboBoxPeer* UIToolkit::createComboBoxPeer( ComboBoxControl* component, ComponentType componentType)
{
	return NULL;
}

ListBoxPeer* UIToolkit::createListboxPeer( ListBoxControl* component, ComponentType componentType)
{
	return NULL;
}

TextPeer* UIToolkit::createTextPeer( TextControl* component, ComponentType componentType)
{
	return NULL;
}	

RichTextPeer* UIToolkit::createRTFPeer( ComponentType componentType)
{
	return NULL;
}

ScrollbarPeer* UIToolkit::createScrollbarPeer( ComponentType componentType)
{
	return NULL;
}

ButtonPeer* UIToolkit::createButtonPeer( CommandButton* component, ComponentType componentType)
{
	return NULL;
}
	
HeaderPeer* UIToolkit::createHeaderPeer( ComponentType componentType)
{
	return NULL;
}

TabbedPagesPeer* UIToolkit::createTabsheetsPeer( ComponentType componentType)
{
	return NULL;
}

DialogPeer* UIToolkit::createDialogPeer( Frame* owner, Dialog* component, ComponentType componentType)
{
	return NULL;
}

DialogPeer* UIToolkit::createDialogPeer()
{
	return NULL;
}

WindowPeer* UIToolkit::createWindowPeer( Control* component, ComponentType componentType)
{
	return new Win32Window( component );
}

ClipboardPeer* UIToolkit::createClipboardPeer()
{
	return NULL;
}

DragDropPeer* UIToolkit::createDragDropPeer()
{
	return NULL;
}

DataObjectPeer* UIToolkit::createDataObjectPeer()
{
	return NULL;
}

Clipboard* UIToolkit::getSystemClipboard()
{
	return this->m_systemClipboard;
}

ApplicationPeer* UIToolkit::createApplicationPeer()
{
	return NULL;
}

SystemPeer* UIToolkit::getSystemPeer()
{
	return new Win32SystemPeer();
}

System* UIToolkit::getSystem()
{
	return m_system;
}

Color* UIToolkit::getSystemColor( const unsigned long& systemColor )
{	
	return NULL;
}

DropTargetPeer* UIToolkit::createDropTargetPeer()
{
	return NULL;
}


ContextPeer* UIToolkit::createContextPeer( const long& contextID )
{
	if ( NULL == m_graphicsToolKit ){
		//throw exception
	}
	return this->m_graphicsToolKit->createContextPeer( contextID );
}

ContextPeer* UIToolkit::createContextPeer( Control* component )
{
	if ( NULL == m_graphicsToolKit ){
		//throw exception
	}
	return NULL;//this->m_graphicsToolKit->createContextPeer( component );
}

ContextPeer* UIToolkit::createContextPeer( const unsigned long& width, const unsigned long& height )
{
	if ( NULL == m_graphicsToolKit ){
		//throw exception
	}
	return this->m_graphicsToolKit->createContextPeer( width, height );
}

MenuItemPeer* UIToolkit::createMenuItemPeer( MenuItem* item )
{
	return NULL;
}

MenuBarPeer* UIToolkit::createMenuBarPeer( MenuBar* menuBar )
{
	return NULL;
}

CommonFileDialogPeer* UIToolkit::createCommonFileOpenDialogPeer( Control* owner )
{
	return NULL;
}

CommonFolderBrowseDialogPeer* UIToolkit::createCommonFolderBrowseDialogPeer( Control* owner )
{
	return NULL;
}

CommonFileDialogPeer* UIToolkit::createCommonFileSaveDialogPeer( Control* owner )
{
	return NULL;
}

CommonColorDialogPeer* UIToolkit::createCommonColorDialogPeer( Control* owner )
{
	return NULL;
}

CommonFontDialogPeer* UIToolkit::createCommonFontDialogPeer( Control* owner )
{
	return NULL;
}

PopupMenuPeer* UIToolkit::createPopupMenuPeer( PopupMenu* popupMenu )
{
	return NULL;
}