/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//AbstractListModel.cpp

#include "AbstractListModel.h"
#include <algorithm>

using namespace VCF;

AbstractListModel::AbstractListModel()
{
	
	m_listContainer.initContainer( m_listItems );
}

AbstractListModel::~AbstractListModel()
{
	std::vector<ListItem*>::iterator it = m_listItems.begin();	
	while ( it != m_listItems.end() ){		
		delete *it;
		it ++;
	}
	m_listItems.clear();
}

void AbstractListModel::addItem( ListItem* item )
{
	this->m_listItems.push_back( item );
	item->setIndex( m_listItems.size() - 1 );
	ListModelEvent event( this, item );
	notify_onItemAdded( &event );
}

void AbstractListModel::deleteItem( const unsigned long& index )
{
	ListItem* item = m_listItems[index];	
	ListModelEvent event( this, item );	
	notify_onItemDeleted( &event );	

	delete item;
	m_listItems.erase( m_listItems.begin() + index );
}

void AbstractListModel::deleteItem( ListItem* item )
{
	std::vector<ListItem*>::iterator it = std::find( m_listItems.begin(), m_listItems.end(), item );
	if ( it != m_listItems.end() ){		
		ListModelEvent event( this, item );
		notify_onItemDeleted( &event );		
		delete *it;
		m_listItems.erase( it );
	}
}

void AbstractListModel::empty()
{
	std::vector<ListItem*>::iterator it = m_listItems.begin();
	ListModelEvent event( this, LIST_MODEL_CONTENTS_DELETED );
	while ( it != m_listItems.end() ){
		event.setListItem( *it );
		notify_onItemDeleted( &event );
		delete *it;
		it ++;
	}
	m_listItems.clear();

	notify_onListModelContentsChanged( &event );
}

void AbstractListModel::insertItem( const unsigned long& index, ListItem* item )
{
	m_listItems.insert( m_listItems.begin() + index, item );
	item->setIndex( index );
	ListModelEvent event( this, item );
	notify_onItemAdded( &event );
}

void AbstractListModel::validate()
{

}

ListItem* AbstractListModel::getItemFromIndex( const unsigned long& index )
{
	ListItem* result = NULL;
	if ( index < m_listItems.size() ){
		result = m_listItems[index];
	}
	return result;
}

Enumerator<ListItem*>* AbstractListModel::getItems()
{
	return m_listContainer.getEnumerator();	
}

unsigned long AbstractListModel::getCount()
{
	return this->m_listItems.size();
}

void AbstractListModel::saveToStream( OutputStream * stream )
{
	Enumerator<ListItem*>* items = this->getItems();
	long count = this->getCount();
	stream->write( count );
	while ( items->hasMoreElements() ){
		ListItem* item = items->nextElement();
		if ( NULL != item ){
			Persistable* persistableItem = dynamic_cast<Persistable*>(item);
			if ( NULL != persistableItem ){
				stream->write( persistableItem );
			}
			else {
				String caption = item->getCaption();
				stream->write( caption );
			}
		}	
	}
}

void AbstractListModel::loadFromStream( InputStream * stream )
{
	long count = 0;
	stream->read( count );
	for (int i=0;i<count;i++){
		String s = "";
		stream->read( s );
	}
}