/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// AbstractTextModel.cpp


#include "AbstractTextModel.h"

using namespace VCF;


AbstractTextModel::AbstractTextModel()
{
	
}

AbstractTextModel::~AbstractTextModel()
{

}

void AbstractTextModel::validate()
{
	ValidationEvent event( this );

	this->notify_onModelValidate( &event );
}

void AbstractTextModel::empty()
{
	this->m_text = "";
	TextEvent event( this );
	
	this->notify_onTextChanged( &event );
}

void AbstractTextModel::setText( const String& text )
{
	TextEvent event( this, text );
	
	this->notify_onTextChanged( &event );
}

void AbstractTextModel::insertText( const unsigned long& index, const String& text )
{
	TextEvent event( this, text );

	this->notify_onTextChanged( &event );
}

void AbstractTextModel::deleteText( const unsigned long& index, const unsigned long& count )
{
	String changeText = this->m_text.substr( index, count );
	TextEvent event( this, changeText );
	
	this->notify_onTextChanged( &event );
}

void AbstractTextModel::appendText( const String& text )
{
	TextEvent event( this, text );

	this->notify_onTextChanged( &event );
}

String AbstractTextModel::getText()
{
	return this->m_text;
}

unsigned long AbstractTextModel::getSize()
{
	return this->m_text.size();
}
