/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "BasicTableItemEditor.h"
#include "TextControl.h"
#include "TableCellItem.h"
#include "TextModel.h"

using namespace VCF;

BasicTableItemEditor::BasicTableItemEditor( TableCellItem* editingItem )
{
	m_editingItem = editingItem;
}

BasicTableItemEditor::~BasicTableItemEditor()
{
	m_editorMap.clear();
}	


bool BasicTableItemEditor::isCellEditable()
{
	return true;
}

void BasicTableItemEditor::setItemToEdit( TableCellItem* itemToEdit )
{
	m_editingItem = itemToEdit;
}

void BasicTableItemEditor::updateItem()
{
	if ( NULL != m_editingItem ){
		TextControl* editingControl = dynamic_cast<TextControl*>( getCurrentEditingControl(m_editingItem) );
		if ( NULL != editingControl ){
			TextModel* tm = editingControl->getTextModel();
			
			String captionOfItem = tm->getText();
			
			void* itemCaptionPtr = (void*)&captionOfItem;
			ItemEditorEvent event( this, itemCaptionPtr );
			event.setItemBeingEdited( m_editingItem );
			
			try {
				notify_onValidateChange( &event );
				
				event.setType( ITEMEDITOR_CHANGED );
				//if we got this far we are OK to send off a change notification !
				
				m_editingItem->setCaption( captionOfItem );
				notify_onChange( &event );						
			}
			catch ( InvalidStateException& ex ){
				
			}
			catch (...){
				throw;//pass it on to someone else !!
			}
		}
	}
}

Control* BasicTableItemEditor::getEditingControl()
{
	if ( NULL != m_editingItem ){
		TextControl* tc = new TextControl();
		m_editorMap[m_editingItem] = tc;
		TextModel* tm = tc->getTextModel();
		tm->setText( m_editingItem->getCaption() );
		return tc;
	}
	
	return NULL;
}

Control* BasicTableItemEditor::getCurrentEditingControl( TableCellItem* currentItem )
{
	Control* result = NULL;
	std::map<TableCellItem*,Control*>::iterator found = m_editorMap.find( currentItem );
	if ( found != m_editorMap.end() ){
		result = found->second;
	}

	return result;
}