/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ClassRegistry.cpp

#include "ClassRegistry.h"
#include <iostream>

using namespace VCF;

#ifdef USE_IMPROV_NEW

std::list<void*> Object::allocatedObjects; 

#endif

ClassRegistry ClassRegistry::m_registryInstance;


ClassRegistry::ClassRegistry()
{
	m_classContainer.initContainer( m_classMap );

	//register "Object" as default class	
	TypedClass<Object>* objClass = new TypedClass<Object>( "VCF::Object", OBJECT_CLASSID, "" );

	m_classMap["VCF::Object"] = objClass;

	objClass->init();	
}

ClassRegistry::~ClassRegistry()
{
	std::map<String,Class*>::iterator it = m_classMap.begin();
	while ( it != m_classMap.end() ){
		delete it->second;
		it ++;
	}
	m_classMap.clear();
}

ClassRegistry* ClassRegistry::getClassRegistry()
{
	return &ClassRegistry::m_registryInstance;
}

Class* ClassRegistry::getClass( const String& className )
{
	Class* result = NULL;

	std::map<String,Class*>::iterator found = 
		m_classMap.find( className );

	if ( found != m_classMap.end() ){
		result = found->second;
	}

	return result;
}

Class* ClassRegistry::getClass( Object* object )
{
	Class* result = NULL;
	std::map<String,Class*>::iterator found = 
		m_classMap.begin();

	Class* clazz = NULL;
	while ( found != m_classMap.end() ){
		clazz = found->second;
		if ( clazz->isEqual( object ) ){
			result = clazz;
			break;
		}
		found++;
	}

	return result;
}

void ClassRegistry::createNewInstance( const String& className, Object** newInstance )
{
	Class* clazz = NULL;
	*newInstance = NULL;
	std::map<String,Class*>::iterator found = 
		m_classMap.find( className );

	if ( found != m_classMap.end() ){
		clazz = found->second;
		clazz->createInstance( newInstance );
	}

}

void ClassRegistry::addClass( const String& className, Class* classToRegister )
{	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();

	std::map<String,Class*>::iterator found = 
		classRegistry->m_classMap.find( className );

	if ( found == classRegistry->m_classMap.end() ){	
		classRegistry->m_classMap[className] = classToRegister;	
		Class* superClass = classToRegister->getSuperClass();
		Class* tmp = NULL;
		while ( NULL != superClass ){
			//copy over properties
			Enumerator<Property*>* props = superClass->getProperties();
			if ( NULL != props ){
				while ( props->hasMoreElements() ){
					Property* prop = props->nextElement();
					if ( NULL != prop ){
						Property* newProp = NULL;
						prop->copy( &newProp );
						classToRegister->addProperty( newProp );
					}
				}
			}

			//copy over events
			Enumerator<EventPropertySet*>* eventSets = superClass->getEventSets();
			if ( NULL != eventSets ){
				while ( eventSets->hasMoreElements() ){
					EventPropertySet* eps = eventSets->nextElement();
					if ( NULL != eps ){
						Enumerator<EventProperty*>* eventProps = eps->getEvents();
						if ( NULL != eventProps ){
							while ( eventProps->hasMoreElements() ){
								EventProperty* eventProp = eventProps->nextElement();
								if ( NULL != eventProp ){
									classToRegister->addEvent( eventProp->getEventClassName(),
																eventProp->getEventMethod(),
																eventProp->getListenerClassName(),
																eps->getAdapterClassName() );
								}
							}
						}	
					}
				}
			}
			tmp = superClass;
			superClass = tmp->getSuperClass();
		}
	}
}

Enumerator<Class*>* ClassRegistry::getClasses()
{
	return m_classContainer.getEnumerator();
}

void ClassRegistry::dump()
{
	std::map<String,Class*>::iterator it = m_classMap.begin();

	while ( it != m_classMap.end() ){	
		std::cout << typeid(*(it->second)).name() << std::endl;
		it++;
	}
}
