/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//ComboBoxControl

#include "ComboBoxControl.h"
#include "UIToolkit.h"
#include "DefaultListModel.h"
#include "ComboBoxPeer.h"
#include "InvalidPeer.h"
#include "Basic3DBorder.h"
#include "ListItem.h"
#include "Font.h"


using namespace VCF;

ComboBoxControl::ComboBoxControl():
	CustomControl( true )//make this a heavyweight control !
{		
	this->init();	
}

void ComboBoxControl::init()
{
	this->setBorder( new Basic3DBorder() );

	m_listModel = new DefaultListModel();
	m_modelHandler = new ListModelHandler( this );
	m_modelHandler->m_itemAdded = (OnListModelEvent)this->onItemAdded;
	m_modelHandler->m_itemDeleted = (OnListModelEvent)this->onItemDeleted;
	m_modelHandler->m_listModelContentsChanged = (OnListModelEvent)this->onListModelContentsChanged;
	m_listModel->addListModelListener( m_modelHandler );	
	this->addEventHandler( "ListModelHandler", m_modelHandler );
	m_selectedIndex = 0;
	m_arrowPressed = false;
}

ComboBoxControl::~ComboBoxControl()
{
	if ( NULL != m_listModel ){
		delete m_listModel;
		m_listModel = NULL;
	}
}

ListModel* ComboBoxControl::getListModel()
{
	return m_listModel;
}

void ComboBoxControl::setListModel(ListModel * model)
{
	m_listModel = model;
}

void ComboBoxControl::paint( GraphicsContext* context )
{
	CustomControl::paint( context );
	UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();
	Color* hilight = toolkit->getSystemColor( SYSCOLOR_HIGHLIGHT );	
	Color* shadow = toolkit->getSystemColor( SYSCOLOR_SHADOW );

	double h = this->getHeight();
	double x = this->getWidth() - h;
	double y = 3.0;
	m_arrowRect.setRect( x, y, (x + h) -4, (y + h) -6 );
	
	Color* cl1 = hilight;
	Color* cl2 = shadow;
	if ( true == this->m_arrowPressed ) {
		cl1 = shadow;
		cl2 = hilight;
		context->rectangle( &m_arrowRect );
		context->setColor( shadow );
		context->fillPath();

		Border* border = this->getBorder();
		Rect innerRect;
		if ( NULL != border ){
			Rect* tmp = border->getClientRect( this );
			innerRect.setRect( tmp->m_left, tmp->m_top, m_arrowRect.m_left, tmp->m_bottom );
			innerRect.inflate( -2, -2 );
			innerRect.m_bottom -=1;
		}
		else {
			innerRect.setRect( 3, 3, m_arrowRect.m_left, m_arrowRect.m_bottom-1 );
		}
		context->moveTo( innerRect.m_left, innerRect.m_bottom );	
		context->lineTo( innerRect.m_left, innerRect.m_top );
		context->lineTo( innerRect.m_right, innerRect.m_top );
		context->setColor( shadow );
		context->strokePath();

		context->moveTo( innerRect.m_right, innerRect.m_top );
		context->lineTo( innerRect.m_right, innerRect.m_bottom );
		context->lineTo( innerRect.m_left, innerRect.m_bottom );
		context->setColor( hilight );
		context->strokePath();
	}
	else {

	}

	context->moveTo( x, y );
	context->lineTo( x, h - 3 );
	context->setColor( cl1 );
	context->strokePath();

	context->moveTo( x+1, y );
	context->lineTo( x+1, h - 3 );
	context->setColor( cl2 );
	context->strokePath();

	std::vector<Point*> triangle;
	Point pt1( x + h/4, h/2 - 2 );
	Point pt2( pt1.m_x + h/2, pt1.m_y );	
	Point pt3( x + h/2, pt1.m_y + h/4 );
	triangle.push_back( &pt1 );
	triangle.push_back( &pt2 );
	triangle.push_back( &pt3 );
	
	context->polyline( triangle );
	context->setColor( Color::getColor( "black" ) );	
	context->fillPath();
	ListItem* selectedItem = getSelectedItem();
	if ( NULL != selectedItem ){
		String caption = selectedItem->getCaption();
		Font* f = context->getCurrentFont();
		double sy = h/2 - (f->getPixelSize()/2);
		context->drawString( 5, sy, caption );	
	}	
}

void ComboBoxControl::onListModelContentsChanged( ListModelEvent* event )
{
	if ( NULL != event ){
		switch ( event->getType() ){
			case LIST_MODEL_CONTENTS_DELETED: {
				
			}
			break;

			case LIST_MODEL_ITEM_CHANGED: {
				ListItem* item = event->getListItem();
				if ( NULL != item ){

				}
			}
			break;		
		}
	}
}

void ComboBoxControl::onItemAdded( ListModelEvent* event )
{
}

void ComboBoxControl::onItemDeleted( ListModelEvent* event )
{
}

void ComboBoxControl::mouseDown( MouseEvent* event )
{
	CustomControl::mouseDown( event );
	
	this->m_arrowPressed = this->m_arrowRect.containsPt( event->getPoint() );
	if ( true == m_arrowPressed ){
		this->keepMouseEvents();
	}
	this->repaint();
}

void ComboBoxControl::mouseMove( MouseEvent* event )
{
	CustomControl::mouseMove( event );
	if ( event->hasLeftButton() ){
		this->m_arrowPressed = this->m_arrowRect.containsPt( event->getPoint() );
		this->repaint();
	}
}

void ComboBoxControl::mouseUp( MouseEvent* event )
{
	CustomControl::mouseUp( event );
	m_arrowPressed = false;
	this->repaint();
	this->releaseMouseEvents();
}

void ComboBoxControl::keyPressed( KeyboardEvent* event )
{
	switch ( event->getVirtualCode() ){
		case VIRT_KEY_UP_ARROW :{
			ulong32 index = m_selectedIndex + 1;
			
			if ( index >=	this->m_listModel->getCount() ){
				index = 0;
			}

			this->setSelectedItemIndex( index );

			this->repaint();
		}
		break;

		case VIRT_KEY_DOWN_ARROW :{
			ulong32 index = m_selectedIndex - 1;
			/**
			* this is done this way because we have an UNSIGNED long so we 
			*won't get negative numbers, anything over the count needs to be wrapped
			*/
			if ( index > this->m_listModel->getCount() ){ 
				index = this->m_listModel->getCount() -1;
			}

			this->setSelectedItemIndex( index );

			this->repaint();
		}
		break;
	}
}

ListItem* ComboBoxControl::getSelectedItem()
{
	ListItem* result = NULL;
	
	if ( NULL != m_listModel ){
		if ( m_listModel->getCount() > 0 ){
			result = m_listModel->getItemFromIndex( this->m_selectedIndex );
		}
	}
		
	return result;
}

void ComboBoxControl::setSelectedItem( ListItem* selectedItem )
{

}

void ComboBoxControl::setSelectedItemIndex( const ulong32& selectedIndex )
{
	m_selectedIndex = selectedIndex;
	if ( NULL != m_listModel ){
		if ( m_listModel->getCount() > 0 ){
			ListItem* item = m_listModel->getItemFromIndex( this->m_selectedIndex );
			if ( NULL != item ){
				item->setSelected( true );
				ItemEvent event( item, ITEM_EVENT_SELECTED );
				notify_onSelectionChange( &event );
				this->repaint();
			}
		}
	}
}