/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//CommandButton.cpp

#include "CommandButton.h"
#include "GraphicsContext.h"
#include "UIToolkit.h"
#include "ClassRegistry.h"
#include "ButtonPeer.h"
#include "InvalidPeer.h"

using namespace VCF;

CommandButton::CommandButton()
{
	
	m_Peer = UIToolkit::getDefaultUIToolkit()->createButtonPeer( this );
	
	if ( NULL == m_Peer ){
		throw InvalidPeer( MAKE_ERROR_MSG(NO_PEER), __LINE__ );
	}
	m_Peer->setControl( this );

	m_buttonPeer = (ButtonPeer*)m_Peer;	

	m_commandType = BC_NONE;
}

CommandButton::~CommandButton()
{

}

void CommandButton::click()
{
	ButtonEvent event( this, 0 );
	notify_onButtonClicked( &event );
}

void CommandButton::setCaption( const std::string& caption )
{
	this->m_Peer->setText( caption );
}

String CommandButton::getCaption()
{
	return this->m_Peer->getText();
}

void CommandButton::paint(GraphicsContext * context)
{
	context->circle( 10, 10, 5 );
	context->fillPath();
}

ButtonCommandType CommandButton::getCommandType()
{
	return this->m_commandType;
}

void CommandButton::setCommandType( const ButtonCommandType& commandType )
{
	m_commandType = commandType;
}