/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#include "Component.h"
#include "Model.h"
#include "GraphicsContext.h"
#include "Border.h"
#include "ClassRegistry.h"
#include "UIToolkit.h"
#include <algorithm>
#include "Container.h"

using namespace VCF;

std::map<String,std::vector<Class*>*> Component::registeredComponentMap;

EnumeratorContainer<std::vector<Class*>,Class*> Component::registeredCompContainer;

EnumeratorContainer<std::vector<String>,String> Component::registeredCategoryContainer;

std::vector<String> Component::registeredCategory;


Component::Component()
{	
	
	m_componentState = CS_NORMAL;
	m_top = 0.0;
	m_left = 0.0;
	m_componentContainer.initContainer( this->m_components );
}

Component::~Component()
{
	std::vector<Component*>::iterator componentIter = this->m_components.begin();
	while ( componentIter != m_components.end() ){
		Component* component = *componentIter;
		delete component;
		component = NULL;
		componentIter++;
	}
	m_components.clear();
}

double Component::getLeft()
{
	return m_left;
}

double Component::getTop()
{
	return m_top;
}

String Component::getName()
{
	return this->m_name;
}

void Component::setName( const String& name )
{
	this->m_name = name;
}

void Component::setLeft( const double& left )
{
	this->m_left = left;
}

void Component::setTop( const double& top )
{
	this->m_top = top;
}

void Component::loadFromStream( InputStream * stream )
{
	if ( NULL != stream ){
		Class* clazz = this->getClass();
		if ( NULL != clazz ){
			String propName = "";	
			String strVal = "";
			long count = 0;
			stream->read( count );
			while ( count > 0 ){
				stream->read( propName );
				Property* prop = clazz->getProperty( propName );
				if ( NULL != prop ){
					VariantData* val = prop->get();
					stream->read( strVal );
					val->setFromString( strVal );
				}
				count --;
			}
		}
	}
}

void Component::saveToStream( OutputStream * stream )
{
	if ( NULL != stream ){
		Class* clazz = this->getClass();
		if ( NULL != clazz ){
			Enumerator<Property*>* props = clazz->getProperties();
			long count = clazz->getPropertyCount();
			stream->write( count );
			while ( props->hasMoreElements() ){
				Property* prop = props->nextElement();
				if ( NULL != prop ){
					stream->write( prop->getName() );
					VariantData* val = prop->get();
					String strVal = "null";
					if ( NULL != val ){
						strVal = val->toString();
						stream->write( strVal );
					}
					else {
						stream->write( strVal );
					}
				}
			}
		}
	}
}

void Component::beforeDestroy( ComponentEvent* event )
{

}

void Component::processComponentEvent( ComponentEvent* componentEvent )
{
	if ( NULL != componentEvent ){
		unsigned long eventType = componentEvent->getType();

		switch ( eventType ){

			case COMPONENT_EVENT_CREATED:{
				this->afterCreate( componentEvent );
				this->notify_onComponentCreated( componentEvent );
			}
			break;

			case COMPONENT_EVENT_DELETED:{
				this->beforeDestroy( componentEvent );
				this->notify_onComponentDeleted( componentEvent );
			}
		}
	}
}

String Component::toString()
{
	String result = "";
	//ComponentDTD dtd;
	//result += dtd.toString() + "\n";
	result += "<object class=\"" + this->getClassName() + "\"> \n";
	Class* clazz = this->getClass();
	if ( NULL != clazz ){
		Enumerator<Property*>* props =  clazz->getProperties();		
		
		while ( props->hasMoreElements() ){
			Property* p = props->nextElement();
			result += "\t" + p->toString() + "\n";
		}
	}
	result += "</object>";
	return result;
}

ComponentState Component::getComponentState()
{
	return this->m_componentState;
}

void Component::setComponentState( const ComponentState& componentState )
{
	this->m_componentState = componentState;
	std::vector<Component*>::iterator it = this->m_components.begin();
	while ( it != m_components.end() ){
		Component* child = *it;
		child->setComponentState( m_componentState );
		it ++;
	}
	
	Container* container = dynamic_cast<Container*>(this);
	if ( NULL != container ){
		Enumerator<Control*>* controls = container->getChildren();
		if ( NULL != controls ){
			while ( true == controls->hasMoreElements() ){
				Control* control = controls->nextElement();
				control->setComponentState( m_componentState );
			}
		}
	}
}

void Component::addComponent( Component* component )
{
	m_components.push_back( component );
	std::vector<Component*>::iterator it = this->m_components.begin();
}

void Component::removeComponent( Component* component )
{
	std::vector<Component*>::iterator found  = std::find( m_components.begin(), m_components.end(), component );
	if ( found != m_components.end() ){
		m_components.erase( found );
	}
}

Enumerator<Component*>* Component::getComponents()
{
	return m_componentContainer.getEnumerator();
}

unsigned long Component::getComponentCount()
{
	return m_components.size();
}

Component* Component::findComponent( const String& componentName ){
	Component* result = NULL;
	//this is very slow !! for the moment !
	std::vector<Component*>::iterator it = this->m_components.begin();
	while ( it != m_components.end() ){
		Component* child = *it;
		if ( child->getName() == componentName ){
			result = child;
			break;
		}
		it ++;
	}

	return result;
}

bool Component::bindVariable( Component** variablePtr, const String& variableName )
{
	bool result = false;
	/**
	*search components first
	*if that fails then try controls - if we are a container
	*/
	*variablePtr = NULL;
	Component* foundComp = this->findComponent( variableName );
	if ( NULL != foundComp ){
		*variablePtr = foundComp;
	}
	else {
		Container* container = dynamic_cast<Container*>(this);
		if ( NULL != container ){
			Control* foundControl = container->findControl( variableName );
			if ( NULL != foundControl ){
				*variablePtr = foundControl;
			}
		}
	}

	return result;
}

bool Component::registerComponent( const String& componentClassName, const String& category )
{	
	bool result = false;
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* componentClass = classRegistry->getClass( componentClassName );
	if ( NULL != componentClass ){
		std::map<String,std::vector<Class*>*>::iterator found = registeredComponentMap.find( category );
		std::vector<Class*>* categoryList = NULL;
		if ( found == registeredComponentMap.end() ){
			categoryList = new std::vector<Class*>();
			registeredCategory.push_back( category );
		}
		else {
			categoryList = found->second;
		}
		if ( NULL != categoryList ){
			categoryList->push_back( componentClass );
			Component::registeredComponentMap[category] = categoryList;			
			result = true;
		}		
	}

	return result;
}

Enumerator<Class*>* Component::getComponentClassesForCategory( const String& category )
{
	Enumerator<Class*>* result = NULL;
	//registeredCompContainer
	std::map<String,std::vector<Class*>*>::iterator found = registeredComponentMap.find( category );
	if ( found != registeredComponentMap.end() ){
		std::vector<Class*>* categoryList = found->second;
		if ( NULL != categoryList ){
			registeredCompContainer.initContainer( *categoryList );
			result = registeredCompContainer.getEnumerator();
		}
	}
	return result;
}

Enumerator<String>* Component::getRegisteredComponentCategories()
{
	registeredCategoryContainer.initContainer( registeredCategory );
	return registeredCategoryContainer.getEnumerator();
}

void Component::clearRegistedComponents()
{
	std::map<String,std::vector<Class*>*>::iterator it = registeredComponentMap.begin();
	while ( it != registeredComponentMap.end() ){
		std::vector<Class*>* compList = it->second;
		delete compList;
		compList = NULL;
		it++;
	}
	registeredComponentMap.clear();

	registeredCategory.clear();
}


/**
*ComponentDTD methods
*/

String ComponentDTD::toString()
{
	String result = "";	
	result += "<!--what follows is the DTD for Framework UI Objects-->\n";
	result += "<!DOCTYPE object [ \n";
	result += "<!ELEMENT object ((property*) | children)> \n";
	result += "<!ELEMENT children (object*)> \n";
	result += "<!ATTLIST object	class	CDATA 	#REQUIRED \n";
	result += "\t\t\t classID CDATA #IMPLIED> \n";
	result += "<!ELEMENT property EMPTY>\n";
	result += "<!ELEMENT objectProperty (property*)>\n";
	result += "<!ATTLIST property \n";
	result += "\t\t\t name CDATA #REQUIRED \n";
	result += "\t\t\t value CDATA #REQUIRED>\n";
	result += "]>\n";
	result += "<!--end of UI.dtd -->\n";
	return result;
}

