/**
*Visual Control Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Control Framework
*/ 

/* Generated by Together */

#include "Control.h"
#include "Model.h"
#include "ControlGraphicsContext.h"
#include "InvalidPeer.h"
#include "ControlPeer.h"
#include "Border.h"
#include "ClassRegistry.h"
#include "UIToolkit.h"
#include "Rect.h"
#include "Point.h"
#include "Font.h"
#include "ContextPeer.h"
#include "PopupMenu.h"
#include "Application.h"
#include "Container.h"


using namespace VCF;

Control* Control::capturedMouseControl = NULL;


Control::Control()
{	
	this->m_Peer = NULL;
	this->m_aligment = ALIGN_NONE;
	this->m_border = NULL;
	this->m_font = new Font();
	this->m_context = new ControlGraphicsContext( this );
	m_parent = NULL;
	this->m_doubleBuffered = true;
	m_hasMouseCapture = false;

	m_bounds = new Rect();
	m_clientBounds = new Rect();
	m_color = new Color;
	
	this->m_popupMenu = NULL;

	UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();
	if ( NULL != toolkit ){
		this->setColor( toolkit->getSystemColor( SYSCOLOR_FACE ) );
	}	
}

Control::~Control()
{
	delete m_bounds;
	delete m_clientBounds;
	delete m_color;

	delete m_context;

	if ( NULL != m_Peer ) {
		delete m_Peer;
		m_Peer = NULL;
	}
	if ( NULL != m_border ){
		delete m_border;
		m_border = NULL;
	}

	if ( NULL != m_font ){
		delete m_font;
		m_font = NULL;
	}
}


Border* Control::getBorder()
{
	return this->m_border;
}

void Control::setBorder( Border* border )
{
	if ( NULL != m_border ){
		delete m_border;
		m_border = NULL;
	}
	this->m_border = border;
}

Rect* Control::getBounds()
{	
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	*m_bounds = *(m_Peer->getBounds()); 

	return this->m_bounds;
}

Rect* Control::getClientBounds( const bool& includeBorder )
{	
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	
	Rect* r = this->m_Peer->getBounds(); 
	m_clientBounds->setRect( 0.0, 0.0, r->getWidth(), r->getHeight() );

	if ( (false == includeBorder) && (NULL != this->m_border) ){
		*m_clientBounds = *m_border->getClientRect( this );
	}

	return this->m_clientBounds;
}

double Control::getLeft()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	double result = 0.0;
	if ( NULL != this->m_Peer ){
		*m_bounds = *(m_Peer->getBounds());		
		result = this->m_bounds->m_left;
	};
	return result;
}

double Control::getWidth()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	double result = 0.0;
	if ( NULL != this->m_Peer ){
		*m_bounds = *(this->m_Peer->getBounds());		
		result = this->m_bounds->getWidth();
	};
	return result;
}

double Control::getTop()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	double result = 0.0;
	if ( NULL != this->m_Peer ){
		*m_bounds = *(m_Peer->getBounds());		
		result = this->m_bounds->m_top;
	};
	return result;
}

double Control::getHeight()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	
	double result = 0.0;
	if ( NULL != this->m_Peer ){
		*m_bounds = *(this->m_Peer->getBounds());		
		result = this->m_bounds->getHeight();
	};

	return result;
}

bool Control::getVisible()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	return this->m_Peer->getVisible();
}

AlignmentType Control::getAlignment()
{
	return this->m_aligment;
}

void Control::setBounds( Rect* rect )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	

	*m_bounds = *rect;
	this->m_Peer->setBounds( m_bounds );

}

void Control::setAlignment( const AlignmentType& alignment )
{
	this->m_aligment = alignment;
	Control* parent = this->getParent();
	if ( NULL != parent ){
		Container* container = dynamic_cast<Container*>( parent );
		if ( NULL != container ){
			container->resizeChildren();
		}
	}
}

void Control::setLeft( const double& left )
{
	Component::setLeft( left );
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	
	if ( NULL != this->m_Peer ){
		double dx = m_bounds->getWidth();
		this->m_bounds->m_left = left;
		this->m_bounds->m_right = left + dx;
		this->m_Peer->setBounds( m_bounds );
	};
}

void Control::setWidth( const double& width )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	if ( NULL != this->m_Peer ){
		this->m_bounds->m_right = this->m_bounds->m_left + width;
		this->m_Peer->setBounds( m_bounds );
	};
}

void Control::setTop( const double& top )
{
	Component::setTop( top );
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	if ( NULL != this->m_Peer ){
		double dy = this->m_bounds->getHeight();
		this->m_bounds->m_top = top;
		this->m_bounds->m_bottom = top + dy;
		this->m_Peer->setBounds( m_bounds );
	};
}

void Control::setHeight( const double& height )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	

	if ( NULL != this->m_Peer ){
		this->m_bounds->m_bottom = this->m_bounds->m_top + height;
		this->m_Peer->setBounds( m_bounds );
	};
}

void Control::setVisible( const bool& visible )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	this->m_Peer->setVisible( visible );
}

void Control::loadFromStream( InputStream * stream )
{
	if ( NULL != stream ){
		Class* clazz = this->getClass();
		if ( NULL != clazz ){
			String propName = "";	
			String strVal = "";
			long count = 0;
			stream->read( count );
			while ( count > 0 ){
				stream->read( propName );
				Property* prop = clazz->getProperty( propName );
				if ( NULL != prop ){
					VariantData* val = prop->get();
					stream->read( strVal );
					val->setFromString( strVal );
				}
				count --;
			}
		}
	}
}

void Control::saveToStream( OutputStream * stream )
{
	if ( NULL != stream ){
		Class* clazz = this->getClass();
		if ( NULL != clazz ){
			Enumerator<Property*>* props = clazz->getProperties();
			long count = clazz->getPropertyCount();
			stream->write( count );
			while ( props->hasMoreElements() ){
				Property* prop = props->nextElement();
				if ( NULL != prop ){
					stream->write( prop->getName() );
					VariantData* val = prop->get();
					String strVal = "null";
					if ( NULL != val ){
						strVal = val->toString();
						stream->write( strVal );
					}
					else {
						stream->write( strVal );
					}
				}
			}
		}
	}
}

void Control::processMouseEvent( MouseEvent* mouseEvent )
{
	if ( NULL != mouseEvent ){
		unsigned long eventType = mouseEvent->getType();

		switch ( eventType ){
			case MOUSE_EVENT_DOWN:{
				//this->mouseDown( mouseEvent );
				//if ( !mouseEvent->isConsumed() )
				this->notify_onMouseDown( mouseEvent );
				if (!mouseEvent->isConsumed())
					this->mouseDown( mouseEvent );
			}
			break;

			case MOUSE_EVENT_MOVE:{
				//this->mouseMove( mouseEvent );
				this->notify_onMouseMove( mouseEvent );
				if (!mouseEvent->isConsumed())
					this->mouseMove( mouseEvent );
			}
			break;

			case MOUSE_EVENT_UP:{
				//this->mouseUp( mouseEvent );
				bool rightBtn = mouseEvent->hasRightButton();
				this->notify_onMouseUp( mouseEvent );
				if (!mouseEvent->isConsumed())
					this->mouseUp( mouseEvent );
				if ( (NULL != this->m_popupMenu) && (true == rightBtn) ){
					m_popupMenu->popup( mouseEvent->getPoint() );	
				}
			}
			break;

			case MOUSE_EVENT_CLICK:{
				this->notify_onMouseClicked( mouseEvent );
				mouseClick( mouseEvent );
			}
			break;

			case MOUSE_EVENT_DBLCLICK:{
				this->notify_onMouseDoubleClicked( mouseEvent );
				mouseDblClick( mouseEvent );
			}
			break;
		}
		//if ( !mouseEvent->isConsumed() )
			//mouseEvent->setConsumed(true);
	}
}

void Control::processKeyboardEvent( KeyboardEvent* keyboardEvent )
{
	if ( NULL != keyboardEvent ){
		unsigned long eventType = keyboardEvent->getType();
		switch ( eventType ){
			case KEYBOARD_EVENT_DOWN:{
				this->keyDown( keyboardEvent );
				this->notify_onKeyDown( keyboardEvent );
			}
			break;

			case KEYBOARD_EVENT_PRESSED:{
				this->keyPressed( keyboardEvent );
				this->notify_onKeyPressed( keyboardEvent );
			}
			break;

			case KEYBOARD_EVENT_UP:{
				this->keyUp ( keyboardEvent );
				this->notify_onKeyUp( keyboardEvent );
			}
			break;
		}
	}
}

void Control::processControlEvent( ControlEvent* ControlEvent )
{
	if ( NULL != ControlEvent ){
		unsigned long eventType = ControlEvent->getType();

		switch ( eventType ){
			case CONTROL_EVENT_SIZED:{
				this->notify_onControlSized( ControlEvent );
			}
			break;

			case CONTROL_EVENT_POSITIONED:{
				this->notify_onControlPositioned( ControlEvent );
			}
			break;

			case CONTROL_EVENT_PARENT_CHANGED:{
				this->notify_onControlParentChanged( ControlEvent );
			}
			break;
		}
	}
}

bool Control::canBeginDragDrop()
{
	return false;
}

void Control::setAutoStartDragDrop(const bool& canAutoStartDragDrop)
{
	m_autoStartDragDrop = canAutoStartDragDrop;
}

bool Control::getAutoStartDragDrop()
{
	return m_autoStartDragDrop;
}

void Control::beginDragDrop()
{

}

ControlPeer* Control::getPeer()
{
	return this->m_Peer;
}

void Control::setParent( Control* parent )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};	
	
	m_parent = parent;
	this->m_Peer->setParent( parent );
}

Control* Control::getParent()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	return m_parent;
}

bool Control::isFocused()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	return this->m_Peer->isFocused();
}

void Control::setFocus( const bool& focused )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	this->m_Peer->setFocus( focused );
}

bool Control::isEnabled()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return this->m_Peer->isEnabled();
}

void Control::setEnabled( const bool& enabled )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	this->m_Peer->setEnabled( enabled );
}

void Control::mouseDown( MouseEvent* event )
{
	this->setFocus( true );
}

void Control::mouseMove( MouseEvent* event )
{

}

void Control::mouseUp( MouseEvent* event )
{
	
}

void Control::keyDown( KeyboardEvent* event )
{

}

void Control::mouseClick(  MouseEvent* event )
{

}
	
void Control::mouseDblClick(  MouseEvent* event )
{

}

void Control::keyPressed( KeyboardEvent* event )
{

}

void Control::keyUp( KeyboardEvent* event )
{

}

void Control::translateToParent( Point* point )
{
	Control* parent = this->getParent();
	if ( NULL != parent ){
		Rect* bounds = parent->getBounds();
		point->m_x += bounds->m_left;
		point->m_y += bounds->m_top;
	}
	else{
		//throw exception ?
	}
}

void Control::translateToLocal( Point* point )
{
	Control* parent = this->getParent();
	if ( NULL != parent ){
		Rect* bounds = this->getBounds();
		point->m_x -= bounds->m_left;
		point->m_y -= bounds->m_top;
	}
	else{
		//throw exception ?
	}
}

void Control::beforeDestroy( ControlEvent* event )
{

}

bool Control::isLighweight()
{	
	return false;
}

Control* Control::getHeavyweightParent()
{	
	Control* parent = NULL;
	Control* result = NULL;
	result = this->getParent();
	while ( true == result->isLighweight() ){
		parent = result;
		result = parent->getParent();
	}

	return result;
}

Color* Control::getColor()
{
	return this->m_color;
}

void Control::setColor( Color* color )
{
	m_color->copy( color );
}

String Control::toString()
{
	return Component::toString();
}

Font* Control::getFont()
{
	return m_font;
}

void Control::setFont( Font* font )
{
	if ( NULL != font ){
		m_font->copy( font );
	}
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	this->m_Peer->setFont( m_font );

}

bool Control::useParentFont()
{
	return m_useParentFont;
}

void Control::setUseParentFont( const bool& useParentFont )
{
	m_useParentFont = useParentFont;
}

void Control::afterCreate( ComponentEvent* event )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	this->m_Peer->setFont( m_font );
}

void Control::repaint( Rect* repaintRect )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	this->m_Peer->repaint( repaintRect );
}

bool Control::isDoubleBuffered()
{
	return this->m_doubleBuffered;
}

void Control::setDoubleBuffered( const bool& doubleBuffered )
{
	m_doubleBuffered = doubleBuffered;
}

void Control::keepMouseEvents()
{	
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_hasMouseCapture = true;
	Control::setCapturedMouseControl( this );
	this->m_Peer->keepMouseEvents();	
}

void Control::releaseMouseEvents()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_hasMouseCapture = false;
	Control::setCapturedMouseControl( NULL );
	this->m_Peer->releaseMouseEvents();	
}

Control* Control::getCapturedMouseControl()
{
	return Control::capturedMouseControl;
}
	
void Control::setCapturedMouseControl( Control* control )
{
	Control::capturedMouseControl = control;
}

GraphicsContext* Control::getContext()
{		
	return this->m_context;
}

PopupMenu* Control::getPopupMenu()
{
	return this->m_popupMenu;
}

void Control::setPopupMenu( PopupMenu* popupMenu )
{
	if ( NULL != m_popupMenu ){
		m_popupMenu->setControl( NULL );
	}
	
	m_popupMenu = popupMenu;

	if ( NULL != m_popupMenu ){
		m_popupMenu->setControl( this );
	}
}