/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//CustomControl.cpp


#include "CustomControl.h"
#include "UIToolkit.h"
#include "Border.h"
#include "FillState.h"

using namespace VCF;


CustomControl::CustomControl():
	Control()
{	
	m_Peer =	UIToolkit::getDefaultUIToolkit()->createControlPeer( this );
	m_isLightweight = false;
	//this->m_customControlPeer = (CustomControlPeer*)m_Peer;
	this->init();
}

CustomControl::CustomControl( const bool& useHeavyweightPeer ):
	Control()
{
	if ( true == useHeavyweightPeer ){
		m_Peer = UIToolkit::getDefaultUIToolkit()->createControlPeer( this, CT_HEAVYWEIGHT );
	}
	else{
		m_Peer = UIToolkit::getDefaultUIToolkit()->createControlPeer( this, CT_LIGHTWEIGHT );
	}
	this->m_customControlPeer = (CustomControlPeer*)m_Peer;
	m_isLightweight = !useHeavyweightPeer;
	this->init();
}

CustomControl::~CustomControl()
{
}

void CustomControl::paint(GraphicsContext * context)
{
	Border* border = this->getBorder();
	
	Rect innerBounds(0,0,this->getWidth(), this->getHeight() );

	if ( NULL != border ){
		border->paint( this, context );
		Rect* tmpBounds = border->getClientRect( this );
		innerBounds.setRect( tmpBounds->m_left, tmpBounds->m_top, tmpBounds->m_right, tmpBounds->m_bottom );
		innerBounds.inflate( -1, -1 );
	}
	if ( (true != m_isTransparent) || (true != m_isLightweight) ){
		FillState fill;
		fill.m_Color = *(this->getColor());
		context->rectangle( innerBounds.m_left, innerBounds.m_top, innerBounds.m_right, innerBounds.m_bottom );
		context->setCurrentFillState( &fill );
		context->fillPath();
	}
}

void CustomControl::init()
{	
	this->setEnabled( true );
	this->setVisible( true );
	this->setFocus( true );
}

bool CustomControl::isLighweight()
{
	return m_isLightweight;
}

bool CustomControl::isTransparent()
{
	return m_isTransparent;
}

void CustomControl::setTransparent( const bool& transparent )
{
	this->m_isTransparent = transparent;
}