/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//DefaultPropertyEditors.cpp

#include "DefaultPropertyEditors.h"
#include "ComboBoxControl.h"
#include "DefaultListItem.h"
#include "ListModel.h"
#include "ItemEvent.h"
#include "ComboBoxListener.h" 
#include "Panel.h"
#include "Basic3DBorder.h"
#include "CommandButton.h"
#include "Color.h"
#include "Dialog.h"
#include "CommonColor.h"
#include "UIToolkit.h"
#include "Font.h"
#include "CommonFont.h"
#include "ListboxControl.h"
#include "ListModel.h"	
#include "TextControl.h"
#include "Label.h"
using namespace VCF;


AbstractPropertyEditor::AbstractPropertyEditor()
{
	m_source = NULL;
	m_property = NULL;
}

AbstractPropertyEditor::~AbstractPropertyEditor()
{

}

Control* AbstractPropertyEditor::getCustomEditor()
{
	return NULL;
}
	
void AbstractPropertyEditor::paintValue( GraphicsContext* context, const Rect& bounds )
{

}

void AbstractPropertyEditor::setSource( Object* source )
{
	m_source = source;
}

void AbstractPropertyEditor::setProperty( Property* property )
{
	m_property = property;
}

VariantData* AbstractPropertyEditor::getValue()
{
	/*VariantData* result = NULL;

	if ( NULL != m_property ){
		if ( NULL == m_source ){
			m_source = m_property->getSource();
		}
		if ( (NULL != this->m_property) && (NULL != m_source) ){
			result = m_property->get( m_source );
		}
	}
	*/
	return &m_data;
}

void AbstractPropertyEditor::setValue( VariantData* value )
{
	/*
	if ( NULL != m_property ){
		if ( NULL == m_source ){
			m_source = m_property->getSource();
		}
		if ( (NULL != this->m_property) && (NULL != m_source) ){
			m_property->set( m_source, value );
		}
	}
	*/
	m_data = *value;
}

bool AbstractPropertyEditor::hasCustomEditor()
{
	return false;
}
	
bool AbstractPropertyEditor::canPaintValue()
{
	return false;
}

String AbstractPropertyEditor::getValueAsText()
{
	/*
	String result = "null";
	if ( NULL != this->m_property ){
		VariantData* val = this->getValue();
		if ( NULL != val ){
			result = val->toString();
		}		
	}
	*/
	return m_data.toString();
}

void AbstractPropertyEditor::setValueAsText( const String& textValue )
{
	/*
	if ( NULL != m_property ){
		if ( NULL == m_source ){
			m_source = m_property->getSource();
		}
		if ( (NULL != this->m_property) && (NULL != m_source) ){
			m_property->set( m_source, textValue );
		}
	}
	*/
	m_data.setFromString( textValue );
}

String AbstractPropertyEditor::getCodeString()
{
	return "";
}


BoolPropertyEditor::BoolPropertyEditor()
{
	this->m_comboBoxAdapter = new ComboBoxAdapter( this );
	
	m_comboBoxAdapter->m_selectionChange = (OnComboBoxEvent)BoolPropertyEditor::onSelectionChange;
	this->addEventHandler( "ComboBoxAdapter", this->m_comboBoxAdapter );
}

BoolPropertyEditor::~BoolPropertyEditor()
{
	
}

Control* BoolPropertyEditor::getCustomEditor()
{
	m_combo = new ComboBoxControl();
	m_combo->addComboBoxListener( m_comboBoxAdapter );
	
	ListModel* lm = m_combo->getListModel();
	if ( NULL != lm ){
		lm->addItem( new DefaultListItem( lm, "true" ) );
		lm->addItem( new DefaultListItem( lm, "false" ) );		
	}

	bool val = *(this->getValue());
	ulong32 selectedIndex = (val) ? 0 : 1;
	m_combo->setSelectedItemIndex( selectedIndex );

	return 	m_combo;
}

void BoolPropertyEditor::onSelectionChange( ItemEvent* event )
{
	ListItem* item = (ListItem*)event->getSource();
	if ( NULL != item ){
		ulong32 selectedIndex = item->getIndex();
		this->m_data = (selectedIndex == 0) ? true : false;
	}
	
}

EnumPropertyEditor::EnumPropertyEditor()
{
	this->m_comboBoxAdapter = new ComboBoxAdapter( this );
	
	m_comboBoxAdapter->m_selectionChange = (OnComboBoxEvent)EnumPropertyEditor::onSelectionChange;
	this->addEventHandler( "ComboBoxAdapter", this->m_comboBoxAdapter );
}

EnumPropertyEditor::~EnumPropertyEditor()
{

}

Control* EnumPropertyEditor::getCustomEditor()
{
	m_combo = new ComboBoxControl();
	m_combo->addComboBoxListener( m_comboBoxAdapter );
	
	ListModel* lm = m_combo->getListModel();
	
	Enum* val = *(this->getValue());

	int current = val->get();

	int start = val->begin();
	String s1 = val->toString();
	int next = val->next();
	String s2 = val->toString();
	if ( NULL != lm ){
		lm->addItem( new DefaultListItem( lm, s1 ) );
		lm->addItem( new DefaultListItem( lm, s2 ) );
		while ( next != start ) {
			next = val->next();
			lm->addItem( new DefaultListItem( lm, val->toString() ) );
		}
	}	
	
	m_combo->setSelectedItemIndex( current );

	return 	m_combo;
}

void EnumPropertyEditor::onSelectionChange( ItemEvent* event )
{
	ListItem* item = (ListItem*)event->getSource();
	if ( NULL != item ){
		ulong32 selectedIndex = item->getIndex();
		Enum* val = m_data;
		val->set( selectedIndex );

		this->m_data = val;
	}
}



template <class T> class ModalPropertyEditorControl : public Panel {
public:
	typedef void (T::*ShowEditorDialog)( VariantData* data );

	ModalPropertyEditorControl( ShowEditorDialog showEditorMethod, VariantData* data, T* source ){
		m_showEditorMethod = showEditorMethod;
		m_data = data;
		m_source = source;

		CommandButton* btn = new CommandButton();
		
		ButtonHandler* handler = new ButtonHandler( this );
		handler->m_buttonClicked = (OnButtonEvent)onBtnClick;
		btn->addButtonListener( handler );
		this->addEventHandler( "ButtonClickHandler", handler );

		btn->setBounds( &Rect(0,0,16,16) );
		btn->setCaption( "..." );
		this->add( btn, ALIGN_RIGHT );		
		
		this->setColor( Color::getColor("white") );
		this->setBorder( NULL );
	}

	virtual ~ModalPropertyEditorControl(){};	

	void onBtnClick( ButtonEvent* event ){
		(m_source->*m_showEditorMethod)( m_data );
	}		
private:
	ShowEditorDialog m_showEditorMethod;
	VariantData* m_data;
	T* m_source;
};

class ColorPanel : public Panel {
public:
	ColorPanel(){
		
	}

	virtual ~ColorPanel(){

	}

};


ColorPropertyEditor::ColorPropertyEditor()
{

}

ColorPropertyEditor::~ColorPropertyEditor()
{

}

void ColorPropertyEditor::paintValue( GraphicsContext* context, const Rect& bounds )
{
	Color* c = (Color*)(Object*)(*(this->getValue()));
	Color* oldColor = context->getColor();
	
	Rect innerBds = bounds;
	
	innerBds.inflate( -2, -2 );
	context->setColor( Color::getColor("black") );
	context->rectangle( &innerBds );
	context->strokePath();

	innerBds.inflate( -3, -3 );
	context->setColor( c );
	context->rectangle( &innerBds );
	context->fillPath();
	context->setColor( oldColor );
}

Control* ColorPropertyEditor::getCustomEditor()
{	
	return new ModalPropertyEditorControl<ColorPropertyEditor>( 
			ColorPropertyEditor::showColorEditor, this->getValue(), this );
}

void ColorPropertyEditor::showColorEditor( VariantData* data )
{
	CommonColor clrDlg;
	Color* c = (Color*)(Object*)(*(this->getValue()));
	clrDlg.setSelectedColor( c );
	if ( true == clrDlg.execute() ){
		VariantData data;
		c->copy( clrDlg.getSelectedColor() );
		data = c;
		this->setValue( &data );
	}
}

FontPropertyEditor::FontPropertyEditor()
{

}

FontPropertyEditor::~FontPropertyEditor()
{

}

void FontPropertyEditor::paintValue( GraphicsContext* context, const Rect& bounds )
{
	Font* f = (Font*)(Object*)(*(this->getValue()));
	Font* oldFont = context->getCurrentFont();
	Color* oldColor = context->getColor();
	Rect innerBds = bounds;
	
	innerBds.inflate( -2, -2 );
	context->setColor( Color::getColor("black") );
	context->rectangle( &innerBds );
	context->strokePath();
	
	context->setColor( oldColor );

	innerBds.inflate( -3, 0 );
	//context->setCurrentFont( f );	
	context->textAt( innerBds.m_left + 2, innerBds.m_top, "ABC" );
	
	context->setCurrentFont( oldFont );	
}

Control* FontPropertyEditor::getCustomEditor()
{	
	return new ModalPropertyEditorControl<FontPropertyEditor>( 
			FontPropertyEditor::showFontEditor, this->getValue(), this );
}

void FontPropertyEditor::showFontEditor( VariantData* data )
{
	CommonFont fontDlg(NULL);
	Font* f = (Font*)(Object*)(*(this->getValue()));
	fontDlg.setSelectedFont( f );
	if ( true == fontDlg.execute() ){
		VariantData data;
		f->copy( fontDlg.getSelectedFont() );
		data = f;
		this->setValue( &data );
	}
}

DefaultMenuItemPropertyEditor::DefaultMenuItemPropertyEditor()
{

}

DefaultMenuItemPropertyEditor::~DefaultMenuItemPropertyEditor()
{

}

Control* DefaultMenuItemPropertyEditor::getCustomEditor()
{
	return new ModalPropertyEditorControl<DefaultMenuItemPropertyEditor>( 
			DefaultMenuItemPropertyEditor::showDefaultMenuItemEditor, this->getValue(), this );
}

void DefaultMenuItemPropertyEditor::showDefaultMenuItemEditor( VariantData* data )
{
	Dialog::showMessage( "DefaultMenuItemPropertyEditor::showDefaultMenuItemEditor( VariantData* data )" );
}

DefaultListModelPropertyEditor::DefaultListModelPropertyEditor()
{

}

DefaultListModelPropertyEditor::~DefaultListModelPropertyEditor()
{

}

Control* DefaultListModelPropertyEditor::getCustomEditor()
{
	return new ModalPropertyEditorControl<DefaultListModelPropertyEditor>( 
			DefaultListModelPropertyEditor::showDefaultListModelEditor, this->getValue(), this );
}


void DefaultListModelPropertyEditor::showDefaultListModelEditor( VariantData* data )
{
	Dialog* d = new Dialog();
	
	d->setBounds( &Rect(400,400,750,650) );
		
	Panel* contents = new Panel();

	Panel* bottom = new Panel();
	bottom->setBounds( &Rect(0,0,10,30) );
	d->add( bottom, ALIGN_BOTTOM );
	d->add( contents, ALIGN_CLIENT );
	contents->setBorder( NULL );
	bottom->setBorder( NULL );

	CommandButton* okBtn = new CommandButton();
	okBtn->setBounds( &Rect(160,4,240,26) );
	okBtn->setCaption( "OK" );
	okBtn->setCommandType( BC_OK );

	CommandButton* cancelBtn = new CommandButton();
	cancelBtn->setBounds( &Rect(252,4,332,26) );
	cancelBtn->setCaption( "Cancel" );
	cancelBtn->setCommandType( BC_CANCEL );
	bottom->add( okBtn );
	bottom->add( cancelBtn );
	
	d->setCaption( "Edit List Model" );
	
	Rect r = *contents->getBounds();
	r.inflate( -5,-5 );
	
	ListBoxControl* lbc = new ListBoxControl();
	lbc->setBounds( &Rect( 5, 5, 120, r.m_bottom ) );
	contents->add( lbc );
	
	ListModel* lm = lbc->getListModel();	

	Label* l = new Label();
	TextControl* tc = new TextControl();
	Rect dlgRect = *d->getBounds();
	tc->setBounds( &Rect( dlgRect.getWidth() - 120, 10, dlgRect.getWidth() - 5, 35 ) );
	l->setBounds( &Rect( tc->getLeft() - 80, 10, tc->getLeft(), 35 ) );
	l->setCaption( "Item Caption" );
	contents->add( l );
	contents->add( tc );

	CommandButton* addBtn = new CommandButton();
	Rect* tcRect = tc->getBounds();
	addBtn->setBounds( &Rect( tc->getLeft(), tcRect->getBottom() + 10, tc->getLeft() + 80, tcRect->getBottom() + 35 ) );
	addBtn->setCaption( "Add Item" );

	tcRect = addBtn->getBounds();
	CommandButton* removeBtn = new CommandButton();
	removeBtn->setBounds( &Rect( tc->getLeft(), tcRect->getBottom() + 10, tc->getLeft() + 80, tcRect->getBottom() + 35 ) );
	removeBtn->setCaption( "Remove Item" );		

	contents->add( addBtn );
	contents->add( removeBtn );

	d->showModal();

	delete d;
	d = NULL;
}