/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "DefaultTabModel.h"
#include <algorithm>


using namespace VCF;

DefaultTabModel::DefaultTabModel()
{
	m_container.initContainer( this->m_pages );
}

DefaultTabModel::~DefaultTabModel()
{
	this->clearTabPages();
}
  
void DefaultTabModel::addTabPage( TabPage* page )
{
	TabModelEvent event( this, TAB_MODEL_EVENT_ITEM_ADDED, page );
	notify_onTabPageAdded( &event );
	m_pages.push_back( page );
}

void DefaultTabModel::insertTabPage( const ulong32& index, TabPage* page )
{
	TabModelEvent event( this, TAB_MODEL_EVENT_ITEM_ADDED, page );
	notify_onTabPageAdded( &event );
	m_pages.insert( m_pages.begin() + index, page );
}
	
void DefaultTabModel::deleteTabPage( TabPage* page )
{
	TabModelEvent event( this, TAB_MODEL_EVENT_ITEM_REMOVED, page );
	notify_onTabPageRemoved( &event );
	std::vector<TabPage*>::iterator found = std::find( m_pages.begin(), m_pages.end(), page );
	if ( found != m_pages.end() ){
		m_pages.erase( found );
		//clean up memory
		delete *found;
	}
}

void DefaultTabModel::deleteTabPage( const ulong32& index )
{
	std::vector<TabPage*>::iterator found = m_pages.begin() + index;
	if ( found != m_pages.end() ){
		TabModelEvent event( this, TAB_MODEL_EVENT_ITEM_REMOVED, *found );
		notify_onTabPageRemoved( &event );
		m_pages.erase( found );
		//clean up memory
		delete *found;
	}	
}

void DefaultTabModel::clearTabPages()
{
	std::vector<TabPage*>::iterator it = m_pages.begin();
	while ( it != m_pages.end() ){
		TabPage* page = *it;
		delete page;
		page = NULL;
		it++;
	}
	m_pages.clear();
}

TabPage* DefaultTabModel::getPageAt( const ulong32& index )
{
	TabPage* result = NULL;
	std::vector<TabPage*>::iterator it = m_pages.begin() + index;
	if ( it != m_pages.end() ){
		result = *it;
	}
	return result;
}

TabPage* DefaultTabModel::getSelectedPage()
{
	TabPage* result = NULL;
	std::vector<TabPage*>::iterator it = m_pages.begin();
	while ( it != m_pages.end() ){
		TabPage* page = *it;
		if ( NULL != page ){
			if ( true == page->isSelected() ){
				result = page;
				break;
			}
		}	
		it++;	
	}
	return result;
}

void DefaultTabModel::setSelectedPage( TabPage* page )
{	
	std::vector<TabPage*>::iterator it = m_pages.begin();
	while ( it != m_pages.end() ){
		TabPage* aPage = *it;
		if ( NULL != page ){
			aPage->setSelected( false );
		}	
		it++;	
	}
	page->setSelected( true );
	TabModelEvent event( this, TAB_MODEL_EVENT_ITEM_SELECTED, page );
	notify_onTabPageSelected( &event );
}

void DefaultTabModel::setSelectedPage( const ulong32& index )
{
	std::vector<TabPage*>::iterator it = m_pages.begin();
	if ( it != m_pages.end() ){
		TabPage* page = *it;
		setSelectedPage( page );
	}
}

Enumerator<TabPage*>* DefaultTabModel::getPages()
{
	return this->m_container.getEnumerator();
}