/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "DefaultTabPage.h"
#include "TabModel.h"
#include "UIToolkit.h"
#include "Color.h"
#include "GraphicsContext.h"
#include "Font.h"


using namespace VCF;

DefaultTabPage::DefaultTabPage( Control* component )
{
	m_preferredHeight = 12;
	this->m_component = NULL;
	this->setPageComponent( component );
}

DefaultTabPage::~DefaultTabPage()
{

}

bool DefaultTabPage::containsPoint( Point * pt )
{
	return this->m_bounds.containsPt( pt );
}

Image* DefaultTabPage::getImage()
{
	return NULL;
}

unsigned long DefaultTabPage::getIndex()
{
	return this->m_index;
}

void DefaultTabPage::setIndex( const unsigned long& index )
{
	m_index = index;
}

void* DefaultTabPage::getData()
{
	return NULL;
}

void DefaultTabPage::setData( void* data )
{
	//do nothing for now;
}

Model* DefaultTabPage::getModel()
{
	return m_model;
}

void DefaultTabPage::setModel( Model* model )
{
	m_model = model;
}

void DefaultTabPage::setPageName( const String& name )
{
	m_pageName = name;
}

String DefaultTabPage::getPageName()
{
	return m_pageName;
}

Control* DefaultTabPage::getPageComponent()
{
	return m_component;
}

void DefaultTabPage::setPageComponent( Control* component )
{
	if ( NULL != m_component ){
		delete m_component;
		m_component = NULL;
	}
	m_component = component;
	
}

bool DefaultTabPage::isSelected()
{
	return m_selected;
}

void DefaultTabPage::setSelected( const bool& selected )
{
	m_selected = selected;
	ItemEvent event( this, ITEM_EVENT_SELECTED );
	this->notify_onItemSelected( &event );	
}

void DefaultTabPage::paint( GraphicsContext* context, Rect* paintRect )
{
	Color* hilite = UIToolkit::getDefaultUIToolkit()->getSystemColor(SYSCOLOR_HIGHLIGHT);
	Color* shadow = UIToolkit::getDefaultUIToolkit()->getSystemColor(SYSCOLOR_SHADOW);
	if ( !this->isSelected() ){
		context->setColor( shadow );
		std::vector<Point*> pts;
		pts.push_back( new Point(paintRect->m_left, paintRect->m_top) );
		pts.push_back( new Point(paintRect->m_left + ((paintRect->getWidth()) - (paintRect->getHeight())), paintRect->m_top) );
		pts.push_back( new Point(paintRect->m_right, paintRect->m_bottom) );
		pts.push_back( new Point(paintRect->m_left, paintRect->m_bottom) );
		pts.push_back( new Point(paintRect->m_left, paintRect->m_top) );
		context->polyline( pts );
		context->fillPath();
		for (std::vector<Point*>::iterator it=pts.begin();it!=pts.end();it++){
			delete *it;
		}
	}

	context->setColor( &Color(0.0f,0.0f,0.0f) );

	context->moveTo( paintRect->m_left, paintRect->m_top );
	context->lineTo( paintRect->m_left + (paintRect->getWidth() - paintRect->getHeight()) , paintRect->m_top );
	context->lineTo( paintRect->m_right, paintRect->m_bottom );
	context->moveTo( paintRect->m_left, paintRect->m_bottom );
	context->lineTo( paintRect->m_left, paintRect->m_top );	
	context->strokePath();

	context->setColor( hilite );
	context->moveTo( paintRect->m_left+1, paintRect->m_top+1 );
	context->lineTo( paintRect->m_left + ((paintRect->getWidth()-2) - (paintRect->getHeight()-2)), paintRect->m_top+1 );
	context->lineTo( paintRect->m_right-2, paintRect->m_bottom-1 );
	context->moveTo( paintRect->m_left+1, paintRect->m_bottom-1 );
	context->lineTo( paintRect->m_left+1, paintRect->m_top+1 );	
	context->strokePath();

	//context->rectangle( paintRect->m_left, paintRect->m_top, paintRect->m_right, paintRect->m_bottom );
	Font textFnt;
	textFnt.setPixelSize( 12 );
	context->setCurrentFont( &textFnt );
	context->textAt( paintRect->m_left + 2, paintRect->m_top+2, this->m_pageName );

	this->m_bounds.setRect( paintRect->m_left, paintRect->m_top, paintRect->m_right, paintRect->m_bottom );
}

ulong32 DefaultTabPage::getPreferredHeight()
{
	return m_preferredHeight;	
}