/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// DefaultTreeItem.cpp


#include "DefaultTreeItem.h"
#include "StringUtils.h"
#include "Color.h"


using namespace VCF;


DefaultTreeItem::DefaultTreeItem( const String& caption, TreeModel* model )
{
	this->init();
	this->m_treeModel = model;
	setCaption( caption );
}

DefaultTreeItem::~DefaultTreeItem()
{
	std::vector<TreeItem*>::iterator it = m_childNodeItems.begin();
	while ( it != m_childNodeItems.end() ){
		delete *it;
		it++;
	}
}

void DefaultTreeItem::init()
{
	

	this->m_caption = "";
	this->m_childNodeItems.clear();
	this->m_image = NULL;
	this->m_parent = NULL;
	this->m_textBold = false;
	this->m_TextColor.setRGB( 0.0, 0.0, 0.0 );
	this->m_userData = NULL;
	m_enumContainer.initContainer( m_childNodeItems );
}

bool DefaultTreeItem::containsPoint( Point * pt )
{
	return false;
}

Image* DefaultTreeItem::getImage()
{
	return m_image;
}

unsigned long DefaultTreeItem::getIndex()
{
	return this->m_index;
}

void DefaultTreeItem::setIndex( const unsigned long& index )
{
	m_index = index;
}

void* DefaultTreeItem::getData()
{
	return m_userData;
}

void DefaultTreeItem::setData( void* data )
{
	m_userData = data;
	ItemEvent event( this );
	notify_onItemChanged( &event );
}

Color* DefaultTreeItem::getTextColor()
{
	return &this->m_TextColor;
}

void DefaultTreeItem::setTextColor(Color* color)
{
	this->m_TextColor = *color;
	ItemEvent event( this );
	notify_onItemChanged( &event );
}

void DefaultTreeItem::setTextBold(const bool& bold)
{
	this->m_textBold = bold;
	ItemEvent event( this );
	notify_onItemChanged( &event );
}

bool DefaultTreeItem::getTextBold()
{
	return this->m_textBold;
}

bool DefaultTreeItem::isRoot()
{
	return ( NULL == m_parent );
}

bool DefaultTreeItem::isLeaf()
{
	return  (0 == m_childNodeItems.size());
}

TreeItem* DefaultTreeItem::getParent()
{
	return m_parent;
}

void DefaultTreeItem::setParent( TreeItem* parent )
{
	this->m_parent = parent;
	ItemEvent event( this );
	notify_onItemChanged( &event );
}

TreeItem* DefaultTreeItem::getNextChildNodeItem()
{
	return NULL;
}

TreeItem* DefaultTreeItem::getPrevChildNodeItem()
{
	return NULL;
}

String DefaultTreeItem::getCaption()
{
	return this->m_caption;	
}

void DefaultTreeItem::setCaption( const String& caption )
{
	StringUtils::trace( "caption = " + caption + "\n" );
	this->m_caption = caption;
	ItemEvent event( this, ITEM_EVENT_TEXT_CHANGED );
	notify_onItemChanged( &event );
}

Enumerator<TreeItem*>* DefaultTreeItem::getChildren()
{
	return this->m_enumContainer.getEnumerator();
}

void DefaultTreeItem::addChild( TreeItem* child )
{
	StringUtils::trace( "Adding child " + child->getCaption() + "\n" );
	this->m_childNodeItems.push_back( child );
	child->setParent( this );
	ItemEvent event( this );
	notify_onItemAdded( &event );
}

void DefaultTreeItem::insertChild( const unsigned long& index, TreeItem* child )
{
	std::vector<TreeItem*>::iterator it = m_childNodeItems.begin();
	this->m_childNodeItems.insert( it + index, child );
	child->setParent( this );
	ItemEvent event( this );
	notify_onItemAdded( &event );
}
	
void DefaultTreeItem::deleteChild( TreeItem* child )
{

}

void DefaultTreeItem::deleteChild( const unsigned long& index )
{

}

void DefaultTreeItem::clearChildren()
{
	for (std::vector<TreeItem*>::iterator it = m_childNodeItems.begin(); it != m_childNodeItems.end(); it ++ ){
		(*it)->clearChildren();
		delete (*it);
		it++;
	}
	this->m_childNodeItems.clear();

	ItemEvent event( this );
	notify_onItemChanged( &event );
}

Model* DefaultTreeItem::getModel()
{
	return this->m_treeModel;
}

void DefaultTreeItem::setModel( Model* model )
{
	this->m_treeModel = dynamic_cast<TreeModel*>( model );
}

void DefaultTreeItem::paint( GraphicsContext* context, Rect* paintRect )
{
	
}
	
bool DefaultTreeItem::isSelected()
{
	return m_selected;
}

void DefaultTreeItem::setSelected( const bool& selected )
{
	m_selected = selected;
}