/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Dialog.cpp

#include "Dialog.h"
#include "UIToolkit.h"
#include "ClassRegistry.h"
#include "Application.h"
#include "InvalidPeer.h"
#include "DialogPeer.h"
#include "Rect.h"


using namespace VCF;

Dialog::Dialog( Frame* owner )
{		
	this->m_Peer = UIToolkit::getDefaultUIToolkit()->createDialogPeer( owner, this );
	if ( NULL == m_Peer ){
		throw InvalidPeer( MAKE_ERROR_MSG(NO_PEER), __LINE__ );
	}
	m_Peer->setControl( this );
	this->m_dialogPeer = (DialogPeer*)m_Peer;
}


Dialog::~Dialog() 
{

}

void Dialog::paint(GraphicsContext * context)
{
	Frame::paint( context );
}

void Dialog::setCaption( const String& caption )
{
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	this->m_caption = caption;
	this->m_dialogPeer->setCaption( m_caption );
}

Rect* Dialog::getClientBounds()
{
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	return this->m_dialogPeer->getClientBounds();
}

void  Dialog::setClientBounds( Rect* bounds )
{
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	m_dialogPeer->setClientBounds( bounds );
}

void Dialog::resizeChildren()
{
	if ( true == alignWork() ){
		Rect* bounds = getClientBounds();
		Rect rect( 0.0, 0.0, bounds->getWidth(), bounds->getHeight() );
		this->doAlign( ALIGN_TOP, &rect );
		this->doAlign( ALIGN_BOTTOM, &rect );
		this->doAlign( ALIGN_LEFT, &rect );
		this->doAlign( ALIGN_RIGHT, &rect );
		this->doAlign( ALIGN_CLIENT, &rect );
	}
}

ModalReturnType Dialog::showModal()
{
	this->setVisible( true );
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	
	return m_dialogPeer->showModal();
}

void Dialog::show()
{
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}

	m_dialogPeer->show();
}


void Dialog::showMessage( const String& message, const String& caption )
{
	DialogPeer* dialogPeer = UIToolkit::getDefaultUIToolkit()->createDialogPeer();// owner, this );
	if ( NULL != dialogPeer ){
		dialogPeer->showMessage( message, caption );
	}

	delete dialogPeer;
	dialogPeer = NULL;
}

ModalReturnType Dialog::showMessage( const String& message, const MessageStyle& messageStyle, const String& caption )
{
	ModalReturnType result = MRT_OK;
	
	DialogPeer* dialogPeer = UIToolkit::getDefaultUIToolkit()->createDialogPeer();// owner, this );
	if ( NULL != dialogPeer ){
		result = dialogPeer->showMessage( message, caption, messageStyle );
	}

	delete dialogPeer;
	dialogPeer = NULL;

	return result;
}

void Dialog::close()
{
	if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}

	m_dialogPeer->close();
}

void Dialog::setFrameStyle( const FrameStyleType& frameStyle )
{
		if ( NULL == m_dialogPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	m_frameStyle = frameStyle;

	if ( CS_DESIGNING != this->getComponentState() ){
		m_dialogPeer->setFrameStyle( m_frameStyle );
	}
	
}