/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Frame.cpp

#include "Frame.h"
#include "Rect.h"
#include "GraphicsContext.h"
#include "InvalidPeer.h"

using namespace VCF;


Frame::Frame()
{
	this->setContainer( this );
	m_isTopmostFrame = false;
	m_frameStyle = FST_SIZEABLE;
}

void Frame::paint( GraphicsContext * context )
{
	Rect paintRect( 0.0, 0.0, this->getWidth(), this->getHeight() );
	Color* fillColor = this->getColor();
	context->setColor( fillColor );
	context->rectangle( &paintRect );
	context->fillPath();
	
	this->paintChildren( context );
}

void Frame::show()
{
	this->setVisible( true );
}

void Frame::hide()
{
	this->setVisible( false );
}

void Frame::setCaption( const String& caption )
{
	this->m_caption = caption;
}

String Frame::getCaption()
{
	return this->m_caption;
}

String Frame::toString()
{
	String result = "";
	ComponentDTD dtd;
	result += dtd.toString() + "\n";
	result += "<object class=\"" + this->getClassName() + "\"> \n";

	Class* clazz = this->getClass();
	if ( NULL != clazz ){
		Enumerator<Property*>* props =  clazz->getProperties();		
		
		while ( props->hasMoreElements() ){
			Property* p = props->nextElement();
			if ( NULL != p ){
				result += "\t" + p->toString() + "\n";
			}
		}
	}

	//now dump out all the components
	result += "<children> \n";
	std::vector<Control*>::iterator it = m_controls.begin();
	while ( it != m_controls.end() ){
		Control* component = *it;
		if ( NULL != component ){
			result += component->toString() + "\n";
		}
		it++;
	}
	result += "</children> \n";
	result += "</object>";
	//m_components
	return result;
}

bool Frame::isFrameTopmost()
{
	return m_isTopmostFrame;
}

void Frame::setFrameTopmost( const bool& isTopmost )
{
	m_isTopmostFrame = isTopmost;
}
