/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#include "ListBoxControl.h"
#include "ListModel.h"
#include "UIToolkit.h"
#include "DefaultListModel.h"
#include "ListItem.h"
#include "ListModelEvent.h"
#include "ListBoxPeer.h"
#include "InvalidPeer.h"

using namespace VCF;

ListBoxControl::ListBoxControl()
{
	m_Peer =	UIToolkit::getDefaultUIToolkit()->createListboxPeer( this );
	m_listBoxPeer = dynamic_cast<ListBoxPeer*>( m_Peer );	

	if ( NULL == this->m_listBoxPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	m_listModel = new DefaultListModel();
	m_modelHandler = new ListModelHandler( this );
	m_modelHandler->m_itemAdded = (OnListModelEvent)this->onItemAdded;
	m_modelHandler->m_itemDeleted = (OnListModelEvent)this->onItemDeleted;
	m_modelHandler->m_listModelContentsChanged = (OnListModelEvent)this->onListModelContentsChanged;
	m_listModel->addListModelListener( m_modelHandler );
}

ListBoxControl::~ListBoxControl()
{
	delete m_listModel;
	delete m_modelHandler;
}

ListModel* ListBoxControl::getListModel()
{
	return m_listModel;
}

void ListBoxControl::setListModel( ListModel * model )
{
	m_listModel = model;
}

void ListBoxControl::onListModelContentsChanged( ListModelEvent* event )
{
	if ( NULL == this->m_listBoxPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	if ( NULL != event ){
		switch ( event->getType() ){
			case LIST_MODEL_CONTENTS_DELETED: {
				
			}
			break;

			case LIST_MODEL_ITEM_CHANGED: {
				ListItem* item = event->getListItem();
				if ( NULL != item ){
					m_listBoxPeer->removeItem( item );
					m_listBoxPeer->addItem( item );
				}
			}
			break;		
		}
	}
}

void ListBoxControl::onItemAdded( ListModelEvent* event )
{
	if ( NULL == this->m_listBoxPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	
	m_listBoxPeer->addItem( event->getListItem() );
}

void ListBoxControl::onItemDeleted( ListModelEvent* event )
{
	if ( NULL == this->m_listBoxPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
}

void ListBoxControl::rangeSelect( const bool & isSelected, ListItem * first, ListItem * last )
{
	if ( NULL == this->m_listBoxPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listBoxPeer->rangeSelect( isSelected, first, last );
}