/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//ListViewControl.cpp

#include "ListViewControl.h"
#include "UIToolkit.h"
#include "DefaultListModel.h"
#include "ClassRegistry.h"
#include "ListviewPeer.h"
#include "InvalidPeer.h"


using namespace VCF;


ListViewControl::ListViewControl()
{
	m_Peer =	UIToolkit::getDefaultUIToolkit()->createListViewPeer( this );
	
	m_listviewPeer = dynamic_cast<ListviewPeer*>( m_Peer );	

	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	m_listModel = new DefaultListModel();
	m_modelHandler = new ListModelHandler( this );
	m_modelHandler->m_itemAdded = (OnListModelEvent)this->onItemAdded;
	m_modelHandler->m_itemDeleted = (OnListModelEvent)this->onItemDeleted;
	m_modelHandler->m_listModelContentsChanged = (OnListModelEvent)this->onListModelContentsChanged;
	m_listModel->addListModelListener( m_modelHandler );
	init();
}


ListViewControl::~ListViewControl()
{
	delete m_listModel;
	delete m_modelHandler;
}

void ListViewControl::init()
{
	m_iconStyle = IS_LARGE_ICON;
}

ListModel* ListViewControl::getListModel()
{
	return m_listModel;
}

void ListViewControl::setListModel(ListModel * model)
{
	m_listModel = model;
}

void ListViewControl::onListModelContentsChanged( ListModelEvent* event )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	if ( NULL != event ){
		switch ( event->getType() ){
			case LIST_MODEL_CONTENTS_DELETED: {
				
			}
			break;

			case LIST_MODEL_ITEM_CHANGED: {
				ListItem* item = event->getListItem();
				if ( NULL != item ){
					m_listviewPeer->deleteItem( item );
					m_listviewPeer->insertItem( item->getIndex(), item );
				}
			}
			break; 
		}
	}
}

void ListViewControl::onItemAdded( ListModelEvent* event )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};

	m_listviewPeer->addItem( event->getListItem() );
}

void ListViewControl::onItemDeleted( ListModelEvent* event )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
}

void ListViewControl::addHeaderColumn( const String& columnName )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->addHeaderColumn( columnName );
}

void ListViewControl::insertHeaderColumn( const unsigned long& index, const String& columnName )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->insertHeaderColumn( index, columnName );
}

void ListViewControl::deleteHeaderColumn( const unsigned long& index )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->deleteHeaderColumn( index );
}

IconStyleType ListViewControl::getIconStyle()
{
	//if ( NULL == this->m_listviewPeer ){
	//	throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	//};
	return m_iconStyle;// m_listviewPeer->getIconStyle();
}

void ListViewControl::setIconStyle( const IconStyleType& iconStyle )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_iconStyle = iconStyle;
	m_listviewPeer->setIconStyle( m_iconStyle );
}

bool ListViewControl::getAllowsMultiSelect()
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return m_listviewPeer->getAllowsMultiSelect();
}

void ListViewControl::setAllowsMultiSelect( const bool& allowsMultiSelect )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->setAllowsMultiSelect( allowsMultiSelect );
}

IconAlignType ListViewControl::getIconAlignment()
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return m_listviewPeer->getIconAlignment();
}

void ListViewControl::setIconAlignment( const IconAlignType& iconAlignType )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->setIconAlignment( iconAlignType );
}

bool ListViewControl::getAllowLabelEditing()
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	return m_listviewPeer->getAllowLabelEditing();
}

void ListViewControl::setAllowLabelEditing( const bool& allowLabelEditing )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->setAllowLabelEditing( allowLabelEditing );
}

void ListViewControl::setItemFocused( ListItem* item )
{
	if ( NULL == this->m_listviewPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	};
	m_listviewPeer->setFocusedItem( item );
}
