/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "MenuBar.h"
#include "UIToolkit.h"
#include "DefaultMenuItem.h"
#include "InvalidPeer.h"


using namespace VCF;

MenuBar::MenuBar()
{
	this->init();
	UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();
	if ( NULL != toolkit ){
		m_Peer = toolkit->createMenuBarPeer( this );
	}
	else {
		//throw exception
	}

	if ( NULL == m_Peer ){
		//throw exception
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
}

MenuBar::~MenuBar()
{
	if ( NULL != m_Peer ){
		delete m_Peer;
		m_Peer = NULL;
	}
}

void MenuBar::init()
{
	m_Peer = NULL;
	m_itemHandler = NULL;
	m_itemHandler = new ItemHandler( this );
	m_itemHandler->m_itemAdded = (OnItemEvent)onMenutItemAdded;
	m_itemHandler->m_itemDeleted = (OnItemEvent)onMenutItemDeleted;

	this->setRootMenuItem( new DefaultMenuItem() );
	MenuItem* item = this->getRootMenuItem();
	item->addItemListener( m_itemHandler );

	this->addEventHandler( "MenuBarHandler", m_itemHandler );
}

void MenuBar::onMenutItemAdded( ItemEvent* event )
{
	m_Peer->update();
}

void MenuBar::onMenutItemDeleted( ItemEvent* event )
{
	m_Peer->update();
}

void MenuBar::setFrame( Frame* frame )
{
	if ( NULL == m_Peer ){
		//throw exception
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	m_Peer->setFrame( frame );
}