/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#include "Object.h"

#pragma warning (disable : 4786)
#include <iostream>
#include <fstream>

#include "Class.h"
#include "ClassRegistry.h"
#include <algorithm>
#include "Listener.h"
#include "StringUtils.h"

using namespace std;
using namespace VCF;

#ifdef USE_VCF_NEW
std::list<void*> Object::allocatedObjects;
#endif

Object::Object()
{
	
	
}

Object::~Object()
{	
	//destroy all the event handlers
	std::map<String,Listener*>::iterator it = m_eventHandlers.begin();
	while ( it != m_eventHandlers.end() ){
		if ( NULL != it->second ){
			delete it->second;
		}
		it++;
	}
	m_eventHandlers.clear();
}

void Object::init()
{

}

Class* Object::getClass()
{
	ClassRegistry* registry = ClassRegistry::getClassRegistry();
	Class* result = (Class*) registry->getClass( this );
	if ( NULL != result ){
		result->setSource( this );
	}
	return result;
}

String Object::getClassName()
{
	String result = "";
	Class* clazz = this->getClass();
	if ( NULL != clazz ){
		result = clazz->getClassName();
	}
	else {
		
#ifdef WIN32 
		std::string tmp = typeid(*this).name();  //put back in when we find typeid 
		tmp = tmp.substr( 6 );
		
		
#ifndef KEEP_NAMESPACE_IN_CLASSNAME
		
		std::string::size_type idx = tmp.find( "::" );
		if ( idx == tmp.npos ) {
			result = tmp;  // not part of a namespace
		} else {
			result = tmp.substr( idx + 2 );  // strip namespace off from string
		}
		
#else
		
		result = tmp;
		
#endif	
		
#else //not supported in GCC compiler yet
		result = "unknown class instance.";
#endif
	}

	return result;
}

String Object::toString()
{
	String result = "";
	char info[256];
	sprintf( info, "%s @ %p", 
		     this->getClassName().c_str(), this );
	result += info;
	return result;
}

void Object::addEventHandler( const String& handlerName, Listener* handler )
{
	m_eventHandlers[handlerName] = handler;
}

Listener* Object::getEventHandler( const String& handlerName )
{
	Listener* result = NULL;
	std::map<String,Listener*>::iterator found = m_eventHandlers.find( handlerName );
	if ( found != m_eventHandlers.end() ){
		result = found->second;
	}
	return result;
}

#ifdef USE_VCF_NEW

int totalCount = 0;

void * Object::operator new( size_t allocSize )
{
    totalCount++;
	void *newPtr = malloc( allocSize );
    if( newPtr != 0 ){
        memset( newPtr, 0, allocSize );
		allocatedObjects.push_back( newPtr );
	}
    return newPtr;
}

void Object::operator delete( void *memPtr )
{
	
	std::list<void*>::iterator it = std::find( allocatedObjects.begin(), allocatedObjects.end(), memPtr );
    if ( it != allocatedObjects.end() ){
		allocatedObjects.erase( it );
	}
	free( memPtr );
}

void Object::dump()
{
	std::list<void*>::iterator it = allocatedObjects.begin();
	StringUtils::trace( "Dumping remaining allocated Objects...\n" );
	StringUtils::trace( StringUtils::toString( totalCount ) + " objects allocated\n" );
	long i = 0;
	Object* obj = NULL;
	while ( it != allocatedObjects.end() ){
		obj = (Object*)(*it);
		try {
			StringUtils::trace( "object " + StringUtils::toString( i ) + "\n" );// + obj->toString() + "\n" );
		}
		catch (... ){
			StringUtils::trace( "unknown object...\n" );
		}
		i++;
		it++;
	}
	StringUtils::trace( "Finished dumping objects ! "  + StringUtils::toString( i ) + " objects still allocated\n" );
}

#endif 

#ifdef WIN32

/*
static std::ofstream _improv_cerr;
static std::ofstream _improv_cout;
static std::ofstream _improv_tmp;
namespace VCFUtility {    

    
    static std::ostream& _redirect(std::ofstream& ofstr,
                           std::ostream& str,
                           std::string& fname) {
        ofstr.open(fname.c_str());
    
        if (!ofstr) {
            std::cerr << "cannot redirect standard error: "
                      <<  fname << " cannot be opened" << std::endl;
            return std::cerr;
        } else {
            str = ofstr;
            return ofstr;
        }
    }

    std::ostream& redirectCerr(std::string& fname) {
        return _redirect(_improv_cerr, std::cerr, fname);
    }

    std::ostream& redirectCout(std::string& fname) {
        return _redirect(_improv_cout, std::cout, fname);
    }

    std::ostream& redirect(ostream& str, std::string& fname) {
        return _redirect(_improv_tmp, str, fname);
    }
    

};
*/
#endif