/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#pragma warning (disable : 4786)
#include "TableControl.h"
#include "DefaultTableModel.h"
#include "Rect.h"
#include "TableCellItem.h"
#include "ItemListener.h"
#include "TableItemEditor.h"

using namespace VCF;



TableControl::TableControl( TableModel* model ):
	CustomControl( true )
{
	m_model = model;
	init();
}

TableControl::~TableControl()
{
	if ( NULL != this->m_model ){
		delete m_model;
		m_model = NULL;
	}
}

void TableControl::paint( GraphicsContext * context )
{
	CustomControl::paint( context );
	

	uint32 rowCount = this->m_model->getRowCount();
	Rect itemRect;
	double x = 0;	
	double y = 0;
	int j = 0;
	for ( int i=0;i<rowCount;i++){
		TableRowItemEnumerator* rowItemEnum = m_model->getRowItemEnumerator( i );
		j = 0;
		x = 0;
		while ( rowItemEnum->hasMoreElements() ){
			TableCellItem* item = rowItemEnum->nextElement();
			if ( NULL != item ){
				itemRect.setRect( x, y, x + this->m_columnWidths[j], y + this->m_rowHeights[i] );
				item->paint( context, &itemRect );
			}
			x += this->m_columnWidths[j];
			j++;				
		}
		y += this->m_rowHeights[i];	
	}

	this->paintChildren( context );
}

void TableControl::init()
{
	m_autoResizeColumns = true;
	m_selectedCellItem = NULL;
	m_tabelCellHandler = NULL;
	m_currentEditingControl = NULL;
	m_currentItemEditor = NULL;
	this->setContainer( this );	
	this->m_defaultColumnWidth = DEFAULT_COLUMN_WIDTH;
	this->m_defaultRowHeight = DEFAULT_ROW_HEIGHT;
	if ( NULL == m_model ){
		m_model = new DefaultTableModel();
	}
	TableModelAdapter* tma = new TableModelAdapter( this, (OnTableChanged)onTableModelChanged );
	this->addEventHandler( "TableModelHandler", tma );
	m_model->addTableModelListener( tma );

	m_tabelCellHandler = new ItemHandler( this );
	m_tabelCellHandler->m_itemSelected = (OnItemEvent)onTableCellItemSelected;
	this->addEventHandler( "TableCellHandler", m_tabelCellHandler );

	MouseHandler* mh = new MouseHandler( this );
	mh->m_mouseDoubleClicked = (OnMouseEvent)OnMouseDblClick;
	this->addEventHandler( "TableMouseHandler", mh );
	this->addMouseListener( mh );
}

void TableControl::onTableModelChanged( TableModelEvent* event )
{
	if ( NULL != m_currentEditingControl ){		
		this->remove( m_currentEditingControl );
		delete m_currentEditingControl;		
	}
	m_currentEditingControl = NULL;
	m_currentItemEditor = NULL;
	switch ( event->getType() ){
		case COLUMN_DELETED:{
			
		}
		break;

		case COLUMN_ADDED:{
			this->m_columnWidths.insert( m_columnWidths.begin() + event->getColumnThatChanged(), 
				                         this->m_defaultColumnWidth );

			uint32 rowCount = this->m_model->getRowCount();
			for ( int i=0;i<rowCount;i++){
				Item* item = this->m_model->getItem( i, event->getColumnThatChanged());
				if ( NULL != item ){
					item->addItemListener( this->m_tabelCellHandler );
				}
			}
			
		}
		break;

		case ROW_DELETED:{
			
		}
		break;

		case ROW_ADDED:{
			this->m_rowHeights.insert( m_rowHeights.begin() + event->getRowThatChanged(),
				                       this->m_defaultRowHeight ); 

			TableRowItemEnumerator* rowItems = this->m_model->getRowItemEnumerator( event->getRowThatChanged() );
			if ( NULL != rowItems ){
				while ( true == rowItems->hasMoreElements() ){
					Item* item = rowItems->nextElement();
					if ( NULL != item ){
						item->addItemListener( this->m_tabelCellHandler );
					}	
				}
			}

		}
		break;

		case ALL_COLUMNS_CHANGED:{
			
		}
		break;

		case ALL_ROWS_CHANGED:{
			
		}
		break;
	}
}

uint32 TableControl::getDefaultRowHeight()
{
	return m_defaultRowHeight;
}

void TableControl::setDefaultRowHeight( const uint32& defaultRowHeight )
{
	this->m_defaultRowHeight = defaultRowHeight;
}

uint32 TableControl::getDefaultColumnWidth()
{
	return m_defaultColumnWidth;
}

void TableControl::setDefaultColumnWidth( const uint32& defaultColumnWidth )
{
	this->m_defaultColumnWidth = defaultColumnWidth;
}

void TableControl::setFixedColumn( const bool& isFixed, const uint32& col )
{
	if ( NULL != this->m_model ){
		int rowCount = this->m_model->getRowCount();
		for (int i=0;i<rowCount;i++){
			TableCellItem* item = m_model->getItem( i, col );			
			if ( NULL != item ){
				item->setFixed( isFixed );
			}
		}		
	}
}

void TableControl::setFixedRow( const bool& isFixed, const uint32& row )
{
	if ( NULL != this->m_model ){
		int colCount = this->m_model->getColumnCount();
		for (int i=0;i<colCount;i++){
			TableCellItem* item = m_model->getItem( row, i );			
			if ( NULL != item ){
				item->setFixed( isFixed );
			}
		}		
	}
}

TableModel* TableControl::getTableModel()
{
	return m_model;
}

void TableControl::setTableModel( TableModel* model )
{
	if ( NULL != m_model ){
		delete m_model;
		m_model = NULL;
	}
	m_model = model;
}

void TableControl::setColumnCount( const uint32& colCount )
{
	
}

void TableControl::setRowCount( const uint32& rowCount )
{

}

void TableControl::onTableCellItemSelected( ItemEvent* event )
{	
	if ( NULL != this->m_selectedCellItem ){
		if ( event->getSource() != m_selectedCellItem ){
			this->m_selectedCellItem->setSelected( false );
			
			this->m_selectedCellItem = dynamic_cast<TableCellItem*>(event->getSource());	
		}		
	}
	else {
		this->m_selectedCellItem = dynamic_cast<TableCellItem*>(event->getSource());
	}

	this->repaint();
}

void TableControl::mouseDown( MouseEvent* event ){
	CustomControl::mouseDown( event );	
	
	if ( NULL != m_currentEditingControl ){
		if ( NULL != m_currentItemEditor ){
			this->m_currentItemEditor->updateItem();
		}
		this->remove( m_currentEditingControl );
		delete m_currentEditingControl;
		m_currentEditingControl = NULL;
		m_currentItemEditor = NULL;
	}

	uint32 rowCount = this->m_model->getRowCount();
	for ( int i=0;i<rowCount;i++){
		TableRowItemEnumerator* rowItems = this->m_model->getRowItemEnumerator( i );
		if ( NULL != rowItems ){
			while ( true == rowItems->hasMoreElements() ){
				Item* item = rowItems->nextElement();
				if ( NULL != item ){
					if ( true == item->containsPoint( event->getPoint() ) ){
						item->setSelected( true );
						break;
					}
				}
			}
		}
	}
}

void TableControl::mouseMove( MouseEvent* event ){
	CustomControl::mouseMove( event );
}

void TableControl::mouseUp( MouseEvent* event ){
	CustomControl::mouseUp( event );
}

void TableControl::mouseDblClick(  MouseEvent* event )
{
	CustomControl::mouseDblClick( event );
	if ( NULL != m_selectedCellItem ){
		if ( true == m_selectedCellItem->isItemEditable() ){
			m_currentItemEditor = m_selectedCellItem->getItemEditor();
			if ( NULL != m_currentItemEditor ){
				m_currentItemEditor->setItemToEdit( m_selectedCellItem );
				Control* editorControl = m_currentItemEditor->getEditingControl();
				if ( NULL != editorControl ){
					Rect bounds;
					uint32 c = m_selectedCellItem->getColumn();
					uint32 r = m_selectedCellItem->getRow();
					for ( int i=1;i<=c;i++){
						bounds.m_left +=  m_columnWidths[i-1];
					}

					for ( int j=1;j<=r;j++){
						bounds.m_top +=  m_rowHeights[j-1];
					}

					bounds.m_right = bounds.m_left + m_columnWidths[c];
					bounds.m_bottom = bounds.m_top + m_rowHeights[r];
					editorControl->setBounds( &bounds );
					
					this->add( editorControl );
					m_currentEditingControl = editorControl;
					m_currentEditingControl->setFocus( true );
				}
			}
		}
	}
}

void TableControl::OnMouseDblClick( MouseEvent* event )
{
	
}

void TableControl::setColumnWidth( const uint32& column, const uint32& width )
{
	m_columnWidths[column] = width;
	this->repaint();
}

void TableControl::setBounds( Rect* rect ) throw( InvalidPeer )
{
	//double oldWidth = this->getWidth();
	//double dxPercent = rect->getWidth() / oldWidth;
	CustomControl::setBounds( rect );
}

void TableControl::setAutoResizeColumns( const bool& autoResizeColumns )
{
	this->m_autoResizeColumns = autoResizeColumns;
}

void TableControl::resetColumnWidths()
{
	
}