/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Rect.h"
#include "GraphicsContext.h"
#include "Control.h"
#include "CustomControl.h"
#include "Container.h"
#include "Image.h"
#include "ToolbarDock.h"
#include "Toolbar.h"

using namespace VCF;

ToolbarDock::ToolbarDock()
{
	setContainer( this );
	m_dockSide = TDS_TOP;
	m_backgroundImage = NULL;
	m_isUsingBackground = false;
	m_allowsDockDragging = false;
	m_backgroundStyle = TBKS_NONE;
	this->setAlignment( ALIGN_TOP );
}

ToolbarDock::~ToolbarDock()
{

}

void ToolbarDock::add( Control* child )
{
	AbstractContainer::add( child );
}

void ToolbarDock::add( Control* child, const AlignmentType& alignment )
{
	AbstractContainer::add( child, alignment );
}

ToolbarDockSide ToolbarDock::getDockSide()
{
	return m_dockSide;
}

void ToolbarDock::setDockSide( const ToolbarDockSide& dockSide )
{
	m_dockSide = dockSide;
}

Image* ToolbarDock::getBackground()
{
	return m_backgroundImage;
}

void ToolbarDock::setBackground( Image* background )
{
	this->m_backgroundImage = background;
}

ToolbarBackgroundStyle ToolbarDock::getBackgroundStyle()
{
	return this->m_backgroundStyle;
}

void ToolbarDock::setBackgroundStyle( const ToolbarBackgroundStyle& backgroundStyle )
{
	m_backgroundStyle = backgroundStyle;
}

bool ToolbarDock::isUsingBackground()
{
	return this->m_isUsingBackground;
}

void ToolbarDock::setUsingBackground( const bool& usingBackground )
{
	m_isUsingBackground = usingBackground;
}

bool ToolbarDock::allowsDragging()
{
	return this->m_allowsDockDragging;
}

void ToolbarDock::setAllowsDragging( const bool& allowsDrag )
{
	m_allowsDockDragging = allowsDrag;
}

void ToolbarDock::paint( GraphicsContext* context )
{
	CustomControl::paint( context );
	if ( CS_DESIGNING == this->getComponentState() ){
		Rect r = *this->getBounds();
		r.inflate( -1, -1 );
		Color* oldColor = context->getColor();
		context->setColor( Color::getColor("black") );
		context->rectangle( &r );
		context->strokePath();
		context->setColor( oldColor );

	}

}

void ToolbarDock::mouseDown( MouseEvent* event )
{

}

void ToolbarDock::mouseMove( MouseEvent* event )
{

}

void ToolbarDock::mouseUp( MouseEvent* event )
{

}

void ToolbarDock::addToolbar( Toolbar* toolbar )
{

}

Toolbar* ToolbarDock::findToolbar( Point* pt )
{
	Toolbar* result = NULL;
	return result;
}