/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Window.cpp

#include "Window.h"
#include "UIToolkit.h"
#include "InvalidPointerException.h"
#include "ClassRegistry.h"
#include "InvalidPeer.h"
#include "Rect.h"



using namespace VCF;

Window::Window()
{
	

	this->m_Peer = UIToolkit::getDefaultUIToolkit()->createWindowPeer( this );
	if ( NULL == m_Peer ){
		throw InvalidPeer( MAKE_ERROR_MSG(NO_PEER), __LINE__ );
	}
	m_Peer->setControl( this );
	m_windowPeer = (WindowPeer*)m_Peer;

	m_menuBar = NULL;
}


Window::~Window()
{
	
}

void Window::paint(GraphicsContext * context)
{
	Frame::paint( context );
}

void Window::setCaption( const String& caption )
{
	Frame::setCaption( caption );
	this->m_Peer->setText( caption );
}

Rect* Window::getClientBounds()
{
	if ( NULL == m_windowPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	return this->m_windowPeer->getClientBounds();
}

void  Window::setClientBounds( Rect* bounds )
{
	if ( NULL == m_windowPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	m_windowPeer->setClientBounds( bounds );
}

void Window::resizeChildren()
{
	if ( true == alignWork() ){
		Rect* bounds = getClientBounds();
		Rect rect( 0.0, 0.0, bounds->getWidth(), bounds->getHeight() );
		this->doAlign( ALIGN_TOP, &rect );
		this->doAlign( ALIGN_BOTTOM, &rect );
		this->doAlign( ALIGN_LEFT, &rect );
		this->doAlign( ALIGN_RIGHT, &rect );
		this->doAlign( ALIGN_CLIENT, &rect );
	}
}

void Window::beforeDestroy( ComponentEvent* event )
{
	delete m_Peer;

	if ( NULL != m_menuBar ){
		delete m_menuBar;
		m_menuBar = NULL;
	}
	m_Peer = NULL;
}

MenuBar* Window::getMenuBar()
{
	return this->m_menuBar;
}

void Window::setMenuBar( MenuBar* menuBar )
{
	this->m_menuBar = menuBar;
	if ( NULL != m_menuBar ){
		m_menuBar->setFrame( this );
	}
}

void Window::close() 
{
	WindowEvent event( this, WINDOW_EVENT_CLOSE );
	notify_onWindowClose( &event );

	if ( NULL == m_windowPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	m_windowPeer->close();
}

void Window::setFrameStyle( const FrameStyleType& frameStyle )
{
	if ( NULL == m_windowPeer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
	}
	
	
	m_frameStyle = frameStyle;

	if ( CS_DESIGNING != this->getComponentState() ){
		m_windowPeer->setFrameStyle( m_frameStyle );
	}
}

void Window::setFrameTopmost( const bool& isTopmost )
{
	Frame::setFrameTopmost( isTopmost );
	
	if ( CS_DESIGNING != this->getComponentState() ){ 
		if ( NULL == m_windowPeer ){
			throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);	
		}
		
		m_windowPeer->setFrameTopmost( isTopmost );
	}
}